#include <dx/dx.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#define NLAT 30
#define NLONG 181

/* This module takes the NS and EW components of current and calculates the
 * ground magnetic field perturbation using the method of Kisabeth and Rostoker
 * 
 * The module has three inputs
 *   In[0] = NS Current - DX Field
 *   In[1] = EW Current - DX Field
 *   In[2] = Stations
 *
 * and it has two outputs 
 *   Out[0] = XY Currents - DX Field
 *   Out[1] = Magnetometer HDZ - DX Field
 *
*/

int ioninterp(Field Base,Field Intrp);

int CalcHDZ(Field Theta,Field Phi,Field MagHDZ);

int IntKReq6(float fCStatTh,float fSStatTh,float fStatLong,
            Field fPhi, Field FTheta,float *fpHDZ);

float KReq6HPhiInt(float fCStatTh, float fSStatTh, float fStatLong, float fCTh,
                   float fSTh, int i, Field fPhi, Field fTheta);

float KReq6H(float fCStatTh, float fSStatTh, float fStatLong, float fCTh,
             float fSTh, float fPhi, float fJew, float fJns);

float KReq6DPhiInt(float fCStatTh, float fSStatTh, float fStatLong, float fCTh,
                   float fSTh, int i, Field fPhi, Field fTheta);

float KReq6D(float fCStatTh, float fSStatTh, float fStatLong, float fCTh,
             float fSTh, float fPhi, float fJew, float fJns);

float KReq6ZPhiInt(float fCStatTh, float fSStatTh, float fStatLong, float fCTh,
                   float fSTh, int i, Field fPhi, Field fTheta);

float KReq6Z(float fCStatTh, float fSStatTh, float fStatLong, float fCTh,
             float fSTh, float fPhi, float fJew, float fJns);

Error m_MagStat(Object *in, Object *out)
{
  /* Inputs */
  Field fJTheta;  /* Theta (NS) current */
  Array aJThetaData; /* Theta Data */
  float *fpJThetaData; /* Ptr to Data elements */
  Array aJThetaPts; /* Theta Pts */
  float *fpJThetaPts; /* Ptr to Pts elements */
  Array aJThetaConnect;  /* Array of connections */
  Field fJPhi;    /* Phi (EW) current */
  Array aJPhiData; /* Phi Data */
  float *fpJPhiData; /* Ptr to Data elements */
  Array aJPhiPts; /* Phi Pts */
  float *fpJPhiPts; /* Ptr to Pts elements */
  Array aJPhiConnect;  /* Array of connections */
  Field fStations; /* Magnetometer station locatations */
  Array aStationsData; /* Stations Data */
  float *fpStationsData; /* Ptr to Data elements */
  Array aStationsPts; /* Stations Pts */
  float *fpStationsPts; /* Ptr to Pts elements */
  Array aStationsConnect;  /* Array of connections */
  
  /* Outputs */
  Array aJxyData=NULL;  /* Array for Jxy Data */
  float *fpJxyData;      /* Ptr Jxy Data */
  Array aJxyPts=NULL;   /* Array for Jxy Pts */
  float *fpJxyPts;       /* Ptr Jxy Pts data */
  Array aJxyConnect=NULL;    /* Array for Jxy Field Connections */
  int *ipJxyConnect;     /* Ptr to connections for Jxy Field */
  Field fJxy;           /* Jxy Field */
  Array aMagHDZData=NULL;  /* Array for Mag Data */
  float *fpMagHDZData;      /* Ptr Mag Data */
  Array aMagHDZPts=NULL;   /* Array for Mag Pts */
  float *fpMagHDZPts;       /* Ptr Mag Pts data */
  Array aMagHDZConnect=NULL;    /* Array for Mag Field Connections */
  int *ipMagHDZConnect;     /* Ptr to connections for Mag Field */
  Field fMagHDZ;           /* Mag Field */
  Array aRegIonData=NULL;  /* Array for RegIon Data */
  float *fpRegIonData;      /* Ptr RegIon Data */
  Array aRegIonPts=NULL;   /* Array for RegIon Pts */
  float *fpRegIonPts;       /* Ptr RegIon Pts data */
  Array aRegIonConnect=NULL;    /* Array for RegIon Field Connections */
  int *ipRegIonConnect;     /* Ptr to connections for RegIon Field */
  Field fRegIon;           /* RegIon Field */
 

  /* Variable Declerations */
  Field fIntrpTheta;
  Array aIntrpThetaData=NULL; /* J Theta on Regular Grid */
  float *fpIntrpThetaData; /* Ptr to data */
  Field fIntrpPhi;
  Array aIntrpPhiData=NULL; /* J Phi on Regular Grid */
  float *fpIntrpPhiData; /* Ptr to data */
  
  int i,j,k; /* Looping vars */
  int njp1,nk2p3,iSize,iNpts,iNStations; /* Size vars */
  int iErr;
  int iRank,iaShape[10],iaDim2D[2],iaLatDim[2]; 
  float *fpX, *fpY, *fpZ, *fpVal;
  float fX,fY,fZ,fR,fCoLat,fAng,fTheta,fPhi,fScale;
  float faCoLat[NLAT], faLong[NLONG];
  float fkDeg2Rad, fDtheta, fDphi;
  Type type;
  Category cat;
  int rank;

  FILE *FpFile;
  int irows,icols;
  float fjx,fjy,fjz,fx,fy,fz;


  if (!in[0]) {
     DXSetError(ERROR_BAD_PARAMETER,"Missing NS Current");
     goto error;
  } else {
     fJTheta = (Field)in[0];
     aJThetaData = (Array)DXGetComponentValue((Field)in[0],"data");
     fpJThetaData = DXGetArrayData(aJThetaData);
     aJThetaPts = (Array)DXGetComponentValue((Field)in[0],"positions");
     fpJThetaPts = DXGetArrayData(aJThetaPts);
     aJThetaConnect = (Array)DXGetComponentValue((Field)in[0],"connections");
     DXQueryGridConnections(aJThetaConnect,&iRank,iaDim2D);
     nk2p3 = iaDim2D[0];
     njp1 = iaDim2D[1];
     iSize = njp1*nk2p3;
     DXMessage("njp1 %d nk2p3 %d",njp1,nk2p3);
  }
  if (!in[1]) {
     DXSetError(ERROR_BAD_PARAMETER,"Missing EW Current");
     goto error;
  } else {
     fJPhi=(Field)in[1];
     aJPhiData = (Array)DXGetComponentValue((Field)in[1],"data");
     fpJPhiData = DXGetArrayData(aJPhiData);
     aJPhiPts = (Array)DXGetComponentValue((Field)in[1],"positions");
     fpJPhiPts = DXGetArrayData(aJPhiData);
  }
  if (!in[2]) {
     DXSetError(ERROR_BAD_PARAMETER,"Missing Magnetometer Stations");
     goto error;
  } else {
     aStationsData = (Array)DXGetComponentValue((Field)in[2],"data");
     fpStationsData = DXGetArrayData(aStationsData);
     aStationsPts = (Array)DXGetComponentValue((Field)in[2],"positions");
     fpStationsPts = DXGetArrayData(aStationsData);
     aStationsConnect = (Array)DXGetComponentValue((Field)in[2],"connections");
     DXQueryGridConnections(aStationsConnect,&iRank,iaShape);
     iNStations = iaShape[0];
     DXMessage("Stations %d",iNStations);
  }

  DXMessage("Welcome to magnetometer");
  DXMessage("NLONG = %d NLAT = %d",NLONG, NLAT);
  iaLatDim[0]=NLONG;
  iaLatDim[1]=NLAT;

  /* Setup regular ionospheric grid */
  aRegIonData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
  DXAddArrayData(aRegIonData,0,NLAT*NLONG,NULL);
  fpRegIonData=DXGetArrayData(aRegIonData);

  aIntrpThetaData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,1);
  DXAddArrayData(aIntrpThetaData,0,NLAT*NLONG,NULL);
  fpIntrpThetaData=DXGetArrayData(aIntrpThetaData);

  aIntrpPhiData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,1);
  DXAddArrayData(aIntrpPhiData,0,NLAT*NLONG,NULL);
  fpIntrpPhiData=DXGetArrayData(aIntrpPhiData);

  aRegIonPts = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
  DXAddArrayData(aRegIonPts,0,NLAT*NLONG,NULL);
  fpRegIonPts=DXGetArrayData(aRegIonPts);

  aMagHDZData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
  DXAddArrayData(aMagHDZData,0,iNStations,NULL);
  fpMagHDZData=DXGetArrayData(aMagHDZData);

  aMagHDZPts = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
  DXAddArrayData(aMagHDZPts,0,iNStations,NULL);
  fpMagHDZPts=DXGetArrayData(aMagHDZPts);

  DXMessage("Done with array allocation");
  fRegIon = DXNewField(); 
  DXSetStringAttribute((Object)aRegIonData,"dep","positions");
  DXSetComponentValue(fRegIon, "data", (Object)aRegIonData);
  DXSetComponentValue(fRegIon, "positions", (Object)aRegIonPts);
  aRegIonConnect = DXMakeGridConnectionsV(2,iaLatDim);
  DXSetComponentValue(fRegIon, "connections", (Object)aRegIonConnect);
  DXMessage("RegIon compelte");

  fIntrpTheta = DXNewField(); 
  DXSetStringAttribute((Object)aIntrpThetaData,"dep","positions");
  DXSetComponentValue(fIntrpTheta, "data", (Object)aIntrpThetaData);
  DXSetComponentValue(fIntrpTheta, "positions", (Object)aRegIonPts);
  DXSetComponentValue(fIntrpTheta, "connections", (Object)aRegIonConnect);
  DXMessage("IntrpTheta compelte");

  fIntrpPhi = DXNewField(); 
  DXSetStringAttribute((Object)aIntrpPhiData,"dep","positions");
  DXSetComponentValue(fIntrpPhi, "data", (Object)aIntrpPhiData);
  DXSetComponentValue(fIntrpPhi, "positions", (Object)aRegIonPts);
  DXSetComponentValue(fIntrpPhi, "connections", (Object)aRegIonConnect);
  DXMessage("IntrpPhi compelte");

  fMagHDZ = DXNewField();
  DXSetStringAttribute((Object)aMagHDZData,"dep","positions");
  DXSetComponentValue(fMagHDZ, "data", (Object)aMagHDZData);
  DXSetComponentValue(fMagHDZ, "positions", (Object)aStationsPts);
  DXSetComponentValue(fMagHDZ, "connections", (Object)aStationsConnect);
  DXMessage("MagHDZ compelte");

  fkDeg2Rad = M_PI/180.0;
  fDtheta = 1.0*fkDeg2Rad;
  fDphi = 2.0*fkDeg2Rad;
  for(i=0;i<NLAT;i++) {
      faCoLat[i] = i*fDtheta;
  }
  for(i=0;i<NLONG;i++) {
      faLong[i] = i*fDphi;
  }
  DXMessage("Arrays creating RegIon");
  for(i=0;i<NLAT;i++) {
    for(j=0;j<NLONG;j++) {
      fpRegIonPts[0+3*(NLAT*j+i)]=sin(faCoLat[i])*cos(faLong[j]);
      fpRegIonPts[1+3*(NLAT*j+i)]=sin(faCoLat[i])*sin(faLong[j]);
      fpRegIonPts[2+3*(NLAT*j+i)]=cos(faCoLat[i]);
      fpRegIonData[0+3*(NLAT*j+i)]=0.0;
      fpRegIonData[1+3*(NLAT*j+i)]=0.0;
      fpRegIonData[2+3*(NLAT*j+i)]=0.0;
    }
  }

  iErr=ioninterp(fJTheta,fIntrpTheta);
  iErr=ioninterp(fJPhi,fIntrpPhi);
  for(i=0;i<NLAT;i++) {
    for(j=0;j<NLONG;j++) {
    fX = fpRegIonPts[0+3*(NLAT*j+i)];
    fY = fpRegIonPts[1+3*(NLAT*j+i)];
    fZ = fpRegIonPts[2+3*(NLAT*j+i)];
    fR = sqrt(fX*fX+fY*fY+fZ*fZ);
    fCoLat = acos(fZ/fR);
    fAng = atan2(fY,fX);
    fTheta = fpIntrpThetaData[NLAT*j+i];
    fPhi = fpIntrpPhiData[NLAT*j+i];
    fpRegIonData[0+3*(NLAT*j+i)]=fTheta*cos(fAng)*cos(fCoLat)-fPhi*sin(fAng);
    fpRegIonData[1+3*(NLAT*j+i)]=fTheta*sin(fAng)*cos(fCoLat)+fPhi*cos(fAng);
    //fpRegIonData[0+3*(NLAT*j+i)]=fTheta;
    //fpRegIonData[1+3*(NLAT*j+i)]=fPhi;
    }
  }
//  FpFile=fopen("/home/wiltbemj/tmp/ioncurrents.txt","r");
//  fscanf(FpFile,"%d %d",&irows,&icols);
//  DXMessage("Rows %d Cols %d",irows,icols);
//  for(i=0;i<NLAT;i++) {
//    for(j=0;j<NLONG;j++) {
//       fscanf(FpFile,"%d %d %f %f %f %f %f %f",&irows,&icols,
//              &fjx,&fjy,&fjz,&fx,&fy,&fz);
//       fpRegIonData[0+3*(NLAT*icols+irows)]=fjx;
//       fpRegIonData[1+3*(NLAT*icols+irows)]=fjy;
//       fpRegIonData[2+3*(NLAT*icols+irows)]=fjz;
//       fpRegIonPts[0+3*(NLAT*icols+irows)]=fx;
//       fpRegIonPts[1+3*(NLAT*icols+irows)]=fy;
//       fpRegIonPts[2+3*(NLAT*icols+irows)]=fz;
//    }
//  }
//  fclose(FpFile);
//  FpFile=fopen("/home/wiltbemj/tmp/ae.txt","r");
//  fscanf(FpFile,"%d %d",&irows,&icols);
//  DXMessage("Rows %d Cols %d",irows,icols);
//  for(i=0;i<iNStations;i++) {
//    fscanf(FpFile,"%d %f %f %f %f %f %f",&icols,
//              &fjx,&fjy,&fjz,&fx,&fy,&fz);
//    DXMessage("Stats %d %f %f %f %f %f %f",icols,
//              fjx,fjy,fjz,fx,fy,fz);
//       fpStationsPts[0+3*i]=fx;
//       fpStationsPts[1+3*i]=fy;
//       fpStationsPts[2+3*icols]=fz;
//       fpMagHDZPts[0+3*i]=fx;
//       fpMagHDZPts[1+3*i]=fy;
//       fpMagHDZPts[2+3*i]=fz;
//  }
//  DXMessage("Done with stations");
//  fclose(FpFile);

  /* For debuging purposes wwe change to SCW 20 in long and 5 deg in lat  
     for(j=0;j<NLAT;j++) {
     for(k=0;k<NLONG;k++) {
        fpIntrpThetaData[NLAT*k+j] = 0.0;
        fpIntrpPhiData[NLAT*k+j] = 0.0;
     }
   }

   for(j=15;j<21;j++) {
     for(k=0;k<2;k++) {
        //fpIntrpPhiData[NLAT*k+j] = -1.8;
        fpIntrpThetaData[NLAT*k+j] = 1.8;
     }
   }

   for(j=15;j<21;j++) {
     for(k=178;k<NLONG;k++) {
        //fpIntrpPhiData[NLAT*k+j] = -1.8;
        fpIntrpThetaData[NLAT*k+j] = 1.8;
     }
   }

  for(i=0;i<iNStations;i++) {
     fpMagHDZData[0+3*i]=0.0;
     fpMagHDZData[1+3*i]=0.0;
     fpMagHDZData[2+3*i]=0.0;
     fpMagHDZPts[0+3*i]=cos((60.0+i*20.0/iNStations)*M_PI/180.0);
     fpMagHDZPts[1+3*i]=0.0;
     fpMagHDZPts[2+3*i]=sin((60.0+i*20.0/iNStations)*M_PI/180.0);
  } 
  DXMessage("Done with station assignment");
*/

  DXMessage("Before CalcHDZ");
  iErr = CalcHDZ(fIntrpTheta,fIntrpPhi,fMagHDZ);

  if(!DXEndField(fRegIon)) {
    DXSetError(ERROR_INTERNAL,"Couldn't complete field");
    goto error;
  }
  if(!DXEndField(fIntrpTheta)) {
    DXSetError(ERROR_INTERNAL,"Couldn't complete field");
    goto error;
  }
  if(!DXEndField(fIntrpPhi)) {
    DXSetError(ERROR_INTERNAL,"Couldn't complete field");
    goto error;
  }
  if(!DXEndField(fMagHDZ)) {
    DXSetError(ERROR_INTERNAL,"Couldn't complete field");
    goto error;
  }

  DXMessage("All done!");
  out[0] = (Object)fRegIon;
  out[1] = (Object)fMagHDZ;
  return OK;

  error:
    DXDelete((Object)fRegIon);
    DXDelete((Object)aRegIonPts);
    DXDelete((Object)aRegIonData);
    DXDelete((Object)aRegIonConnect);

}

int ioninterp(Field fBase, Field fIntrp)
{
   
   int iNpts, i, j, k, jj, kk, g, h;
   int nlat, nlong,iRank;
   int iaDimsIntrp[2], iaDims[2], nj, nk, nk2, nSize;
   int *ipJ, *ipK, *ipFlag, iVeclen;
   float *fpXi, *fpYi, *fpZi,*fpValue;
   float fDx, fDy, fDxm1, fDym1;
   float *fpX, *fpData;
   char caFieldType[256];
  
   Array aBaseData,aBasePts,aBaseConnect;
   float *fpBaseData,*fpBasePts;
   Array aIntrpData,aIntrpPts,aIntrpConnect;
   float *fpIntrpData,*fpIntrpPts;

   DXMessage("ioninterp %d",fBase);
   i = DXEmptyField(fBase);
   DXMessage("Empty %d",i);
   /* Step zero setup pointers to data/position compoents */
   aBaseData = (Array)DXGetComponentValue(fBase,"data");
   if (aBaseData == NULL) DXMessage("abasedata failed");
   fpBaseData = DXGetArrayData(aBaseData);
   aBasePts = (Array)DXGetComponentValue(fBase,"positions");
   fpBasePts = DXGetArrayData(aBasePts);
   aBaseConnect = (Array)DXGetComponentValue(fBase,"connections");
   DXQueryGridConnections(aBaseConnect,&iRank,iaDims);
   nk = iaDims[0];
   nj = iaDims[1];
   nk2 = nk/2;
   nSize = nj*nk;
   DXMessage("base info %d %d",nj,nk);

   aIntrpData = (Array)DXGetComponentValue(fIntrp,"data");
   fpIntrpData = DXGetArrayData(aIntrpData);
   aIntrpPts = (Array)DXGetComponentValue(fIntrp,"positions");
   fpIntrpPts = DXGetArrayData(aIntrpPts);
   aIntrpConnect = (Array)DXGetComponentValue(fIntrp,"connections");
   DXQueryGridConnections(aIntrpConnect,&iRank,iaDims);
   nlat = iaDims[1];
   nlong = iaDims[0];
   iNpts = nlat*nlong;
   DXMessage("interp info LONG %d LAT %d",nlong,nlat);

   /* First get the locations to interpolate to */
   fpXi = (float *) malloc(iNpts*sizeof(float));
   fpYi = (float *) malloc(iNpts*sizeof(float));
   fpZi = (float *) malloc(iNpts*sizeof(float));
   for(g=0;g<nlat;g++) {
     for(h=0;h<nlong;h++) {
       fpXi[nlat*h+g] = fpIntrpPts[0+3*(nlat*h+g)];
       fpYi[nlat*h+g] = fpIntrpPts[1+3*(nlat*h+g)];
       fpZi[nlat*h+g] = fpIntrpPts[2+3*(nlat*h+g)];
     }
   }
   DXMessage("Got interp pts");
  


   /* Find out information about the field we are interpolating from */
   fpX = (float *) malloc(nj*sizeof(float));

   for(j=0;j<nj;j++) {
     fpX[j] = fpBasePts[0+3*(j+nj)];
     DXMessage("fpX[%d] %f",j,fpX[j]);
   }
   DXMessage("fpBasePts[1+3*(nj/2)] %f fpBasePts[1+3*(nj/2+(nk-1)*nj)] %f",
              fpBasePts[1+3*(nj/2)],fpBasePts[1+3*(nj/2+(nk-1)*nj)]);
   DXMessage("Got base pts");
   ipJ = (int *) malloc(iNpts*sizeof(int));
   ipK = (int *) malloc(iNpts*sizeof(int));
   ipFlag = (int *) malloc(iNpts*sizeof(int));

   /* First make sure point is inside grid */
   for(g=0;g<nlat;g++) {
     for(h=0;h<nlong;h++) {
         ipFlag[nlat*h+g] = 0;
         if( (fpXi[nlat*h+g] <= fpX[0]) && (fpXi[nlat*h+g] >= fpX[nj-1])) {
            if ((fpYi[nlat*h+g] <= fpBasePts[1+3*(nj/2)]) 
                && (fpYi[nlat*h+g] >= fpBasePts[1+3*(nj/2+(nk-1)*nj)])) {
                   ipFlag[nlat*h+g] = 1;
            }
         }
     }
   }
   DXMessage("Checked grid");
    
   /* Location of points surounding the interpolation pt */
   for(g=0;g<nlat;g++) {
     for(h=0;h<nlong;h++) {
        jj = 0;
        if(ipFlag[nlat*h+g]) {
           for(j=0;j<nj-1;j++) {
             if((fpX[j+1] <= fpXi[nlat*h+g]) && 
                (fpXi[nlat*h+g] <= fpX[j])) jj = j;  
           }
        }
        ipJ[nlat*h+g] = jj;
     }
   }
   DXMessage("Found J locs");

   for(g=0;g<nlat;g++) {
     for(h=0;h<nlong;h++) {
        kk = 0;
        if(ipFlag[nlat*h+g]) {
           if (fpYi[nlat*h+g] > 0) {
              for(k=0;k<nk2;k++) {
               if((fpBasePts[1+3*(ipJ[nlat*h+g]+nj*(k+1))] <= 
                   fpYi[nlat*h+g]) &&
                  (fpYi[nlat*h+g] <= fpBasePts[1+3*(ipJ[nlat*h+g]+nj*k)]))
                   kk = k;
              }
           } else {
              for(k=nk2-1;k<nk-1;k++) {
               if((fpBasePts[1+3*(ipJ[nlat*h+g]+nj*(k+1))] <= 
                   fpYi[nlat*h+g]) &&
                   (fpYi[nlat*h+g] <= fpBasePts[1+3*(ipJ[nlat*h+g]+nj*k)])) 
                    kk = k;
              }
          }
        }
        ipK[nlat*h+g] = kk;
     }
  }
  DXMessage("Found K locs");
  DXMessage("ipJ[0,0] %d ipK[0,0] %d ",ipJ[0],ipK[0]);

  /* Do the bilinear interpolation */
  for(g=0;g<nlat;g++) {
    for(h=0;h<nlong;h++) {
       if(ipFlag[nlat*h+g]) {
         fDx = (fpXi[nlat*h+g]-fpX[ipJ[nlat*h+g]+1])/
               (fpX[ipJ[nlat*h+g]]-fpX[ipJ[nlat*h+g]+1]);
         fDy = (fpYi[nlat*h+g]-fpBasePts[1+
                3*(ipJ[nlat*h+g]+nj*ipK[nlat*h+g])])/
               (fpBasePts[1+3*(ipJ[g]+nj*(ipK[nlat*h+g]+1))]-
                fpBasePts[1+3*(ipJ[g]+nj*ipK[nlat*h+g])]);
         fDxm1 = (1.0-fDx);
         fDym1 = (1.0-fDy);
         fpIntrpData[nlat*h+g] = 
                 fDxm1*fDym1*fpBaseData[(ipJ[nlat*h+g]+1+nj*ipK[nlat*h+g])] +
                 fDx*fDym1*fpBaseData[(ipJ[nlat*h+g]+nj*ipK[nlat*h+g])] +
                 fDx*fDy*fpBaseData[(ipJ[nlat*h+g]+nj*(ipK[nlat*h+g]+1))] +
                 fDxm1*fDy*fpBaseData[(ipJ[nlat*h+g]+1+nj*(ipK[nlat*h+g]+1))]; 
         if (0==g && 0==h) {
           DXMessage("fDx %f",fDx);
           DXMessage("fDy %f",fDy);
           DXMessage("fpBaseData[ipj+1,ipk] %f", 
             fpBaseData[(ipJ[nlat*h+g]+1+nj*ipK[nlat*h+g])]);
           DXMessage("fpBaseData[ipj,ipk] %f", 
             fpBaseData[(ipJ[nlat*h+g]+nj*ipK[nlat*h+g])]);
           DXMessage("fpBaseData[ipj,ipk+1] %f", 
             fpBaseData[(ipJ[nlat*h+g]+nj*(ipK[nlat*h+g]+1))]);
           DXMessage("fpBaseData[ipj+1,ipk+1] %f", 
             fpBaseData[(ipJ[nlat*h+g]+1+nj*(ipK[nlat*h+g]+1))]);
         }
       } else {
         fpIntrpData[nlat*h+g] = -9999.9;
       }
    }
  }
  DXMessage("Completed interp");


  /* Clean up the allocated temproary space */
  free(fpXi);
  free(fpYi);
  free(fpZi);
  free(fpX);
  free(ipJ);
  free(ipK);
  free(ipFlag);
     
  return 1; 
}

int CalcHDZ(Field fTheta, Field fPhi, Field fMagHDZ)
{
   int i,j,k, iErr;
   int nlat, nlong, iNpts;   
   int nstats;
   float fkDeg2Rad, fDtheta, fDphi;
   float faCoLat[NLAT], faLong[NLONG], faHDZ[3];
   float *fpStatX, *fpStatY, *fpStatZ;
   float fR, fCoLat, fAng,*fpStatCoLat, *fpStatLong;
   float fCStatTh, fSStatTh;
   float *fpData;

   Array aMagHDZData,aMagHDZPts,aMagHDZConnect;
   float *fpMagHDZData,*fpMagHDZPts;
   int iRank, iaDims[10];

   DXMessage("Inside CalcHDZ");
   aMagHDZData = (Array)DXGetComponentValue(fMagHDZ,"data");
   fpMagHDZData = DXGetArrayData(aMagHDZData);
   aMagHDZPts = (Array)DXGetComponentValue(fMagHDZ,"positions");
   fpMagHDZPts = DXGetArrayData(aMagHDZPts);
   aMagHDZConnect = (Array)DXGetComponentValue(fMagHDZ,"connections");
   DXQueryGridConnections(aMagHDZConnect,&iRank,iaDims);
   nstats = iaDims[0];
   DXMessage("Number of stations %d",nstats);

   fpStatX = (float *) malloc(nstats*sizeof(float));
   fpStatY = (float *) malloc(nstats*sizeof(float));
   fpStatZ = (float *) malloc(nstats*sizeof(float));
   fpStatCoLat = (float *) malloc(nstats*sizeof(float));
   fpStatLong = (float *) malloc(nstats*sizeof(float));
   for(i=0;i<nstats;i++) {
      fpStatX[i] = fpMagHDZPts[0+3*i];
      fpStatY[i] = fpMagHDZPts[1+3*i];
      fpStatZ[i] = fpMagHDZPts[2+3*i];
      fR = sqrt(fpStatX[i]*fpStatX[i]+fpStatY[i]*fpStatY[i]+
                fpStatZ[i]*fpStatZ[i]);
      fpStatCoLat[i] = acos(fpStatZ[i]/fR);
      fpStatLong[i] = atan2(fpStatY[i],fpStatX[i]);
   }

   /* Now we determine the componets from the EW currents using KR eq 6 */
   for(i=0;i<nstats;i++) {
      fCStatTh = cos(fpStatCoLat[i]);
      fSStatTh = sin(fpStatCoLat[i]); 
      iErr = IntKReq6(fCStatTh,fSStatTh,fpStatLong[i],fPhi,fTheta,faHDZ);
      fpMagHDZData[0+3*i] = faHDZ[0];
      fpMagHDZData[1+3*i] = faHDZ[1];
      fpMagHDZData[2+3*i] = faHDZ[2];
   }
   
   
   /*Clean up space allocated for temporary varible */
   free(fpStatX);
   free(fpStatY);
   free(fpStatZ);
   free(fpStatCoLat);
   free(fpStatLong);

   return 1;
}

int IntKReq6(float fCStatTh, float fSStatTh, float fStatLong, 
             Field fPhi, Field fTheta, float *fpHDZ)
{
   int i,j,k;
   int nlat, nlong, iNpts;
   float fkDeg2Rad, fDtheta, fDphi;
   float kfMuq4Pi, kfRe, kfRi;
   float faCoLat[NLAT];
   float fH, fD, fZ, fAH, fBH, fAD, fBD, fAZ, fBZ;
   float fCTh, fSTh;

   kfMuq4Pi = 1.0e-7;
   kfRe = 6370.0e3;
   kfRi = kfRe + 100.0e3;

   /*First we create the information about the regular ionospheric grid */
   nlat = NLAT;
   nlong = NLONG;
   iNpts = nlat*nlong;
   fkDeg2Rad = M_PI/180.0;
   fDtheta = 1.0*fkDeg2Rad;
   fDphi = 2.0*fkDeg2Rad;
   
   for(i=0;i<nlat;i++) {
      faCoLat[i] = i*fDtheta;
   }

   /* Now we determine the H componet from KR eq 6a */
   fH = 0;
   fD = 0;
   fZ = 0;
   for(i=0;i<nlat-1;i++) {
     fCTh = cos(faCoLat[i]);
     fSTh = sin(faCoLat[i]);
     fH += KReq6HPhiInt(fCStatTh,fSStatTh,fStatLong,fCTh,fSTh,i,fPhi,
                        fTheta);
     fD += KReq6DPhiInt(fCStatTh,fSStatTh,fStatLong,fCTh,fSTh,i,fPhi,
                        fTheta);
     fZ += KReq6ZPhiInt(fCStatTh,fSStatTh,fStatLong,fCTh,fSTh,i,fPhi,
                        fTheta);
   }
   fCTh = cos(0.0);
   fSTh = sin(0.0);
   fAH = KReq6HPhiInt(fCStatTh,fSStatTh,fStatLong,fCTh,fSTh,0,fPhi,fTheta);
   fAD = KReq6DPhiInt(fCStatTh,fSStatTh,fStatLong,fCTh,fSTh,0,fPhi,fTheta);
   fAZ = KReq6ZPhiInt(fCStatTh,fSStatTh,fStatLong,fCTh,fSTh,0,fPhi,fTheta);
   fCTh = cos(faCoLat[nlat-1]);
   fSTh = sin(faCoLat[nlat-1]);
   fBH = KReq6HPhiInt(fCStatTh,fSStatTh,fStatLong,fCTh,fSTh,nlat-1,fPhi,
                      fTheta);
   fBD = KReq6DPhiInt(fCStatTh,fSStatTh,fStatLong,fCTh,fSTh,nlat-1,fPhi,
                      fTheta);
   fBZ = KReq6ZPhiInt(fCStatTh,fSStatTh,fStatLong,fCTh,fSTh,nlat-1,fPhi,
                      fTheta);
   fH = 0.5*fDtheta*(fAH+fBH+2*fH);
   fD = 0.5*fDtheta*(fAD+fBD+2*fD);
   fZ = 0.5*fDtheta*(fAZ+fBZ+2*fZ);
   fH *= kfRi*kfRi*kfMuq4Pi;
   fD *= kfRi*kfRi*kfMuq4Pi;
   fZ *= kfRi*kfRi*kfMuq4Pi;
   
   fpHDZ[0] = -1.0*fH;
   fpHDZ[1] = fD;
   fpHDZ[2] = -1.0*fZ;
   return 1;
}

float KReq6HPhiInt(float fCStatTh, float fSStatTh, float fStatLong, float fCTh,
                   float fSTh, int i, Field fPhi, Field fTheta )
{
   int j,k;
   int nlat, nlong, iNpts;
   float fkDeg2Rad, fDtheta, fDphi, fJew, fJew1, fJns, fJns1;
   float faLong[NLONG];
   float fH, fD, fZ, fA, fB;

   /*First we create the information about the regular ionospheric grid */
   nlat = NLAT;
   nlong = NLONG;
   iNpts = nlat*nlong;
   fkDeg2Rad = M_PI/180.0;
   fDtheta = 1.0*fkDeg2Rad;
   fDphi = 2.0*fkDeg2Rad;

   Array aPhiData,aThetaData;
   float *fpPhiData, *fpThetaData;

   aPhiData = (Array)DXGetComponentValue(fPhi,"data");
   fpPhiData = DXGetArrayData(aPhiData);
   aThetaData = (Array)DXGetComponentValue(fTheta,"data");
   fpThetaData = DXGetArrayData(aThetaData);
   
   
   for(k=0;k<nlong;k++) {
      faLong[k] = k*fDphi; 
   }   

   fH = 0.0;
   for(j=1;j<nlong-1;j++) {
      fJew = fpPhiData[nlat*j+i];
      fJns = fpThetaData[nlat*j+i];
      fH += KReq6H(fCStatTh,fSStatTh,fStatLong,fCTh,fSTh,faLong[j],fJew,fJns);
   }
   fJew = fpPhiData[nlat*0+i];
   fJew1 = fpPhiData[nlat*(nlong-1)+i];
   fJns = fpThetaData[nlat*0+i];
   fJns1 = fpThetaData[nlat*(nlong-1)+i];
   fH = 0.5*fDphi*(KReq6H(fCStatTh,fSStatTh,fStatLong,fCTh,fSTh,0.0,fJew,fJns) +
                   KReq6H(fCStatTh,fSStatTh,fStatLong,fCTh,fSTh,
                          faLong[nlong-1],fJew1,fJns1) + 2*fH);
   return fH;
}

float KReq6H(float fCStatTh, float fSStatTh, float fStatLong, float fCTh, 
             float fSTh, float fPhi, float fJew, float fJns) 
{
  
   float kfMuq4Pi, kfRe, kfRi, fAns;
   float fCStatPhimPhi, fSStatPhimPhi;
   float fA11, fA22, fA33, fFactor, fK23;
   float fA23, fA32, fK22;
   kfMuq4Pi = 1.0e-7;
   kfRe = 6370.0e3;
   kfRi = kfRe + 100.0e3;

   fCStatPhimPhi = cos(fStatLong-fPhi);
   fSStatPhimPhi = sin(fStatLong-fPhi);
   fA11 = fSStatTh*fSTh*fCStatPhimPhi+fCStatTh*fCTh;
   fA22 = fCStatTh*fCTh*fCStatPhimPhi+fSStatTh*fSTh;
   fA33 = fCStatPhimPhi;
   fFactor = sqrt(kfRi*kfRi+kfRe*kfRe-2*kfRi*kfRe*fA11);
   fFactor = pow(1.0/fFactor,3.0);
   fK23 = fFactor*(kfRe*fA33-kfRi*fA22);
 
   /* To include the Jns term we need to add K22J2  */
   fA23 = fCStatTh*fSStatPhimPhi;
   fA32 = -1.0*fCStatTh*fSStatPhimPhi;
   fK22 = fFactor*(kfRi*fA23+kfRe*fA32);
   fAns = (fK23*fJew+fK22*fJns)*fSTh;
   
   return fAns;
}
   
float KReq6DPhiInt(float fCStatTh, float fSStatTh, float fStatLong, float fCTh,
                   float fSTh, int i, Field fPhi, Field fTheta)
{
   int j,k;
   int nlat, nlong, iNpts;
   float fkDeg2Rad, fDtheta, fDphi, fJew, fJew1, fJns, fJns1;
   float faLong[NLONG];
   float fH, fD, fZ, fA, fB;

   Array aPhiData,aThetaData;
   float *fpPhiData, *fpThetaData;

   aPhiData = (Array)DXGetComponentValue(fPhi,"data");
   fpPhiData = DXGetArrayData(aPhiData);
   aThetaData = (Array)DXGetComponentValue(fTheta,"data");
   fpThetaData = DXGetArrayData(aThetaData);

   /*First we create the information about the regular ionospheric grid */
   nlat = NLAT;
   nlong = NLONG;
   iNpts = nlat*nlong;
   fkDeg2Rad = M_PI/180.0;
   fDtheta = 1.0*fkDeg2Rad;
   fDphi = 2.0*fkDeg2Rad;
   
   for(k=0;k<nlong;k++) {
      faLong[k] = k*fDphi; 
   }   

   fD = 0.0;
   for(j=1;j<nlong-1;j++) {
      fJew = fpPhiData[nlat*j+i];
      fJns = fpThetaData[nlat*j+i];
      fD += KReq6D(fCStatTh,fSStatTh,fStatLong,fCTh,fSTh,faLong[j],fJew,fJns);
   }
   fJew = fpPhiData[nlat*0+i];
   fJew1 = fpPhiData[nlat*(nlong-1)+i];
   fJns = fpThetaData[nlat*0+i];
   fJns1 = fpThetaData[nlat*(nlong-1)+i];
   fD = 0.5*fDphi*(KReq6D(fCStatTh,fSStatTh,fStatLong,fCTh,fSTh,0.0,fJew,fJns) +
                   KReq6D(fCStatTh,fSStatTh,fStatLong,fCTh,fSTh,
                          faLong[nlong-1],fJew1,fJns1) + 2*fD);
   return fD;
}

float KReq6D(float fCStatTh, float fSStatTh, float fStatLong, float fCTh, 
             float fSTh, float fPhi, float fJew, float fJns) 
{
  
   float kfMuq4Pi, kfRe, kfRi, fAns;
   float fCStatPhimPhi, fSStatPhimPhi;
   float fA11, fA23, fA32, fFactor, fK33;
   float fA22, fA33, fK32;
   kfMuq4Pi = 1.0e-7;
   kfRe = 6370.0e3;
   kfRi = kfRe + 100.0e3;

   fCStatPhimPhi = cos(fStatLong-fPhi);
   fSStatPhimPhi = sin(fStatLong-fPhi);
   fA11 = fSStatTh*fSTh*fCStatPhimPhi+fCStatTh*fCTh;
   fA23 = fCStatTh*fSStatPhimPhi;
   fA32 = -1.0*fCTh*fSStatPhimPhi;
   fFactor = sqrt(kfRi*kfRi+kfRe*kfRe-2*kfRi*kfRe*fA11);
   fFactor = pow(1.0/fFactor,3.0);
   fK33 = -1.0*fFactor*(kfRe*fA23+kfRi*fA32);

   /* To include the Jns effects we need to add k32J2 term  */
   fA22 = fCStatTh*fCTh*fCStatPhimPhi+fSStatTh*fSTh;
   fA33 = fCStatPhimPhi;
   fK32 = fFactor*(kfRi*fA33-kfRe*fA22);
   fAns = (fK32*fJns+fK33*fJew)*fSTh;
   return fAns;
}

float KReq6ZPhiInt(float fCStatTh, float fSStatTh, float fStatLong, float fCTh,
                   float fSTh, int i, Field fPhi, Field fTheta)
{
   int j,k;
   int nlat, nlong, iNpts;
   float fkDeg2Rad, fDtheta, fDphi, fJew, fJew1, fJns, fJns1;
   float faLong[NLONG];
   float fH, fD, fZ, fA, fB;

   Array aPhiData,aThetaData;
   float *fpPhiData, *fpThetaData;

   aPhiData = (Array)DXGetComponentValue(fPhi,"data");
   fpPhiData = DXGetArrayData(aPhiData);
   aThetaData = (Array)DXGetComponentValue(fTheta,"data");
   fpThetaData = DXGetArrayData(aThetaData);
   /*First we create the information about the regular ionospheric grid */
   nlat = NLAT;
   nlong = NLONG;
   iNpts = nlat*nlong;
   fkDeg2Rad = M_PI/180.0;
   fDtheta = 1.0*fkDeg2Rad;
   fDphi = 2.0*fkDeg2Rad;
   
   for(k=0;k<nlong;k++) {
      faLong[k] = k*fDphi; 
   }   

   fZ = 0.0;
   for(j=1;j<nlong-1;j++) {
      fJew = fpPhiData[nlat*j+i];
      fJns = fpThetaData[nlat*j+i];
      fB =  KReq6Z(fCStatTh,fSStatTh,fStatLong,fCTh,fSTh,faLong[j],fJew,fJns);
      fZ += fB;
   }
   fJew = fpPhiData[nlat*0+i];
   fJew1 = fpPhiData[nlat*(nlong-1)+i];
   fJns = fpThetaData[nlat*0+i];
   fJns1 = fpThetaData[nlat*(nlong-1)+i];
   fZ = 0.5*fDphi*(KReq6Z(fCStatTh,fSStatTh,fStatLong,fCTh,fSTh,0.0,fJew,fJns) +
                   KReq6Z(fCStatTh,fSStatTh,fStatLong,fCTh,fSTh,
                          faLong[nlong-1],fJew1,fJns1) + 2*fZ);
   return fZ;
}

float KReq6Z(float fCStatTh, float fSStatTh, float fStatLong, float fCTh, 
             float fSTh, float fPhi, float fJew, float fJns) 
{
  
   float kfMuq4Pi, kfRe, kfRi, fAns;
   float fCStatPhimPhi, fSStatPhimPhi;
   float fA11, fA12, fFactor, fK13;
   float fA13, fK12;
   kfMuq4Pi = 1.0e-7;
   kfRe = 6370.0e3;
   kfRi = kfRe + 100.0e3;

   fCStatPhimPhi = cos(fStatLong-fPhi);
   fSStatPhimPhi = sin(fStatLong-fPhi);
   fA11 = fSStatTh*fSTh*fCStatPhimPhi+fCStatTh*fCTh;
   fA12 = fSStatTh*fCTh*fCStatPhimPhi-fCStatTh*fSTh;
   fFactor = sqrt(kfRi*kfRi+kfRe*kfRe-2*kfRi*kfRe*fA11);
   fFactor = pow(1.0/fFactor,3.0);
   fK13 = -1.0*fFactor*kfRi*fA12;

   /* To include the Jns term we need to add K12J2 term */
   fA13 = fSStatTh*fSStatPhimPhi;
   fK12 = fFactor*kfRi*fA13;
   fAns = (fK12*fJns+fK13*fJew)*fSTh;
   
   return fAns;
}
