#include "netcdf.h"
#include <dx/dx.h>
#include <stdio.h>
#include <math.h>

/* ---------------------------------------------------------------------------
 * readAMIE
 *
 * INPUTS
 *   filename: Name of file with data
 *   timestep: timestep in data file to read
 *  
 * OUTPUTS
 *   ion: field containing ionospheric data
 *   ut: Vector containing ut array [yr mon day hr min sec]
 */
int GetAMIEVar(char *cpName,int iStep,int iFileId, 
               int iLat,int iLon,int iNumVars,
               int iLoc,float *fpData);
#define ij(i,j) (((j)+iLon*(i)))
#define cij(c,i,j) ((c)+3*((j)+iLon*(i)))
#define vij(c,i,j) ((c)+iNumVars*((j)+iLon*(i)))

Error m_ReadAMIE(Object *in, Object *out) 
{

  /* Variable Declerations */
  /* Inputs */
  char *cpFileName = NULL; /*Ptr to filename */
  int iStep;               /*Timestep to read */
  char *cpHeight = NULL;   /* Ptr to Height */
  
  /* Outputs */
  Array aGrid = NULL;           /* Array for Grid locations */
  float *fpGrid;                /* Ptr to complete Grid  location data */
  Array aData = NULL;        /* Array for MHD Field data */
  float *fpData;             /* Ptr to MHD data */
  Array aConnect = NULL;     /* Array containing conncetions for MHD field */
  int *ipConnect;            /* Ptr to MHD Connection data */
  Field fIon = NULL;            /* MHD data Field */
  double *dpLon,*dpLat;   /* Temp space for coordinate reads */
  int iLev, iLon, iLat, iSteps;         /* Size of coord dims */
  double *dpTmp;                 /* Variable Data */
  Array aUT = NULL;              /* UT Array */
  int *ipUT;                     /* Ptr to UT array Data */
  
  /* Other varibles */
  int iStatus, iDimId, iVarId, iFileId; /* netCDF vars */
  int iaEdge2d[3], iaStart[2],iaCount[2];  /* netCDF vars */
  int iReadFlag; /* Flag for mallocs completed */
  int iRdPts, iNumPts; /* Size of arrays */
  int h,i,j,k;
  int iNumVars,iCount; /* Num of varibles in output field */
  float fDeg2Rad, fRe, fReInv;
  int *ipYear, *ipMon, *ipDay, *ipHr, *ipMin; /* UT Data */
  
  
   
  /* Get input parameters */
  if(!in[0]) {
    DXSetError(ERROR_BAD_PARAMETER,"Missing Data Filename");
    goto error;
  } else if (!DXExtractString(in[0],&cpFileName)) {
    DXSetError(ERROR_BAD_PARAMETER,"Data Filename must be string");
    goto error;
  }
  if(!in[1]) {
    DXSetError(ERROR_BAD_PARAMETER,"Missing Timestep");
    goto error;
  } else if (!DXExtractInteger(in[1],&iStep)) {
    DXSetError(ERROR_BAD_PARAMETER,"Step must be integer");
    goto error;
  }
   

  iReadFlag = 0;
  iStatus = nc_open(cpFileName,NC_NOWRITE,&iFileId);
  if(iStatus != NC_NOERR) {
     DXSetError(ERROR_BAD_PARAMETER,"Unable to open netCDF file %s",cpFileName);
     goto error;
  }
  
  /* Check to see if requested step is valid */
  iStatus = nc_inq_dimid(iFileId,"time",&iDimId);
  iStatus = nc_inq_dimlen(iFileId,iDimId,&iSteps);
  if ((iStep < 0) || (iStep >= iSteps)) {
     DXSetError(ERROR_BAD_PARAMETER,
       "Requested step, %d, not valid.  Max Steps %d",iStep,iSteps-1);
     goto error;
  }
    
  /* Get UT from for step */
  ipYear = (int*)malloc(sizeof(int)*iSteps);
  ipMon = (int*)malloc(sizeof(int)*iSteps);
  ipDay = (int*)malloc(sizeof(int)*iSteps);
  ipHr = (int*)malloc(sizeof(int)*iSteps);
  ipMin = (int*)malloc(sizeof(int)*iSteps);
  iStatus = nc_inq_varid(iFileId,"year",&iVarId);
  iStatus = nc_get_var_int(iFileId,iVarId,ipYear);
  iStatus = nc_inq_varid(iFileId,"month",&iVarId);
  iStatus = nc_get_var_int(iFileId,iVarId,ipMon);
  iStatus = nc_inq_varid(iFileId,"day",&iVarId);
  iStatus = nc_get_var_int(iFileId,iVarId,ipDay);
  iStatus = nc_inq_varid(iFileId,"hour",&iVarId);
  iStatus = nc_get_var_int(iFileId,iVarId,ipHr);
  iStatus = nc_inq_varid(iFileId,"minute",&iVarId);
  iStatus = nc_get_var_int(iFileId,iVarId,ipMin);
  aUT = DXNewArray(TYPE_INT,CATEGORY_REAL,1,6);
  DXAddArrayData(aUT, 0, 1, NULL);
  ipUT = (int *)DXGetArrayData(aUT);
  ipUT[0]= ipYear[iStep];  /* Year */
  ipUT[1] = ipMon[iStep];  /* Mon */
  ipUT[2] = ipDay[iStep];  /* Day */
  ipUT[3] = ipHr[iStep];  /* Hr */
  ipUT[4] = ipMin[iStep];  /* Min */
  ipUT[5] = 0;  /* Sec */

  /* Get coordinate arrays and sizes */
  
  iStatus = nc_inq_dimid(iFileId,"lon",&iDimId);
  iStatus = nc_inq_dimlen(iFileId,iDimId,&iLon);
  dpLon = (double*)malloc(sizeof(double)*iLon);
  iStatus = nc_inq_varid(iFileId,"lon",&iVarId);
  iStatus = nc_get_var_double(iFileId,iVarId,dpLon);

  iStatus = nc_inq_dimid(iFileId,"lat",&iDimId);
  iStatus = nc_inq_dimlen(iFileId,iDimId,&iLat);
  dpLat = (double*)malloc(sizeof(double)*iLat);
  iStatus = nc_inq_varid(iFileId,"lat",&iVarId);
  iStatus = nc_get_var_double(iFileId,iVarId,dpLat);

  iReadFlag = 1;
 
#ifdef DEBUG
  DXMessage("After coord read %d %d %d",iSteps,iLon,iLat);
#endif

  iaEdge2d[0]=iLat;
  iaEdge2d[1]=iLon;
  iRdPts = iLat*iLon;
  iNumPts = iLat*(iLon);  

  aGrid = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
  DXAddArrayData(aGrid,0,iNumPts,NULL);
  fpGrid = (float*)DXGetArrayData(aGrid);
  fDeg2Rad = M_PI/180.0;
  fRe = 6371.2; 
  fReInv = 1.0/fRe;
  iCount = 0;
  /* Use Geopotential Height as radius in calculation */
  for(i=0;i<iLat;i++) {
    for(j=0;j<iLon;j++) {
    iCount++;
      fpGrid[cij(0,i,j)] = cos(dpLat[i]*fDeg2Rad)*
                           cos(dpLon[j]*fDeg2Rad)*1.0;
      fpGrid[cij(1,i,j)] = cos(dpLat[i]*fDeg2Rad)*
                           sin(dpLon[j]*fDeg2Rad)*1.0;
      fpGrid[cij(2,i,j)] = sin(dpLat[i]*fDeg2Rad)*1.0;
    }
  }
 
  iNumVars = 20;
  aData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,iNumVars);
  DXAddArrayData(aData,0,iNumPts,NULL);
  fpData = (float*)DXGetArrayData(aData);

  iStatus = GetAMIEVar("pot_model",iStep,iFileId,iLat,iLon,iNumVars,0,fpData);
  iStatus = GetAMIEVar("ekv_model",iStep,iFileId,iLat,iLon,iNumVars,1,fpData);
  iStatus = GetAMIEVar("efx_model",iStep,iFileId,iLat,iLon,iNumVars,2,fpData);
  iStatus = GetAMIEVar("tped_model",iStep,iFileId,iLat,iLon,iNumVars,3,
                         fpData);
  iStatus = GetAMIEVar("thall_model",iStep,iFileId,iLat,iLon,iNumVars,4,
                         fpData);
  iStatus = GetAMIEVar("pot",iStep,iFileId,iLat,iLon,iNumVars,5,fpData);
  iStatus = GetAMIEVar("ekv",iStep,iFileId,iLat,iLon,iNumVars,6,fpData);
  iStatus = GetAMIEVar("efx",iStep,iFileId,iLat,iLon,iNumVars,7,fpData);
  iStatus = GetAMIEVar("tped",iStep,iFileId,iLat,iLon,iNumVars,8,fpData);
  iStatus = GetAMIEVar("thall",iStep,iFileId,iLat,iLon,iNumVars,9,fpData);
  iStatus = GetAMIEVar("ped",iStep,iFileId,iLat,iLon,iNumVars,10,fpData);
  iStatus = GetAMIEVar("hall",iStep,iFileId,iLat,iLon,iNumVars,11,fpData);
  iStatus = GetAMIEVar("E_east",iStep,iFileId,iLat,iLon,iNumVars,12,fpData);
  iStatus = GetAMIEVar("E_north",iStep,iFileId,iLat,iLon,iNumVars,13,fpData);
  iStatus = GetAMIEVar("JEN",iStep,iFileId,iLat,iLon,iNumVars,14,fpData);
  iStatus = GetAMIEVar("JEE",iStep,iFileId,iLat,iLon,iNumVars,15, fpData);
  iStatus = GetAMIEVar("FAC",iStep,iFileId,iLat,iLon,iNumVars,16,fpData);
  iStatus = GetAMIEVar("JOULE",iStep,iFileId,iLat,iLon,iNumVars,17,fpData);
  iStatus = GetAMIEVar("EQCURR",iStep,iFileId,iLat,iLon,iNumVars,18,fpData);
  iStatus = GetAMIEVar("CURRPOT",iStep,iFileId,iLat,iLon,iNumVars,19,fpData);
    
  
  fIon = DXNewField();
  if(!fIon) {
    DXSetError(ERROR_NO_MEMORY,"Creation of Ion field failed");
    goto error;
  }
 
  /* Set the dependence of the Data Array to be on positons*/
  if(!DXSetStringAttribute((Object)aData,"dep","positions")) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array Dep");
     goto error;
  }
  /* Set the MHD Data array as the data component of the Ion Field*/
  if(!DXSetComponentValue(fIon,"data",(Object)aData)) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array of Ion Field");
     goto error;
  }
  /* Set the MHD Pts array as the position component of the Ion Field*/
  if(!DXSetComponentValue(fIon,"positions",(Object)aGrid)) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Position Array of Ion Field");
     goto error;
  }
  aConnect = DXMakeGridConnectionsV(2,iaEdge2d);
  if (!aConnect) {
    DXSetError(ERROR_INTERNAL,"Couldn't Make Ion Grid Connections");
    goto error;
  }

  if (!DXSetComponentValue(fIon,"connections",(Object)aConnect)) {
      DXSetError(ERROR_INTERNAL,"Couldn't set Ion Grid Connections");
      goto error;
  }

  if(!DXEndField(fIon)) {
     DXSetError(ERROR_INTERNAL,"Couldn't complete field");
     goto error;
  }

#ifdef DEBUG
  DXMessage("After Field");
#endif
 
  
  out[0] = (Object) fIon;
  out[1] = (Object) aUT;
 
  free(dpLon);
  free(dpLat);
  free(ipYear);
  free(ipMon);
  free(ipDay);
  free(ipHr);
  free(ipMin);
 
  return OK;

  error: 
    DXDelete((Object) fIon);
    DXDelete((Object) aData);
    DXDelete((Object) aGrid);
    DXDelete((Object) aConnect);
    if(1 == iReadFlag) {
      free(dpLon);
      free(dpLat);
      free(dpTmp);
      free(ipYear);
      free(ipMon);
      free(ipDay);
      free(ipHr);
      free(ipMin);
    }
}

int GetAMIEVar(char *cpName,int iStep,int iFileId, 
               int iLat,int iLon,int iNumVars,
               int iLoc,float *fpData)
{
  int i,j,k;
  int iRdPts,iaStart[3],iaCount[3],iVarId,iStatus;
  double *dpTmp;

  iRdPts = iLat*iLon;
  dpTmp = (double*)malloc(sizeof(double)*iRdPts);
  iaStart[0] = iStep; iaStart[1] = 0 ; iaStart[2] = 0; 
  iaCount[0] = 1; iaCount[1] = iLat; iaCount[2] = iLon;
  iStatus = nc_inq_varid(iFileId,cpName,&iVarId);
  iStatus = nc_get_vara_double(iFileId, iVarId, iaStart, iaCount, dpTmp);

  for(i=0;i<iLat;i++) {
    for(j=0;j<iLon;j++) {
      fpData[vij(iLoc,i,j)] = dpTmp[ij(i,j)];
    }
  }
  free(dpTmp);
  return iStatus;
}

