#include <dx/dx.h>
#include <stdio.h>
#include <math.h>

/*--------------------------------------------------------------------------
 *
 * This function attempts to create a 3D set of cross hair axes
 *
 * inputs
 * xmin - min of x-axis
 * xmax - max of x-axis
 *
 * output
 *
 * out - axis geometry as a group
 *
 * -------------------------------------------------------------------------
 */
 
static RGBColor rgbWhite = {1.0,1.0,1.0};

#define AXIS(x0,y0,z0,x1,y1,z1,rgb) { \
    Line line;                          \
    line.p = iAxisPt;                        \
    line.q = iAxisPt+1;                      \
    DXAddLine(FAxis, iAxisLines++, line);           \
    DXAddPoint(FAxis, line.p, DXPt(x0,y0,z0));      \
    DXAddPoint(FAxis, line.q, DXPt(x1,y1,z1));      \
    DXAddColor(FAxis, line.p, rgb);                 \
    DXAddColor(FAxis, line.q, rgb);         \
    iAxisPt += 2;                            \
}   

#define TICK(x0,y0,z0,x1,y1,z1,rgb) { \
    Line line;                          \
    line.p = iTickPt;                        \
    line.q = iTickPt+1;                      \
    DXAddLine(FTick, iTickLines++, line);           \
    DXAddPoint(FTick, line.p, DXPt(x0,y0,z0));      \
    DXAddPoint(FTick, line.q, DXPt(x1,y1,z1));      \
    DXAddColor(FTick, line.p, rgb);                 \
    DXAddColor(FTick, line.q, rgb);         \
    iTickPt += 2;                            \
}   

Error 
m_Axes(Object *in, Object *out) {

  /* Input Varibles */
  float fXmin, fXmax, fXTickDelta;   /* X min and max*/
  float fYmin, fYmax, fYTickDelta;   /* Y min and max*/
  float fZmin, fZmax, fZTickDelta;   /* Z min and max*/
  float fTickSize;		     /* Size of Ticks*/
  float fLabelSize;		     /* Scale Factor for Labels*/

  /* Outputs and varibles used in their creation */
  Group GAxes;		/* Group to hold composite object of axis,tics, etc*/
  Field FAxis;		/* Field to each axis*/
  int iAxisPt=0;        /* Currnet Point within Axis field*/
  int iAxisLines=0;	/* Current Line within Axis field*/
  Field FTick;		/* Field to hold each tic mark*/
  int iTickPt=0;        /* Currnet Point within tics field*/
  int iTickLines=0;	/* Current Line within tics field*/
  float *fpXTickLoc;	/* Ptr to array of XTick locations*/
  int iNumXTick;        /* Number of Ticks on the Xaxis*/
  float *fpYTickLoc;	/* Ptr to array of YTick locations*/
  int iNumYTick;        /* Number of Ticks on the Yaxis*/
  float *fpZTickLoc;	/* Ptr to array of ZTick locations*/
  int iNumZTick;        /* Number of Ticks on the Zaxis*/
  
  /* Others */
  int i,j;
  float fTick,fWidth;
  Matrix MScale;
  Object OFont, OText;
  static float sfAscent;
  Xform xform = NULL;
  char **cppXLabels, *cpTmp, caTmp[100];

  /* Get inputs*/
  if(!in[0]) {
    DXSetError(ERROR_BAD_PARAMETER,"Missing Xmin");
    goto error;
  } else if (!DXExtractFloat(in[0],&fXmin)) {
    DXSetError(ERROR_BAD_PARAMETER,"Xmin must be float");
    goto error;
  }
  if(!in[1]) {
    DXSetError(ERROR_BAD_PARAMETER,"Missing Xmax");
    goto error;
  } else if (!DXExtractFloat(in[1],&fXmax)) {
    DXSetError(ERROR_BAD_PARAMETER,"Xmax must be float");
    goto error;
  }
  if(!in[2]) {
    DXSetError(ERROR_BAD_PARAMETER,"Missing XTickDelta");
    goto error;
  } else if (!DXExtractFloat(in[2],&fXTickDelta)) {
    DXSetError(ERROR_BAD_PARAMETER,"XTickDelta must be float");
    goto error;
  }

  if(!in[3]) {
    DXSetError(ERROR_BAD_PARAMETER,"Missing Ymin");
    goto error;
  } else if (!DXExtractFloat(in[3],&fYmin)) {
    DXSetError(ERROR_BAD_PARAMETER,"Ymin must be float");
    goto error;
  }
  if(!in[4]) {
    DXSetError(ERROR_BAD_PARAMETER,"Missing Ymax");
    goto error;
  } else if (!DXExtractFloat(in[4],&fYmax)) {
    DXSetError(ERROR_BAD_PARAMETER,"Ymax must be float");
    goto error;
  }
  if(!in[5]) {
    DXSetError(ERROR_BAD_PARAMETER,"Missing YTickDelta");
    goto error;
  } else if (!DXExtractFloat(in[5],&fYTickDelta)) {
    DXSetError(ERROR_BAD_PARAMETER,"YTickDelta must be float");
    goto error;
  }


  if(!in[6]) {
    DXSetError(ERROR_BAD_PARAMETER,"Missing Zmin");
    goto error;
  } else if (!DXExtractFloat(in[6],&fZmin)) {
    DXSetError(ERROR_BAD_PARAMETER,"Zmin must be float");
    goto error;
  }
  if(!in[7]) {
    DXSetError(ERROR_BAD_PARAMETER,"Missing Zmax");
    goto error;
  } else if (!DXExtractFloat(in[7],&fZmax)) {
    DXSetError(ERROR_BAD_PARAMETER,"Zmax must be float");
    goto error;
  }
  if(!in[8]) {
    DXSetError(ERROR_BAD_PARAMETER,"Missing ZTickDelta");
    goto error;
  } else if (!DXExtractFloat(in[8],&fZTickDelta)) {
    DXSetError(ERROR_BAD_PARAMETER,"ZTickDelta must be float");
    goto error;
  }

  if(!in[9]) {
    DXSetError(ERROR_BAD_PARAMETER,"Missing TickSize");
    goto error;
  } else if (!DXExtractFloat(in[9],&fTickSize)) {
    DXSetError(ERROR_BAD_PARAMETER,"TickSize must be float");
    goto error;
  }

  if(!in[10]) {
    DXSetError(ERROR_BAD_PARAMETER,"Missing LabelSize");
    goto error;
  } else if (!DXExtractFloat(in[10],&fLabelSize)) {
    DXSetError(ERROR_BAD_PARAMETER,"LabelSize must be float");
    goto error;
  }


  GAxes = DXNewGroup();
  if (!GAxes) goto error;

  /* create the field for ticks */
  FTick = DXNewField();
  if (!FTick) goto error;
  DXSetFloatAttribute((Object)FTick,"fuzz",2);
  DXSetMember(GAxes,NULL,(Object)FTick);
 
  /* create the field for ticks */
  FAxis = DXNewField();
  if (!FAxis) goto error;
  DXSetFloatAttribute((Object)FAxis,"fuzz",1);
  DXSetMember(GAxes,NULL,(Object)FAxis);

  /* Now we add the three lines which make up the XYZ axes*/
  AXIS(fXmin,0.0,0.0,fXmax,0.0,0.0,rgbWhite); /* X axis*/
  AXIS(0.0,fYmin,0.0,0.0,fYmax,0.0,rgbWhite); /* Y axis*/
  AXIS(0.0,0.0,fZmin,0.0,0.0,fZmax,rgbWhite); /* Z axis*/


  iNumXTick = (fXmax - fXmin)/fXTickDelta + 1;
  cppXLabels = (char **)DXAllocate(iNumXTick*sizeof(char *));
  OFont = DXGetFont("fixed",&sfAscent,NULL);
  for(i=0;i<3;i++) {
     MScale.b[i] = 0.0;
     for(j=0;j<3;j++) {
       MScale.A[i][j] = 0.0;
     }
  }
  MScale.A[0][0] = fLabelSize;
  MScale.A[1][1] = fLabelSize;
  MScale.A[2][2] = 1.0;
  MScale = DXScale(fLabelSize,fLabelSize,1.0);
  for(i=0;i<iNumXTick;i++) { 
    fTick = fXmin+i*fXTickDelta;
    TICK(fTick,-fTickSize,0.0,fTick,fTickSize,0.0,rgbWhite);
    TICK(fTick,0.0,-fTickSize,fTick,0.0,fTickSize,rgbWhite);
    cpTmp = (char*) DXAllocate(100*sizeof(char));
    sprintf(cpTmp,"%3.1f",fTick);
    cppXLabels[i] = cpTmp;
    sprintf(caTmp,"%s",cppXLabels[i]);
    OText = DXGeometricText(caTmp,OFont,&fWidth);
    xform = DXNewXform((Object)OText,MScale);
    xform = DXNewXform((Object)xform,DXTranslate(DXVec(fTick,0,0)));
    DXSetMember(GAxes,NULL,(Object)xform);
  }
  iNumYTick = (fYmax - fYmin)/fYTickDelta+1;
  for(i=0;i<iNumYTick;i++) { 
    fTick = fYmin+i*fYTickDelta;
    TICK(-fTickSize,fTick,0.0,fTickSize,fTick,0.0,rgbWhite);
    TICK(0.0,fTick,-fTickSize,0.0,fTick,fTickSize,rgbWhite);
  }
  iNumZTick = (fZmax - fZmin)/fZTickDelta+1;
  for(i=0;i<iNumZTick;i++) { 
    fTick = fZmin+i*fZTickDelta;
    TICK(-fTickSize,0.0,fTick,fTickSize,0.0,fTick,rgbWhite);
    TICK(0.0,-fTickSize,fTick,0.0,fTickSize,fTick,rgbWhite);
  }

  
  out[0] = (Object)GAxes;

  /* Cleanup*/
  for(i=0;i<iNumXTick;i++) {
     DXFree(cppXLabels[i]);
  }
  DXFree(cppXLabels);
  return OK;

error:
  DXDelete((Object)GAxes);
  return ERROR;
}
  
  



