#include <dx/dx.h>
#include <stdio.h>
#include <math.h>

/* --------------------------------------------------------------------------
 * readbatsrus
 *
 * This module reads the Finite element data files created by the BATSRUS code
 *
 * INPUTS
 *   filename: Name of file with data
 *
 * OUTPUTS
 *   mhd: field containing mhd data
 *
 */

#define SIZE 256
Error m_ReadBATSRUS(Object *in, Object *out) 

{

  /* Varible Declarations */

  /* Inputs */
  char *cpMHDName = NULL; 	/* Ptr to MHD Data file name */
  
  /* Outputs */

  Array aGrid = NULL; 		/* Array for Grid locations */
  float *fpGrid;		/* Ptr to complete Grid  location data */
  Array aMHDData = NULL;	/* Array for MHD Field data */
  float *fpMHDData;		/* Ptr to MHD data */ 
  Array aMHDConnect = NULL;	/* Array containing conncetions for MHD field */
  int *ipMHDConnect;		/* Ptr to MHD Connection data */
  Field fMHD = NULL;		/* MHD data Field */

  /* Other varibles */
  FILE *FpMHDFile;
  char caLine[SIZE], caVars[SIZE], caEType[SIZE], *cpPtr;
  int i,j;
  int iNumPts, iNumCells, iNumVars, iNumVarsm3, iPtsPerCell,iReadFlag;
  float *fpDataRead;
  int *ipCellRead;

  if(!in[0]) {
    DXSetError(ERROR_BAD_PARAMETER,"Missing Data Filename");
    goto error;
  } else if (!DXExtractString(in[0],&cpMHDName)) {
    DXSetError(ERROR_BAD_PARAMETER,"Data Filename must be string");
    goto error;
  }

  /* First we figure out the Number of Varilbe, Points and Cells by processing
   * the header information in the data file
   */

  FpMHDFile = fopen(cpMHDName,"r");
  if (NULL == FpMHDFile) {
     DXSetError(ERROR_BAD_PARAMETER,"Unable to open file");
     goto error;
  }
  /* First line contains run description which I ignore */
  fgets(caLine,SIZE,FpMHDFile);
  /* Second line contains varible names and units */
  fgets(caVars,SIZE,FpMHDFile);
  printf("%s\n",caVars);
  cpPtr = strtok(caVars,",");
  iNumVars = 0;
  while (NULL != cpPtr) {
    cpPtr = strtok(NULL,",");
    iNumVars++;
  }
  iNumVarsm3 = iNumVars-3;
  /* Thrid line contains information on number of points and cells as well as
   * the type of elements.  The format of this line is very specific and I have
   * used that information to construct the correct fscanf statement.
   */
   fscanf(FpMHDFile,"%s %s %s %d %s %s %d %s %s %s",caLine,caLine,caLine,&iNumPts,
                                        caLine,caLine, &iNumCells,
                                        caLine,caLine,caEType);

   /* Output File Size info to user */
   DXMessage("File contains %d points with %d variables making %d cells",
             iNumPts,iNumVars,iNumCells);

    /* Setup the nubmer of points per cell using cell type information */
    if(0 == strcmp(caEType,"ET=BRICK")) {
       iPtsPerCell = 8;
    }
    if(0 == strcmp(caEType,"ET=QUADRILATERAL")) {
       iPtsPerCell = 4;
    }
   /* Now we allocate some temporary space to efficently read in the varbile
    * data before placing it DX Arrays
    */

   DXMessage("Cell Type %s with %d Pts per cell",caEType,iPtsPerCell);

   fpDataRead = (float*)malloc(iNumPts*iNumVars*sizeof(float));
   ipCellRead = (int*)malloc(iNumCells*iPtsPerCell*sizeof(int));
   iReadFlag = 1;
    

   /* The cell locations and data are the first in the data file */
   for(i=0;i<iNumPts*iNumVars;i++) {
     fscanf(FpMHDFile,"%g",fpDataRead+i);
   }
  
   /* then the elements of each cell are listed */
   for(i=0;i<iNumCells*iPtsPerCell;i++) {
     fscanf(FpMHDFile,"%d",ipCellRead+i);
   }


  /* Now that we know how big things are we allocate space for the grid array
   * and then assign the gird locations
   */

   DXMessage("Preparing to allocate Grid Arrays");
   aGrid = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
   if(!aGrid) {
     DXSetError(ERROR_NO_MEMORY,"Creation of Grid array failed");
     goto error;
   }
   if(!DXAddArrayData(aGrid,0,iNumPts,NULL)) {
     DXSetError(ERROR_NO_MEMORY,"Allocation of Grid data array failed");
     goto error;
   }
   fpGrid = (float *)DXGetArrayData(aGrid);
   /* Now that we have space lets assign the grid locations */

   DXMessage("Pts %g %g %g \n",fpDataRead[0],fpDataRead[1],fpDataRead[2]);
   for(i=0;i<iNumPts;i++) {
      fpGrid[0+3*i] = fpDataRead[0+iNumVars*i];  /* X is first varible */
      fpGrid[1+3*i] = fpDataRead[1+iNumVars*i];  /* Y is second varible */
      fpGrid[2+3*i] = fpDataRead[2+iNumVars*i];  /* Z is third varible */
   }
   DXMessage("Grid %g %g %g \n",fpGrid[0],fpGrid[1],fpGrid[2]);

   /* Now we allocate space for the MHD data */
   
   DXMessage("Preparing to allocate MHD Arrays");
  aMHDData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,iNumVarsm3);
  if(!aMHDData) {
    DXSetError(ERROR_NO_MEMORY,"Creation of MHD Data Array failed");
    goto error;
  }
  if(!DXAddArrayData(aMHDData,0,iNumPts,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of MHD Data Array failed");
    goto error;
  }

  /* Get Ptr to Data portion of MHDData Array*/
  fpMHDData = (float*)DXGetArrayData(aMHDData);

  /* Then we assign the MHD data to the array structure */
  for(i=0;i<iNumPts;i++) {
    for(j=0;j<(iNumVarsm3);j++) {
      fpMHDData[j+(iNumVarsm3)*i] = fpDataRead[j+3+iNumVars*i];  /* Skip xyz  */
    }
  }

  /* Now that we have all the data we can create the output field */
   DXMessage("Preparing to allocate MHD Field");
  fMHD = DXNewField();
  if(!fMHD) {
    DXSetError(ERROR_NO_MEMORY,"Creation of MHD field failed");
    goto error;
  }

  /* Set the dependence of the Data Array to be on positons*/
  if(!DXSetStringAttribute((Object)aMHDData,"dep","positions")) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array Dep");
     goto error;
  }
  /* Set the MHD Data array as the data component of the MHD Field*/
  if(!DXSetComponentValue(fMHD,"data",(Object)aMHDData)) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array of MHD Field");
     goto error;
  }
  /* Set the MHD Pts array as the position component of the MHD Field*/
  if(!DXSetComponentValue(fMHD,"positions",(Object)aGrid)) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Position Array of MHD Field");
     goto error;
  }


  /* Connections from the BASTRUS data files are different than OpenDX
   * connections in two ways.  First OpenDX are C based so they start with zero
   * instead of 1 so all values must be reduced by 1.  Secondly, the order of
   * points in the OpenDX Quads is LL UL LR UR whereas the BATSRUS order is UL
   * UR LR LL (clockwise).  The following loops take care of both these issues
   * while assigning the data to MHDConnect array
   */

   DXMessage("Preparing to allocate MHD Connections");
  aMHDConnect = DXNewArray(TYPE_INT,CATEGORY_REAL,1,iPtsPerCell);
  if (!aMHDConnect) {
    DXSetError(ERROR_NO_MEMORY,"Creation MHD Grid Connections Array Failed");
    goto error;
  }
  if(!DXAddArrayData(aMHDConnect,0,iNumCells,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of MHD Grid Connections Failed");
    goto error;
  }
  if (4 == iPtsPerCell) {
    if(!DXSetStringAttribute((Object)aMHDConnect,"element type","quads")) {
       DXSetError(ERROR_INTERNAL,"Couldn't set Element type");
       goto error;
    }
  }
  if (8 == iPtsPerCell) {
    if(!DXSetStringAttribute((Object)aMHDConnect,"element type","cubes")) {
       DXSetError(ERROR_INTERNAL,"Couldn't set Element type");
       goto error;
    }
  }
  ipMHDConnect = (int*)DXGetArrayData(aMHDConnect);
  DXMessage("Beginning cell assignments");
  for(i=0;i<iNumCells;i++) {
    if(4 == iPtsPerCell) {
      ipMHDConnect[0+iPtsPerCell*i] = ipCellRead[3+iPtsPerCell*i]-1;
      ipMHDConnect[1+iPtsPerCell*i] = ipCellRead[0+iPtsPerCell*i]-1;
      ipMHDConnect[2+iPtsPerCell*i] = ipCellRead[2+iPtsPerCell*i]-1;
      ipMHDConnect[3+iPtsPerCell*i] = ipCellRead[1+iPtsPerCell*i]-1;
    }
    if(8 == iPtsPerCell) {
      ipMHDConnect[0+iPtsPerCell*i] = ipCellRead[3+iPtsPerCell*i]-1;
      ipMHDConnect[1+iPtsPerCell*i] = ipCellRead[0+iPtsPerCell*i]-1;
      ipMHDConnect[2+iPtsPerCell*i] = ipCellRead[2+iPtsPerCell*i]-1;
      ipMHDConnect[3+iPtsPerCell*i] = ipCellRead[1+iPtsPerCell*i]-1;
      ipMHDConnect[4+iPtsPerCell*i] = ipCellRead[7+iPtsPerCell*i]-1;
      ipMHDConnect[5+iPtsPerCell*i] = ipCellRead[4+iPtsPerCell*i]-1;
      ipMHDConnect[6+iPtsPerCell*i] = ipCellRead[6+iPtsPerCell*i]-1;
      ipMHDConnect[7+iPtsPerCell*i] = ipCellRead[5+iPtsPerCell*i]-1;
    }
  }
  DXMessage("Done with cell assignments");
  if (!DXSetComponentValue(fMHD,"connections",(Object)aMHDConnect)) {
    DXSetError(ERROR_INTERNAL,"Could set MHD Grid Connections");
    goto error;
  }
  /* End Field sets default attributes */
  if(!DXEndField(fMHD)) {
     DXSetError(ERROR_INTERNAL,"Couldn't complete field");
     goto error;
  }


  /* Output the field */
  out[0] = (Object)fMHD;

  /* Clean up temporaries */
  DXMessage("Before data free");
  free(fpDataRead);
  free(ipCellRead); 
  DXMessage("After data free");

  return OK;

  error:
    DXDelete((Object) fMHD);
    DXDelete((Object) aMHDData);
    DXDelete((Object) aGrid);
    DXDelete((Object) aMHDConnect);
    if(1 == iReadFlag) {
      free(fpDataRead);
      free(ipCellRead); 
    }
  return ERROR;
}

