#include <dx/dx.h>
#include "fits.h"

/* --------------------------------------------------------------------------
 * readfits
 *
 * This module reads any FITS formated files
 *
 * INPUTS
 *   filename: Name of file with data
 *
 *   hdu: Number of starting FITS file extension
 *
 * OUTPUTS
 *   mhd: 3D field containing mhd data
 */

Error m_ReadFITS(Object *in, Object *out)

{
   /* Inputs */
  char *filename = NULL;

  /* Outputs */
  Array aGrid = NULL;
  float *fpGrid;
  Array aMHDData = NULL;
  float *fpMHDData;
  Array aMHDConnect = NULL;
  Field MHD = NULL;
  
  /* Other Variables */
  int status, nfound,anynull;
  int i,j,k;
  long naxes[2],fpixel,npixels;
  float nullval,cntx,cnty;
  char datet[26], timet[26], dates[26], comment[80];
   
  if(!in[0]) {
    DXSetError(ERROR_BAD_PARAMETER,"Missing Data Filename");
    goto error;
  } else if (!DXExtractString(in[0],&filename)) {
    DXSetError(ERROR_BAD_PARAMETER,"Data Filename must be string");
    goto error;
  }

#ifdef DEBUG
 DXMessage("Filename %s",filename);
#endif
 status = 0;

 fitsfile *fptr;       /* pointer to the FITS file, defined in fitsio.h */
 if ( fits_open_file(&fptr, filename, READONLY, &status) ) 
 printerror( status );
 

 /* read the NAXIS1 and NAXIS2 keyword to get image size */

  if ( fits_read_keys_lng(fptr, "NAXIS", 1, 2, naxes, &nfound, &status) )
    printerror( status );
  npixels = naxes[0] * naxes[1];
#ifdef DEBUG
  DXMessage("%d %d %d %d",nfound,naxes[0],naxes[1],npixels);
#endif
  aMHDData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,1);
  DXAddArrayData(aMHDData,0,npixels,NULL);
  fpMHDData = (float*)DXGetArrayData(aMHDData);
  fpixel=1;
  nullval=0;
  if ( fits_read_img(fptr, TFLOAT, fpixel, npixels, &nullval,
                  fpMHDData, &anynull, &status) )
    printerror( status );
 

  /* Make the grid positions pixel locations */
  aGrid = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,2);
  DXAddArrayData(aGrid,0,npixels,NULL);
  fpGrid = (float*)DXGetArrayData(aGrid);
  for(j=0;j<naxes[0];j++)
     for(k=0;k<=naxes[1];k++)
       {

         fpGrid[0+2*(k+(naxes[1])*(j))]= k;
         fpGrid[1+2*(k+(naxes[1])*(j))]= j;

       }


  /* Do the final steps for the DXField */
  MHD = DXNewField();
 if(!MHD) {
   DXSetError(ERROR_NO_MEMORY,"Creation of MHD field failed");
   goto error;
 }
 
 /* Set the dependence of the Data Array to be on positons*/
 if(!DXSetStringAttribute((Object)aMHDData,"dep","positions")) {
   DXSetError(ERROR_INTERNAL,"Couldn't set Data Array Dep");
   goto error;
 }
 /* Set the MHD Data array as the data component of the MHD Field*/
 if(!DXSetComponentValue(MHD,"data",(Object)aMHDData)) {
   DXSetError(ERROR_INTERNAL,"Couldn't set Data Array of MHD Field");
   goto error;
 }
 /* Set the MHD Pts array as the position component of the MHD Field*/
 if(!DXSetComponentValue(MHD,"positions",(Object)aGrid)) {
   DXSetError(ERROR_INTERNAL,"Couldn't set Position Array of MHD Field");
   goto error;
 }
 
 aMHDConnect = DXMakeGridConnections(2,naxes[0],naxes[1]);
 if (!aMHDConnect) {
   DXSetError(ERROR_INTERNAL,"Couldn't Make MHD Grid Connections");
   goto error;
 }

 if (!DXSetComponentValue(MHD,"connections",(Object)aMHDConnect)) {
   DXSetError(ERROR_INTERNAL,"Couldn't set MHD Grid Connections");
   goto error;
 }
 
 if(!DXEndField(MHD)) {
   DXSetError(ERROR_INTERNAL,"Couldn't complete field");
   goto error;
 }

#ifdef DEBUG
 DXMessage("After Field ");
#endif

 /* Reading possible key information */
 if(fits_read_key(fptr,TFLOAT, "CRPIX1",&cntx,comment, &status)){
   DXMessage("Failed to read CRPIX1");
 }
 if(fits_read_key(fptr,TFLOAT, "CRPIX2",&cnty,comment, &status)){
   DXMessage("Failed to read CRPIX2");
 }
 if(fits_read_key(fptr,TSTRING, "DATE-OBS",datet,comment, &status)){
   DXMessage("Failed to read DATE-OBS");
 }
 if(fits_read_key(fptr,TSTRING, "TIME-OBS",timet,comment, &status)){
   DXMessage("Failed to read TIME-OBS");
 }

  if ( fits_close_file(fptr, &status) )
   printerror( status );
 status=sprintf(dates,"%s %s",datet,timet);
#ifdef DEBUG
 DXMessage("%f %f %s",cntx,cnty,datet);
#endif
 
 out[0] = (Object) MHD;
 out[1]=(Object)DXMakeFloat(cntx);
 out[2]=(Object)DXMakeFloat(cnty);
 out[3]=(Object)DXMakeString(dates);
 return OK;
 
 error:
 DXDelete((Object) MHD);
 DXDelete((Object) aMHDData);
 DXDelete((Object) aGrid);
 DXDelete((Object) aMHDConnect);
 return ERROR;
}
