#include "netcdf.h"
#include <dx/dx.h>
#include <stdio.h>
#include <math.h>

/* --------------------------------------------------------------------------
 * readHelio
 *
 * This module reads the data files for the solar wind created by the CORHEL code
 *
 * INPUTS
 *   filename: Name of file with data
 *
 * OUTPUTS
 *   mhd: field containing mhd data
 *
 */

#define SIZE 256
Error m_ReadHelio(Object *in, Object *out) 

{

  /* Varible Declarations */

  /* Inputs */
  char *cpMHDName = NULL; 	/* Ptr to MHD Data file name */
  
  /* Outputs */

  Array aGrid = NULL; 		/* Array for Grid locations */
  float *fpGrid;		/* Ptr to complete Grid  location data */
  Array aMHDData = NULL;	/* Array for MHD Field data */
  float *fpMHDData;		/* Ptr to MHD data */ 
  Array aMHDConnect = NULL;	/* Array containing conncetions for MHD field */
  int *ipMHDConnect;		/* Ptr to MHD Connection data */
  Field fMHD = NULL;		/* MHD data Field */
  float *fpXC1,*fpXC2,*fpXC3;   /* Temp space for coordinate reads */
  int iXC1, iXC2, iXC3;		/* Size of coord dims */
  float *fpVar;			/* Variable Data */
  float *fpTmp;			/* Temporary Space for Vector Data */

  /* Other varibles */
  FILE *FpMHDFile;
  int h,i,j,k,iStatus,iReadFlag;
  int iNumPts, iRdPts, iNumVars, iNx, iNy, iNz ;
  int liFileId, liIndex,liSdsId, liAtrId, liErr; /* HDF Access Vars */
  int liRank, liNumType, liAttr;
  int liaDimSize[MAX_VAR_DIMS], liStart, liEdge, liStride;
  int liaStart3d[3], liaEdge3d[3], liaStride3d[3], liDimId;
  char caDataName[MAX_NC_NAME];
  float fM2AU;

  char *varname[17]= {"XC1","XC2","XC3","XH1","XH2","XH3","TIME","DTSTEP",
                      "GAMMA","D","T","V1","V2","V3","B1","B2","B3"};


  if(!in[0]) {
    DXSetError(ERROR_BAD_PARAMETER,"Missing Data Filename");
    goto error;
  } else if (!DXExtractString(in[0],&cpMHDName)) {
    DXSetError(ERROR_BAD_PARAMETER,"Data Filename must be string");
    goto error;
  }

  DXMessage("Filename %s",cpMHDName);
  iStatus = nc_open(cpMHDName,NC_NOWRITE,&liFileId);
  /* Open the NetCDF data file */
  if(iStatus != NC_NOERR ){
     DXSetError(ERROR_BAD_PARAMETER,"Unable to open netCDF file");
     goto error;
  }
  iReadFlag = 0;

  /* Get coordinate arrays and sizes */
  /* XC1 is radial direction */
  iStatus = nc_inq_dimid(liFileId,"n1",&liDimId);
  iStatus = nc_inq_dimlen(liFileId,liDimId,&iXC1);
  fpXC1 = (float*)malloc(sizeof(float)*iXC1);
  iStatus = nc_inq_varid(liFileId,"X1",&liSdsId);
  iStatus = nc_get_var_float(liFileId,liSdsId,fpXC1);

  /* XC2 is theta direction */
  iStatus = nc_inq_dimid(liFileId,"n2",&liDimId);
  iStatus = nc_inq_dimlen(liFileId,liDimId,&iXC2);
  iStatus = nc_inq_varid(liFileId,"X2",&liSdsId);
  fpXC2 = (float*)malloc(sizeof(float)*iXC2);
  iStatus = nc_get_var_float(liFileId,liSdsId,fpXC2);

  /* XC3 is phi direction */
  iStatus = nc_inq_dimid(liFileId,"n3",&liDimId);
  iStatus = nc_inq_dimlen(liFileId,liDimId,&iXC3);
  fpXC3 = (float*)malloc(sizeof(float)*iXC3);
  iStatus = nc_inq_varid(liFileId,"X3",&liSdsId);
  iStatus = nc_get_var_float(liFileId,liSdsId,fpXC3);

  liaEdge3d[0] = iXC3+1;  /* +1 for peroidic nature of phi direction */
  liaEdge3d[1] = iXC2;
  liaEdge3d[2] = iXC1;
  iReadFlag=1;

  DXMessage("After Coord Read %d %d %d",iXC1,iXC2,iXC3);

  /* Make the positions array for the DXField */
  iRdPts = iXC1*iXC2*iXC3;
  iNumPts = iXC1*iXC2*(iXC3+1); /* +1 for peroidic nature of phi direction */

  aGrid = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
  DXAddArrayData(aGrid,0,iNumPts,NULL);
  fpGrid = (float*)DXGetArrayData(aGrid);
  fM2AU = 1.0/1.496e11;  /* Convert distances to AU */
  for(k=0;k<iXC3;k++) {
    for(j=0;j<iXC2;j++) {
      for(i=0;i<iXC1;i++) {
         fpGrid[0+3*(i+iXC1*(j+iXC2*k))]=
           fpXC1[i]*sin(fpXC2[j])*cos(fpXC3[k])*fM2AU; 
         fpGrid[1+3*(i+iXC1*(j+iXC2*k))]=
           fpXC1[i]*sin(fpXC2[j])*sin(fpXC3[k])*fM2AU; 
         fpGrid[2+3*(i+iXC1*(j+iXC2*k))]=
           fpXC1[i]*cos(fpXC2[j])*fM2AU; 
//         fpGrid[0+3*(i+iXC1*(j+iXC2*k))]=fpXC1[i]; 
//         fpGrid[1+3*(i+iXC1*(j+iXC2*k))]=fpXC2[j]; 
//         fpGrid[2+3*(i+iXC1*(j+iXC2*k))]=fpXC3[k]; 
      }
    }
  }
  /* Since Phi is peroidic copy 0 into last point to make vis appear continious
   * when displayed
   */
  for(j=0;j<iXC2;j++) {
    for(i=0;i<iXC1;i++) {
       fpGrid[0+3*(i+iXC1*(j+iXC2*iXC3))]=
         fpXC1[i]*sin(fpXC2[j])*cos(fpXC3[0])*fM2AU; 
       fpGrid[1+3*(i+iXC1*(j+iXC2*iXC3))]=
         fpXC1[i]*sin(fpXC2[j])*sin(fpXC3[0])*fM2AU; 
       fpGrid[2+3*(i+iXC1*(j+iXC2*iXC3))]=
         fpXC1[i]*cos(fpXC2[j])*fM2AU; 
    }
  }

  
  /* Create the data array for the DX field */

  iNumVars = 10;
  aMHDData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,iNumVars);
  DXAddArrayData(aMHDData,0,iNumPts,NULL);
  fpVar = (float*)DXGetArrayData(aMHDData);
  fpTmp = malloc(sizeof(float)*iNumPts);

  /* Get MHD parameters into the DX field */
  /* For MHDData we have the follow assignment of varibles
   *
   * 0 - Density originally [kg/m^3]
   * 1 - Temperature originally [K] converted to [kK]
   * 2 - Vx originally [m/s] converted to [km/s]
   * 3 - Vy originally [m/s] converted to [km/s]
   * 4 - Vz originally [m/s] converted to [km/s]
   * 5 - Bx originally [T] converted to [nT]
   * 6 - By originally [T] converted to [nT]
   * 7 - Bz originally [T] converted to [nT]
   * 8 - Radius in AU
   */
  /* Density is scalar so its just a direct copy into element 1*/

  iStatus = nc_inq_varid(liFileId,"D",&liSdsId);
  iStatus = nc_get_var_float(liFileId,liSdsId,fpTmp);
  for(k=0;k<iXC3;k++) {
    for(j=0;j<iXC2;j++) {
      for(i=0;i<iXC1;i++) {
        fpVar[0+iNumVars*(i+iXC1*(j+iXC2*k))]= fpTmp[i+iXC1*(j+iXC2*k)];
      }
    }
  }
		
  /* Temperature is scalar so its just a direct copy into element 1*/
  iStatus = nc_inq_varid(liFileId,"T",&liSdsId);
  iStatus = nc_get_var_float(liFileId,liSdsId,fpTmp);
  for(k=0;k<iXC3;k++) {
    for(j=0;j<iXC2;j++) {
      for(i=0;i<iXC1;i++) {
        fpVar[1+iNumVars*(i+iXC1*(j+iXC2*k))]= fpTmp[i+iXC1*(j+iXC2*k)]*1.0e-3;
      }
    }
  }

  /* Velocity is spherical vector so we must convert it into cartiesian
   * coordinates for display in OpenDX
   */
  iStatus = nc_inq_varid(liFileId,"V1",&liSdsId);
  iStatus = nc_get_var_float(liFileId,liSdsId,fpTmp);
  for(k=0;k<iXC3;k++) {
    for(j=0;j<iXC2;j++) {
      for(i=0;i<iXC1;i++) {
         /* For Vr use r = i sin(Th)cos(Phi)+j sin(Th)sin(Phi)+k cos(Th) */
         fpVar[2+iNumVars*(i+iXC1*(j+iXC2*k))]=
           fpTmp[i+iXC1*(j+iXC2*k)]*sin(fpXC2[j])*cos(fpXC3[k])*1.0e-3; 
         fpVar[3+iNumVars*(i+iXC1*(j+iXC2*k))]=
           fpTmp[i+iXC1*(j+iXC2*k)]*sin(fpXC2[j])*sin(fpXC3[k])*1.0e-3; 
         fpVar[4+iNumVars*(i+iXC1*(j+iXC2*k))]=
           fpTmp[i+iXC1*(j+iXC2*k)]*cos(fpXC2[j])*1.0e-3; 
      }
    }
  }

  iStatus = nc_inq_varid(liFileId,"V2",&liSdsId);
  iStatus = nc_get_var_float(liFileId,liSdsId,fpTmp);
  for(k=0;k<iXC3;k++) {
    for(j=0;j<iXC2;j++) {
      for(i=0;i<iXC1;i++) {
         /* For Vth use r = i cos(Th)cos(Phi)+j cos(Th)sin(Phi)-k sin(Th) */
         fpVar[2+iNumVars*(i+iXC1*(j+iXC2*k))]= 
           fpVar[2+iNumVars*(i+iXC1*(j+iXC2*k))] +
           fpTmp[i+iXC1*(j+iXC2*k)]*cos(fpXC2[j])*cos(fpXC3[k])*1.0e-3; 
         fpVar[3+iNumVars*(i+iXC1*(j+iXC2*k))]= 
           fpVar[3+iNumVars*(i+iXC1*(j+iXC2*k))] +
           fpTmp[i+iXC1*(j+iXC2*k)]*cos(fpXC2[j])*sin(fpXC3[k])*1.0e-3; 
         fpVar[4+iNumVars*(i+iXC1*(j+iXC2*k))]= 
           fpVar[4+iNumVars*(i+iXC1*(j+iXC2*k))] +
           -1.0*fpTmp[i+iXC1*(j+iXC2*k)]*sin(fpXC2[j])*1.0e-3; 
      }
    }
  }
  iStatus = nc_inq_varid(liFileId,"V3",&liSdsId);
  iStatus = nc_get_var_float(liFileId,liSdsId,fpTmp);
  for(k=0;k<iXC3;k++) {
    for(j=0;j<iXC2;j++) {
      for(i=0;i<iXC1;i++) {
         /* For Vphi use r = -i sin(Phi)+j cos(Phi) */
         fpVar[2+iNumVars*(i+iXC1*(j+iXC2*k))]= 
           fpVar[2+iNumVars*(i+iXC1*(j+iXC2*k))] +
           -1.0*fpTmp[i+iXC1*(j+iXC2*k)]*sin(fpXC3[k])*1.0e-3; 
         fpVar[3+iNumVars*(i+iXC1*(j+iXC2*k))]= 
           fpVar[3+iNumVars*(i+iXC1*(j+iXC2*k))] +
           fpTmp[i+iXC1*(j+iXC2*k)]*cos(fpXC3[k])*1.0e-3; 
      }
    }
  }
  /* Magnetic Field is spherical vector so we must convert it into cartiesian
   * coordinates for display in OpenDX
   */
  iStatus = nc_inq_varid(liFileId,"B1",&liSdsId);
  iStatus = nc_get_var_float(liFileId,liSdsId,fpTmp);
  for(k=0;k<iXC3;k++) {
    for(j=0;j<iXC2;j++) {
      for(i=0;i<iXC1;i++) {
         /* For Br use r = i sin(Th)cos(Phi)+j sin(Th)sin(Phi)+k cos(Th) */
         fpVar[5+iNumVars*(i+iXC1*(j+iXC2*k))]=
           fpTmp[i+iXC1*(j+iXC2*k)]*sin(fpXC2[j])*cos(fpXC3[k])*1.0e9; 
         fpVar[6+iNumVars*(i+iXC1*(j+iXC2*k))]=
           fpTmp[i+iXC1*(j+iXC2*k)]*sin(fpXC2[j])*sin(fpXC3[k])*1.0e9; 
         fpVar[7+iNumVars*(i+iXC1*(j+iXC2*k))]=
           fpTmp[i+iXC1*(j+iXC2*k)]*cos(fpXC2[j])*1.0e9; 
      }
    }
  }

  iStatus = nc_inq_varid(liFileId,"B2",&liSdsId);
  iStatus = nc_get_var_float(liFileId,liSdsId,fpTmp);
  for(k=0;k<iXC3;k++) {
    for(j=0;j<iXC2;j++) {
      for(i=0;i<iXC1;i++) {
         /* For Bth use r = i cos(Th)cos(Phi)+j cos(Th)sin(Phi)-k sin(Th) */
         fpVar[5+iNumVars*(i+iXC1*(j+iXC2*k))]= 
           fpVar[5+iNumVars*(i+iXC1*(j+iXC2*k))] +
           fpTmp[i+iXC1*(j+iXC2*k)]*cos(fpXC2[j])*cos(fpXC3[k])*1.0e9; 
         fpVar[6+iNumVars*(i+iXC1*(j+iXC2*k))]= 
           fpVar[6+iNumVars*(i+iXC1*(j+iXC2*k))] +
           fpTmp[i+iXC1*(j+iXC2*k)]*cos(fpXC2[j])*sin(fpXC3[k])*1.0e9; 
         fpVar[7+iNumVars*(i+iXC1*(j+iXC2*k))]= 
            fpVar[7+iNumVars*(i+iXC1*(j+iXC2*k))] +
           -1.0*fpTmp[i+iXC1*(j+iXC2*k)]*sin(fpXC2[j])*1.0e9; 
      }
    }
  }
  iStatus = nc_inq_varid(liFileId,"B3",&liSdsId);
  iStatus = nc_get_var_float(liFileId,liSdsId,fpTmp);
  for(k=0;k<iXC3;k++) {
    for(j=0;j<iXC2;j++) {
      for(i=0;i<iXC1;i++) {
         /* For Bphi use r = -i sin(Phi)+j cos(Phi) */
         fpVar[5+iNumVars*(i+iXC1*(j+iXC2*k))]= 
           fpVar[5+iNumVars*(i+iXC1*(j+iXC2*k))] +
           -1.0*fpTmp[i+iXC1*(j+iXC2*k)]*sin(fpXC3[k])*1.0e9; 
         fpVar[6+iNumVars*(i+iXC1*(j+iXC2*k))]= 
           fpVar[6+iNumVars*(i+iXC1*(j+iXC2*k))] +
           fpTmp[i+iXC1*(j+iXC2*k)]*cos(fpXC3[k])*1.0e9; 
      }
    }
  }
  for(k=0;k<iXC3;k++) {
    for(j=0;j<iXC2;j++) {
      for(i=0;i<iXC1;i++) {
       /* For Radius use r = sqrt(x^2+y^2+z^2) */
       fpVar[8+iNumVars*(i+iXC1*(j+iXC2*k))]= sqrt(
         fpGrid[0+3*(i+iXC1*(j+iXC2*iXC3))]*fpGrid[0+3*(i+iXC1*(j+iXC2*iXC3))]+
         fpGrid[1+3*(i+iXC1*(j+iXC2*iXC3))]*fpGrid[1+3*(i+iXC1*(j+iXC2*iXC3))]+
         fpGrid[2+3*(i+iXC1*(j+iXC2*iXC3))]*fpGrid[2+3*(i+iXC1*(j+iXC2*iXC3))]);
      }
    }
  }

  iStatus = nc_inq_varid(liFileId,"BP",&liSdsId);
  iStatus = nc_get_var_float(liFileId,liSdsId,fpTmp);
  for(k=0;k<iXC3;k++) {
    for(j=0;j<iXC2;j++) {
      for(i=0;i<iXC1;i++) {
        fpVar[9+iNumVars*(i+iXC1*(j+iXC2*k))]= fpTmp[i+iXC1*(j+iXC2*k)];
      }
    }
  }
  /* Since Phi is peroidic copy 0 into last point to make vis appear continious
   * when displayed
   */
  for(j=0;j<iXC2;j++) {
    for(i=0;i<iXC1;i++) {
      for(h=0;h<iNumVars;h++)   {
       fpVar[h+iNumVars*(i+iXC1*(j+iXC2*iXC3))]=
         fpVar[h+iNumVars*(i+iXC1*(j+iXC2*0))];
      }
    }
  }
  /* Do the final steps for the DXField */
  fMHD = DXNewField();
  if(!fMHD) {
    DXSetError(ERROR_NO_MEMORY,"Creation of MHD field failed");
    goto error;
  }

  /* Set the dependence of the Data Array to be on positons*/
  if(!DXSetStringAttribute((Object)aMHDData,"dep","positions")) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array Dep");
     goto error;
  }
  /* Set the MHD Data array as the data component of the MHD Field*/
  if(!DXSetComponentValue(fMHD,"data",(Object)aMHDData)) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array of MHD Field");
     goto error;
  }
  /* Set the MHD Pts array as the position component of the MHD Field*/
  if(!DXSetComponentValue(fMHD,"positions",(Object)aGrid)) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Position Array of MHD Field");
     goto error;
  }
  aMHDConnect = DXMakeGridConnectionsV(3,liaEdge3d);
  if (!aMHDConnect) {
    DXSetError(ERROR_INTERNAL,"Could Make MHD Grid Connections");
    goto error;
  }

  if (!DXSetComponentValue(fMHD,"connections",(Object)aMHDConnect)) {
      DXSetError(ERROR_INTERNAL,"Could set MHD Grid Connections");
      goto error;
  }

  if(!DXEndField(fMHD)) {
     DXSetError(ERROR_INTERNAL,"Couldn't complete field");
     goto error;
  }

  DXMessage("After Field ");

  out[0] = (Object) fMHD;

  free(fpXC1);
  free(fpXC2);
  free(fpXC3);
  free(fpTmp);
  
  return OK;

  error:
    DXDelete((Object) fMHD);
    DXDelete((Object) aMHDData);
    DXDelete((Object) aGrid);
    DXDelete((Object) aMHDConnect);
    if(1 == iReadFlag) {
      free(fpXC1);
      free(fpXC2);
      free(fpXC3);
    }
  return ERROR;
}

