#include <stdio.h>
#include <math.h>
#include <dx/dx.h>
#include "param.h"
#define MAX_NAME_SIZE 256

/* This module is designed to use the magnetic flux data from the readmhdhdf
 * series of modules to compute the set last closed field lines.  In addition,
 * it outputs the location of this boundary in the ionosphere.
 *
 * This module has two inputs
 *   In[0] = Magnetic Flux field from LFM
 *   In[1] = Number of field lines to compute
 *
 * This module has three outputs
 *   Out[0] = Set of last closed field lines
 *   Out[1] = Set of points composing in last closed field lines
 *   Out[2] = Boundary in northern hemisphere (Not yet implemented)
 *   Out[3] = Boundary in southern hemisphere (Not yet implemented)
 */

#define MAXPTS 10000 /* Max number of points in a given field line */
                     /* This should match NLNM in bline.inc */

Error
m_Lastcloseddbl(Object *in, Object *out)
{
   /* Inputs */

   Field fFlux = NULL; /* Magnetic Flux Data */
   int iNumLines;    /* Number of Magnetic field lines to compute */

   /* Outputs */
   Group gLines = NULL; /* Group containing last closed field lines */

   /* Other variables */
   Field fLine = NULL;  /* Field object for Field Lines */
   Array aPos = NULL;   /* Array for field line positions */
   float *fpPos;        /* Ptr to Position data */
   Array aData = NULL;  /* Array for field line data */
   float *fpData;       /* Ptr to Data data */ 
   Array aCon = NULL;   /* Array for field line connections */
   Field fSurf = NULL;  /* Field object for Aggrate Surface */
   Array aSurfPos = NULL;   /* Array for field line positions */
   float *fpSurfPos;        /* Ptr to Surface Position data */
   Array aSurfData = NULL;  /* Array for Surface line data */
   float *fpSurfData;       /* Ptr to Surface data */ 
   int iaShape[10];     /* Array to hold dimensioning information */
   int iRank,iItems;    /* Rank and number of items in arrays */
   int ni,nj,nk;        /* Dimension info */
   int nip1,njp1,nkp1;  /* Dimension info */
   Category CCat;       /* Array category */
   Type TType;          /* Array Type */
   int i,j,k;           /* Loop counters */
   int iNumPts;         /* Number of Pts in a field line */
   int iActLines;       /* Actual Number of Field Lines computed */
   int iTotalPts;       /* Total Number Pts in all field lines */
   int iPts;            /* Pt counter */
   float faXn[3],faXs[3],faXmax[3]; /* Info about field line */
   float *fpBx,*fpBy,*fpBz; /* Magnetic flux */
   float *fpX,*fpY,*fpZ,*fpGrid;    /* Magnetic positions */
   float *fpXLine,*fpYLine,*fpZLine; /* Locations along field */

   /* Extract input parameters */
   if(!in[0]) {
     DXSetError(ERROR_BAD_PARAMETER,"Missing number of Field Lines");
     goto error;
   } else if (!DXExtractInteger(in[0],&iNumLines)) {
     DXSetError(ERROR_BAD_PARAMETER,"Number of Field Lines must be Integer");
     goto error;
   }
   if(!in[1]) {
     DXSetError(ERROR_BAD_PARAMETER,"Missing Magnetic Flux");
     goto error;
   } else {
     /* Get flux field */
     aCon = (Array)DXGetComponentValue((Field)in[1],"connections");
     /* Confirm array size is consistent with module type */
     DXQueryGridConnections(aCon, &iRank, iaShape);
     DXMessage("Array info %d Rank %d ni %d nj %d nk",iRank, iaShape[0],
                iaShape[1], iaShape[2]);
     ni   = NI;
     nj   = NJ;
     nk   = NK;
     nip1 = NI+1;
     njp1 = NJ+1;
     nkp1 = NK+1;
     iItems = nip1*njp1*nkp1;
     if( (iaShape[0] != nkp1) || (iaShape[1] != njp1) ||
         (iaShape[2] != nip1)) {
        DXSetError(ERROR_DATA_INVALID,"Imcompatable ni,nj,nk");
        goto error;
     }
     
     /* Extract magnetic field data into format for blast */
     aData = (Array)DXGetComponentValue((Field)in[1],"data");
     fpData = DXGetArrayData(aData);

     fpBx = (float*)malloc(iItems*sizeof(float));
     fpBy = (float*)malloc(iItems*sizeof(float));
     fpBz = (float*)malloc(iItems*sizeof(float));

     /* Bx needs to be Fortran indexed with size nip1,nj,nk */
     for (k=0; k<nk; k++) { 
       for (j=0; j<nj; j++) {
         for ( i=0; i<nip1; i++) {
           fpBx[i+nip1*(j+k*nj)] = fpData[0+3*(i+nip1*(j+njp1*k))];
         }
       }
     }
      
     /* By needs to be Fortran indexed with size ni,njp1,nk */
     for (k=0; k<nk; k++) { 
       for (j=0; j<njp1; j++) {
         for ( i=0; i<ni; i++) {
           fpBy[i+ni*(j+k*njp1)] = fpData[1+3*(i+nip1*(j+njp1*k))];
         }
       }
     }

     /* Bz needs to be Fortran indexed with size ni,nj,nkp1 */
     for (k=0; k<nkp1; k++) { 
       for (j=0; j<nj; j++) {
         for ( i=0; i<ni; i++) {
           fpBz[i+ni*(j+k*nj)] = fpData[2+3*(i+nip1*(j+njp1*k))];
         }
       }
     }
     /* Removed debug message
     DXMessage("X[0] %g X[1] %g Y[0] %g Y[1] %g Z[0] %g Z[1] %g",
               fpBx[0],fpBx[1],fpBy[0],fpBy[1],fpBz[0],fpBz[1]);
     */

     /* Extract position data into format for blast */

     aPos = (Array)DXGetComponentValue((Field)in[1],"positions");
     fpPos = DXGetArrayData(aPos);
    
     fpX  = (float*)malloc(iItems*sizeof(float));
     fpY  = (float*)malloc(iItems*sizeof(float));
     fpZ  = (float*)malloc(iItems*sizeof(float));
     fpGrid  = (float*)malloc(3*iItems*sizeof(float));

     for(k=0;k<iItems;k++) {
       /* X,Y,Z are Fortran indexed with size nip1,njp1,nkp1 */
       fpX[k] = fpPos[0+3*k];
       fpY[k] = fpPos[1+3*k];
       fpZ[k] = fpPos[2+3*k];
       /* Grid is Fortran indexed with size nip1,njp1,nkp1,3 */
       fpGrid[k] = fpX[k];
       fpGrid[iItems+k] = fpY[k];
       fpGrid[2*iItems+k] = fpZ[k];
     }
     /* loadgrid transfers information into meter common block */
     floadgrid_(fpGrid);
     free(fpGrid);

   }

   /* Allocate temporary space to field line locations */
   fpXLine = (float*)malloc(MAXPTS*sizeof(float));
   fpYLine = (float*)malloc(MAXPTS*sizeof(float));
   fpZLine = (float*)malloc(MAXPTS*sizeof(float));

   gLines = DXNewGroup();
   iActLines = 0;
   iTotalPts = 0;
   for(i=0;i<iNumLines;i++) {
      /* Get the field line from blast */
      fblast_(fpBx,fpBy,fpBz,&i,&iNumLines,&iNumPts,fpXLine,fpYLine,fpZLine,
             faXn,faXs,faXmax);
      if (iNumPts > 1) {
        /* Create the field line for output */
        fLine = DXNewField();
        /* Transfer position information to Field */
        aPos = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
        DXAddArrayData(aPos,0,iNumPts,NULL);
        fpPos = (float*)DXGetArrayData(aPos);
        for(j=0;j<iNumPts;j++) {
          fpPos[0+3*j]=fpXLine[j];
          fpPos[1+3*j]=fpYLine[j];
          fpPos[2+3*j]=fpZLine[j];
        }
        DXSetComponentValue(fLine,"positions",(Object)aPos);
  
        /* Set point number as dummy data for field */
        aData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,1);
        DXAddArrayData(aData,0,iNumPts,NULL);
        fpData = (float*)DXGetArrayData(aData);
        for(j=0;j<iNumPts;j++) {
          fpData[j+0]=j;
        }
        DXSetStringAttribute((Object)aData,"dep","positions");
        DXSetComponentValue(fLine,"data",(Object)aData);
  
        /* Line is 1d so connections are straight forward */
        aCon = DXMakeGridConnections(1,iNumPts);
        DXSetComponentValue(fLine,"connections",(Object)aCon);
        DXSetComponentAttribute(fLine,"connections","element type",
                                (Object)DXNewString("lines"));      
        /* Add the field line to the Group output */
        DXEndField(fLine);
        DXSetEnumeratedMember(gLines,iActLines++,(Object)fLine);
        /* Keep track of total number of points in order to create field with
         * points from all field lines 
         */
	iTotalPts += iNumPts;
        /* Remove debug message
        DXMessage("Total Pts %d NumPts %d",iTotalPts,iNumPts);
        */
     }
   }
   /* Create field with Pts of all Field Lines */
   fSurf = DXNewField();
   aSurfPos = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
   DXAddArrayData(aSurfPos,0,iTotalPts,NULL);
   fpSurfPos = (float*)DXGetArrayData(aSurfPos);
   
   iPts = 0;
   /* Loop over each field line and transfer position information */
   for(i=0;i<iActLines;i++) {
     fLine = (Field) DXGetEnumeratedMember((Group)gLines,i,NULL);
     aPos = (Array)DXGetComponentValue((Field)fLine,"positions");
     fpPos = DXGetArrayData(aPos);
     DXGetArrayInfo(aPos,&iItems,&TType,&CCat,&iRank,iaShape);
     /* Copy Pts from line to surface */
     for(k=0;k<iItems;k++) {
       fpSurfPos[0+3*iPts] = fpPos[0+3*k];
       fpSurfPos[1+3*iPts] = fpPos[1+3*k];
       fpSurfPos[2+3*iPts] = fpPos[2+3*k];
       iPts++;
     }
   }
   DXMessage("Pts %d",iPts);
   DXSetComponentValue(fSurf,"positions",(Object)aSurfPos);
  
   /* Set pt number as dummy data for Surf field */
   aSurfData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,1);
   DXAddArrayData(aSurfData,0,iTotalPts,NULL);
   fpSurfData = (float*)DXGetArrayData(aSurfData);
   for(j=0;j<iTotalPts;j++) {
       fpSurfData[j+0]=j;
   }
   DXSetStringAttribute((Object)aSurfData,"dep","positions");
   DXSetComponentValue(fSurf,"data",(Object)aSurfData);

   DXEndField(fSurf);

   /* Set outputs */
   out[0] = (Object)gLines;
   out[1] = (Object)fSurf;

   /* Cleanup temporary allocations */
   free(fpBx);    free(fpBy);    free(fpBz);
   free(fpX);     free(fpY);     free(fpZ);
   free(fpXLine); free(fpYLine); free(fpZLine);
   return OK;

   error:
     DXDelete((Object)fLine);
     DXDelete((Object)aPos);
     DXDelete((Object)aData);
     DXDelete((Object)aCon);
     DXDelete((Object)fSurf);
     DXDelete((Object)aSurfPos);
     DXDelete((Object)aSurfData);
   return ERROR;
 
}
