#include "mfhdf.h"
#include "hdf.h"
#include <dx/dx.h>
#include <stdio.h>
#include <math.h>
#include "funcs.h"
#include "debug.h"

/* This modules is based upon Ravi's read_mhd_hdf.c code which imports the old
 * style HDF files into Data Explorer.  This routine will import the data from
 * the parallel verison of the code into data explorer.  The new file format has
 * the grid information in one file and the MHD data in another file.  The files
 * also contain information about the grid size which can be used to keep this
 * module flexable.  
 *
 * This module has two inputs
 *     In[0] = file name to read
 *     In[1] = Normalization - float
*
 *  and it has two outputs
 *     Out[0] = ION Data Field - Field float 15 vector contains ion data
 *     Out[1] = Time - float simulation time in seconds
 */

Error
m_ReadLFMParaIon(Object *in, Object *out)
{
  /* Variable Declarations */
  int lstep, dirlen;
  /* Inputs */
  char *cpDataName = NULL; // Ptr to Data File Name
  float fNorm;             // Normalization Constant
  
  /* Outputs */
  Array aIonData = NULL;  /* Array for Ion Field Data */
  float *fpIonData;       /* Ptr for Ion Data */
  Array aIonPts = NULL;   /* Array for Ion Pts */
  float *fpIonPts;        /* Ptr for Ion Grid Pts */
  Array aIonConnect=NULL; /* Array for Ion Connections */
  Field fION = NULL;      /* Ion Data Field */
  float fSimTime;         /* Simulation Time */

  /* Other Variables */
  int32 liFileId, liIndex,liSdsId, liAtrId, liErr; /* HDF Access Vars */
  int ni,nip1,nj,njp1,njp2,nk,nkp1, iaMHDDim[3],iNumPts;   /* Grid Size Vars */
  int iNumFluxPts, iaFluxDim[3], iNumDivPts, iaDivDim[3];  /* Grid Size Vars */
  int ionnjp1, ionnk2p3, iaIonDim[2], ionNumPts; /* Ion Size Vars */
  int i,j,k; /* Looping Vars */
  int iErr;
  int ncid;

  
  cpDataName = calloc(256,1);

  /* Get Input Values */

  /* Get Data Filename*/
  if(!in[0]) { 
    DXSetError(ERROR_BAD_PARAMETER,"Missing File name");
    goto error;
  } else if (!DXExtractString(in[0],&cpDataName)) {
    DXSetError(ERROR_BAD_PARAMETER,"Ion File Name must be string");
    goto error;
  }

  /* Get Normalization Constant*/
  if(!in[1]) {
    fNorm = 6.38e8;
  } else if (!DXExtractFloat(in[1],&fNorm)) {
    DXSetError(ERROR_BAD_PARAMETER,"Normalization Constant must be float");
    goto error;
  }


  //#ifdef DEBUG
  /* Give some outut to the user about the input parameters used */
  DXMessage("Ion Data Filename: %s",cpDataName);
  //#endif 

  /* Main Body of Function */

  /* Now we process the ionospheric data file */

  if (DFishdf(cpDataName) !=0) {
    DXSetError(ERROR_BAD_PARAMETER,"DX says %s isn't HDF file",cpDataName);
    goto error;
  }
  if((liFileId = SDstart(cpDataName,
  			 DFACC_RDONLY)) < 0) {
    DXMessage("FileId: %d",liFileId);
    DXSetError(ERROR_BAD_PARAMETER,"Unable to open Grid File");
    goto error;
  } 

  liIndex = SDnametoindex(liFileId,"x2ion");
  liSdsId = SDselect(liFileId,liIndex);
  liAtrId = SDfindattr(liSdsId,"ni");
  liErr = SDreadattr(liSdsId,liAtrId,&ionnjp1);
  liAtrId = SDfindattr(liSdsId,"nj");
  liErr = SDreadattr(liSdsId,liAtrId,&ionnk2p3);
  liErr = SDendaccess(liSdsId);
  ionNumPts = ionnjp1*ionnk2p3; // Acutal size of renderable grid
  iaIonDim[1] = ionnjp1;
  iaIonDim[0] = ionnk2p3;

  // Tell the user how big the grid is
#ifdef DEBUG
  DXMessage("Ion Grid Size %d %d %d",ionnjp1,ionnk2p3,ionNumPts);
#endif

  // Now that we have the size of the grid we can create the DX Arrays that will
  // hold the data

  // First the IonData Array - which is a 16 vector containing Ion Data
  aIonData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,16);
  if(!aIonData) {
    DXSetError(ERROR_NO_MEMORY,"Creation of Ion Data Array failed");
    goto error;
  }
  if(!DXAddArrayData(aIonData,0,ionNumPts,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of Ion Data Array failed");
    goto error;
  }

  // Get Ptr to Data portion of IonData Array
  fpIonData = (float*)DXGetArrayData(aIonData);

  // Second the IonPts Array - location of grid points
  aIonPts = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
  if(!aIonPts) {
    DXSetError(ERROR_NO_MEMORY,"Creation of Ion Pts Array failed");
    goto error;
  }
  if(!DXAddArrayData(aIonPts,0,ionNumPts,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of Ion Pts Array failed");
    goto error;
  }

  // Get Ptr to Data portion of IonPts Array
  fpIonPts = (float*)DXGetArrayData(aIonPts);
#ifdef DEBUG
  DXMessage("fpIonPts %x fpIonPts[0] %g",fpIonPts,fpIonPts[0]);
#endif


  /* Now we do the real work of this module */
  /* First step is to get the grid Returns a Fortran Order Array with x,y,z as
     the least rapidly varying quanity
  */

#ifdef DEBUG
  DXMessage("Before Get Ion Grid");
#endif
  iErr = PGetIonGrid(liFileId,fNorm,fpIonPts);
  /* Now that we are done with the Grid File we can close it */

#ifdef DEBUG
  DXMessage("Before GetIon Data");
#endif
  iErr = PGetIonData(liFileId,ionnjp1,ionnk2p3,fpIonData);
  /* Now that we are done with the Grid File we can close it */
  SDend(liFileId);

  /* Create the Ion output field */
  fION = DXNewField();
  if(!fION) {
    DXSetError(ERROR_NO_MEMORY,"Creation of ION field failed");
    goto error;
  }

  /* Set the dependence of the Data Array to be on positons */
  if(!DXSetStringAttribute((Object)aIonData,"dep","positions")) { 
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array Dep");
     goto error;
  }
  /* Set the Ion Data array as the data component of the ION Field */
  if(!DXSetComponentValue(fION,"data",(Object)aIonData)) { 
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array of Flux Field");
     goto error;
  }
  /* Set the Ion Pts array as the position component of the ION Field */
  if(!DXSetComponentValue(fION,"positions",(Object)aIonPts)) { 
     DXSetError(ERROR_INTERNAL,"Couldn't set Position Array of ION Field");
     goto error;
  }
  /* Make the connections */
  aIonConnect = DXMakeGridConnectionsV(2,iaIonDim);
  if (!aIonConnect) {
    DXSetError(ERROR_INTERNAL,"Could Make Ion Grid Connections");
    goto error;
  }
  if (!DXSetComponentValue(fION,"connections",(Object)aIonConnect)) {
    DXSetError(ERROR_INTERNAL,"Could set Ion Grid Connections");
    goto error;
  }
	
  /* End Field sets default attributes */
  if(!DXEndField(fION)) {
     DXSetError(ERROR_INTERNAL,"Couldn't complete field");
     goto error;
  }

  /* Get the simulation time */
  out[0] = (Object)fION;
  out[1] = (Object)DXMakeFloat(fSimTime);

  return OK;

error:
  DXDelete((Object) fION);
  DXDelete((Object) aIonData);
  DXDelete((Object) aIonPts);
  DXDelete((Object) aIonConnect);
  return ERROR;
}



  

  
