#include "mfhdf.h"
#include "hdf.h"
#include <dx/dx.h>
#include <stdio.h>
#include <math.h>
#include "funcs.h"
#include "fpointers.h"
#include "debug.h"

/* This modules is based upon Ravi's read_mhd_hdf.c code which imports the old
 * style HDF files into Data Explorer.  This routine will import the data from
 * the parallel verison of the code into data explorer.  The new file format has
 * the grid information in one file and the MHD data in another file.  The files
 * also contain information about the grid size which can be used to keep this
 * module flexable.  
 *
 * This module has two inputs
 *     In[0] = filename to be read
 *     In[1] = Normalization - float
 *
 *  and it has four outputs
 *     Out[0] = MHD Data Field - Field float 14 vector contains mhd data
 *     Out[1] = Magnetic flux Field - Field float 3 vector
 *     Out[2] = DivB field
 *     Out[3] = Time - float simulation time in seconds
 */

Error
m_ReadLFMParaMHD(Object *in, Object *out)
{
  /* Variable Declarations */
  int lstep, dirlen;
  /* Inputs */
  char *cpDataName = NULL; // Ptr to Data File Name
  float fNorm;             // Normalization Constant
  
  /* Outputs */
  Array aMHDData = NULL;  /* Array for MHD Field Data */
  float *fpMHDData;       /* Ptr to Grid Data */
  Array aMHDPts = NULL;   /* Array for MHD Field Pts */
  float *fpMHDPts;        /* Ptr to Grid Pts */
  Array aMHDConnect=NULL; /* Array for MHD connections */
  Field fMHD = NULL;      /* MHD Data Field */
  Array aFluxData = NULL;  /* Array for Flux Field Data */
  float *fpFluxData;       /* Ptr to Grid Data */
  Array aFluxPts = NULL;   /* Array for Flux Field Pts */
  float *fpFluxPts;        /* Ptr to Grid Pts */
  Array aFluxConnect=NULL; /* Array for Flux connections */
  Field fFlux = NULL;      /* Flux Data Field */
  Array aDivBData = NULL;  /* Array for DivB Field Data */
  float *fpDivBData;       /* Ptr to Grid Data */
  Array aDivBPts = NULL;   /* Array for DivB Field Pts */
  float *fpDivBPts;        /* Ptr to Grid Pts */
  Array aDivBConnect=NULL; /* Array for Flux connections */
  Field fDivB = NULL;      /* DivB Data Field */
  float fSimTime;         /* Simulation Time */

  /* Other Variables */
  int32 liFileId, liIndex,liSdsId, liAtrId, liErr; /* HDF Access Vars */
  int ni,nip1,nj,njp1,njp2,nk,nkp1, iaMHDDim[3],iNumPts;   /* Grid Size Vars */
  int iNumFluxPts, iaFluxDim[3], iNumDivPts, iaDivDim[3];  /* Grid Size Vars */
  int ionnjp1, ionnk2p3, iaIonDim[2], ionNumPts; /* Ion Size Vars */
  int i,j,k; /* Looping Vars */
  int iErr;
  int ncid;

  
  cpDataName = calloc(256,1);

  /* Get Input Values */

  /* Get INPUT Filename*/
  if(!in[0]) { 
    DXSetError(ERROR_BAD_PARAMETER,"Missing File or Path");
    goto error;
  } else if (!DXExtractString(in[0],&cpDataName)) {
    DXSetError(ERROR_BAD_PARAMETER,"MHD Path_name must be string");
    goto error;
  }


  /* Get Normalization Constant*/
  if(!in[1]) {
    fNorm = 6.38e8;
  } else if (!DXExtractFloat(in[1],&fNorm)) {
    DXSetError(ERROR_BAD_PARAMETER,"Normalization Constant must be float");
    goto error;
  }

  //#ifdef DEBUG
  /* Give some outut to the user about the input parameters used */
  DXMessage("MHD Data Filename: %s",cpDataName);
  DXMessage("Normalization Constant: %f",fNorm);
  //#endif 

  /* Main Body of Function */

  /* Process MHD Data */
  if (DFishdf(cpDataName) !=0) {
    DXSetError(ERROR_BAD_PARAMETER,"DX says %s isn't HDF file",cpDataName);
    goto error;
  }
#ifdef DEBUG
  DXMessage("It's an HDF File");
#endif
  DXMessage("Hi Mike %s data file\n",cpDataName);
  /* First step is to open grid file and determine size of comp grid*/
  if((liFileId = SDstart(cpDataName,
  			 DFACC_READ)) < 0) {
    DXMessage("FileId: %d",liFileId);
    DXSetError(ERROR_BAD_PARAMETER,"Unable to open Data File");
    goto error;
  } 

  liIndex = SDnametoindex(liFileId,"X_grid");
  liSdsId = SDselect(liFileId,liIndex);
  liAtrId = SDfindattr(liSdsId,"ni");
  liErr = SDreadattr(liSdsId,liAtrId,&nip1);
  liAtrId = SDfindattr(liSdsId,"nj");
  liErr = SDreadattr(liSdsId,liAtrId,&njp1);
  liAtrId = SDfindattr(liSdsId,"nk");
  liErr = SDreadattr(liSdsId,liAtrId,&nkp1);
  liErr = SDendaccess(liSdsId);
  ni = nip1 - 1;
  nj = njp1 - 1;
  nk = nkp1 - 1;
  njp2 = njp1 + 1;
  iaMHDDim[0] = nkp1;
  iaMHDDim[1] = njp2;
  iaMHDDim[2] = ni;
#ifdef DEBUG  
  DXMessage("dimensions ni: %d, nj: %d, nk: %d\n",ni,nj,nk);
#endif

  iNumPts = ni*njp2*nkp1; /* Acutal size of renderable grid */
  iaFluxDim[0] = nkp1;
  iaFluxDim[1] = njp1;
  iaFluxDim[2] = nip1;
  iNumFluxPts = nip1*njp1*nkp1;
  iaDivDim[0] = nkp1;
  iaDivDim[1] = nj;
  iaDivDim[2] = ni;
  iNumDivPts = ni*nj*nkp1;

#ifdef DEBUG
  /* Tell the user how big the grid is*/
  DXMessage("Grid Size %d %d %d",ni,nj,nk);
#endif

  /* Now that we have the size of the grid we can create the DX Arrays that will*/
  /* hold the data*/


  /*   MHD data arrays  */

  /* First the MHDData Array - which is a 14 vector containing MHD Data*/
  aMHDData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,14);
  if(!aMHDData) {
    DXSetError(ERROR_NO_MEMORY,"Creation of MHD Data Array failed");
    goto error;
  }
  if(!DXAddArrayData(aMHDData,0,iNumPts,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of MHD Data Array failed");
    goto error;
  }

  /* Get Ptr to Data portion of MHDData Array*/
  fpMHDData = (float*)DXGetArrayData(aMHDData);

  /* Second the MHDPts Array - location of grid points*/
  aMHDPts = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
  if(!aMHDPts) {
    DXSetError(ERROR_NO_MEMORY,"Creation of MHD Pts Array failed");
    goto error;
  }
  if(!DXAddArrayData(aMHDPts,0,iNumPts,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of MHD Pts Array failed");
    goto error;
  }

  /* Get Ptr to Data portion of MHDPts Array*/
  fpMHDPts = (float*)DXGetArrayData(aMHDPts);
#ifdef DEBUG
  DXMessage("fpMHDPts %x fpMHDPts[0] %g",fpMHDPts,fpMHDPts[0]);
#endif

  /*     B fluxes   */

  /* First the FluxData Array - which is a 3 vector containing Flux Data*/
  aFluxData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
  if(!aFluxData) {
    DXSetError(ERROR_NO_MEMORY,"Creation of Flux Data Array failed");
    goto error;
  }
  if(!DXAddArrayData(aFluxData,0,iNumFluxPts,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of Flux Data Array failed");
    goto error;
  }

  /* Get Ptr to Data portion of FluxData Array*/
  fpFluxData = (float*)DXGetArrayData(aFluxData);

  /* Second the FluxPts Array - location of grid points*/
  aFluxPts = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
  if(!aFluxPts) {
    DXSetError(ERROR_NO_MEMORY,"Creation of Flux Pts Array failed");
    goto error;
  }
  if(!DXAddArrayData(aFluxPts,0,iNumFluxPts,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of Flux Pts Array failed");
    goto error;
  }

  /* Get Ptr to Data portion of FluxPts Array*/
  fpFluxPts = (float*)DXGetArrayData(aFluxPts);
#ifdef DEBUG
  DXMessage("fpFluxPts %x fpFluxPts[0] %g",fpFluxPts,fpFluxPts[0]);
#endif

  /*   Div B check   */

  /* First the DivBData Array - which is a 1 vector containing DivB Data*/
  aDivBData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,1);
  if(!aDivBData) {
    DXSetError(ERROR_NO_MEMORY,"Creation of DivB Data Array failed");
    goto error;
  }
  if(!DXAddArrayData(aDivBData,0,iNumDivPts,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of DivB Data Array failed");
    goto error;
  }

  /* Get Ptr to Data portion of DivBData Array*/
  fpDivBData = (float*)DXGetArrayData(aDivBData);

  /* Second the DivBPts Array - location of grid points*/
  aDivBPts = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
  if(!aDivBPts) {
    DXSetError(ERROR_NO_MEMORY,"Creation of DivB Pts Array failed");
    goto error;
  }
  if(!DXAddArrayData(aDivBPts,0,iNumDivPts,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of DivB Pts Array failed");
    goto error;
  }

  /* Get Ptr to Data portion of DivBPts Array*/
  fpDivBPts = (float*)DXGetArrayData(aDivBPts);
#ifdef DEBUG
  DXMessage("fpDivBPts %x fpDivBPts[0] %g",fpDivBPts,fpDivBPts[0]);
#endif

  /* Now we do the real work of this module */
  /**/
  /* First step is to get the grid Returns a Fortran Order Array with x,y,z as*/
  /* the least rapidly varying quanity*/

#ifdef DEBUG
  DXMessage("Hi Mike Before Get Grid");
#endif
  iErr = PGetGrid(liFileId,fNorm,fpMHDPts,fpFluxPts,fpDivBPts);
  /* Now that we are done with the Grid File we can close it*/
  DXMessage("After Get Grid");
  SDend(liFileId);

  if((liFileId = SDstart(cpDataName,
  			 DFACC_RDONLY)) < 0) {
    DXMessage("FileId: %d",liFileId);
    DXSetError(ERROR_BAD_PARAMETER,"Unable to open Data File");
    goto error;
  } 

#ifdef DEBUG
  DXMessage("Before GetMHD Data");
#endif
  iErr = PGetMHDData(liFileId,ni,nj,nk,fpMHDData,fpFluxData,fpDivBData);
#ifdef DEBUG
  DXMessage("After GetMHD Data");
#endif
  /* Use this data file to extract the simulation time */
  liIndex = SDnametoindex(liFileId,"rho_");
  liSdsId = SDselect(liFileId,liIndex);
  liAtrId = SDfindattr(liSdsId,"time");
  liErr = SDreadattr(liSdsId,liAtrId,&fSimTime);
  DXMessage("Simulation time: %g",fSimTime);

  /* Now that we are done with the MHD Data File we can close it */
  SDend(liFileId);

  /* Create the MHD output field*/
  fMHD = DXNewField();
  if(!fMHD) {
    DXSetError(ERROR_NO_MEMORY,"Creation of MHD field failed");
    goto error;
  }

  /* Set the dependence of the Data Array to be on positons*/
  if(!DXSetStringAttribute((Object)aMHDData,"dep","positions")) { 
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array Dep");
     goto error;
  }
  /* Set the MHD Data array as the data component of the MHD Field*/
  if(!DXSetComponentValue(fMHD,"data",(Object)aMHDData)) { 
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array of MHD Field");
     goto error;
  }
  /* Set the MHD Pts array as the position component of the MHD Field*/
  if(!DXSetComponentValue(fMHD,"positions",(Object)aMHDPts)) { 
     DXSetError(ERROR_INTERNAL,"Couldn't set Position Array of MHD Field");
     goto error;
  }


  /* Determine the connections between points in field */
  aMHDConnect = DXMakeGridConnectionsV(3,iaMHDDim);
  if (!aMHDConnect) {
    DXSetError(ERROR_INTERNAL,"Could Make Ion Grid Connections");
    goto error;
  }
  if (!DXSetComponentValue(fMHD,"connections",(Object)aMHDConnect)) {
    DXSetError(ERROR_INTERNAL,"Could set Ion Grid Connections");
    goto error;
  }
  /* End Field sets default attributes */
  if(!DXEndField(fMHD)) {
     DXSetError(ERROR_INTERNAL,"Couldn't complete field");
     goto error;
  }



  /* Create the Flux output field*/
  fFlux = DXNewField();
  if(!fFlux) {
    DXSetError(ERROR_NO_MEMORY,"Creation of Flux field failed");
    goto error;
  }

  /* Set the dependence of the Data Array to be on positons*/
  if(!DXSetStringAttribute((Object)aFluxData,"dep","positions")) { 
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array Dep");
     goto error;
  }
  /* Set the Flux Data array as the data component of the Flux Field*/
  if(!DXSetComponentValue(fFlux,"data",(Object)aFluxData)) { 
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array of Flux Field");
     goto error;
  }
  /* Set the Flux Pts array as the position component of the Flux Field*/
  if(!DXSetComponentValue(fFlux,"positions",(Object)aFluxPts)) { 
     DXSetError(ERROR_INTERNAL,"Couldn't set Position Array of Flux Field");
     goto error;
  }


  /* Determine the connections between points in field */
  aFluxConnect = DXMakeGridConnectionsV(3,iaFluxDim);
  if (!aFluxConnect) {
    DXSetError(ERROR_INTERNAL,"Could Make Ion Grid Connections");
    goto error;
  }
  if (!DXSetComponentValue(fFlux,"connections",(Object)aFluxConnect)) {
    DXSetError(ERROR_INTERNAL,"Could set Ion Grid Connections");
    goto error;
  }
  /* End Field sets default attributes */
  if(!DXEndField(fFlux)) {
     DXSetError(ERROR_INTERNAL,"Couldn't complete field");
     goto error;
  }


  /* Create the Div B output field*/
  fDivB = DXNewField();
  if(!fDivB) {
    DXSetError(ERROR_NO_MEMORY,"Creation of DivB field failed");
    goto error;
  }

  /* Set the dependence of the Data Array to be on positons*/
  if(!DXSetStringAttribute((Object)aDivBData,"dep","positions")) { 
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array Dep");
     goto error;
  }
  /* Set the DivB Data array as the data component of the DivB Field*/
  if(!DXSetComponentValue(fDivB,"data",(Object)aDivBData)) { 
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array of DivB Field");
     goto error;
  }
  /* Set the DivB Pts array as the position component of the DivB Field*/
  if(!DXSetComponentValue(fDivB,"positions",(Object)aDivBPts)) { 
     DXSetError(ERROR_INTERNAL,"Couldn't set Position Array of DivB Field");
     goto error;
  }


  /* Determine the connections between points in field */
  aDivBConnect = DXMakeGridConnectionsV(3,iaDivDim);
  if (!aDivBConnect) {
    DXSetError(ERROR_INTERNAL,"Could Make Ion Grid Connections");
    goto error;
  }
  if (!DXSetComponentValue(fDivB,"connections",(Object)aDivBConnect)) {
    DXSetError(ERROR_INTERNAL,"Could set Ion Grid Connections");
    goto error;
  }
  /* End Field sets default attributes */
  if(!DXEndField(fDivB)) {
     DXSetError(ERROR_INTERNAL,"Couldn't complete field");
     goto error;
  }

  /* Get the simulation time */
  out[0] = (Object)fMHD;
  out[1] = (Object)fFlux;
  out[2] = (Object)fDivB;
  out[3] = (Object)DXMakeFloat(fSimTime);

  return OK;

error:
  DXDelete((Object) fMHD);
  DXDelete((Object) aMHDData);
  DXDelete((Object) aMHDPts);
  DXDelete((Object) aMHDConnect);
  return ERROR;
}



  

  
