#include "mfhdf.h"
#include <dx/dx.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "funcs.h"
#include "fpointers.h"

#define MAXRANK 3

/* This module reads in the 'new' style HDF files from the SMP version of the
 * LFM.  The 'new' style HDF files have several advantages: 1) All ionospheric
 * data is stored in the file, 2) grid size information is accessible from file
 * 3) random access via step number information
 *
 * The module has three inputs
 *   In[0] = Filename - string
 *   In[1] = Step number - int
 *   In[2] = Normalization - float
 *   In[3] = Grid Trace - int
 *
 * and it has seven outputs
 *   Out[0] = MHD Data field - Field float 14 vector contains mhd data
 *   Out[1] = ION Data field - Field float 16 vector contains ion data
 *   Out[2] = Magnetic Flux Field - Field float 3 vector
 *   Out[3] = Time - float simulation time in seconds
 *   Out[4] = Div B Field - Field float 1  vector
 *   Out[5] = Nulls - Field float 1 vector
 *   Out[6] = Flux Tubes - Field float 3 vector contains field aligned quanties
 *
 * NB - Use of this function requires linking against a HDF library compiled
 * with the -DHAVE_NETCDF flag
 */



Error m_ReadLFM(Object *in, Object *out)
{
   /***********************/
   /*  Declare variables  */
   /***********************/

  /* Variable Declarations */
  /* Inputs */
  int iStep;               	/* Step Number */
  char *cpFileName=NULL;        /* Ptr to File Name */
  float fNorm;             	/* Normalization Constant */
  int iComputeTubes;            /* Compute flux tube quanties */
  FILE *fptr;			/* File pointer to verify file */

  /* Outputs */
  Array aMHDData=NULL;		/* Array for MHD Field Data */
  float *fpMHDData;		/* Ptr to MHD data */
  Array aMHDPts=NULL;		/* Array for MHD Field Pts */
  float *fpMHDPts;		/* Ptr to MHD Pts data */
  Array aMHDConnect=NULL;	/* Array for MHD Field Connections */
  int   *ipMHDConnect;          /* Ptr to MHD Connection data */
  Field fMHD=NULL;		/* MHD Data Field */
  Array aTubeData=NULL;		/* Array for Tube Field Data */
  float *fpTubeData;		/* Ptr to Tube data */
  Array aTubePts=NULL;		/* Array for Tube Field Pts */
  float *fpTubePts;		/* Ptr to Tube Pts data */
  Array aTubeConnect=NULL;	/* Array for Tube Field Connections */
  int   *ipTubeConnect;          /* Ptr to Tube Connection data */
  Field fTube=NULL;		/* Tube Data Field */
  Array aFluxData = NULL;  /* Array for Flux Field Data */
  float *fpFluxData;       /* Ptr to Grid Data */
  Array aFluxPts = NULL;   /* Array for Flux Field Pts */
  float *fpFluxPts;        /* Ptr to Grid Pts */
  Array aFluxConnect=NULL; /* Array for Flux connections */
  Field fFlux = NULL;      /* Flux Data Field */
  Array aNullData = NULL;  /* Array for Null Field Data */
  float *fpNullData;       /* Ptr to Grid Data */
  Array aNullPts = NULL;   /* Array for Null Field Pts */
  float *fpNullPts;        /* Ptr to Grid Pts */
  Array aNullConnect=NULL; /* Array for Null connections */
  Field fNull = NULL;      /* Null Data Field */
  Array aDivBData = NULL;  /* Array for DivB Field Data */
  float *fpDivBData;       /* Ptr to Grid Data */
  Array aDivBPts = NULL;   /* Array for DivB Field Pts */
  float *fpDivBPts;        /* Ptr to Grid Pts */
  Array aDivBConnect=NULL; /* Array for Flux connections */
  Field fDivB = NULL;      /* DivB Data Field */
  Array aIONData=NULL;		/* Array for ION Field Data */
  float *fpIONData;		/* Ptr to ION data */
  Array aIONPts=NULL;		/* Array for ION Field Pts */
  float *fpIONPts;		/* Ptr to ION Pts data */
  Array aIONConnect=NULL;	/* Array for ION Field Connections */
  Field fION=NULL;		/* ION Data Field */
  float fSimTime;		/* Simualtion Time */

  /* Other Variables */
  int32 liFileId, liIndex,liSdsId, liErr; /* HDF Access Vars */
  int32 liAttrId, liStat, liNumType, liSize, liRank, liCount;
  int32 liaDimSize[MAX_VAR_DIMS], *lipStart, *lipEdge, *lipStride; 
  int32 liSecId;
  char caLabel[MAX_NC_NAME];
  int ni,nip1,nj,njp1,njp2,nk,nkp1,nk2p3,iaMHDDim[3],iNumPts;/*Grid Size Vars */
  int iNumFluxPts, iaFluxDim[3], iNumDivPts, iaDivDim[3];  /* Grid Size Vars */
  int ionnjp1, ionnk2p3, iaIONDim[2], ionNumPts; /* ION Size Vars */
  int iErr;
  int i,j,k; /* Looping Vars */
  int nv;
  float *fpBiFlip, *fpBjFlip, *fpBkFlip; /* Null Finding Vars */
  int iInitial, iFinal, iInterval, iStepFix;
  int koff,koff2,joff,joff2;

  /* Get Input Values */

   if (!in[0]) {
      DXSetError(ERROR_BAD_PARAMETER,"missing filename");
      goto error;
   } else if (!DXExtractString(in[0], &cpFileName)) {   
    DXSetError(ERROR_BAD_PARAMETER, "filename must be a string");
    goto error;
  }
  if (!in[1]) {
    DXSetError(ERROR_BAD_PARAMETER,"missing Time Step ");
    goto error;
  } else if (!DXExtractInteger(in[1], &iStep)) {   
    DXSetError(ERROR_BAD_PARAMETER, "Time Step must be an integer");
    goto error;
  }
  if (!in[2]) {
    DXSetError(ERROR_BAD_PARAMETER,"missing Normalization factor ");
    goto error;
  } else if (!DXExtractFloat(in[2], &fNorm)) {   
    DXSetError(ERROR_BAD_PARAMETER, "Normalization factor must be a float");
    goto error;
  }
  if (!in[3]) {
    iComputeTubes = 0;  /* If not present don't compute tubes */
  } else if (!DXExtractInteger(in[3], &iComputeTubes)) {   
    DXSetError(ERROR_BAD_PARAMETER, "Grid Flag must be an integer");
    goto error;
  }

#ifdef DEBUG
   DXMessage("Filename: %s",cpFileName);
   DXMessage("Step: %d",iStep);
   DXMessage("Normalization: %g",fNorm);
#endif

   /* Main Body of Function */
 
   fptr = fopen(cpFileName,"r");  
   if(fptr == NULL) {
     DXSetError(ERROR_BAD_PARAMETER,"FILE %s does not exist",cpFileName);
     goto error;
   }
   fclose(fptr);
   if (DFishdf(cpFileName) !=0) {
     DXSetError(ERROR_BAD_PARAMETER,"DX says %s isn't HDF file",cpFileName);
    goto error;
   }

 /* First step is to open file and determine grid size */ 
   if((liFileId = SDstart(cpFileName,
                        DFACC_RDONLY)) < 0) {
    DXMessage("FileId: %d",liFileId);
    DXSetError(ERROR_BAD_PARAMETER,"Unable to open Grid File");
    goto error;

  } 
#ifdef DEBUG
  DXMessage("File Id %d",liFileId);
#endif 

  /* grid_x2 will give us nip1 and njp1  */
#ifdef DEBUG
  DXMessage("Getting grid size");
#endif 
  liIndex = SDnametoindex(liFileId,"grid_x2");
  liSecId = SDselect(liFileId,liIndex);
  liStat = SDgetinfo(liSecId,caLabel,&liRank,liaDimSize,&liNumType,&liAttrId);
  nip1 = liaDimSize[1];
  njp1 = liaDimSize[0];
  ni = nip1-1;
  nj = njp1-1;
  njp2 = njp1+1;
  liStat = SDendaccess(liSecId);

  /* grid_phi will give us nkp1 */
  liIndex = SDnametoindex(liFileId,"grid_phi");
  liSecId = SDselect(liFileId,liIndex);
  liStat = SDgetinfo(liSecId,caLabel,&liRank,liaDimSize,&liNumType,&liAttrId);
  nkp1 = liaDimSize[0];
  nk = nkp1-1;
  liStat = SDendaccess(liSecId);

  iaMHDDim[0]=nkp1;
  iaMHDDim[1]=njp2;
  iaMHDDim[2]=ni;
  iNumPts = ni*njp2*nkp1; /* Actual size of renderable grid */
  iaFluxDim[0] = nkp1;
  iaFluxDim[1] = njp1;
  iaFluxDim[2] = nip1;
  iNumFluxPts = nip1*njp1*nkp1;
  iaDivDim[0] = nkp1;
  iaDivDim[1] = nj;
  iaDivDim[2] = ni;
  iNumDivPts = ni*nj*nkp1;

#ifdef DEBUG
  DXMessage("NI NJ NK %d %d %d\n",ni,nj,nk);
  DXMessage("NumFluxPts %d\n",iNumFluxPts);
  DXMessage("NumPts %d\n",iNumPts);
#endif

  /* Now that we have the size of the grid we can create the DX arrays that will
   * hold the data
   */

  /* First the MHDData Array - which is a 14 vector containing MHD Data*/
  aMHDData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,14);
  if(!aMHDData) {
    DXSetError(ERROR_NO_MEMORY,"Creation of MHD Data Array failed");
    goto error;
  }
  if(!DXAddArrayData(aMHDData,0,iNumPts,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of MHD Data Array failed");
    goto error;
  }

  /* Get Ptr to Data portion of MHDData Array*/
  fpMHDData = (float*)DXGetArrayData(aMHDData);

  /* Second the MHDPts Array - location of grid points*/
  aMHDPts = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
  if(!aMHDPts) {
    DXSetError(ERROR_NO_MEMORY,"Creation of MHD Pts Array failed");
    goto error;
  }
  if(!DXAddArrayData(aMHDPts,0,iNumPts,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of MHD Pts Array failed");
    goto error;
  }

  /* Get Ptr to Data portion of MHDPts Array*/
  fpMHDPts = (float*)DXGetArrayData(aMHDPts);

  /* Next the Flux Tube Array - which is a 5 vector containing Flux tube Data*/
  aTubeData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,4);
  if(!aTubeData) {
    DXSetError(ERROR_NO_MEMORY,"Creation of Tube Data Array failed");
    goto error;
  }
  if(!DXAddArrayData(aTubeData,0,iNumPts,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of Tube Data Array failed");
    goto error;
  }

  /* Get Ptr to Data portion of TubeData Array*/
  fpTubeData = (float*)DXGetArrayData(aTubeData);

  /* Second the TubePts Array - location of grid points*/
  aTubePts = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
  if(!aTubePts) {
    DXSetError(ERROR_NO_MEMORY,"Creation of Tube Pts Array failed");
    goto error;
  }
  if(!DXAddArrayData(aTubePts,0,iNumPts,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of Tube Pts Array failed");
    goto error;
  }

  /* Get Ptr to Data portion of TubePts Array*/
  fpTubePts = (float*)DXGetArrayData(aTubePts);

  /*     B fluxes   */

  /* First the FluxData Array - which is a 3 vector containing Flux Data*/
  aFluxData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
  if(!aFluxData) {
    DXSetError(ERROR_NO_MEMORY,"Creation of Flux Data Array failed");
    goto error;
  }
  if(!DXAddArrayData(aFluxData,0,iNumFluxPts,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of Flux Data Array failed");
    goto error;
  }

  /* Get Ptr to Data portion of FluxData Array*/
  fpFluxData = (float*)DXGetArrayData(aFluxData);

  /* Second the FluxPts Array - location of grid points*/
  aFluxPts = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
  if(!aFluxPts) {
    DXSetError(ERROR_NO_MEMORY,"Creation of Flux Pts Array failed");
    goto error;
  }
  if(!DXAddArrayData(aFluxPts,0,iNumFluxPts,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of Flux Pts Array failed");
    goto error;
  }

  /* Get Ptr to Data portion of FluxPts Array*/
  fpFluxPts = (float*)DXGetArrayData(aFluxPts);
#ifdef DEBUG
  DXMessage("fpFluxPts %x fpFluxPts[0] %g",fpFluxPts,fpFluxPts[0]);
#endif
  /*     Null   */

  /* First the NullData Array - which is a 3 vector containing Null Data*/
  aNullData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,1);
  if(!aNullData) {
    DXSetError(ERROR_NO_MEMORY,"Creation of Null Data Array failed");
    goto error;
  }
  if(!DXAddArrayData(aNullData,0,iNumFluxPts,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of Null Data Array failed");
    goto error;
  }

  /* Get Ptr to Data portion of NullData Array*/
  fpNullData = (float*)DXGetArrayData(aNullData);

  /* Second the NullPts Array - location of grid points*/
  aNullPts = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
  if(!aNullPts) {
    DXSetError(ERROR_NO_MEMORY,"Creation of Null Pts Array failed");
    goto error;
  }
  if(!DXAddArrayData(aNullPts,0,iNumFluxPts,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of Null Pts Array failed");
    goto error;
  }

  /* Get Ptr to Data portion of NullPts Array*/
  fpNullPts = (float*)DXGetArrayData(aNullPts);

  /*   Div B check   */

  /* First the DivBData Array - which is a 1 vector containing DivB Data*/
  aDivBData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,1);
  if(!aDivBData) {
    DXSetError(ERROR_NO_MEMORY,"Creation of DivB Data Array failed");
    goto error;
  }
  if(!DXAddArrayData(aDivBData,0,iNumDivPts,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of DivB Data Array failed");
    goto error;
  }

  /* Get Ptr to Data portion of DivBData Array*/
  fpDivBData = (float*)DXGetArrayData(aDivBData);

  /* Second the DivBPts Array - location of grid points*/
  aDivBPts = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
  if(!aDivBPts) {
    DXSetError(ERROR_NO_MEMORY,"Creation of DivB Pts Array failed");
    goto error;
  }
  if(!DXAddArrayData(aDivBPts,0,iNumDivPts,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of DivB Pts Array failed");
    goto error;
  }

  /* Get Ptr to Data portion of DivBPts Array*/
  fpDivBPts = (float*)DXGetArrayData(aDivBPts);
#ifdef DEBUG
  DXMessage("fpDivBPts %x fpDivBPts[0] %g",fpDivBPts,fpDivBPts[0]);
#endif


  /* Now that we have space we can get the Grid information */
  iErr = GetGrid(liFileId,fNorm,ni,nj,nk,fpMHDPts,fpFluxPts, fpDivBPts);
  /* Tube Pts are the same as MHD Pts */
  for(i=0;i<3*iNumPts;i++) fpTubePts[i]=fpMHDPts[i];

  /* then we get information about timesteps in file */
  liAttrId = SDfindattr(liFileId,"initial_step");
  liStat = SDattrinfo(liFileId,liAttrId,caLabel,&liNumType,&liCount);
  liStat = SDreadattr(liFileId,liAttrId,&iInitial);
  liAttrId = SDfindattr(liFileId,"final_step");
  liStat = SDattrinfo(liFileId,liAttrId,caLabel,&liNumType,&liCount);
  liStat = SDreadattr(liFileId,liAttrId,&iFinal);
  liAttrId = SDfindattr(liFileId,"dump_interval");
  liStat = SDattrinfo(liFileId,liAttrId,caLabel,&liNumType,&liCount);
  liStat = SDreadattr(liFileId,liAttrId,&iInterval);

  /* Now that we have the information on the steps check to make sure the
   * requested step is valid
   */

  iStepFix = ChkStep(iStep,iInitial,iFinal,iInterval);

  /* Get the MHD data */
  iErr = GetMHDData(liFileId,iStepFix,ni,nj,nk,iComputeTubes,
                    fpMHDData,fpFluxData,fpDivBData, fpTubeData);

  /* Compute the location of Nulls */
  /* A null occurs inside a cell if all three fluxes pass through zero inside
   * its computational volume.  This implies that the sign must change on either
   * side of the interface so we compute the product, because only cells with
   * negative products will have passed through zero, then we get the union of
   * cells with all three sign flips 
   */
  fpBiFlip = malloc(sizeof(float)*iNumFluxPts);
  fpBjFlip = malloc(sizeof(float)*iNumFluxPts);
  fpBkFlip = malloc(sizeof(float)*iNumFluxPts);
  nv = 3;
  for(i=0;i<nip1-2;i++) {
    for(j=0;j<njp1-1;j++) {
      for(k=0;k<nkp1-1;k++) {
        fpBiFlip[ijk1(i,j,k)]= fpFluxData[vi1j1k(0,i,j,k)]*
                               fpFluxData[vi1j1k(0,i+1,j,k)];
      }
    }
  }
  for(i=0;i<nip1-1;i++) {
    for(j=0;j<njp1-2;j++) {
      for(k=0;k<nkp1-1;k++) {
        fpBjFlip[ijk1(i,j,k)]=fpFluxData[vijk1(1,i,j,k)]*
                               fpFluxData[vijk1(1,i,j+1,k)];
      }
    }
  }
  for(i=0;i<nip1-1;i++) {
    for(j=0;j<njp1-1;j++) {
      for(k=0;k<nkp1-2;k++) {
        fpBkFlip[ijk1(i,j,k)]=fpFluxData[vijk1(2,i,j,k)]*
                               fpFluxData[vijk1(2,i,j,k+1)];
      }
    }
  }
  for(i=0;i<nip1-1;i++) {
    for(j=0;j<njp1-1;j++) {
      for(k=0;k<nkp1-1;k++) {
        if ( fpBiFlip[ijk1(i,j,k)] < 0.0 && fpBjFlip[ijk1(i,j,k)] < 0.0 &&
             fpBkFlip[ijk1(i,j,k)] < 0.0) {
           fpNullData[ijk1(i,j,k)] = 1.0;
        } else {
           fpNullData[ijk1(i,j,k)] = 0.0;
        }
      }
    }
  }
  /* Copy the points over to output field */
  for(i=0;i<iNumFluxPts-1;i++) 
    fpNullPts[i] = fpFluxPts[i];

  free(fpBiFlip);
  free(fpBjFlip);
  free(fpBkFlip);
  /* Get the simulation time */
  sprintf(caLabel,"time_step_%d",iStepFix);
  liIndex = SDnametoindex(liFileId,caLabel);
  liSdsId = SDselect(liFileId,liIndex);
  liAttrId = SDfindattr(liSdsId,"time");
  liStat = SDreadattr(liSdsId,liAttrId,&fSimTime);
  SDendaccess(liSdsId);
  
  /* Now we output the data */
  fMHD = DXNewField();
  if (!DXSetStringAttribute((Object)aMHDData, "dep", "positions")) {
    DXSetError(ERROR_INTERNAL,"Failed to set dep");
    goto error;
  }
  /* set the data array as the data component of f */
  if (!DXSetComponentValue(fMHD, "data", (Object)aMHDData)) {
    DXSetError(ERROR_INTERNAL,"Failed to set data");
    goto error;
  }
  /* Load position data */
  if (!DXSetComponentValue(fMHD, "positions", (Object)aMHDPts)) {
    DXSetError(ERROR_INTERNAL,"Failed to set positions");
    goto error;
  }
  /* aMHDConnect = DXMakeGridConnectionsV(3,iaMHDDim); */
  aMHDConnect = DXNewArray(TYPE_INT,CATEGORY_REAL,1,8);
  if (!aMHDConnect) {
    DXSetError(ERROR_INTERNAL,"Failed to allocate space for Connections");
    goto error;
  }
  if (!DXAddArrayData(aMHDConnect,0,(ni-1)*njp1*nkp1,NULL)) {
    DXSetError(ERROR_INTERNAL,"Failed to allocate space for connections");
    goto error;
  }
  ipMHDConnect = (int*)DXGetArrayData(aMHDConnect);
  
  for(k=0;k<nkp1;k++) {
     koff = njp2*ni*k; koff2 = koff+njp2*ni;
     for (j=0; j<njp2-1; j++ ) {
       joff = ni*j; joff2 = ni*(j+1);
       for (i=0; i<ni-1; i++) {
         *ipMHDConnect = i + joff +koff;
         *(ipMHDConnect+1) = i+1 + joff + koff;
         *(ipMHDConnect+2) = i + joff2 + koff;
         *(ipMHDConnect+3) = i+1 + joff2 + koff;
         *(ipMHDConnect+4) = (i + joff + koff2)%iNumPts;
         *(ipMHDConnect+5) = (i+1 + joff + koff2)%iNumPts;
         *(ipMHDConnect+6) = (i + joff2 + koff2)%iNumPts;
         *(ipMHDConnect+7) = (i+1 + joff2 + koff2)%iNumPts;
         ipMHDConnect +=8;
       }
     }
   }

/*  if (!DXSetComponentValue(fMHD, "connections", (Object)aMHDConnect)) {
    DXSetError(ERROR_INTERNAL,"Failed to set connections");
    goto error;
  }
*/
  if (!DXSetConnections(fMHD,"cubes",(Object)aMHDConnect)) {
    DXSetError(ERROR_INTERNAL,"Failed to set connections");
    goto error;
  }
  /* End Field sets default attributes */
  if(!DXEndField(fMHD)) {
     DXSetError(ERROR_INTERNAL,"Couldn't complete field");
     goto error;
  }

/*
  nv = 4;
  for(k=0;k<nk;k++){
    for(j=0;j<nj;j++){
      for(i=0;i<ni;i++){
	DXMessage("%d %d %d %.8e",i,j,k,fpTubeData[vijk(3,i,j,k)]);
  }}}
*/


  /* Now we output the Tube data */
  fTube = DXNewField();
  if (!DXSetStringAttribute((Object)aTubeData, "dep", "positions")) {
    DXSetError(ERROR_INTERNAL,"Failed to set dep");
    goto error;
  }
  /* set the data array as the data component of f */
  if (!DXSetComponentValue(fTube, "data", (Object)aTubeData)) {
    DXSetError(ERROR_INTERNAL,"Failed to set data");
    goto error;
  }
  /* Load position data */
  if (!DXSetComponentValue(fTube, "positions", (Object)aTubePts)) {
    DXSetError(ERROR_INTERNAL,"Failed to set positions");
    goto error;
  }
  /* aTubeConnect = DXMakeGridConnectionsV(3,iaTubeDim); */
  aTubeConnect = DXNewArray(TYPE_INT,CATEGORY_REAL,1,8);
  if (!aTubeConnect) {
    DXSetError(ERROR_INTERNAL,"Failed to allocate space for Connections");
    goto error;
  }
  if (!DXAddArrayData(aTubeConnect,0,(ni-1)*njp1*nkp1,NULL)) {
    DXSetError(ERROR_INTERNAL,"Failed to allocate space for connections");
    goto error;
  }
  ipTubeConnect = (int*)DXGetArrayData(aTubeConnect);
  
  for(k=0;k<nkp1;k++) {
     koff = njp2*ni*k; koff2 = koff+njp2*ni;
     for (j=0; j<njp2-1; j++ ) {
       joff = ni*j; joff2 = ni*(j+1);
       for (i=0; i<ni-1; i++) {
         *ipTubeConnect = i + joff +koff;
         *(ipTubeConnect+1) = i+1 + joff + koff;
         *(ipTubeConnect+2) = i + joff2 + koff;
         *(ipTubeConnect+3) = i+1 + joff2 + koff;
         *(ipTubeConnect+4) = (i + joff + koff2)%iNumPts;
         *(ipTubeConnect+5) = (i+1 + joff + koff2)%iNumPts;
         *(ipTubeConnect+6) = (i + joff2 + koff2)%iNumPts;
         *(ipTubeConnect+7) = (i+1 + joff2 + koff2)%iNumPts;
         ipTubeConnect +=8;
       }
     }
   }

/*  if (!DXSetComponentValue(fTube, "connections", (Object)aTubeConnect)) {
    DXSetError(ERROR_INTERNAL,"Failed to set connections");
    goto error;
  }
*/
  if (!DXSetConnections(fTube,"cubes",(Object)aTubeConnect)) {
    DXSetError(ERROR_INTERNAL,"Failed to set connections");
    goto error;
  }
  /* End Field sets default attributes */
  if(!DXEndField(fTube)) {
     DXSetError(ERROR_INTERNAL,"Couldn't complete field");
     goto error;
  }
  
  /* Create the Flux output field*/
  fFlux = DXNewField();
  if(!fFlux) {
    DXSetError(ERROR_NO_MEMORY,"Creation of Flux field failed");
    goto error;
  }

  /* Set the dependence of the Data Array to be on positons*/
  if(!DXSetStringAttribute((Object)aFluxData,"dep","positions")) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array Dep");
     goto error;
  }
  /* Set the Flux Data array as the data component of the Flux Field*/
  if(!DXSetComponentValue(fFlux,"data",(Object)aFluxData)) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array of Flux Field");
     goto error;
  }
  /* Set the Flux Pts array as the position component of the Flux Field*/
  if(!DXSetComponentValue(fFlux,"positions",(Object)aFluxPts)) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Position Array of Flux Field");
     goto error;
  }


  /* Determine the connections between points in field */
  aFluxConnect = DXMakeGridConnectionsV(3,iaFluxDim);
  if (!aFluxConnect) {
    DXSetError(ERROR_INTERNAL,"Couldn't Make Flux Grid Connections");
    goto error;
  }
  if (!DXSetComponentValue(fFlux,"connections",(Object)aFluxConnect)) {
    DXSetError(ERROR_INTERNAL,"Couldn't set Ion Flux Connections");
    goto error;
  }
  /* End Field sets default attributes */
  if(!DXEndField(fFlux)) {
     DXSetError(ERROR_INTERNAL,"Couldn't complete Flux field");
     goto error;
  }

  /* Create the Null output field*/
  fNull = DXNewField();
  if(!fNull) {
    DXSetError(ERROR_NO_MEMORY,"Creation of Null field failed");
    goto error;
  }

  /* Set the dependence of the Data Array to be on positons*/
  if(!DXSetStringAttribute((Object)aNullData,"dep","positions")) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array Dep");
     goto error;
  }
  /* Set the Null Data array as the data component of the Null Field*/
  if(!DXSetComponentValue(fNull,"data",(Object)aNullData)) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array of Null Field");
     goto error;
  }
  /* Set the Null Pts array as the position component of the Null Field*/
  if(!DXSetComponentValue(fNull,"positions",(Object)aNullPts)) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Position Array of Null Field");
     goto error;
  }


  /* Determine the connections between points in field */
  aNullConnect = DXMakeGridConnectionsV(3,iaFluxDim);
  if (!aNullConnect) {
    DXSetError(ERROR_INTERNAL,"Couldn't Make Null Grid Connections");
    goto error;
  }
  if (!DXSetComponentValue(fNull,"connections",(Object)aNullConnect)) {
    DXSetError(ERROR_INTERNAL,"Couldn't set Null Grid Connections");
    goto error;
  }
  /* End Field sets default attributes */
  if(!DXEndField(fNull)) {
     DXSetError(ERROR_INTERNAL,"Couldn't complete Null field");
     goto error;
  }

  /* Create the Div B output field*/
  fDivB = DXNewField();
  if(!fDivB) {
    DXSetError(ERROR_NO_MEMORY,"Creation of DivB field failed");
    goto error;
  }

  /* Set the dependence of the Data Array to be on positons*/
  if(!DXSetStringAttribute((Object)aDivBData,"dep","positions")) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array Dep");
     goto error;
  }
  /* Set the DivB Data array as the data component of the DivB Field*/
  if(!DXSetComponentValue(fDivB,"data",(Object)aDivBData)) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array of DivB Field");
     goto error;
  }
  /* Set the DivB Pts array as the position component of the DivB Field*/
  if(!DXSetComponentValue(fDivB,"positions",(Object)aDivBPts)) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Position Array of DivB Field");
     goto error;
  }


  /* Determine the connections between points in field */
  aDivBConnect = DXMakeGridConnectionsV(3,iaDivDim);
  if (!aDivBConnect) {
    DXSetError(ERROR_INTERNAL,"Could Make Ion Grid Connections");
    goto error;
  }
  if (!DXSetComponentValue(fDivB,"connections",(Object)aDivBConnect)) {
    DXSetError(ERROR_INTERNAL,"Could set Ion Grid Connections");
    goto error;
  }
  /* End Field sets default attributes */
  if(!DXEndField(fDivB)) {
     DXSetError(ERROR_INTERNAL,"Couldn't complete field");
     goto error;
  }

  /* Now we process the ionospheric information */
  nk2p3 = nk/2+3;
  iaIONDim[1]=njp1;
  iaIONDim[0]=nk2p3;
  ionNumPts = njp1*nk2p3;
  
  /* Setup arrays to hold the data */
   aIONData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,18);
  if(!aIONData) {
    DXSetError(ERROR_NO_MEMORY,"Creation of ION Data Array failed");
    goto error;
  }
  if(!DXAddArrayData(aIONData,0,ionNumPts,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of ION Data Array failed");
    goto error;
  }

  // Get Ptr to Data portion of IONData Array
  fpIONData = (float*)DXGetArrayData(aIONData);

  // Second the IONPts Array - location of grid points
  aIONPts = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
  if(!aIONPts) {
    DXSetError(ERROR_NO_MEMORY,"Creation of ION Pts Array failed");
    goto error;
  }
  if(!DXAddArrayData(aIONPts,0,ionNumPts,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of ION Pts Array failed");
    goto error;
  }

  // Get Ptr to Data portion of IONPts Array
  fpIONPts = (float*)DXGetArrayData(aIONPts);

  /* Now we get the ionospheric grid */
  iErr = CalcIonGrid(fNorm,nip1,njp2,nk2p3,fpIONPts);
 
  /* then we get the ionospheric data */
  iErr = GetIonData(liFileId,iStepFix,nip1,njp1,nk2p3,fpIONData);

  /* and then setup the ionospheric field */

  fION = DXNewField();
  if(!fION) {
    DXSetError(ERROR_NO_MEMORY,"Creation of ION field failed");
    goto error;
  }

  /* Set the dependence of the Data Array to be on positons */
  if(!DXSetStringAttribute((Object)aIONData,"dep","positions")) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array Dep");
    goto error;
  }
  /* Set the ION Data array as the data component of the ION Field */
  if(!DXSetComponentValue(fION,"data",(Object)aIONData)) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array of MHD Field");
     goto error;
  }
  /* Set the ION Pts array as the position component of the ION Field */
  if(!DXSetComponentValue(fION,"positions",(Object)aIONPts)) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Position Array of ION Field");
     goto error;
  }
  /* Make the connections */
  aIONConnect = DXMakeGridConnectionsV(2,iaIONDim);
  if (!aIONConnect) {
    DXSetError(ERROR_INTERNAL,"Could Make ION Grid Connections");
    goto error;
  }
  if (!DXSetComponentValue(fION,"connections",(Object)aIONConnect)) {
    DXSetError(ERROR_INTERNAL,"Could set ION Grid Connections");
    goto error;
  }
 /* End Field sets default attributes */
  if(!DXEndField(fION)) {
     DXSetError(ERROR_INTERNAL,"Couldn't complete field");
     goto error;
  }
  

  SDend(liFileId);
  out[0]=(Object)fMHD;
  out[1]=(Object)fION;
  out[2]=(Object)fFlux;
  out[3]=(Object)DXMakeFloat(fSimTime);
  out[4]=(Object)fDivB;
  out[5]=(Object)fNull;
  out[6]=(Object)fTube;
  return OK;

  error:
    DXDelete((Object)fMHD);
    DXDelete((Object)aMHDData);
    DXDelete((Object)aMHDPts);
    DXDelete((Object)aMHDConnect);
    DXDelete((Object)fTube);
    DXDelete((Object)aTubeData);
    DXDelete((Object)aTubePts);
    DXDelete((Object)aTubeConnect);
    DXDelete((Object)fFlux);
    DXDelete((Object)aFluxData);
    DXDelete((Object)aFluxPts);
    DXDelete((Object)aFluxConnect);
    DXDelete((Object)fDivB);
    DXDelete((Object)aDivBData);
    DXDelete((Object)aDivBPts);
    DXDelete((Object)aDivBConnect);
    DXDelete((Object)fION);
    DXDelete((Object)aIONData);
    DXDelete((Object)aIONPts);
    DXDelete((Object)aIONConnect);
    return ERROR;
    
}

int ChkStep(int iTimeStep, int iInital, int iFinal, int iInterval) {

  if( iTimeStep >= iInital && iTimeStep <= iFinal) {
    if( iTimeStep % iInterval == 0 ) {
      return iTimeStep;
    } else {
      fprintf(stderr,"Requested timestep is not VALID!!\n");
      fprintf(stderr,"Changed from %d to %d\n",iTimeStep,
                iTimeStep-iTimeStep%iInterval);
      return iTimeStep - iTimeStep % iInterval;
    }
  } else {
    if (iTimeStep < iInital) {
      fprintf(stderr,"Requested timestep is not VALID!!\n");
      fprintf(stderr,"Changed from %d to %d\n",iTimeStep,iInital);
      return iInital;
    } else {
      fprintf(stderr,"Requested timestep is not VALID!!\n");
      fprintf(stderr,"Changed from %d to %d\n",iTimeStep,iFinal);
      return iFinal;
    }
  }
}
