#include "mfhdf.h"
#include <dx/dx.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "funcs.h"
#include "fpointers.h"

#define MAXRANK 3

/* This module reads in the 'new' style HDF files from the SMP version of the
 * LFM.  The 'new' style HDF files have several advantages: 1) All ionospheric
 * data is stored in the file, 2) grid size information is accessible from file
 * 3) random access via step number information
 *
 * The module has three inputs
 *   In[0] = Filename - string
 *   In[2] = Normalization - float
 *
 * and it has four outputs
 *   Out[0] = MHD Data field - Field float 14 vector contains mhd data
 *
 * NB - Use of this function requires linking against a HDF library compiled
 * with the -DHAVE_NETCDF flag
 */



Error m_ReadLFMSpectrum(Object *in, Object *out)
{
   /***********************/
   /*  Declare variables  */
   /***********************/

  /* Variable Declarations */
  /* Inputs */
  int iStep;               	/* Step Number */
  char *cpFileName=NULL;        /* Ptr to File Name */
  float fNorm;             	/* Normalization Constant */
  FILE *fptr;			/* File pointer to verify file */

  /* Outputs */
  Array aMHDData=NULL;		/* Array for MHD Field Data */
  float *fpMHDData;		/* Ptr to MHD data */
  Array aMHDPts=NULL;		/* Array for MHD Field Pts */
  float *fpMHDPts;		/* Ptr to MHD Pts data */
  Array aMHDConnect=NULL;	/* Array for MHD Field Connections */
  int   *ipMHDConnect;          /* Ptr to MHD Connection data */
  Field fMHD=NULL;		/* MHD Data Field */
  float *fpFluxPts, *fpDivPts;   /* Temp Pointers */

  /* Other Variables */
  int32 liFileId, liIndex,liSdsId, liErr; /* HDF Access Vars */
  int32 liAttrId, liStat, liNumType, liSize, liRank, liCount;
  int32 liaDimSize[MAX_VAR_DIMS], *lipStart, *lipEdge, *lipStride; 
  int32 liSecId;
  char caLabel[MAX_NC_NAME];
  int ni,nip1,nj,njp1,njp2,nk,nkp1,nk2p3,iaMHDDim[3],iNumPts;/*Grid Size Vars */
  int iNumFluxPts, iNumDivPts;
  int nv;
  int ionnjp1, ionnk2p3, iaIONDim[2], ionNumPts; /* ION Size Vars */
  int iErr;
  int i,j,k; /* Looping Vars */
  int iInitial, iFinal, iInterval, iStepFix;
  int koff,koff2,joff,joff2;

  /* Get Input Values */

   if (!in[0]) {
      DXSetError(ERROR_BAD_PARAMETER,"missing filename");
      goto error;
   } else if (!DXExtractString(in[0], &cpFileName)) {   
    DXSetError(ERROR_BAD_PARAMETER, "filename must be a string");
    goto error;
  }
  if (!in[1]) {
    DXSetError(ERROR_BAD_PARAMETER,"missing Normalization factor ");
    goto error;
  } else if (!DXExtractFloat(in[1], &fNorm)) {   
    DXSetError(ERROR_BAD_PARAMETER, "Normalization factor must be a float");
    goto error;
  }

#ifdef DEBUG
   DXMessage("Filename: %s",cpFileName);
   DXMessage("Normalization: %g",fNorm);
#endif

   /* Main Body of Function */
 
   fptr = fopen(cpFileName,"r");  
   if(fptr == NULL) {
     DXSetError(ERROR_BAD_PARAMETER,"FILE %s does not exist",cpFileName);
     goto error;
   }
   if (DFishdf(cpFileName) !=0) {
     DXSetError(ERROR_BAD_PARAMETER,"DX says %s isn't HDF file",cpFileName);
    goto error;
   }

 /* First step is to open file and determine grid size */ 
   if((liFileId = SDstart(cpFileName,
                        DFACC_RDONLY)) < 0) {
    DXMessage("FileId: %d",liFileId);
    DXSetError(ERROR_BAD_PARAMETER,"Unable to open Grid File");
    goto error;

  } 
#ifdef DEBUG
  DXMessage("File Id %d",liFileId);
#endif 

  /* grid_x2 will give us nip1 and njp1  */
#ifdef DEBUG
  DXMessage("Getting grid size");
#endif 
  liIndex = SDnametoindex(liFileId,"grid_x2");
  liSecId = SDselect(liFileId,liIndex);
  liStat = SDgetinfo(liSecId,caLabel,&liRank,liaDimSize,&liNumType,&liAttrId);
  nip1 = liaDimSize[1];
  njp1 = liaDimSize[0];
  ni = nip1-1;
  nj = njp1-1;
  njp2 = njp1+1;
  liStat = SDendaccess(liSecId);

  /* grid_phi will give us nkp1 */
  liIndex = SDnametoindex(liFileId,"grid_phi");
  liSecId = SDselect(liFileId,liIndex);
  liStat = SDgetinfo(liSecId,caLabel,&liRank,liaDimSize,&liNumType,&liAttrId);
  nkp1 = liaDimSize[0];
  nk = nkp1-1;
  liStat = SDendaccess(liSecId);

  iaMHDDim[0]=nkp1;
  iaMHDDim[1]=njp2;
  iaMHDDim[2]=ni;
  iNumPts = ni*njp2*nkp1; /* Actual size of renderable grid */
  iNumFluxPts = nip1*njp1*nkp1;
  iNumDivPts = ni*nj*nkp1;
#ifdef DEBUG
  DXMessage("NI NJ NK %d %d %d\n",ni,nj,nk);
#endif

  /* For power spectrum files we also need the number  freqs used */
  liIndex = SDnametoindex(liFileId,"powerspec");
  liSecId = SDselect(liFileId,liIndex);
  liStat = SDgetinfo(liSecId,caLabel,&liRank,liaDimSize,&liNumType,&liAttrId);
  nv = liaDimSize[0];
  DXMessage("Freqs %d",nv,liRank);
  liStat = SDendaccess(liSecId);

  /* Now that we have the size of the grid we can create the DX arrays that will
   * hold the data
   */

  /* First the MHDData Array - which is a 14 vector containing MHD Data*/
  aMHDData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,nv);
  if(!aMHDData) {
    DXSetError(ERROR_NO_MEMORY,"Creation of MHD Data Array failed");
    goto error;
  }
  if(!DXAddArrayData(aMHDData,0,iNumPts,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of MHD Data Array failed");
    goto error;
  }

  /* Get Ptr to Data portion of MHDData Array*/
  fpMHDData = (float*)DXGetArrayData(aMHDData);

  /* Second the MHDPts Array - location of grid points*/
  aMHDPts = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
  if(!aMHDPts) {
    DXSetError(ERROR_NO_MEMORY,"Creation of MHD Pts Array failed");
    goto error;
  }
  if(!DXAddArrayData(aMHDPts,0,iNumPts,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of MHD Pts Array failed");
    goto error;
  }

  /* Get Ptr to Data portion of MHDPts Array*/
  fpMHDPts = (float*)DXGetArrayData(aMHDPts);

  /* Now that we have space we can get the Grid information */
  fpFluxPts = (float*)malloc(3*iNumFluxPts*sizeof(float));
  fpDivPts = (float*)malloc(3*iNumDivPts*sizeof(float));
  iErr = GetGrid(liFileId,fNorm,ni,nj,nk,fpMHDPts,fpFluxPts,fpDivPts);
  DXMessage("After Grid");

  /* Get the Power Spectrum  */
  iErr = GetPwrSpctrm(liFileId,nv,ni,nj,nk,fpMHDData);
  DXMessage("After Spctrm");

  /* Now we output the data */
  fMHD = DXNewField();
  if (!DXSetStringAttribute((Object)aMHDData, "dep", "positions")) {
    DXSetError(ERROR_INTERNAL,"Failed to set dep");
    goto error;
  }
  /* set the data array as the data component of f */
  if (!DXSetComponentValue(fMHD, "data", (Object)aMHDData)) {
    DXSetError(ERROR_INTERNAL,"Failed to set data");
    goto error;
  }
  /* Load position data */
  if (!DXSetComponentValue(fMHD, "positions", (Object)aMHDPts)) {
    DXSetError(ERROR_INTERNAL,"Failed to set positions");
    goto error;
  }
  /* aMHDConnect = DXMakeGridConnectionsV(3,iaMHDDim); */
  aMHDConnect = DXNewArray(TYPE_INT,CATEGORY_REAL,1,8);
  if (!aMHDConnect) {
    DXSetError(ERROR_INTERNAL,"Failed to allocate space for Connections");
    goto error;
  }
  if (!DXAddArrayData(aMHDConnect,0,(ni-1)*njp1*nkp1,NULL)) {
    DXSetError(ERROR_INTERNAL,"Failed to allocate space for connections");
    goto error;
  }
  ipMHDConnect = (int*)DXGetArrayData(aMHDConnect);
  
  for(k=0;k<nkp1;k++) {
     koff = njp2*ni*k; koff2 = koff+njp2*ni;
     for (j=0; j<njp2-1; j++ ) {
       joff = ni*j; joff2 = ni*(j+1);
       for (i=0; i<ni-1; i++) {
         *ipMHDConnect = i + joff +koff;
         *(ipMHDConnect+1) = i+1 + joff + koff;
         *(ipMHDConnect+2) = i + joff2 + koff;
         *(ipMHDConnect+3) = i+1 + joff2 + koff;
         *(ipMHDConnect+4) = (i + joff + koff2)%iNumPts;
         *(ipMHDConnect+5) = (i+1 + joff + koff2)%iNumPts;
         *(ipMHDConnect+6) = (i + joff2 + koff2)%iNumPts;
         *(ipMHDConnect+7) = (i+1 + joff2 + koff2)%iNumPts;
         ipMHDConnect +=8;
       }
     }
   }

/*  if (!DXSetComponentValue(fMHD, "connections", (Object)aMHDConnect)) {
    DXSetError(ERROR_INTERNAL,"Failed to set connections");
    goto error;
  }
*/
  if (!DXSetConnections(fMHD,"cubes",(Object)aMHDConnect)) {
    DXSetError(ERROR_INTERNAL,"Failed to set connections");
    goto error;
  }
  /* End Field sets default attributes */
  if(!DXEndField(fMHD)) {
     DXSetError(ERROR_INTERNAL,"Couldn't complete field");
     goto error;
  }

  SDend(liFileId);
  out[0]=(Object)fMHD;
  return OK;

  error:
    DXDelete((Object)fMHD);
    DXDelete((Object)aMHDData);
    DXDelete((Object)aMHDPts);
    DXDelete((Object)aMHDConnect);
    return ERROR;
    
}

