#include "mfhdf.h"
#include <dx/dx.h>
#include <stdio.h>
#include <math.h>
#include "funcs.h"
#include "debug.h"

/* This modules is based upon Ravi's read_mhd_hdf.c code which imports the old
 * style HDF files into Data Explorer.  This routine will import the data from
 * the parallel verison of the code into data explorer.  The new file format has
 * the grid information in one file and the MHD data in another file.  The files
 * also contain information about the grid size which can be used to keep this
 * module flexable.  
 *
 * This module has three inputs
 *     In[0] = directory containing data - string
 *     In[1] = time step to be read - string
 *     In[2] = Normalization - float
 *
 *  and it has four outputs
 *     Out[1] = MHD Data Field - Field float 14 vector contains mhd data
 *     Out[2] = ION Data Field - Field float 15 vector contains ion data
 *     Out[4] = Magnetic flux Field - Field float 3 vector
 *     Out[3] = Time - float simulation time in seconds
 */

Error
m_ReadLFMPara(Object *in, Object *out)
{
  /* Variable Declarations */
  int lstep, dirlen;
  /* Inputs */
  char *cpGridName = NULL; // Ptr to Grid File Name
  char *cpDataName = NULL; // Ptr to Data File Name
  char *cpIonGridName = NULL; // Ptr to Ion Grid File Name
  char *cpIonDataName = NULL; // Ptr to Ion Data File Name
  char *cpDirName = NULL; // Ptr to directory name
  float fNorm;             // Normalization Constant
  
  /* Outputs */
  Array aMHDData = NULL;  /* Array for MHD Field Data */
  float *fpMHDData;       /* Ptr to Grid Data */
  Array aMHDPts = NULL;   /* Array for MHD Field Pts */
  float *fpMHDPts;        /* Ptr to Grid Pts */
  Array aMHDConnect=NULL; /* Array for MHD connections */
  Field fMHD = NULL;      /* MHD Data Field */
  Array aIonData = NULL;  /* Array for Ion Field Data */
  float *fpIonData;       /* Ptr for Ion Data */
  Array aIonPts = NULL;   /* Array for Ion Pts */
  float *fpIonPts;        /* Ptr for Ion Grid Pts */
  Array aIonConnect=NULL; /* Array for Ion Connections */
  Field fION = NULL;      /* Ion Data Field */
  float fSimTime;         /* Simulation Time */

  /* Other Variables */
  int32 liFileId, liIndex,liSdsId, liAtrId, liErr; /* HDF Access Vars */
  int ni,nip1,nj,njp1,njp2,nk,nkp1, iaMHDDim[3],iNumPts;  /* Grid Size Vars */
  int ionnjp1, ionnk2p3, iaIonDim[2], ionNumPts; /* Ion Size Vars */
  int i,j,k; /* Looping Vars */
  int iErr;
  int ncid;

  
  cpIonGridName = calloc(256,1);
  cpGridName = calloc(256,1);
  cpIonDataName = calloc(256,1);
  cpDataName = calloc(256,1);
  cpDirName = calloc(256,1);

  /* Get Input Values */

  /* Get Grid Filename*/
  if(!in[0]) { 
    DXSetError(ERROR_BAD_PARAMETER,"Missing Directory Path");
    goto error;
  } else if (!DXExtractString(in[0],&cpDirName)) {
    DXSetError(ERROR_BAD_PARAMETER,"MHD Directory Path must be string");
    goto error;
  }

  strcat(cpDirName,"/");
  strcpy(cpGridName,cpDirName);
  strcpy(cpDataName,cpDirName);
  strcpy(cpIonGridName,cpDirName);
  strcpy(cpIonDataName,cpDirName);

  strcat(cpGridName,"DUMPMHD.hdf");
  strcat(cpIonGridName,"DUMPION.hdf");


  /* Get Time Step */
  if(!in[1]) { 
    DXSetError(ERROR_BAD_PARAMETER,"Missing Time Step");
    goto error;
  } else if (!DXExtractInteger(in[1],&lstep)) {
    DXSetError(ERROR_BAD_PARAMETER,"Time step must be an integer");
    goto error;
  }
  
  dirlen = strlen(cpDirName);
  sprintf(&cpDataName[dirlen],"DUMP_%7.7d.hdf\0",lstep);
  sprintf(&cpIonDataName[dirlen],"ION_%7.7d.hdf\0",lstep);

  /* Get Normalization Constant*/
  if(!in[2]) {
    fNorm = 6.38e8;
  } else if (!DXExtractFloat(in[2],&fNorm)) {
    DXSetError(ERROR_BAD_PARAMETER,"Normalization Constant must be float");
    goto error;
  }

#ifdef DEBUG
  /* Give some outut to the user about the input parameters used */
  DXMessage("MHD Grid Filename: %s",cpGridName);
  DXMessage("MHD Data Filename: %s",cpDataName);
  DXMessage("Ion Grid Filename: %s",cpIonGridName);
  DXMessage("Ion Data Filename: %s",cpIonDataName);
  DXMessage("Normalization Constant: %f",fNorm);
#endif 

  /* Main Body of Function */

  /* Process MHD Data */
  if (DFishdf(cpGridName) !=0) {
    DXSetError(ERROR_BAD_PARAMETER,"DX says %s isn't HDF file",cpGridName);
    goto error;
  }
  
#ifdef DEBUG
  DXMessage("It's an HDF File");
#endif
  /* First step is to open grid file and determine size of comp grid*/
  if((liFileId = SDstart(cpGridName,
  			 DFACC_RDONLY)) < 0) {
    DXSetError(ERROR_BAD_PARAMETER,"Unable to open Grid File");
    goto error;
  } 

  liIndex = SDnametoindex(liFileId,"grid_x2");
  liSdsId = SDselect(liFileId,liIndex);
  liAtrId = SDfindattr(liSdsId,"ni");
  liErr = SDreadattr(liSdsId,liAtrId,&nip1);
  liAtrId = SDfindattr(liSdsId,"nj");
  liErr = SDreadattr(liSdsId,liAtrId,&njp1);
  liErr = SDendaccess(liSdsId);
  liIndex = SDnametoindex(liFileId,"grid_phi");
  liSdsId = SDselect(liFileId,liIndex);
  liAtrId = SDfindattr(liSdsId,"ni");
  SDreadattr(liSdsId, liAtrId,&nkp1);
  liErr = SDendaccess(liSdsId);
  ni = nip1 - 1;
  nj = njp1 - 1;
  nk = nkp1 - 1;
  njp2 = njp1 + 1;
  iaMHDDim[0] = nkp1;
  iaMHDDim[1] = njp2;
  iaMHDDim[2] = ni;
  iNumPts = ni*njp2*nkp1; /* Acutal size of renderable grid */

#ifdef DEBUG
  /* Tell the user how big the grid is*/
  DXMessage("Grid Size %d %d %d",ni,nj,nk);
#endif

  /* Now that we have the size of the grid we can create the DX Arrays that will*/
  /* hold the data*/

  /* First the MHDData Array - which is a 14 vector containing MHD Data*/
  aMHDData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,14);
  if(!aMHDData) {
    DXSetError(ERROR_NO_MEMORY,"Creation of MHD Data Array failed");
    goto error;
  }
  if(!DXAddArrayData(aMHDData,0,iNumPts,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of MHD Data Array failed");
    goto error;
  }

  /* Get Ptr to Data portion of MHDData Array*/
  fpMHDData = (float*)DXGetArrayData(aMHDData);

  /* Second the MHDPts Array - location of grid points*/
  aMHDPts = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
  if(!aMHDPts) {
    DXSetError(ERROR_NO_MEMORY,"Creation of MHD Pts Array failed");
    goto error;
  }
  if(!DXAddArrayData(aMHDPts,0,iNumPts,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of MHD Pts Array failed");
    goto error;
  }

  /* Get Ptr to Data portion of MHDPts Array*/
  fpMHDPts = (float*)DXGetArrayData(aMHDPts);
#ifdef DEBUG
  DXMessage("fpMHDPts %x fpMHDPts[0] %g",fpMHDPts,fpMHDPts[0]);
#endif


  /* Now we do the real work of this module */
  /**/
  /* First step is to get the grid Returns a Fortran Order Array with x,y,z as*/
  /* the least rapidly varying quanity*/

#ifdef DEBUG
  DXMessage("Before Get Grid");
#endif
  iErr = GetGrid(liFileId,fNorm,fpMHDPts);
  /* Now that we are done with the Grid File we can close it*/
  SDend(liFileId);

  if((liFileId = SDstart(cpDataName,
  			 DFACC_RDONLY)) < 0) {
    DXSetError(ERROR_BAD_PARAMETER,"Unable to open Data File");
    goto error;
  } 

#ifdef DEBUG
  DXMessage("Before GetMHD Data");
  DXMessage("FileId: %d",liFileId);
#endif
  iErr = GetMHDData(liFileId,ni,nj,nk,fpMHDData);
#ifdef DEBUG
  DXMessage("After GetMHD Data");
#endif
  /* Use this data file to extract the simulation time */
  liIndex = SDnametoindex(liFileId,"rho_");
  liSdsId = SDselect(liFileId,liIndex);
  liAtrId = SDfindattr(liSdsId,"time");
  liErr = SDreadattr(liSdsId,liAtrId,&fSimTime);

  /* Now that we are done with the MHD Data File we can close it */
  SDend(liFileId);

  /* Create the MHD output field*/
  fMHD = DXNewField();
  if(!fMHD) {
    DXSetError(ERROR_NO_MEMORY,"Creation of MHD field failed");
    goto error;
  }

  /* Set the dependence of the Data Array to be on positons*/
  if(!DXSetStringAttribute((Object)aMHDData,"dep","positions")) { 
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array Dep");
     goto error;
  }
  /* Set the MHD Data array as the data component of the MHD Field*/
  if(!DXSetComponentValue(fMHD,"data",(Object)aMHDData)) { 
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array of MHD Field");
     goto error;
  }
  /* Set the MHD Pts array as the position component of the MHD Field*/
  if(!DXSetComponentValue(fMHD,"positions",(Object)aMHDPts)) { 
     DXSetError(ERROR_INTERNAL,"Couldn't set Position Array of MHD Field");
     goto error;
  }


  /* Determine the connections between points in field */
  aMHDConnect = DXMakeGridConnectionsV(3,iaMHDDim);
  if (!aMHDConnect) {
    DXSetError(ERROR_INTERNAL,"Could Make Ion Grid Connections");
    goto error;
  }
  if (!DXSetComponentValue(fMHD,"connections",(Object)aMHDConnect)) {
    DXSetError(ERROR_INTERNAL,"Could set Ion Grid Connections");
    goto error;
  }
  /* End Field sets default attributes */
  if(!DXEndField(fMHD)) {
     DXSetError(ERROR_INTERNAL,"Couldn't complete field");
     goto error;
  }

  /* Now we process the ionospheric data file */

  if (DFishdf(cpIonGridName) !=0) {
    DXSetError(ERROR_BAD_PARAMETER,"DX says %s isn't HDF file",cpIonGridName);
    goto error;
  }
  if((liFileId = SDstart(cpIonGridName,
  			 DFACC_RDONLY)) < 0) {
    DXSetError(ERROR_BAD_PARAMETER,"Unable to open Grid File");
    goto error;
  } 

  liIndex = SDnametoindex(liFileId,"x2ion");
  liSdsId = SDselect(liFileId,liIndex);
  liAtrId = SDfindattr(liSdsId,"ni");
  liErr = SDreadattr(liSdsId,liAtrId,&ionnjp1);
  liAtrId = SDfindattr(liSdsId,"nj");
  liErr = SDreadattr(liSdsId,liAtrId,&ionnk2p3);
  liErr = SDendaccess(liSdsId);
  ionNumPts = ionnjp1*ionnk2p3; // Acutal size of renderable grid
  iaIonDim[1] = ionnjp1;
  iaIonDim[0] = ionnk2p3;

  // Tell the user how big the grid is
#ifdef DEBUG
  DXMessage("Ion Grid Size %d %d",ionnjp1,ionnk2p3);
#endif

  // Now that we have the size of the grid we can create the DX Arrays that will
  // hold the data

  // First the IonData Array - which is a 12 vector containing MHD Data
  aIonData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,16);
  if(!aIonData) {
    DXSetError(ERROR_NO_MEMORY,"Creation of Ion Data Array failed");
    goto error;
  }
  if(!DXAddArrayData(aIonData,0,ionNumPts,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of Ion Data Array failed");
    goto error;
  }

  // Get Ptr to Data portion of IonData Array
  fpIonData = (float*)DXGetArrayData(aIonData);

  // Second the IonPts Array - location of grid points
  aIonPts = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
  if(!aIonPts) {
    DXSetError(ERROR_NO_MEMORY,"Creation of Ion Pts Array failed");
    goto error;
  }
  if(!DXAddArrayData(aIonPts,0,ionNumPts,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of Ion Pts Array failed");
    goto error;
  }

  // Get Ptr to Data portion of IonPts Array
  fpIonPts = (float*)DXGetArrayData(aIonPts);
#ifdef DEBUG
  DXMessage("fpIonPts %x fpIonPts[0] %g",fpIonPts,fpIonPts[0]);
#endif


  /* Now we do the real work of this module */
  /* First step is to get the grid Returns a Fortran Order Array with x,y,z as
     the least rapidly varying quanity
  */

#ifdef DEBUG
  DXMessage("Before Get Ion Grid");
#endif
  iErr = GetIonGrid(liFileId,fNorm,fpIonPts);
  /* Now that we are done with the Grid File we can close it */
  SDend(liFileId);

  if((liFileId = SDstart(cpIonDataName,
  			 DFACC_RDONLY)) < 0) {
    DXMessage("FileId: %d",liFileId);
    DXSetError(ERROR_BAD_PARAMETER,"Unable to open Ion Data File");
    goto error;
  } 

#ifdef DEBUG
  DXMessage("Before GetIon Data");
#endif
  iErr = GetIonData(liFileId,ionnjp1,ionnk2p3,fpIonData);
  /* Now that we are done with the Grid File we can close it */
  SDend(liFileId);

  /* Create the Ion output field */
  fION = DXNewField();
  if(!fION) {
    DXSetError(ERROR_NO_MEMORY,"Creation of ION field failed");
    goto error;
  }

  /* Set the dependence of the Data Array to be on positons */
  if(!DXSetStringAttribute((Object)aIonData,"dep","positions")) { 
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array Dep");
     goto error;
  }
  /* Set the Ion Data array as the data component of the ION Field */
  if(!DXSetComponentValue(fION,"data",(Object)aIonData)) { 
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array of MHD Field");
     goto error;
  }
  /* Set the Ion Pts array as the position component of the ION Field */
  if(!DXSetComponentValue(fION,"positions",(Object)aIonPts)) { 
     DXSetError(ERROR_INTERNAL,"Couldn't set Position Array of ION Field");
     goto error;
  }
  /* Make the connections */
  aIonConnect = DXMakeGridConnectionsV(2,iaIonDim);
  if (!aIonConnect) {
    DXSetError(ERROR_INTERNAL,"Could Make Ion Grid Connections");
    goto error;
  }
  if (!DXSetComponentValue(fION,"connections",(Object)aIonConnect)) {
    DXSetError(ERROR_INTERNAL,"Could set Ion Grid Connections");
    goto error;
  }
	
  /* End Field sets default attributes */
  if(!DXEndField(fION)) {
     DXSetError(ERROR_INTERNAL,"Couldn't complete field");
     goto error;
  }

  /* Get the simulation time */
  out[0] = (Object)fMHD;
  out[1] = (Object)fION;
  out[2] = (Object)DXMakeFloat(fSimTime);

  return OK;

error:
  //DXDelete((Object) fMHD);
  //DXDelete((Object) aMHDData);
  //DXDelete((Object) aMHDPts);
  //DXDelete((Object) aMHDConnect);
  //DXDelete((Object) fION);
  //DXDelete((Object) aIonData);
  //DXDelete((Object) aIonPts);
  //DXDelete((Object) aIonConnect);
  return ERROR;
}



  

  
