c
c	This collection of subroutines was adapted from the
c	GEOPACK.F set by N.A. Tsyganenko for use by the SPOF
c	Major changes include: modification of subroutine 
c	IGRF to accept dates beyond 1990, adoption of RECOMP
c	instead of RECALC (RECOMP has been renamed RECALC and
c	the original RECALC has been removed from the packet),
c	addition of new routine BTOT to compute the total field.
c
C
C
c	Compiled and modified by: Mauricio Peredo
c				  Hughes STX at NASA/GSFC
c				  September 1992
c
c
c
C------------------------------------------------------------------------
        SUBROUTINE BTOT(X,Y,Z,EXNAME,IOPT,BX,BY,BZ,count)
C
C
C       Written by:     Mauricio Peredo
C                       Hughes STX at NASA/GSFC, Code 695
C                       September 1992
C

C       This routine computes the total magnetic field at each
C       (input) point (X,Y,Z). It is assumed that subroutine
C       RECALC has already been called to set quantities in
C       the common block (in particular, the call to RECALC
C       sets the dipole tilt angle, and all angles needed for
C       conversions between various coordinate systems). The
C       internal field is computed from the IGRF model (for
C       appropriate year). In calling IGRF, we set the order
C       of harmonics to use according to the following rules:
C       if the radius is 10 Re or more we use a dipole
C       approximation (IHARM=1), if the radius is between
C       6 Re and 10 Re, we use dipole+quadrupole (IHARM=2),
C       inside 6 Re we go for the full blown expansion with
C       IHARM=10.  The external field is computed from the
C       routine identified by the current value of EXNAME,
C       and the "version" of that model selected depends on
C       the value of the flag IOPT [For example if we have
C       EXNAME=EX89KP and IOPT=1 the external field computation
C       is according to the T89 model with Kp = 0, 0+, and so on]
C
C
C------ INPUT:  X,Y,Z = Cartesian GSM coordinates of the point where
C                       the field is to be computed (in Re)
C               EXNAME = Name of the subroutine to use for computation
C                        of the external field. Valid values are: EXT87L,
C                        EXT87T, EX89KP, EX89AE (more will be added later)
C               IOPT = Flag indication which "version" of the model to
C                      use, the options correspond to different levels of
C                      magnetospheric activity.  EXT87L, EX89KP and EX89AE
C                      each allow IOPT = 1 to 6, EXT87T has IOPT = 1 to 8.
C
C------ OUTPUT: BX,BY,BZ = Cartesian GSM components of the total (internal
C                          plus external) magnetic field vector at (X,Y,Z)
C
C       ATTENTION: IT IS ASSUMED THAT SUBROUTINE RECALC HAS BEEN CALLED
C                  PRIOR TO CALLING THIS SUBROUTINE!!!!!!!
C
C
C       Duplicate the common block from RECOMP here
C
	IMPLICIT NONE

	INTEGER IY,K,count,i,iopt,IHARM

	EXTERNAL EXNAME

	REAL ST0,CT0,SL0,CL0,CTCL,STCL,CTSL,STSL,SFI,CFI,SPS,CPS,
     1   SHI,CHI,HI,PSI,XMUT,A11,A21,A31,A12,A22,A32,A13,A23,A33,
     2   DS3,X(count),Y(count),Z(count),BX(count),BY(count),
     3   BZ(count),BXEXT,BYEXT,BZEXT,XGEO,YGEO,ZGEO,R,T,F,FX,FY,
     4   FZ,BXINT,BYINT,BZINT,BR,BF,BT,BA(8)

       COMMON/C1/ ST0,CT0,SL0,CL0,CTCL,STCL,CTSL,STSL,SFI,CFI,SPS,CPS,
     * SHI,CHI,HI,PSI,XMUT,A11,A21,A31,A12,A22,A32,A13,A23,A33,DS3,
     * K,IY,BA

        do 100 i = 1, count

C       Compute external field

           CALL EXNAME(IOPT,PSI,X(i),Y(i),Z(i),BXEXT,BYEXT,BZEXT)
c
C       To compute internal field, we convert input point to GEO cartesian
C       coordinates, then to GEO spherical coordinates, call IGRF with the
C       appropriate value for IHARM (see comments above), then convert the
C       resulting internal field to cartesian GEO coordinates and finally to
C       cartesian GSM coordinates.
C
           CALL GEOGSM(XGEO,YGEO,ZGEO,X(i),Y(i),Z(i),-1)

           CALL SPHCAR(R,T,F,XGEO,YGEO,ZGEO,-1)
           IF (R.GE.10.) THEN
              IHARM = 1
           ELSEIF (R.GE.6.) THEN
              IHARM = 2
           ELSE
              IHARM = 10
           ENDIF

           CALL IGRF(IY,IHARM,R,T,F,BR,BT,BF)

           CALL BSPCAR(T,F,BR,BT,BF,FX,FY,FZ)

           CALL GEOGSM(FX,FY,FZ,BXINT,BYINT,BZINT,1)

C
C       Sum up internal and external contributions
C
           BX(i) = BXINT + BXEXT
           BY(i) = BYINT + BYEXT
           BZ(i) = BZINT + BZEXT

 100    continue

        return
        end
C
c
c
      SUBROUTINE IGRF(IY,NM,R,T,F,BR,BT,BF)
C
c
C	MODIFIED TO ACCEPT DATES BETWEEN 1965 AND 2000; COEFFICIENTS
C	FOR IGRF 1985 HAVE BEEN REPLACED WITH DGRF1985 COEFFICIENTS
C	[EOS TRANS. AGU APRIL 21, 1992, P. 182].  ALSO, CODE MODIFIED
C	TO ACCEPT DATES BEYOND 1990, AND TO USE LINEAR EXTRAPOLATION
C	BETWEEN 1990 AND 2000 BASED ON THE IGRF COEFFICIENTS FROM THE
C	SAME EOS ARTICLE
c
c
c	Modified by:	Mauricio Peredo
c			Hughes STX at NASA/GSFC
c			September 1992
c
c
C  CALCULATES COMPONENTS OF MAIN GEOMAGNETIC FIELD IN SPHERICAL
C  GEOGRAPHICAL COORD SYSTEM BY USING THIRD GENERATION IGRF MODEL
C  (J. GEOMAG. GEOELECTR.(1982), V.34, P.313-315,
C  GEOMAGN. AND AERONOMY (1986), V.26, P.523-525).
C  UPDATING OF COEFFICIENTS TO A GIVEN EPOCH IS MADE DURING THE FIRST
C  CALL AND AFTER EVERY CHANGE OF PARAMETER IY.
C------INPUT PARAMETERS:
C  IY - YEAR NUMBER (FROM 1965 UP TO 1990)
C  NM - MAXIMAL ORDER OF HARMONICS TAKEN INTO ACCOUNT (NOT MORE THAN 10)
C  R,T,F - SPHERICAL COORDINATES OF THE POINT (R IN UNITS RE=6371.2 KM,
C  COLATITUDE T AND LONGITUDE F IN RADIANS)
C----- OUTPUT PARAMETERS:
C  BR,BT,BF - SPHERICAL COMPONENTS OF MAIN GEOMAGN.FIELD IN NANOTESLA
C
C
C                   AUTHOR: NIKOLAI A. TSYGANENKO
C                           INSTITUTE OF PHYSICS
C                           ST.-PETERSBURG STATE UNIVERSITY
C                           STARY PETERGOF 198904
C                           ST.-PETERSBURG
C                           RUSSIA
C
	IMPLICIT NONE
C
      REAL A(11),B(11),G(66),H(66),REC(66),G65(66),H65(66),G70(66),
     *H70(66),G75(66),H75(66),G80(66),H80(66),G85(66),H85(66),G90(66),
     *H90(66),DG90(45),DH90(45)

	REAL R,T,F,BR,BT,BF,DT,F2,F1,S,P,AA,PP,D,BBR,BBF,U,CF,SF,
     1       C,W,X,Y,Z,Q,BI,P2,D2,AN,E,HH,BBT,QQ,XK,DP,PM

      LOGICAL BK,BM

	INTEGER IY,NM,MA,IPR,IYR,KNM,N,N2,M,MNN,MN,K,MM
c
      DATA G65/0.,-30334.,-2119.,-1662.,2997.,1594.,1297.,-2038.,1292.,
     *856.,957.,804.,479.,-390.,252.,-219.,358.,254.,-31.,-157.,-62.,
     *45.,61.,8.,-228.,4.,1.,-111.,75.,-57.,4.,13.,-26.,-6.,13.,1.,13.,
     *5.,-4.,-14.,0.,8.,-1.,11.,4.,8.,10.,2.,-13.,10.,-1.,-1.,5.,1.,-2.,
     *-2.,-3.,2.,-5.,-2.,4.,4.,0.,2.,2.,0./
      DATA H65/0.,0.,5776.,0.,-2016.,114.,0.,-404.,240.,-165.,0.,148.,
     *-269.,13.,-269.,0.,19.,128.,-126.,-97.,81.,0.,-11.,100.,68.,-32.,
     *-8.,-7.,0.,-61.,-27.,-2.,6.,26.,-23.,-12.,0.,7.,-12.,9.,-16.,4.,
     *24.,-3.,-17.,0.,-22.,15.,7.,-4.,-5.,10.,10.,-4.,1.,0.,2.,1.,2.,
     *6.,-4.,0.,-2.,3.,0.,-6./
c
      DATA G70/0.,-30220.,-2068.,-1781.,3000.,1611.,1287.,-2091.,1278.,
     *838.,952.,800.,461.,-395.,234.,-216.,359.,262.,-42.,-160.,-56.,
     *43.,64.,15.,-212.,2.,3.,-112.,72.,-57.,1.,14.,-22.,-2.,13.,-2.,
     *14.,6.,-2.,-13.,-3.,5.,0.,11.,3.,8.,10.,2.,-12.,10.,-1.,0.,3.,
     *1.,-1.,-3.,-3.,2.,-5.,-1.,6.,4.,1.,0.,3.,-1./
      DATA H70/0.,0.,5737.,0.,-2047.,25.,0.,-366.,251.,-196.,0.,167.,
     *-266.,26.,-279.,0.,26.,139.,-139.,-91.,83.,0.,-12.,100.,72.,-37.,
     *-6.,1.,0.,-70.,-27.,-4.,8.,23.,-23.,-11.,0.,7.,-15.,6.,-17.,6.,
     *21.,-6.,-16.,0.,-21.,16.,6.,-4.,-5.,10.,11.,-2.,1.,0.,1.,1.,3.,
     *4.,-4.,0.,-1.,3.,1.,-4./
c
      DATA G75/0.,-30100.,-2013.,-1902.,3010.,1632.,1276.,-2144.,1260.,
     *830.,946.,791.,438.,-405.,216.,-218.,356.,264.,-59.,-159.,-49.,
     *45.,66.,28.,-198.,1.,6.,-111.,71.,-56.,1.,16.,-14.,0.,12.,-5.,
     *14.,6.,-1.,-12.,-8.,4.,0.,10.,1.,7.,10.,2.,-12.,10.,-1.,-1.,4.,
     *1.,-2.,-3.,-3.,2.,-5.,-2.,5.,4.,1.,0.,3.,-1./
      DATA H75/0.,0.,5675.,0.,-2067.,-68.,0.,-333.,262.,-223.,0.,191.,
     *-265.,39.,-288.,0.,31.,148.,-152.,-83.,88.,0.,-13.,99.,75.,-41.,
     *-4.,11.,0.,-77.,-26.,-5.,10.,22.,-23.,-12.,0.,6.,-16.,4.,-19.,6.,
     *18.,-10.,-17.,0.,-21.,16.,7.,-4.,-5.,10.,11.,-3.,1.,0.,1.,1.,3.,
     *4.,-4.,-1.,-1.,3.,1.,-5./
c
      DATA G80/0.,-29992.,-1956.,-1997.,3027.,1663.,1281.,-2180.,1251.,
     *833.,938.,782.,398.,-419.,199.,-218.,357.,261.,-74.,-162.,-48.,
     *48.,66.,42.,-192.,4.,14.,-108.,72.,-59.,2.,21.,-12.,1.,11.,-2.,
     *18.,6.,0.,-11.,-7.,4.,3.,6.,-1.,5.,10.,1.,-12.,9.,-3.,-1.,7.,2.,
     *-5.,-4.,-4.,2.,-5.,-2.,5.,3.,1.,2.,3.,0./
      DATA H80/0.,0.,5604.,0.,-2129.,-200.,0.,-336.,271.,-252.,0.,212.,
     *-257.,53.,-297.,0.,46.,150.,-151.,-78.,92.,0.,-15.,93.,71.,-43.,
     *-2.,17.,0.,-82.,-27.,-5.,16.,18.,-23.,-10.,0.,7.,-18.,4.,-22.,9.,
     *16.,-13.,-15.,0.,-21.,16.,9.,-5.,-6.,9.,10.,-6.,2.,0.,1.,0.,3.,
     *6.,-4.,0.,-1.,4.,0.,-6./
c
      DATA G85/0.,-29873.,-1905.,-2072.,3044.,1687.,1296.,-2208.,1247.,
     *829.,936.,780.,361.,-424.,170.,-214.,355.,253.,-93.,-164.,-46.,
     *53.,65.,51.,-185.,4.,16.,-102.,74.,-62.,3.,24.,-6.,4.,10.,0.,21.,
     *6.,0.,-11.,-9.,4.,4.,4.,-4.,5.,10.,1.,-12.,9.,-3.,-1.,7.,1.,-5.,
     *-4.,-4.,3.,-5.,-2.,5.,3.,1.,2.,3.,0./
      DATA H85/0.,0.,5500.,0.,-2197.,-306.,0.,-310.,284.,-297.,0.,232.,
     *-249.,69.,-297.,0.,47.,150.,-154.,-75.,95.,0.,-16.,88.,69.,-48.,
     *-1.,21.,0.,-83.,-27.,-2.,20.,17.,-23.,-7.,0.,8.,-19.,5.,-23.,11.,
     *14.,-15.,-11.,0.,-21.,15.,9.,-6.,-6.,9.,9.,-7.,2.,0.,1.,0.,3.,
     *6.,-4.,0.,-1.,4.,0.,-6./
c
      DATA G90/0.,-29775.,-1851.,-2136.,3058.,1693.,1315.,-2240.,1246.,
     *807.,939.,782.,324.,-423.,142.,-211.,353.,244.,-111.,-166.,-37.,
     *61.,64.,60.,-178.,2.,17.,-96.,77.,-64.,4.,28.,1.,6.,10.,0.,22.,
     *5.,-1.,-11.,-12.,4.,4.,3.,-6.,4.,10.,1.,-12.,9.,-4.,-1.,7.,2.,-6.,
     *-4.,-4.,2.,-5.,-2.,4.,3.,1.,2.,3.,0./
      DATA H90/0.,0.,5411.,0.,-2278.,-380.,0.,-287.,293.,-348.,0.,248.,
     *-240.,87.,-299.,0.,47.,153.,-154.,-69.,98.,0.,-16.,83.,68.,-52.,
     *2.,27.,0.,-81.,-27.,1.,20.,16.,-23.,-5.,0.,10.,-20.,7.,-22.,12.,
     *11.,-16.,-11.,0.,-21.,15.,10.,-6.,-6.,9.,9.,-7.,2.,0.,1.,0.,3.,
     *6.,-4.,0.,-1.,4.,0.,-6./
      DATA DG90 /0.,18.0,10.6,-12.9,2.4,0.0,3.3,-6.7,0.1,-5.9,0.5,0.6,
     *-7.0,0.5,-5.5,0.6,-0.1,-1.6,-3.1,-0.1,2.3,1.3,-0.2,1.8,1.3,-0.2,
     *0.1,1.2,0.6,-0.5,-0.3,0.6,1.6,0.2,0.2,0.3,0.2,-0.7,-0.2,0.1,-1.1,
     *0.0,-0.1,-0.5,-0.6/
      DATA DH90 /0.,0.,-16.1,0.,-15.8,-13.8,0.,4.4,1.6,-10.6,0.,2.6,1.8,
     *3.1,-1.4,0.,-0.1,0.5,0.4,1.7,0.4,0.,0.2,-1.3,0.0,-0.9,0.5,1.2,
     *0.,0.6,0.2,0.8,-0.5,-0.2,0.0,0.0,0.,0.5,-0.2,0.3,0.3,0.4,-0.5,
     *-0.3,0.6/
c
      DATA MA,IYR,IPR/0,0,0/
      IF(MA.NE.1) GOTO 10
      IF(IY.NE.IYR) GOTO 30
      GOTO 130
10    MA=1
      KNM=15
C
      DO 20 N=1,11
         N2=2*N-1
         N2=N2*(N2-2)
         DO 20 M=1,N
            MN=N*(N-1)/2+M
20    REC(MN)=FLOAT((N-M)*(N+M-2))/FLOAT(N2)
C
30    IYR=IY
      IF (IYR.LT.1965) IYR=1965
      IF (IYR.GT.2000) IYR=2000
      IF (IY.NE.IYR.AND.IPR.EQ.0) write(*,999)IY,IYR
      IF (IYR.NE.IY) IPR=1
      IF (IYR.LT.1970) GOTO 50		!INTERPOLATE BETWEEN 1965 - 1970
      IF (IYR.LT.1975) GOTO 60		!INTERPOLATE BETWEEN 1970 - 1975
      IF (IYR.LT.1980) GOTO 70		!INTERPOLATE BETWEEN 1975 - 1980
      IF (IYR.LT.1985) GOTO 80		!INTERPOLATE BETWEEN 1980 - 1985
      IF (IYR.LT.1990) GOTO 90		!INTERPOLATE BETWEEN 1985 - 1990
C
C
C	EXTRAPOLATE BETWEEN 1990 - 2000
C
      DT=FLOAT(IYR)-1990.
      DO 40 N=1,66
         G(N)=G90(N)
         H(N)=H90(N)
         IF (N.GT.45) GOTO 40
         G(N)=G(N)+DG90(N)*DT
         H(N)=H(N)+DH90(N)*DT
40    CONTINUE
      GOTO 300
C
C
C	INTERPOLATE BETWEEEN 1965 - 1970
C
50    F2=(IYR-1965)/5.
      F1=1.-F2
      DO 55 N=1,66
         G(N)=G65(N)*F1+G70(N)*F2
55       H(N)=H65(N)*F1+H70(N)*F2
      GOTO 300
C
C
C	INTERPOLATE BETWEEN 1970 - 1975
C
60    F2=(IYR-1970)/5.
      F1=1.-F2
      DO 65 N=1,66
         G(N)=G70(N)*F1+G75(N)*F2
65       H(N)=H70(N)*F1+H75(N)*F2
      GOTO 300
C
C
C	INTERPOLATE BETWEEN 1975 - 1980
C
70    F2=(IYR-1975)/5.
      F1=1.-F2
      DO 75 N=1,66
         G(N)=G75(N)*F1+G80(N)*F2
75       H(N)=H75(N)*F1+H80(N)*F2
      GOTO 300
C
C
C	INTERPOLATE BETWEEN 1980 - 1985
C
80    F2=(IYR-1980)/5.
      F1=1.-F2
      DO 85 N=1,66
         G(N)=G80(N)*F1+G85(N)*F2
85       H(N)=H80(N)*F1+H85(N)*F2
      GOTO 300
C
C
C	INTERPOLATE BETWEEN 1985 - 1990
C
90    F2=(IYR-1985)/5.
      F1=1.-F2
      DO 95 N=1,66
         G(N)=G85(N)*F1+G90(N)*F2
95       H(N)=H85(N)*F1+H90(N)*F2
      GOTO 300
C
C
C	GET HERE WHEN COEFFICIENTS FOR APPROPRIATE IGRF MODEL HAVE BEEN ASSIGNED
C
300   S=1.
      DO 120 N=2,11
         MN=N*(N-1)/2+1
         S=S*FLOAT(2*N-3)/FLOAT(N-1)
         G(MN)=G(MN)*S
         H(MN)=H(MN)*S
         P=S
         DO 120 M=2,N
            AA=1.
            IF (M.EQ.2) AA=2.
            P=P*SQRT(AA*FLOAT(N-M+1)/FLOAT(N+M-2))
            MNN=MN+M-1
            G(MNN)=G(MNN)*P
120         H(MNN)=H(MNN)*P
C
130   IF(KNM.EQ.NM) GO TO 140
      KNM=NM
      K=KNM+1
140   PP=1./R
      P=PP
      DO 150 N=1,K
         P=P*PP
         A(N)=P
150      B(N)=P*N
      P=1.
      D=0.
      BBR=0.
      BBT=0.
      BBF=0.
      U=T
      CF=COS(F)
      SF=SIN(F)
      C=COS(U)
      S=SIN(U)
      BK=(S.LT.1.E-5)
      DO 200 M=1,K
         BM=(M.EQ.1)
         IF(BM) GOTO 160
         MM=M-1
         W=X
         X=W*CF+Y*SF
         Y=Y*CF-W*SF
         GOTO 170
160      X=0.
         Y=1.
170      Q=P
         Z=D
         BI=0.
         P2=0.
         D2=0.
         DO 190 N=M,K
            AN=A(N)
            MN=N*(N-1)/2+M
            E=G(MN)
            HH=H(MN)
            W=E*Y+HH*X
            BBR=BBR+B(N)*W*Q
            BBT=BBT-AN*W*Z
            IF(BM) GOTO 180
            QQ=Q
            IF(BK) QQ=Z
            BI=BI+AN*(E*X-HH*Y)*QQ
180         XK=REC(MN)
            DP=C*Z-S*Q-XK*D2
            PM=C*Q-XK*P2
            D2=Z
            P2=Q
            Z=DP
190        Q=PM
         D=S*D+C*P
         P=S*P
         IF(BM) GOTO 200
         BI=BI*MM
         BBF=BBF+BI
200   CONTINUE
C
      BR=BBR
      BT=BBT
      IF(BK) GOTO 210
      BF=BBF/S
      GOTO 220
210   IF(C.LT.0.) BBF=-BBF
      BF=BBF
220   CONTINUE
      RETURN
C
999   FORMAT(//1X,
     * 'IGRF WARNS:**** YEAR IS OUT OF INTERVAL 1965-2000: IY =',I5,/,
     *',        CALCULATIONS WILL BE DONE FOR IYR =',I5,' ****'//)
      END
C
C
c---------------------------------------------------------------------
c
       SUBROUTINE DIP(PS,X,Y,Z,BX,BY,BZ)
C
C  CALCULATES GSM COMPONENTS OF GEODIPOLE FIELD WITH THE DIPOLE MOMENT
C  CORRESPONDING TO EPOCH=1980.
C------------INPUT PARAMETERS:
C   PS - GEODIPOLE TILT ANGLE IN RADIANS, X,Y,Z - GSM COORDINATES IN RE
C------------OUTPUT PARAMETERS:
C   BX,BY,BZ - FIELD COMPONENTS IN GSM SYSTEM, IN NANOTESLA.
C
C
C                   AUTHOR: NIKOLAI A. TSYGANENKO
C                           INSTITUTE OF PHYSICS
C                           ST.-PETERSBURG STATE UNIVERSITY
C                           STARY PETERGOF 198904
C                           ST.-PETERSBURG
C                           RUSSIA
C
	IMPLICIT NONE
C
	REAL PS,X,Y,Z,BX,BY,BZ,PSI,SPS,CPS,P,U,V,T,Q
	INTEGER M

      DATA M,PSI/0,5./
      IF(M.EQ.1.AND.ABS(PS-PSI).LT.1.E-5) GOTO 1
      SPS=SIN(PS)
      CPS=COS(PS)
      PSI=PS
      M=1
  1   P=X**2
      U=Z**2
      V=3.*Z*X
      T=Y**2
      Q=30574./SQRT(P+T+U)**5
      BX=Q*((T+U-2.*P)*SPS-V*CPS)
      BY=-3.*Y*Q*(X*SPS+Z*CPS)
      BZ=Q*((P+T-2.*U)*CPS-V*SPS)
      RETURN
      END
c------------------------------------------------------------------------
c
c
C
      SUBROUTINE EXT87L(IOPT,PS,X,Y,Z,BX,BY,BZ)
C
C      'LONG' VERSION OF THE MAGNETOSPHERIC MAGNETIC FIELD MODEL
C  COMPUTES GSM COMPONENTS OF THE MAGNETIC FIELD OF EXTRATERRESTRIAL
C  CURRENT SYSTEMS UP TO GEOCENTRIC RADIAL DISTANCES ABOUT 70 RE .
C  CORRESPONDS TO MAGNETOSPHERIC FIELD MODEL (N.A.TSYGANENKO, PLANET.
C  SPACE SCI., V.35, P.1347, 1987)
C  BASED ON IMP-A,C,D,E,F,G,H,I,J (YEARS 1966-1974)
C  AND HEOS-1,2 (1969-1974) SATELLITE MERGED DATA SET.
C----- INPUT PARAMETERS:  IOPT - SPECIFIES MODEL VERSION, ACCORDING TO
C    GROUND DISTURBANCE LEVEL, IN THE FOLLOWING WAY:
C
C  IOPT=   1       2         3          4         5        6
C         CORRESPOND TO:
C   KP=   0,0+   1-,1,1+   2-,2,2+    3-,3,3+   4-,4,4+   >=5-
C
C    PS - GEODIPOLE TILT ANGLE IN RADIANS
C    X,Y,Z - GSM COORDINATES OF THE POINT IN EARTH RADII
C------ OUTPUT PARAMETERS:   BX,BY,BZ - GSM COMPONENTS OF THE EXTERNAL
C    MAGNETIC FIELD IN NANOTESLA
C
	IMPLICIT NONE

	REAL PS,X,Y,Z,BX,BY,BZ,PSI,HPI,FC,RT,X1,X2,C1,RRC2,DSTR,XN,
     1     RH,DY,DELX,B0,B1,B2,XN21,XN2,XNR,XN22,ADLN,SPS,CPS,RPS,
     2     ZS,ZP,ZM,FY,XNX,XNX2,XC1,XC2,XC22,XR2,XC12,B20,B2P,B2M,
     3     B,BP,BM,XA1,XAP1,XAM1,XA2,XAP2,XAM2,XNA,XNAP,XNAM,F,FP,
     4     FM,XLN1,XLNP1,XLNM1,XLN2,XLNP2,XLNM2,ALN,S0,S0P,S0M,S1,
     5     S1P,S1M,S2,S2P,S2M,G1,G1P,G1M,G2,G2P,G2M,EX1,EX2,Y2,Z2,
     6     YZ,XSM,ZSM,RR2,RR,ZN,BRSM,BZSM,BY1,BXSM

	INTEGER IOPT,IP,I

      REAL GA1(32),GA2(32),GA3(32),GA4(32),GA5(32),GA6(32),PA(32)
      DATA GA1/-.09673,-10.63,1.21,34.57,-.04502,-.06553,-.02952,.3852,
     *-.03665,-2.084,.001795,.00638,-23.49,.06082,.01642,-.02137,32.21,
     *-.04373,-.02311,-.2832,-.002303,-.000631,-6.397,-967.,-8650.,
     *-20.55,5.18,-2.796,2.715,13.58,8.038,29.21/
      DATA GA2/-.485,-12.84,1.856,40.06,-.0294,-.09071,-.02993,.5465,
     *-.04928,-2.453,.001587,.007402,-29.41,.08101,.02322,-.1091,40.75,
     *-.07995,-.03859,-.2755,-.002759,-.000408,-6.189,-957.8,-7246.,
     *-25.51,5.207,-4.184,2.641,16.56,7.795,29.36/
      DATA GA3/-1.132,-18.05,2.625,48.55,-.004868,-.1087,-.03824,.8514,
     *-.0522,-2.881,-.000295,.009055,-29.48,.06394,.03864,-.2288,41.77,
     *-.05849,-.06443,-.4683,.001222,-.000519,-3.696,-991.1,-6955.,
     *-31.43,4.878,-3.151,3.277,19.19,7.248,28.99/
      DATA GA4/-1.003,-16.98,3.14,52.81,-.08625,-.1478,-.03501,.55,
     *-.07778,-2.97,.002086,.01275,-26.79,.06328,.03622,.08345,39.72,
     *-.06009,-.07825,-.9698,.000178,-.000573,-.9328,-872.5,-5851.,
     *-39.68,4.902,-3.848,2.79,20.91,6.193,26.81/
      DATA GA5/-1.539,-14.29,3.479,53.36,-.004201,-.2043,-.03932,.6409,
     *-.1058,-3.221,-.00114,.02166,-30.43,.04049,.05464,.008884,42.,
     *-.01035,-.1053,-1.63,.003802,-.001029,4.204,-665.6,-1011.,-43.49,
     *4.514,-2.948,2.99,21.59,6.005,22./
      DATA GA6/-2.581,-7.726,5.045,53.31,.02262,-.1972,-.01981,.428,
     *-.1055,-5.075,.002762,.03277,-27.35,.04986,.06119,-.1211,47.48,
     *-.0502,-.1477,.838,-.01008,-.0057,9.231,-674.3,-900.,-74.43,
     *4.658,-3.245,3.39,21.8,5.62,25.17/
      DATA IP,PSI,HPI,FC,RT,X1,X2/100,10.,1.5707963,0.3183099031,
     *30.,4.,5./
      IF (IOPT.EQ.IP) GOTO 2
      IP=IOPT
      DO 1 I=1,32
      IF (IP.EQ.1) PA(I)=GA1(I)
      IF (IP.EQ.2) PA(I)=GA2(I)
      IF (IP.EQ.3) PA(I)=GA3(I)
      IF (IP.EQ.4) PA(I)=GA4(I)
      IF (IP.EQ.5) PA(I)=GA5(I)
      IF (IP.EQ.6) PA(I)=GA6(I)
  1   CONTINUE
      C1=PA(29)**2
      RRC2=PA(27)**2
      DSTR=PA(26)/RRC2*4.
      XN=PA(28)
      RH=PA(31)
      DY=PA(30)
      DELX=PA(32)
      B0=PA(23)
      B1=PA(24)
      B2=PA(25)
      XN21=(XN-X1)**2
      XN2=XN-X2
      XNR=1./XN2
      XN22=XN2**2
      ADLN=ALOG(XN22/XN21)
  2   IF(ABS(PS-PSI).LT.1.E-6) GOTO 3
      PSI=PS
      SPS=SIN(PS)
      CPS=COS(PS)
      RPS=RH*SPS
C
C   COMPUTATION BEGINS HERE IF PARAMETERS IOPT AND PS REMAINED UNCHANGED
C   AFTER THE PRECEDING CALL OF THIS SUBROUTINE
C
  3   ZS=Z-RPS
      ZP=Z-RT
      ZM=Z+RT
      FY=FC/(1.+(Y/DY)**2)
      XNX=XN-X
      XNX2=XNX**2
      XC1=X-X1
      XC2=X-X2
      XC22=XC2**2
      XR2=XC2*XNR
      XC12=XC1**2
      B20=ZS**2+C1
      B2P=ZP**2+C1
      B2M=ZM**2+C1
      B=SQRT(B20)
      BP=SQRT(B2P)
      BM=SQRT(B2M)
      XA1=XC12+B20
      XAP1=XC12+B2P
      XAM1=XC12+B2M
      XA2=1./(XC22+B20)
      XAP2=1./(XC22+B2P)
      XAM2=1./(XC22+B2M)
      XNA=XNX2+B20
      XNAP=XNX2+B2P
      XNAM=XNX2+B2M
      F=B20-XC22
      FP=B2P-XC22
      FM=B2M-XC22
      XLN1=ALOG(XN21/XNA)
      XLNP1=ALOG(XN21/XNAP)
      XLNM1=ALOG(XN21/XNAM)
      XLN2=XLN1+ADLN
      XLNP2=XLNP1+ADLN
      XLNM2=XLNM1+ADLN
      ALN=0.25*(XLNP1+XLNM1-2.*XLN1)
      S0=(ATAN(XNX/B)+HPI)/B
      S0P=(ATAN(XNX/BP)+HPI)/BP
      S0M=(ATAN(XNX/BM)+HPI)/BM
      S1=(XLN1*.5+XC1*S0)/XA1
      S1P=(XLNP1*.5+XC1*S0P)/XAP1
      S1M=(XLNM1*.5+XC1*S0M)/XAM1
      S2=(XC2*XA2*XLN2-XNR-F*XA2*S0)*XA2
      S2P=(XC2*XAP2*XLNP2-XNR-FP*XAP2*S0P)*XAP2
      S2M=(XC2*XAM2*XLNM2-XNR-FM*XAM2*S0M)*XAM2
      G1=(B20*S0-0.5*XC1*XLN1)/XA1
      G1P=(B2P*S0P-0.5*XC1*XLNP1)/XAP1
      G1M=(B2M*S0M-0.5*XC1*XLNM1)/XAM1
      G2=((0.5*F*XLN2+2.*S0*B20*XC2)*XA2+XR2)*XA2
      G2P=((0.5*FP*XLNP2+2.*S0P*B2P*XC2)*XAP2+XR2)*XAP2
      G2M=((0.5*FM*XLNM2+2.*S0M*B2M*XC2)*XAM2+XR2)*XAM2
      BX=FY*(B0*(ZS*S0-0.5*(ZP*S0P+ZM*S0M))
     * +B1*(ZS*S1-0.5*(ZP*S1P+ZM*S1M))+B2*(ZS*S2-0.5*(ZP*S2P+ZM*S2M)))
      BY=0.
      BZ=FY*(B0*ALN+B1*(G1-0.5*(G1P+G1M))+B2*(G2-0.5*(G2P+G2M)))
C
C    CALCULATION OF THE MAGNETOTAIL CURRENT CONTRIBUTION IS FINISHED
C
      EX1=EXP(X/DELX)
      EX2=EX1**2
      Y2=Y**2
      Z2=Z**2
      YZ=Y*Z
      BX=BX+(EX1*PA(1)+EX2*PA(3))*Z*CPS+(EX1*PA(2)+EX2*(PA(4)+PA(5)*
     *Y2+PA(6)*Z2))*SPS
      BY=(EX1*PA(7)+EX2*PA(9))*YZ*CPS+(EX1*PA(8)+EX2*(PA(10)+PA(11)*
     *Y2+PA(12)*Z2))*Y*SPS
      BZ=BZ+(EX1*(PA(13)+PA(14)*Y2+PA(15)*Z2)+EX2*(PA(17)+PA(18)*Y2+
     *PA(19)*Z2))*CPS+(EX1*PA(16)+EX2*(PA(20)+PA(21)*Y2+PA(22)*Z2))*
     *Z*SPS
C
C   DCF AND FAC CONTRIBUTION HAS BEEN ADDED TO BX,BY, AND BZ
C
      XSM=X*CPS-Z*SPS
      ZSM=X*SPS+Z*CPS
      Z2=ZSM**2
      RR2=XSM**2+Y**2
      RR=SQRT(RR2)
      ZN=SQRT((RR2+Z2)/RRC2+4.)**5
      BRSM=DSTR*3.*ZSM/ZN
      BZSM=DSTR*(2.*Z2-RR2+8.*RRC2)/ZN
      BY1=BRSM*Y
      BXSM=BRSM*XSM
      BX=BX+BXSM*CPS+BZSM*SPS
      BZ=BZ-BXSM*SPS+BZSM*CPS
      BY=BY+BY1
C
C   RING CURRENT FIELD HAS BEEN TAKEN INTO ACCOUNT
C
      RETURN
      END
C------------------------------------------------------------------------
C
      SUBROUTINE EXT87T(IOPT,PS,X,Y,Z,BX,BY,BZ)
C
C  COMPUTES GSM COMPONENTS OF THE MAGNETIC FIELD OF EXTRATERRESTRIAL
C  CURRENT SYSTEMS UP TO GEOCENTRIC RADIAL DISTANCES ABOUT 30 RE .
C  CORRESPONDS TO MAGNETOSPHERIC FIELD MODEL (N.A.TSYGANENKO, Planet.
C  Space Sci.,v.35, p.1347, 1987), 'TRUNCATED VERSION',
C  BASED ON IMP-A,C,D,E,F,G,H,I,J (YEARS 1966-1980)
C  AND HEOS-1,2 (1969-1974) SATELLITE MERGED DATA SET.
C----- INPUT PARAMETERS:  IOPT - SPECIFIES MODEL VERSION, ACCORDING TO
C    GROUND DISTURBANCY LEVEL, IN THE FOLLOWING WAY:
C
C  IOPT=   1       2       3       4        5        6        7      8
C                      CORRESPOND TO:
C   KP=   0,0+    1-,1    1+,2-   2,2+   3-,3,3+   4-,4,4+   >=5-   >=5+
C
C    PS - GEODIPOLE TILT ANGLE IN RADIANS
C    X,Y,Z - GSM COORDINATES OF THE POINT IN EARTH RADII
C------ OUTPUT PARAMETERS:   BX,BY,BZ - GSM COMPONENTS OF THE EXTERNAL
C    MAGNETIC FIELD IN NANOTESLA
C
C
C                   AUTHOR: NIKOLAI A. TSYGANENKO
C                           INSTITUTE OF PHYSICS
C			    ST.-PETERSBURG STATE UNIVERSITY
C			    STARY PETERGOF 198904
C			    ST.-PETERSBURG
C			    RUSSIA
C
	IMPLICIT NONE

	REAL PS,X,Y,Z,BX,BY,BZ,PSI,HPI,FC,RT,X1,C1,RRC2,DSTR,XN,
     1     RH,DY,DELX,B0,B1,XN21,SPS,CPS,RPS,
     2     ZS,ZP,ZM,FY,XNX,XNX2,XC1,XC12,B20,B2P,B2M,
     3     B,BP,BM,XA1,XAP1,XAM1,XNA,XNAP,XNAM,
     4     XLN1,XLNP1,XLNM1,ALN,S0,S0P,S0M,S1,
     5     S1P,S1M,G1,G1P,G1M,EX,Y2,Z2,
     6     YZ,XSM,ZSM,RR2,RR,ZN,BRSM,BZSM,BY1,BXSM

	INTEGER IOPT,IP,I

      REAL GA1(24),GA2(24),GA3(24),GA4(24),GA5(24),GA6(24),GA7(24),
     *  GA8(24),PA(24)
      DATA GA1/1.126,26.66,-.077,-.06102,-.06197,-2.048,.00327,.008473,
     *12.72,-.00867,-.01953,-.3437,-.002903,-.000999,18.41,-270.3,
     *-25.94,5.21,-6.2,2.29,11.96,8.315,44.22,11.15/
      DATA GA2/1.403,29.24,-.0693,-.0864,-.07202,-2.068,.00286,.007438,
     *16.37,-.02705,-.0281,-.604,-.002256,.000152,20.2,-140.1,-29.65,
     *5.62,-5.52,2.02,14.66,8.06,27.76,10.94/
      DATA GA3/1.589,31.07,-.06527,-.07447,-.07632,-2.413,.002719,
     *.01098,16.2,-.02355,-.03475,-.4377,-.002169,-.001383,18.7,-292.6,
     *-35.25,5.29,-5.18,2.21,14.03,7.66,17.56,10.9/
      DATA GA4/1.699,36.28,-.07514,-.1448,-.08049,-2.209,.000919,
     *.01084,17.38,-.03516,-.03886,-1.169,.004239,.000881,21.79,-162.,
     *-41.87,5.15,-3.62,2.35,17.26,7.61,17.99,10.74/
      DATA GA5/2.141,41.51,-.1518,-.1857,-.1015,-2.929,.004584,.01589,
     *18.29,-.02514,-.05927,-1.336,.00185,.001066,21.31,-358.8,-47.91,
     *5.13,-3.74,2.07,17.23,6.33,32.51,9.73/
      DATA GA6/2.252,39.35,-.04525,-.2062,-.1491,-3.059,-.000183,
     *.02614,15.48,-.02144,-.06608,-1.855,.006199,-.00013,23.91,-161.,
     *-51.48,4.61,-3.32,1.68,15.22,6.68,.6765,8.007/
      DATA GA7/2.773,40.95,.00667,-.133,-.1304,-5.187,.004623,.03651,
     *20.,-.03765,-.09066,.5838,-.01462,-.007189,24.87,-186.1,-74.81,
     *4.57,-4.03,1.7,12.15,6.87,-1.746,8.9/
      DATA GA8/2.919,34.96,2*0.,-.1609,-5.077,2*0.,22.1,-.05915,-.1051,
     *.6321,2*0.,28.11,-330.1,-86.82,4.,-3.,1.73,12.56,5.11,4.,7.866/
      DATA IP,PSI,HPI,FC,RT/100,10.,1.5707963,0.3183099031,30./
      IF (IOPT.EQ.IP) GOTO 2
      IP=IOPT
      DO 1 I=1,24
      IF (IP.EQ.1) PA(I)=GA1(I)
      IF (IP.EQ.2) PA(I)=GA2(I)
      IF (IP.EQ.3) PA(I)=GA3(I)
      IF (IP.EQ.4) PA(I)=GA4(I)
      IF (IP.EQ.5) PA(I)=GA5(I)
      IF (IP.EQ.6) PA(I)=GA6(I)
      IF (IP.EQ.7) PA(I)=GA7(I)
      IF (IP.EQ.8) PA(I)=GA8(I)
  1   CONTINUE
      C1=PA(20)**2
      RRC2=PA(18)**2
      DSTR=PA(17)/RRC2*4.
      XN=PA(19)
      RH=PA(22)
      X1=PA(23)
      DY=PA(21)
      DELX=PA(24)
      B0=PA(15)
      B1=PA(16)
      XN21=(XN-X1)**2
  2   IF(ABS(PS-PSI).LT.1.E-6) GOTO 3
      PSI=PS
      SPS=SIN(PS)
      CPS=COS(PS)
      RPS=RH*SPS
C
C   COMPUTATION BEGINS HERE IF PARAMETERS IOPT AND PS REMAINED UNCHANGED
C   AFTER THE PRECEDING CALL OF THIS SUBROUTINE
C
  3   ZS=Z-RPS
      ZP=Z-RT
      ZM=Z+RT
      FY=FC/(1.+(Y/DY)**2)
      XNX=XN-X
      XNX2=XNX**2
      XC1=X-X1
      XC12=XC1**2
      B20=ZS**2+C1
      B2P=ZP**2+C1
      B2M=ZM**2+C1
      B=SQRT(B20)
      BP=SQRT(B2P)
      BM=SQRT(B2M)
      XA1=XC12+B20
      XAP1=XC12+B2P
      XAM1=XC12+B2M
      XNA=XNX2+B20
      XNAP=XNX2+B2P
      XNAM=XNX2+B2M
      XLN1=ALOG(XN21/XNA)
      XLNP1=ALOG(XN21/XNAP)
      XLNM1=ALOG(XN21/XNAM)
      ALN=0.25*(XLNP1+XLNM1-2.*XLN1)
      S0=(ATAN(XNX/B)+HPI)/B
      S0P=(ATAN(XNX/BP)+HPI)/BP
      S0M=(ATAN(XNX/BM)+HPI)/BM
      S1=(XLN1*.5+XC1*S0)/XA1
      S1P=(XLNP1*.5+XC1*S0P)/XAP1
      S1M=(XLNM1*.5+XC1*S0M)/XAM1
      G1=(B20*S0-0.5*XC1*XLN1)/XA1
      G1P=(B2P*S0P-0.5*XC1*XLNP1)/XAP1
      G1M=(B2M*S0M-0.5*XC1*XLNM1)/XAM1
      BX=FY*(B0*(ZS*S0-0.5*(ZP*S0P+ZM*S0M))
     * +B1*(ZS*S1-0.5*(ZP*S1P+ZM*S1M)))
      BY=0.
      BZ=FY*(B0*ALN+B1*(G1-0.5*(G1P+G1M)))
C
C    CALCULATION OF THE MAGNETOTAIL CURRENT CONTRIBUTION IS FINISHED
C
      EX=EXP(X/DELX)
      Y2=Y**2
      Z2=Z**2
      YZ=Y*Z
      BX=BX+EX*(CPS*PA(1)*Z+SPS*(PA(2)+PA(3)*Y2+PA(4)*Z2))
      BY=EX*(CPS*PA(5)*YZ+SPS*Y*(PA(6)+PA(7)*Y2+PA(8)*Z2))
      BZ=BZ+EX*(CPS*(PA(9)+PA(10)*Y2+PA(11)*Z2)+SPS*Z*(PA(12)
     *  +PA(13)*Y2+PA(14)*Z2))
C
C   DCF AND FAC CONTRIBUTION HAS BEEN ADDED TO BX,BY, AND BZ
C
      XSM=X*CPS-Z*SPS
      ZSM=X*SPS+Z*CPS
      Z2=ZSM**2
      RR2=XSM**2+Y**2
      RR=SQRT(RR2)
      ZN=SQRT((RR2+Z2)/RRC2+4.)**5
      BRSM=DSTR*3.*ZSM/ZN
      BZSM=DSTR*(2.*Z2-RR2+8.*RRC2)/ZN
      BY1=BRSM*Y
      BXSM=BRSM*XSM
      BX=BX+BXSM*CPS+BZSM*SPS
      BZ=BZ-BXSM*SPS+BZSM*CPS
      BY=BY+BY1
C
C   RING CURRENT FIELD HAS BEEN TAKEN INTO ACCOUNT
C
      RETURN
      END
c
c
c
 	      	SUBROUTINE EX89KP(IOPT,PS,X,Y,Z,BX,BY,BZ)
C   COMPUTES GSM COMPONENTS OF THE MAGNETIC FIELD PRODUCED BY EXTRA-
C   TERRESTRIAL CURRENT SYSTEMS IN THE GEOMAGNETOSPHERE. THE MODEL IS
C   VALID UP TO GEOCENTRIC DISTANCES OF 70 RE AND IS BASED ON THE MER-
C   GED IMP-A,C,D,E,F,G,H,I,J (1966-1974) AND HEOS-1 AND -2 (1969-1974)
C   SPACECRAFT DATA SET.  REFERENCE: N.A. TSYGANENKO, A MAGNETOSPHERIC
C   MAGNETIC FIELD MODEL WITH A WARPED TAIL CURRENT SHEET: PLANET.SPACE
C   SCI., V.37, PP.5-20, 1989.
C
C----INPUT PARAMETERS: IOPT - SPECIFIES THE GROUND DISTURBANCE LEVEL:
C
C   IOPT= 1         2         3          4           5           6
C                  CORRESPOND TO:
C    KP= 0,0+    1-,1,1+    2-,2,2+    3-,3,3+     4-,4,4+      >=5-
C
C    PS - GEODIPOLE TILT ANGLE IN RADIANS
C    X, Y, Z  - GSM COORDINATES OF THE POINT IN EARTH RADII
C
C----OUTPUT PARAMETERS: BX,BY,BZ - GSM COMPONENTS OF THE MODEL MAGNETIC
C                        FIELD IN NANOTESLAS
C
C
C              AUTHOR:     NIKOLAI A. TSYGANENKO
C                          INSTITUTE OF PHYSICS, ST.-PETERSBURG UNIVERSITY
C			   STARY PETERGOF 198904
C			   ST.-PETERSBURG
C			   RUSSIA
C
	IMPLICIT NONE

	REAL PS,X,Y,Z,BX,BY,BZ,PSI,DELX,ADR,D0,DD,RC,G,A,DY,SX,HA,
     1       HA02,RDYC2,HLWC2M,DRDYCM,HLDXM,DDEL,RDY2,DRDY2M,HXLW2M,
     2       DADD05,DADD18,SPS,CPS,HTPS,GSPM,XSM,ZSM,X2SM,Y2,RO2,
     3       XXD,XXD2L2,RSQXDL,H,HSX,XSIXT,XSIXTD,SXSIX,DDOP,DDOPDX,
     4       D,DDX,DDY,XRC,SXRC16,Y4,Y410,HY,HYS,ZS,DZSX,DZSY,XSX,
     5       RQ,SRQ,FY,W,DWX,DWY,ZR,T,AT,S1,F5,F7,F3,F9,XWYW,FR,FS,
     6       WT,WTFS,BRZR1,BRZR2,BXT,BYT,BZT,RX2A2,SRX2A2,FDR,DFDRX,
     7       DDR,TDR,ADRT,ADRT2,ADT2R2,FK1,FK2,BXDR,EX,Z2,YZ,BXCF,
     8       BYCF,BZCF,FCY,XSXC,RQC2,SRQC2,WC,DWCX,DWCY,XWCYWC,
     9       ZP,ZM,SP,SM,WCSP,WCSM,RSPRT,RSMRT,FXP,FXM,FYP,FYM

	REAL FZP,FZM,AA4SPS,BXC,BYC,BZC,A02,DYC,XLWC2,XLD2,DEL,XLW2,
     1       DADD,XD,GAM,F1,SXC,RT

	INTEGER IOPT,IP,I

      REAL GA1(28),GA2(28),GA3(28),GA4(28),GA5(28),GA6(28),PA(28)
      DATA GA1/-98.72,-10014.,15.03,76.62,-10237.,1.813,31.1,-0.07464,
     *   -0.07764,0.003303,-1.129,0.001663,0.000988,18.21,-0.03018,
     *   4*0.,24.74,8.16,2.08,-0.88,9.08,3.84,13.55,26.94,5.75/
      DATA GA2/-35.65,-12800.,14.37,124.5,-13543.,2.316,35.64,-0.0741,
     * -0.1081,0.003924,-1.451,0.00202,0.00111,21.37,-0.04567,
     * 4*0.,22.33,8.12,1.664,0.932,9.24,2.43,13.81,28.83,6.05/
      DATA GA3/-77.45,-14588.,64.85,123.9,-16229.,2.641,42.46,-0.07611,
     * -0.1579,0.004078,-1.391,0.00153,0.000727,21.86,-0.04199,
     * 4*0.,20.9,6.28,1.54,4.18,9.61,6.59,15.08,30.57,7.43/
      DATA GA4/-70.12,-16125.,90.71,38.08,-19630.,3.181,47.5,-0.1327,
     * -0.1864,0.01382,-1.488,0.002962,0.000897,22.74,-0.04095,
     * 4*0.,18.64,6.27,0.935,5.39,8.57,5.94,15.63,31.47,8.10/
      DATA GA5/-162.5,-15806.,160.6,5.888,-27534.,3.607,51.1,-0.1006,
     * -0.1927,0.03353,-1.392,0.001594,0.002439,22.41,-0.04925,
     * 4*0.,18.31,6.2,0.768,5.07,10.06,6.67,16.1,30.04,8.26/
      DATA GA6/-128.4,-16184.,149.1,215.5,-36435.,4.09,49.09,-0.0231,
     * -0.1359,0.01989,-2.298,0.004911,0.003421,21.79,-0.05447,
     * 4*0.,19.48,5.83,0.332,6.47,10.47,9.08,15.85,25.27,7.98/
      DATA DEL,GAM,DYC,XD,XLD2,XLW2,A02,RT,SXC,XLWC2,DADD
     * /0.01,4.,20.,0.,40.,170.,25.,30.,4.,50.,1./
      DATA IP,PSI/100,10./
	IF (IOPT.EQ.IP) GOTO 2
C
C   INITIAL CALCULATION OF CONSTANTS FROM A GIVEN SET OF MODEL PARAMETERS
C   SPECIFIED BY THE OPTION NUMBER (IOPT)
C
	IP=IOPT
	DO 1 I=1,28
	IF (IP.EQ.1) PA(I)=GA1(I)
        IF (IP.EQ.2) PA(I)=GA2(I)
	IF (IP.EQ.3) PA(I)=GA3(I)
	IF (IP.EQ.4) PA(I)=GA4(I)
	IF (IP.EQ.5) PA(I)=GA5(I)
	IF (IP.EQ.6) PA(I)=GA6(I)
   1    CONTINUE
	DELX=PA(20)
	ADR=PA(21)
	D0=PA(22)
	DD=PA(23)
	RC=PA(24)
	G=PA(25)
	A=PA(26)
	DY=PA(27)
	SX=PA(28)
	HA=0.5*A
	HA02=0.5*A02
	RDYC2=1./DYC**2
        HLWC2M=-0.5*XLWC2
	DRDYCM=-2.*RDYC2
	HLDXM=-0.5*XLD2
	DDEL=2.*DEL
	RDY2=1./DY**2
	DRDY2M=-2.*RDY2
	HXLW2M=-0.5*XLW2
	DADD05=DADD*0.5
	DADD18=-18.*DADD
C
C   COEFFICIENTS PA(16)-PA(19) ARE FOUND FROM PA(6)-PA(13)
C   SO THAT THE MAGNETIC FIELD BE DIVERGENCELESS
C
	PA(16)=-0.5*(PA(6)/DELX+PA(10))
	PA(17)=-(PA(7)/DELX+PA(11))
	PA(18)=-(PA(8)/DELX+3.*PA(12))
	PA(19)=-(PA(9)/DELX+PA(13))/3.
C
	GOTO 3
   2    IF(ABS(PS-PSI).LT.1.E-6) GOTO 4
   3    PSI=PS
	SPS=SIN(PS)
	CPS=COS(PS)
	HTPS=SPS/(2.*CPS)
	GSPM=-G*SPS
C
C   COMPUTATION BEGINS HERE IF IOPT AND PS ARE THE SAME
C
   4    XSM=X*CPS-Z*SPS
	ZSM=Z*CPS+X*SPS
	X2SM=XSM**2
  	Y2=Y*Y
	RO2=X2SM+Y2
	XXD=XSM-XD
	XXD2L2=1./(XXD**2+XLD2)
	RSQXDL=SQRT(XXD2L2)
	H=0.5*(1.+XXD*RSQXDL)
	HSX=-HLDXM*XXD2L2*RSQXDL
	XSIXT=XSM+16.
	XSIXTD=1./(XSIXT**2+36.)
	SXSIX=SQRT(XSIXTD)
	DDOP=DADD05*(1.-XSIXT*SXSIX)
	DDOPDX=DADD18*XSIXTD*SXSIX
	D=D0+DEL*Y2+GAM*H+DDOP
	DDX=GAM*HSX+DDOPDX
	DDY=DDEL*Y
	XRC=XSM+RC
	SXRC16=SQRT(XRC**2+16.)
	Y4=Y2*Y2
	Y410=1./(Y4+1.E4)
	HY=Y2*Y410*Y
	HYS=HY*Y410*4.E4
	HY=HY*Y
	ZS=HTPS*(XRC-SXRC16)+GSPM*HY
	DZSX=HTPS*(1.-XRC/SXRC16)
	DZSY=GSPM*HYS
C
C  ZS=ZS(XSM,YSM) DEFINES THE SHAPE OF THE WARPED CURRENT SHEET
C
	XSX=XSM-SX
	RQ=1./(XSX**2+XLW2)
	SRQ=SQRT(RQ)
	FY=1./(1.+Y2*RDY2)
	W=0.5*(1.-XSX*SRQ)*FY
	DWX=HXLW2M*RQ*SRQ*FY
	DWY=DRDY2M*W*Y*FY
	ZR=ZSM-ZS
	T=SQRT(ZR**2+D**2)
	AT=A+T
	S1=SQRT(AT**2+RO2)
	F5=1./S1
	F7=1./(S1+AT)
	F1=F5*F7
	F3=F5**3
	F9=AT*F3
	XWYW=XSM*DWX+Y*DWY
	FR=ZR*(XSM*DZSX+Y*DZSY)
	FS=FR-D*(XSM*DDX+Y*DDY)
	WT=W/T
	WTFS=WT*FS
	BRZR1=WT*F1
	BRZR2=WT*F3
	BXT=(PA(1)*BRZR1+PA(2)*BRZR2)*ZR
	BYT=BXT*Y
	BXT=BXT*XSM
	BZT=PA(1)*(W*F5+XWYW*F7+WTFS*F1)+PA(2)*(W*F9+XWYW*F1+WTFS*F3)
C
C   CONTRIBUTION FROM CENTRAL TAIL CURRENT SHEET (BXT,BYT,BZT)
C   IS FOUND.  NOW LET US PROCEED TO THE RING CURRENT FIELD:
C
	RX2A2=1./(X2SM+A02)
	SRX2A2=SQRT(RX2A2)
	FDR=0.5*(1.+XSM*SRX2A2)
	DFDRX=HA02*RX2A2*SRX2A2
	DDR=D0+DD*FDR+DDOP
	TDR=SQRT(ZR**2+DDR**2)
	ADRT=ADR+TDR
	ADRT2=ADRT**2
	ADT2R2=1./(ADRT2+RO2)
	FK1=ADT2R2**2*SQRT(ADT2R2)
	FK2=3.*ADRT*FK1/TDR
	BXDR=PA(5)*ZR*FK2
	BYT=BYT+BXDR*Y
	BXT=BXT+BXDR*XSM
	BZT=BZT+PA(5)*((2.*ADRT2-RO2)*FK1+FK2*(FR-DDR*(DD*DFDRX
     *  +DDOPDX)*XSM))
C
C   NOW CALCULATE CHAPMAN-FERRARO FIELD
C   PLUS AVERAGE FIELD-ALIGNED CURRENT CONTRIBUTION
C
	EX=EXP(X/DELX)
	Z2=Z*Z
	YZ=Y*Z
	BXCF=EX*(CPS*PA(6)*Z+SPS*(PA(7)+PA(8)*Y2+PA(9)*Z2))
	BYCF=EX*(CPS*PA(10)*YZ+SPS*Y*(PA(11)+PA(12)*Y2+PA(13)*Z2))
        BZCF=EX*(CPS*(PA(14)+PA(15)*Y2+PA(16)*Z2)+SPS*Z*(PA(17)+
     *    PA(18)*Y2+PA(19)*Z2))
C
C   MAGNETOTAIL RETURN CURRENT FIELD COMPONENTS (BXC,BYC,BZC):
C
	FCY=1./(1.+Y2*RDYC2)
	XSXC=X-SXC
	RQC2=1./(XSXC**2+XLWC2)
	SRQC2=SQRT(RQC2)
	WC=0.5*(1.-XSXC*SRQC2)*FCY
	DWCX=HLWC2M*RQC2*SRQC2*FCY
	DWCY=DRDYCM*WC*Y*FCY
	XWCYWC=X*DWCX+Y*DWCY
	RO2=Y2+X**2
	ZP=Z+RT
	ZM=Z-RT
	SP=SQRT(ZP**2+RO2)
	SM=SQRT(ZM**2+RO2)
	WCSP=WC/SP
	WCSM=WC/SM
	RSPRT=1./(SP+ZP)
	RSMRT=1./(SM-ZM)
	FXP=WCSP*RSPRT
	FXM=-WCSM*RSMRT
	FYP=FXP*Y
	FYM=FXM*Y
	FXP=FXP*X
	FXM=FXM*X
	FZP=WCSP+XWCYWC*RSPRT
	FZM=WCSM+XWCYWC*RSMRT
	AA4SPS=PA(4)*SPS
	BXC=PA(3)*(FXP+FXM)+(FXP-FXM)*AA4SPS
	BYC=PA(3)*(FYP+FYM)+(FYP-FYM)*AA4SPS
	BZC=PA(3)*(FZP+FZM)+(FZP-FZM)*AA4SPS
C
C   AT LAST, SUM UP THE FIELDS.  THE CENTRAL CURRENT SHEET
C   FIELD COMPONENTS ARE TRANSFORMED INTO GSM COORDINATES
C
	BX=BXC+BXT*CPS+BZT*SPS+BXCF
	BY=BYC+BYT+BYCF
	BZ=BZC+BZT*CPS-BXT*SPS+BZCF
	RETURN
	END
c------------------------------------------------------------------------
c
 	      	SUBROUTINE EX89AE(IOPT,PS,X,Y,Z,BX,BY,BZ)
C
C   COMPUTES GSM COMPONENTS OF THE MAGNETIC FIELD PRODUCED BY EXTRA-
C   TERRESTRIAL CURRENT SYSTEMS IN THE GEOMAGNETOSPHERE. THE MODEL IS
C   VALID UP TO GEOCENTRIC DISTANCES OF 70 RE AND IS BASED ON THE MER-
C   GED IMP-A,C,D,E,F,G,H,I,J (1966-1974) AND HEOS-1 AND -2 (1969-1974)
C   SPACECRAFT DATA SET.  REFERENCE: N.A. TSYGANENKO, A MAGNETOSPHERIC
C   MAGNETIC FIELD MODEL WITH A WARPED TAIL CURRENT SHEET: PLANET.SPACE
C   SCI., V.37, PP.5-20, 1989.
C
C----INPUT PARAMETERS: IOPT - SPECIFIES THE GROUND DISTURBANCE LEVEL:
C
C   IOPT= 1         2         3          4           5           6
C                  CORRESPOND TO:
C    AE=0-50     50-100    100-150    150-250     250-400      >=400
C
C    PS - GEODIPOLE TILT ANGLE IN RADIANS
C    X, Y, Z  - GSM COORDINATES OF THE POINT IN EARTH RADII
C
C----OUTPUT PARAMETERS: BX,BY,BZ - GSM COMPONENTS OF THE MODEL MAGNETIC
C                        FIELD IN NANOTESLAS
C
C              AUTHOR:     NIKOLAI A. TSYGANENKO
C                          INSTITUTE OF PHYSICS, ST.-PETERSBURG UNIVERSITY
C			   STARY PETERGOF 198904
C			   ST.-PETERSBURG
C			   RUSSIA
C
C
	IMPLICIT NONE

	REAL PS,X,Y,Z,BX,BY,BZ,PSI,DELX,ADR,D0,DD,RC,G,A,DY,SX,HA,
     1       HA02,RDYC2,HLWC2M,DRDYCM,HLDXM,DDEL,RDY2,DRDY2M,HXLW2M,
     2       DADD05,DADD18,SPS,CPS,HTPS,GSPM,XSM,ZSM,X2SM,Y2,RO2,
     3       XXD,XXD2L2,RSQXDL,H,HSX,XSIXT,XSIXTD,SXSIX,DDOP,DDOPDX,
     4       D,DDX,DDY,XRC,SXRC16,Y4,Y410,HY,HYS,ZS,DZSX,DZSY,XSX,
     5       RQ,SRQ,FY,W,DWX,DWY,ZR,T,AT,S1,F5,F7,F3,F9,XWYW,FR,FS,
     6       WT,WTFS,BRZR1,BRZR2,BXT,BYT,BZT,RX2A2,SRX2A2,FDR,DFDRX,
     7       DDR,TDR,ADRT,ADRT2,ADT2R2,FK1,FK2,BXDR,EX,Z2,YZ,BXCF,
     8       BYCF,BZCF,FCY,XSXC,RQC2,SRQC2,WC,DWCX,DWCY,XWCYWC,
     9       ZP,ZM,SP,SM,WCSP,WCSM,RSPRT,RSMRT,FXP,FXM,FYP,FYM

	REAL FZP,FZM,AA4SPS,BXC,BYC,BZC,A02,DYC,XLWC2,XLD2,DEL,XLW2,
     1       DADD,XD,GAM,F1,SXC,RT

	INTEGER IOPT,IP,I

      REAL GA1(28),GA2(28),GA3(28),GA4(28),GA5(28),GA6(28),PA(28)
      DATA GA1/-77.38,-10921.,-2.89,193.9,-10954.,2.048,28.83,-0.0367,
     *   -0.0625,0.00104,-1.246,0.00166,0.000792,20.81,-0.03608,
     *   4*0.,25.82,7.656,2.106,-0.30,8.753,2.733,13.17,27.09,5.184/
      DATA GA2/-59.2,-13647.,11.6,237.2,-14465.,2.326,36.54,-0.07084,
     * -0.1182,0.000146,-1.626,0.002466,0.001355,23.49,-0.04602,
     * 4*0.,23.15,7.6,1.6,1.457,8.373,3.883,14.18,28.79,5.97/
      DATA GA3/-65.91,-15267.,64.48,230.6,-14870.,2.712,43.28,-0.09687,
     * -0.1671,0.009814,-1.835,0.0026,0.0017,22.73,-0.05049,
     * 4*0.,20.76,6.314,1.42,4.141,9.436,6.266,15.07,30.35,6.852/
      DATA GA4/-57.97,-16106.,86.28,199.7,-16796.,3.033,46.80,-0.1057,
     * -0.1992,0.01509,-1.534,0.001783,-0.000542,22.29,-0.04664,
     * 4*0.,19.79,5.788,1.177,5.007,9.60,7.32,15.78,33.53,7.401/
      DATA GA5/-118.1,-16473.,136.3,158.7,-21134.,3.135,49.78,-0.1088,
     * -0.2245,0.01759,-1.874,0.003243,-0.000268,21.97,-0.04913,
     * 4*0.,18.48,6.25,0.8643,5.821,9.552,6.051,16.14,31.27,9.062/
      DATA GA6/-223.7,-15054.,219.1,83.84,-31140.,3.777,51.08,-0.1261,
     * -0.2393,0.02507,-1.426,0.001678,0.002039,19.10,-0.05344,
     * 4*0.,16.73,6.532,0.382,5.807,8.099,6.726,16.21,25.63,9.431/
      DATA DEL,GAM,DYC,XD,XLD2,XLW2,A02,RT,SXC,XLWC2,DADD
     * /0.01,4.,20.,0.,40.,170.,25.,30.,4.,50.,1./
      DATA IP,PSI/100,10./
	IF (IOPT.EQ.IP) GOTO 2
C
C   INITIAL CALCULATION OF CONSTANTS FROM A GIVEN SET OF MODEL PARAMETERS
C   SPECIFIED BY THE OPTION NUMBER (IOPT)
C
	IP=IOPT
	DO 1 I=1,28
	IF (IP.EQ.1) PA(I)=GA1(I)
        IF (IP.EQ.2) PA(I)=GA2(I)
	IF (IP.EQ.3) PA(I)=GA3(I)
	IF (IP.EQ.4) PA(I)=GA4(I)
	IF (IP.EQ.5) PA(I)=GA5(I)
	IF (IP.EQ.6) PA(I)=GA6(I)
   1    CONTINUE
	DELX=PA(20)
	ADR=PA(21)
	D0=PA(22)
	DD=PA(23)
	RC=PA(24)
	G=PA(25)
	A=PA(26)
	DY=PA(27)
	SX=PA(28)
	HA=0.5*A
	HA02=0.5*A02
	RDYC2=1./DYC**2
        HLWC2M=-0.5*XLWC2
	DRDYCM=-2.*RDYC2
	HLDXM=-0.5*XLD2
	DDEL=2.*DEL
	RDY2=1./DY**2
	DRDY2M=-2.*RDY2
	HXLW2M=-0.5*XLW2
	DADD05=DADD*0.5
	DADD18=-18.*DADD
C
C   COEFFICIENTS PA(16)-PA(19) ARE FOUND FROM PA(6)-PA(13)
C   SO THAT THE MAGNETIC FIELD BE DIVERGENCELESS
C
	PA(16)=-0.5*(PA(6)/DELX+PA(10))
	PA(17)=-(PA(7)/DELX+PA(11))
	PA(18)=-(PA(8)/DELX+3.*PA(12))
	PA(19)=-(PA(9)/DELX+PA(13))/3.
C
	GOTO 3
   2    IF(ABS(PS-PSI).LT.1.E-6) GOTO 4
   3    PSI=PS
	SPS=SIN(PS)
	CPS=COS(PS)
	HTPS=SPS/(2.*CPS)
	GSPM=-G*SPS
C
C   COMPUTATION BEGINS HERE IF IOPT AND PS ARE THE SAME
C
   4    XSM=X*CPS-Z*SPS
	ZSM=Z*CPS+X*SPS
	X2SM=XSM**2
  	Y2=Y*Y
	RO2=X2SM+Y2
	XXD=XSM-XD
	XXD2L2=1./(XXD**2+XLD2)
	RSQXDL=SQRT(XXD2L2)
	H=0.5*(1.+XXD*RSQXDL)
	HSX=-HLDXM*XXD2L2*RSQXDL
	XSIXT=XSM+16.
	XSIXTD=1./(XSIXT**2+36.)
	SXSIX=SQRT(XSIXTD)
	DDOP=DADD05*(1.-XSIXT*SXSIX)
	DDOPDX=DADD18*XSIXTD*SXSIX
	D=D0+DEL*Y2+GAM*H+DDOP
	DDX=GAM*HSX+DDOPDX
	DDY=DDEL*Y
	XRC=XSM+RC
	SXRC16=SQRT(XRC**2+16.)
	Y4=Y2*Y2
	Y410=1./(Y4+1.E4)
	HY=Y2*Y410*Y
	HYS=HY*Y410*4.E4
	HY=HY*Y
	ZS=HTPS*(XRC-SXRC16)+GSPM*HY
	DZSX=HTPS*(1.-XRC/SXRC16)
	DZSY=GSPM*HYS
C
C  ZS=ZS(XSM,YSM) DEFINES THE SHAPE OF THE WARPED CURRENT SHEET
C
	XSX=XSM-SX
	RQ=1./(XSX**2+XLW2)
	SRQ=SQRT(RQ)
	FY=1./(1.+Y2*RDY2)
	W=0.5*(1.-XSX*SRQ)*FY
	DWX=HXLW2M*RQ*SRQ*FY
	DWY=DRDY2M*W*Y*FY
	ZR=ZSM-ZS
	T=SQRT(ZR**2+D**2)
	AT=A+T
	S1=SQRT(AT**2+RO2)
	F5=1./S1
	F7=1./(S1+AT)
	F1=F5*F7
	F3=F5**3
	F9=AT*F3
	XWYW=XSM*DWX+Y*DWY
	FR=ZR*(XSM*DZSX+Y*DZSY)
	FS=FR-D*(XSM*DDX+Y*DDY)
	WT=W/T
	WTFS=WT*FS
	BRZR1=WT*F1
	BRZR2=WT*F3
	BXT=(PA(1)*BRZR1+PA(2)*BRZR2)*ZR
	BYT=BXT*Y
	BXT=BXT*XSM
	BZT=PA(1)*(W*F5+XWYW*F7+WTFS*F1)+PA(2)*(W*F9+XWYW*F1+WTFS*F3)
C
C   CONTRIBUTION FROM CENTRAL TAIL CURRENT SHEET (BXT,BYT,BZT)
C   IS FOUND.  NOW LET US PROCEED TO THE RING CURRENT FIELD:
C
	RX2A2=1./(X2SM+A02)
	SRX2A2=SQRT(RX2A2)
	FDR=0.5*(1.+XSM*SRX2A2)
	DFDRX=HA02*RX2A2*SRX2A2
	DDR=D0+DD*FDR+DDOP
	TDR=SQRT(ZR**2+DDR**2)
	ADRT=ADR+TDR
	ADRT2=ADRT**2
	ADT2R2=1./(ADRT2+RO2)
	FK1=ADT2R2**2*SQRT(ADT2R2)
	FK2=3.*ADRT*FK1/TDR
	BXDR=PA(5)*ZR*FK2
	BYT=BYT+BXDR*Y
	BXT=BXT+BXDR*XSM
	BZT=BZT+PA(5)*((2.*ADRT2-RO2)*FK1+FK2*(FR-DDR*(DD*DFDRX
     *  +DDOPDX)*XSM))
C
C   NOW CALCULATE CHAPMAN-FERRARO FIELD
C   PLUS AVERAGE FIELD-ALIGNED CURRENT CONTRIBUTION
C
	EX=EXP(X/DELX)
	Z2=Z*Z
	YZ=Y*Z
	BXCF=EX*(CPS*PA(6)*Z+SPS*(PA(7)+PA(8)*Y2+PA(9)*Z2))
	BYCF=EX*(CPS*PA(10)*YZ+SPS*Y*(PA(11)+PA(12)*Y2+PA(13)*Z2))
        BZCF=EX*(CPS*(PA(14)+PA(15)*Y2+PA(16)*Z2)+SPS*Z*(PA(17)+
     *    PA(18)*Y2+PA(19)*Z2))
C
C   MAGNETOTAIL RETURN CURRENT FIELD COMPONENTS (BXC,BYC,BZC):
C
	FCY=1./(1.+Y2*RDYC2)
	XSXC=X-SXC
	RQC2=1./(XSXC**2+XLWC2)
	SRQC2=SQRT(RQC2)
	WC=0.5*(1.-XSXC*SRQC2)*FCY
	DWCX=HLWC2M*RQC2*SRQC2*FCY
	DWCY=DRDYCM*WC*Y*FCY
	XWCYWC=X*DWCX+Y*DWCY
	RO2=Y2+X**2
	ZP=Z+RT
	ZM=Z-RT
	SP=SQRT(ZP**2+RO2)
	SM=SQRT(ZM**2+RO2)
	WCSP=WC/SP
	WCSM=WC/SM
	RSPRT=1./(SP+ZP)
	RSMRT=1./(SM-ZM)
	FXP=WCSP*RSPRT
	FXM=-WCSM*RSMRT
	FYP=FXP*Y
	FYM=FXM*Y
	FXP=FXP*X
	FXM=FXM*X
	FZP=WCSP+XWCYWC*RSPRT
	FZM=WCSM+XWCYWC*RSMRT
	AA4SPS=PA(4)*SPS
	BXC=PA(3)*(FXP+FXM)+(FXP-FXM)*AA4SPS
	BYC=PA(3)*(FYP+FYM)+(FYP-FYM)*AA4SPS
	BZC=PA(3)*(FZP+FZM)+(FZP-FZM)*AA4SPS
C
C   AT LAST, SUM UP THE FIELDS.  THE CENTRAL CURRENT SHEET
C   FIELD COMPONENTS ARE TRANSFORMED INTO GSM COORDINATES
C
	BX=BXC+BXT*CPS+BZT*SPS+BXCF
	BY=BYC+BYT+BYCF
	BZ=BZC+BZT*CPS-BXT*SPS+BZCF
	RETURN
	END
c--------------------------------------------------------------------
c
      SUBROUTINE SUN(IYR,IDAY,IHOUR,MIN,ISEC,GST,SLONG,SRASN,SDEC)
C
C  CALCULATES FOUR QUANTITIES NECESSARY FOR COORDINATE TRANSFORMATIONS
C  WHICH DEPEND ON SUN POSITION (AND, HENCE, ON UNIVERSAL TIME AND SEASON)
C
C-------  INPUT PARAMETERS:
C  IYR,IDAY,IHOUR,MIN,ISEC -  YEAR, DAY, AND UNIVERSAL TIME IN HOURS, MINUTES,
C    AND SECONDS  (IDAY=1 CORRESPONDS TO JANUARY 1).
C
C-------  OUTPUT PARAMETERS:
C  GST - GREENWICH MEAN SIDEREAL TIME, SLONG - LONGITUDE ALONG ECLIPTIC
C  SRASN - RIGHT ASCENSION,  SDEC - DECLINATION  OF THE SUN (RADIANS)
C  THIS SUBROUTINE HAS BEEN COMPILED FROM: RUSSELL C.T., COSM.ELECTRO-
C  DYN., 1971, V.2,PP.184-196.
C
C
C                   AUTHOR: Gilbert D. Mead
C
C
	IMPLICIT NONE

	REAL GST,SLONG,SRASN,SDEC,RAD,T,VL,G,OBLIQ,SOB,SLP,SIND,
     1       COSD,SC

	INTEGER IYR,IDAY,IHOUR,MIN,ISEC

      DOUBLE PRECISION DJ,FDAY
      DATA RAD/57.295779513/
      IF(IYR.LT.1901.OR.IYR.GT.2099) RETURN
      FDAY=DFLOAT(IHOUR*3600+MIN*60+ISEC)/86400.D0
      DJ=365*(IYR-1900)+(IYR-1901)/4+IDAY-0.5D0+FDAY
      T=DJ/36525.
      VL=DMOD(279.696678+0.9856473354*DJ,360.D0)
      GST=DMOD(279.690983+.9856473354*DJ+360.*FDAY+180.,360.D0)/RAD
      G=DMOD(358.475845+0.985600267*DJ,360.D0)/RAD
      SLONG=(VL+(1.91946-0.004789*T)*SIN(G)+0.020094*SIN(2.*G))/RAD
      IF(SLONG.GT.6.2831853) SLONG=SLONG-6.2831853
      IF (SLONG.LT.0.) SLONG=SLONG+6.2831853
      OBLIQ=(23.45229-0.0130125*T)/RAD
      SOB=SIN(OBLIQ)
      SLP=SLONG-9.924E-5
C
C   THE LAST CONSTANT IS A CORRECTION FOR THE ANGULAR ABERRATION  DUE TO
C   THE ORBITAL MOTION OF THE EARTH
C
      SIND=SOB*SIN(SLP)
      COSD=SQRT(1.-SIND**2)
      SC=SIND/COSD
      SDEC=ATAN(SC)
      SRASN=3.141592654-ATAN2(COS(OBLIQ)/SOB*SC,-COS(SLP)/COSD)
      RETURN
      END
c-----------------------------------------------------------------------
c
      SUBROUTINE SPHCAR(R,TETA,PHI,X,Y,Z,J)
C
C   CONVERTS SPHERICAL COORDS INTO CARTESIAN ONES AND VICA VERSA
C    (TETA AND PHI IN RADIANS).
C
C                  J>0            J<0
C-----INPUT:   J,R,TETA,PHI     J,X,Y,Z
C----OUTPUT:      X,Y,Z        R,TETA,PHI
C
C
C                   AUTHOR: NIKOLAI A. TSYGANENKO
C                           INSTITUTE OF PHYSICS
C                           ST.-PETERSBURG STATE UNIVERSITY
C                           STARY PETERGOF 198904
C                           ST.-PETERSBURG
C                           RUSSIA
C
	IMPLICIT NONE

	REAL R,TETA,PHI,X,Y,Z,SQ

	INTEGER J

      IF(J.GT.0) GOTO 3
      SQ=X**2+Y**2
      R=SQRT(SQ+Z**2)
      IF (SQ.NE.0.) GOTO 2
      PHI=0.
      IF (Z.LT.0.) GOTO 1
      TETA=0.
      RETURN
  1   TETA=3.141592654
      RETURN
  2   SQ=SQRT(SQ)
      PHI=ATAN2(Y,X)
      TETA=ATAN2(SQ,Z)
      IF (PHI.LT.0.) PHI=PHI+6.28318531
      RETURN
  3   SQ=R*SIN(TETA)
      X=SQ*COS(PHI)
      Y=SQ*SIN(PHI)
      Z=R*COS(TETA)
      RETURN
      END
c--------------------------------------------------------------------
c
      SUBROUTINE BSPCAR(TETA,PHI,BR,BTET,BPHI,BX,BY,BZ)
C   CALCULATES CARTESIAN FIELD COMPONENTS FROM SPHERICAL ONES
C-----INPUT:   TETA,PHI - SPHERICAL ANGLES OF THE POINT IN RADIANS
C              BR,BTET,BPHI -  SPHERICAL COMPONENTS OF THE FIELD
C-----OUTPUT:  BX,BY,BZ - CARTESIAN COMPONENTS OF THE FIELD
C
C
C                   AUTHOR: NIKOLAI A. TSYGANENKO
C                           INSTITUTE OF PHYSICS
C                           ST.-PETERSBURG STATE UNIVERSITY
C                           STARY PETERGOF 198904
C                           ST.-PETERSBURG
C                           RUSSIA
C
	IMPLICIT NONE

	REAL TETA,PHI,BR,BTET,BPHI,BX,BY,BZ,S,C,SF,CF,BE

      S=SIN(TETA)
      C=COS(TETA)
      SF=SIN(PHI)
      CF=COS(PHI)
      BE=BR*S+BTET*C
      BX=BE*CF-BPHI*SF
      BY=BE*SF+BPHI*CF
      BZ=BR*C-BTET*S
      RETURN
      END
c
C----------------------------------------------------------------------
C
      SUBROUTINE RECALC(IYR,IDAY,IHOUR,MIN,ISEC)
C
C
C	THIS IS A MODIFIED VERSION OF THE SUBROUTINE RECOMP WRITTEN BY
C	N.A. TSYGANENKO.  SINCE I WANT TO USE IT IN PLACE OF SUBROUTINE
C	RECALC I HAVE RENAMED THIS ROUTINE RECALC AND ELIMINATED THE
C	ORIGINAL RECALC FROM THIS VERSION OF THE GEOPACK.F PACKET. THIS
C	WAY ALL ORIGINAL CALLS TO RECALC WILL CONTINUE TO WORK WITHOUT 
C	HAVING TO CHANGE THEM TO CALLS TO RECOMP.
C
C
C  AN ALTERNATIVE VERSION OF THE SUBROUTINE RECALC FROM THE GEOPACK PACKAGE
C  BASED ON A DIFFERENT APPROACH TO DERIVATION OF ROTATION MATRIX ELEMENTS
C
C THIS SUBROUTINE WORKS BY 20% FASTER THAN RECALC AND IS EASIER TO UNDERSTAND
C
C   ################################################
C   #  WRITTEN BY  N.A. TSYGANENKO ON DEC.1, 1991  #
C   ################################################
C
c
c	Modified by:	Mauricio Peredo
c			Hughes STX at NASA/GSFC Code 695
c			September 1992
c
c
c	Modified to accept dates up to year 2000 and updated IGRF
C	coeeficients for 1985.
c
C   OTHER SUBROUTINES CALLED BY THIS ONE: SUN
C
C     IYR = YEAR NUMBER (FOUR DIGITS)
C     IDAY = DAY OF YEAR (DAY 1 = JAN 1)
C     IHOUR = HOUR OF DAY (00 TO 23)
C     MIN = MINUTE OF HOUR (00 TO 59)
C     ISEC = SECONDS OF DAY(00 TO 59)
C
	IMPLICIT NONE

	REAL ST0,CT0,SL0,CL0,CTCL,STCL,CTSL,STSL,SFI,CFI,SPS,CPS,
     1       SHI,CHI,HI,PSI,XMUT,A11,A21,A31,A12,A22,A32,A13,A23,
     2       A33,DS3,F2,F1,G10,G11,H11,DT,SQ,SQQ,SQR,S1,S2,
     3       S3,CGST,SGST,DIP1,DIP2,DIP3,Y1,Y2,Y3,Y,Z1,Z2,Z3,DJ,
     4       T,OBLIQ,DZ1,DZ2,DZ3,DY1,DY2,DY3,EXMAGX,EXMAGY,EXMAGZ,
     5       EYMAGX,EYMAGY,GST,SLONG,SRASN,SDEC,BA(8)

	INTEGER IYR,IDAY,IHOUR,MIN,ISEC,K,IY,IDE,IYE,IPR

       COMMON/C1/ ST0,CT0,SL0,CL0,CTCL,STCL,CTSL,STSL,SFI,CFI,SPS,CPS,
     * SHI,CHI,HI,PSI,XMUT,A11,A21,A31,A12,A22,A32,A13,A23,A33,DS3,
     * K,IY,BA
c
      DATA IYE,IDE,IPR/3*0/
      IF (IYR.EQ.IYE.AND.IDAY.EQ.IDE) GOTO 5
C
C  IYE AND IDE ARE THE CURRENT VALUES OF YEAR AND DAY NUMBER
C
      IY=IYR
      IDE=IDAY
      IF(IY.LT.1965) IY=1965
      IF(IY.GT.2000) IY=2000
C
C  WE ARE RESTRICTED BY THE INTERVAL 1965-2000,
C  FOR WHICH THE IGRF COEFFICIENTS ARE KNOWN; IF IYR IS OUTSIDE THIS INTERVAL
C   THE SUBROUTINE GIVES A WARNING (BUT DOES NOT REPEAT IT AT THE NEXT CALLS)
C
      IF(IY.NE.IYR.AND.IPR.EQ.0) PRINT 10,IYR,IY
      IF(IY.NE.IYR) IPR=1
      IYE=IY
C
C  LINEAR INTERPOLATION OF THE GEODIPOLE MOMENT COMPONENTS BETWEEN THE
C  VALUES FOR THE NEAREST EPOCHS:
C
	IF (IY.LT.1970) THEN				!1965-1970
	   F2=(FLOAT(IY)+FLOAT(IDAY)/365.-1965.)/5.
	   F1=1.D0-F2
	   G10=30334.*F1+30220.*F2
	   G11=-2119.*F1-2068.*F2
	   H11=5776.*F1+5737.*F2
	ELSEIF (IY.LT.1975) THEN			!1970-1975
	   F2=(FLOAT(IY)+FLOAT(IDAY)/365.-1970.)/5.
	   F1=1.D0-F2
	   G10=30220.*F1+30100.*F2
	   G11=-2068.*F1-2013.*F2
	   H11=5737.*F1+5675.*F2
	ELSEIF (IY.LT.1980) THEN			!1975-1980
	   F2=(DFLOAT(IY)+DFLOAT(IDAY)/365.-1975.)/5.
	   F1=1.D0-F2
	   G10=30100.*F1+29992.*F2
	   G11=-2013.*F1-1956.*F2
	   H11=5675.*F1+5604.*F2
 	ELSEIF (IY.LT.1985) THEN			!1980-1985
	   F2=(FLOAT(IY)+FLOAT(IDAY)/365.-1980.)/5.
	   F1=1.D0-F2
	   G10=29992.*F1+29873.*F2
	   G11=-1956.*F1-1905.*F2
	   H11=5604.*F1+5500.*F2
	ELSEIF (IY.LT.1990) THEN			!1985-1990
	   F2=(FLOAT(IY)+FLOAT(IDAY)/365.-1985.)/5.
	   F1=1.D0-F2
	   G10=29873.*F1+29775.*F2
	   G11=-1905.*F1-1851.*F2
	   H11=5500.*F1+5411.*F2
	ELSE						!1990-2000
	   DT=FLOAT(IY)+FLOAT(IDAY)/365.-1990.
	   G10=29775.-18.0*DT
	   G11=-1851.+10.6*DT
	   H11=5411.-16.1*DT
	ENDIF
C
C  NOW CALCULATE THE COMPONENTS OF THE UNIT VECTOR EzMAG IN GEO COORD.SYSTEM:
C   SIN(TETA0)*COS(LAMBDA0), SIN(TETA0)*SIN(LAMBDA0), AND COS(TETA0)
C         ST0 * CL0                ST0 * SL0                CT0
C
      SQ=G11**2+H11**2
      SQQ=SQRT(SQ)
      SQR=SQRT(G10**2+SQ)
      SL0=-H11/SQQ
      CL0=-G11/SQQ
      ST0=SQQ/SQR
      CT0=G10/SQR
      STCL=ST0*CL0
      STSL=ST0*SL0
      CTSL=CT0*SL0
      CTCL=CT0*CL0
C
C      THE CALCULATIONS ARE TERMINATED IF ONLY GEO-MAG TRANSFORMATION
C       IS TO BE DONE  (IHOUR>24 IS THE AGREED CONDITION FOR THIS CASE):
C
   5   IF (IHOUR.GT.24) RETURN
C
      CALL SUN(IY,IDAY,IHOUR,MIN,ISEC,GST,SLONG,SRASN,SDEC)
C
C  S1,S2, AND S3 ARE THE COMPONENTS OF THE UNIT VECTOR EXGSM=EXGSE IN THE
C   SYSTEM GEI POINTING FROM THE EARTH'S CENTER TO THE SUN:
C
      S1=COS(SRASN)*COS(SDEC)
      S2=SIN(SRASN)*COS(SDEC)
      S3=SIN(SDEC)
      CGST=COS(GST)
      SGST=SIN(GST)
C
C  DIP1, DIP2, AND DIP3 ARE THE COMPONENTS OF THE UNIT VECTOR EZSM=EZMAG
C   IN THE SYSTEM GEI:
C
      DIP1=STCL*CGST-STSL*SGST
      DIP2=STCL*SGST+STSL*CGST
      DIP3=CT0
C
C  NOW CALCULATE THE COMPONENTS OF THE UNIT VECTOR EYGSM IN THE SYSTEM GEI
C   BY TAKING THE VECTOR PRODUCT D x S AND NORMALIZING IT TO UNIT LENGTH:
C
      Y1=DIP2*S3-DIP3*S2
      Y2=DIP3*S1-DIP1*S3
      Y3=DIP1*S2-DIP2*S1
      Y=SQRT(Y1*Y1+Y2*Y2+Y3*Y3)
      Y1=Y1/Y
      Y2=Y2/Y
      Y3=Y3/Y
C
C   THEN IN THE GEI SYSTEM THE UNIT VECTOR Z = EZGSM = EXGSM x EYGSM = S x Y
C    HAS THE COMPONENTS:
C
      Z1=S2*Y3-S3*Y2
      Z2=S3*Y1-S1*Y3
      Z3=S1*Y2-S2*Y1
C
C    THE VECTOR EZGSE (HERE DZ) IN GEI HAS THE COMPONENTS (0,-SIN(DELTA),
C     COS(DELTA)) = (0.,-0.397823,0.917462); HERE DELTA = 23.44214 DEG FOR
C   THE EPOCH 1978 (SEE THE BOOK BY GUREVICH OR OTHER ASTRONOMICAL HANDBOOKS).
C    HERE THE MOST ACCURATE TIME-DEPENDENT FORMULA IS USED:
C
      DJ=FLOAT(365*(IY-1900)+(IY-1901)/4 +IDAY)-0.5+FLOAT(ISEC)/86400.
      T=DJ/36525.
      OBLIQ=(23.45229-0.0130125*T)/57.2957795
      DZ1=0.
      DZ2=-SIN(OBLIQ)
      DZ3=COS(OBLIQ)
C
C  THEN THE UNIT VECTOR EYGSE IN GEI SYSTEM IS THE VECTOR PRODUCT DZ x S :
C
      DY1=DZ2*S3-DZ3*S2
      DY2=DZ3*S1-DZ1*S3
      DY3=DZ1*S2-DZ2*S1
C
C   THE ELEMENTS OF THE MATRIX GSE TO GSM ARE THE SCALAR PRODUCTS:
C   CHI=EM22=(EYGSM,EYGSE), SHI=EM23=(EYGSM,EZGSE), EM32=(EZGSM,EYGSE)=-EM23,
C     AND EM33=(EZGSM,EZGSE)=EM22
C
      CHI=Y1*DY1+Y2*DY2+Y3*DY3
      SHI=Y1*DZ1+Y2*DZ2+Y3*DZ3
      HI=ASIN(SHI)
C
C    TILT ANGLE: PSI=ARCSIN(DIP,EXGSM)
C
      SPS=DIP1*S1+DIP2*S2+DIP3*S3
      CPS=SQRT(1.-SPS**2)
      PSI=ASIN(SPS)
C
C    THE ELEMENTS OF THE MATRIX MAG TO SM ARE THE SCALAR PRODUCTS:
C CFI=GM22=(EYSM,EYMAG), SFI=GM23=(EYSM,EXMAG); THEY CAN BE DERIVED AS FOLLOWS:
C
C IN GEO THE VECTORS EXMAG AND EYMAG HAVE THE COMPONENTS (CT0*CL0,CT0*SL0,-ST0)
C  AND (-SL0,CL0,0), RESPECTIVELY.    HENCE, IN GEI THE COMPONENTS ARE:
C  EXMAG:    CT0*CL0*COS(GST)-CT0*SL0*SIN(GST)
C            CT0*CL0*SIN(GST)+CT0*SL0*COS(GST)
C            -ST0
C  EYMAG:    -SL0*COS(GST)-CL0*SIN(GST)
C            -SL0*SIN(GST)+CL0*COS(GST)
C             0
C  THE COMPONENTS OF EYSM IN GEI WERE FOUND ABOVE AS Y1, Y2, AND Y3;
C  NOW WE ONLY HAVE TO COMBINE THE QUANTITIES INTO SCALAR PRODUCTS:
C
      EXMAGX=CT0*(CL0*CGST-SL0*SGST)
      EXMAGY=CT0*(CL0*SGST+SL0*CGST)
      EXMAGZ=-ST0
      EYMAGX=-(SL0*CGST+CL0*SGST)
      EYMAGY=-(SL0*SGST-CL0*CGST)
      CFI=Y1*EYMAGX+Y2*EYMAGY
      SFI=Y1*EXMAGX+Y2*EXMAGY+Y3*EXMAGZ
C
      XMUT=(ATAN2(SFI,CFI)+3.1415926536)*3.8197186342
C
C  THE ELEMENTS OF THE MATRIX GEO TO GSM ARE THE SCALAR PRODUCTS:
C
C   A11=(EXGEO,EXGSM), A12=(EYGEO,EXGSM), A13=(EZGEO,EXGSM),
C   A21=(EXGEO,EYGSM), A22=(EYGEO,EYGSM), A23=(EZGEO,EYGSM),
C   A31=(EXGEO,EZGSM), A32=(EYGEO,EZGSM), A33=(EZGEO,EZGSM),
C
C   ALL THE UNIT VECTORS IN BRACKETS ARE ALREADY DEFINED IN GEI:
C
C  EXGEO=(CGST,SGST,0), EYGEO=(-SGST,CGST,0), EZGEO=(0,0,1)
C  EXGSM=(S1,S2,S3),  EYGSM=(Y1,Y2,Y3),   EZGSM=(Z1,Z2,Z3)
C                                                           AND  THEREFORE:
C
      A11=S1*CGST+S2*SGST
      A12=-S1*SGST+S2*CGST
      A13=S3
      A21=Y1*CGST+Y2*SGST
      A22=-Y1*SGST+Y2*CGST
      A23=Y3
      A31=Z1*CGST+Z2*SGST
      A32=-Z1*SGST+Z2*CGST
      A33=Z3
C
 10   FORMAT(//1X,
     * '****RECOMP WARNS:  YEAR IS OUT OF INTERVAL 1965-2000: IYR=',I4,
     * /,6X,'CALCULATIONS WILL BE DONE FOR IYR=',I4,/)
      RETURN
      END
C--------------------------------------------------------------------------
C
      SUBROUTINE GEOMAG(XGEO,YGEO,ZGEO,XMAG,YMAG,ZMAG,J,IYR)
C
C CONVERTS GEOGRAPHIC (GEO) TO DIPOLE (MAG) COORDINATES OR VICA VERSA.
C IYR IS YEAR NUMBER (FOUR DIGITS).
C
C                           J>0                J<0
C-----INPUT:  J,XGEO,YGEO,ZGEO,IYR   J,XMAG,YMAG,ZMAG,IYR
C-----OUTPUT:    XMAG,YMAG,ZMAG        XGEO,YGEO,ZGEO
C
C
C                   AUTHOR: NIKOLAI A. TSYGANENKO
C                           INSTITUTE OF PHYSICS
C                           ST.-PETERSBURG STATE UNIVERSITY
C                           STARY PETERGOF 198904
C                           ST.-PETERSBURG
C                           RUSSIA
C
	IMPLICIT NONE

	REAL XGEO,YGEO,ZGEO,XMAG,YMAG,ZMAG,ST0,CT0,SL0,CL0,CTCL,
     1       STCL,CTSL,STSL,AB(19),BB(8)

	INTEGER J,IYR,K,IY,II

      COMMON/C1/ ST0,CT0,SL0,CL0,CTCL,STCL,CTSL,STSL,AB,K,IY,BB
      DATA II/1/
      IF(IYR.EQ.II) GOTO 1
      II=IYR
      CALL RECALC(II,0,25,0,0)
  1   CONTINUE
      IF(J.LT.0) GOTO 2
      XMAG=XGEO*CTCL+YGEO*CTSL-ZGEO*ST0
      YMAG=YGEO*CL0-XGEO*SL0
      ZMAG=XGEO*STCL+YGEO*STSL+ZGEO*CT0
      RETURN
  2   XGEO=XMAG*CTCL-YMAG*SL0+ZMAG*STCL
      YGEO=XMAG*CTSL+YMAG*CL0+ZMAG*STSL
      ZGEO=ZMAG*CT0-XMAG*ST0
      RETURN
      END
C------------------------------------------------------------------------
C
      SUBROUTINE MAGSM(XMAG,YMAG,ZMAG,XSM,YSM,ZSM,J)
C
C CONVERTS DIPOLE (MAG) TO SOLAR MAGNETIC (SM) COORDINATES OR VICA VERSA
C
C                    J>0              J<0
C-----INPUT: J,XMAG,YMAG,ZMAG     J,XSM,YSM,ZSM
C----OUTPUT:    XSM,YSM,ZSM       XMAG,YMAG,ZMAG
C  ATTENTION:  SUBROUTINE  RECALC  MUST BE CALLED BEFORE MAGSM IN TWO CASES:
C     /A/  BEFORE THE FIRST USE OF MAGSM
C     /B/  IF THE CURRENT VALUES OF IYEAR,IDAY,IHOUR,MIN,ISEC ARE DIFFERENT
C          FROM THOSE IN THE PRECEDING CALL OF  MAGSM
C
C                   AUTHOR: NIKOLAI A. TSYGANENKO
C                           INSTITUTE OF PHYSICS
C                           ST.-PETERSBURG STATE UNIVERSITY
C                           STARY PETERGOF 198904
C                           ST.-PETERSBURG
C                           RUSSIA
C
	IMPLICIT NONE

	REAL XMAG,YMAG,ZMAG,XSM,YSM,ZSM,SFI,CFI,A(8),B(7),
     1       AB(10),BA(8)

	INTEGER J,K,IY

      COMMON/C1/ A,SFI,CFI,B,AB,K,IY,BA
      IF (J.LT.0) GOTO 1
      XSM=XMAG*CFI-YMAG*SFI
      YSM=XMAG*SFI+YMAG*CFI
      ZSM=ZMAG
      RETURN
  1   XMAG=XSM*CFI+YSM*SFI
      YMAG=YSM*CFI-XSM*SFI
      ZMAG=ZSM
      RETURN
      END
C-------------------------------------------------------------------------
C
       SUBROUTINE GSMGSE(XGSM,YGSM,ZGSM,XGSE,YGSE,ZGSE,J)
C
C CONVERTS SOLAR MAGNETOSPHERIC (GSM) TO SOLAR ECLIPTICAL (GSE) COORDS
C   OR VICA VERSA.
C                    J>0                J<0
C-----INPUT: J,XGSM,YGSM,ZGSM    J,XGSE,YGSE,ZGSE
C----OUTPUT:   XGSE,YGSE,ZGSE      XGSM,YGSM,ZGSM
C  ATTENTION:  SUBROUTINE  RECALC  MUST BE CALLED BEFORE GSMGSE IN TWO CASES:
C     /A/  BEFORE THE FIRST CALL OF GSMGSE
C     /B/  IF THE CURRENT VALUES OF IYEAR,IDAY,IHOUR,MIN,ISEC ARE DIFFERENT
C          FROM THOSE IN THE PRECEDING  CALL  OF GSMGSE
C
C
C                   AUTHOR: NIKOLAI A. TSYGANENKO
C                           INSTITUTE OF PHYSICS
C                           ST.-PETERSBURG STATE UNIVERSITY
C                           STARY PETERGOF 198904
C                           ST.-PETERSBURG
C                           RUSSIA
C
	IMPLICIT NONE

	REAL XGSM,YGSM,ZGSM,XGSE,YGSE,ZGSE,SHI,CHI,
     1       A(12),AB(13),BA(8)
	INTEGER J,K,IY

      COMMON/C1/ A,SHI,CHI,AB,K,IY,BA
      IF(J.LT.0) GOTO 1
      XGSE=XGSM
      YGSE=YGSM*CHI-ZGSM*SHI
      ZGSE=YGSM*SHI+ZGSM*CHI
      RETURN
1     XGSM=XGSE
      YGSM=YGSE*CHI+ZGSE*SHI
      ZGSM=ZGSE*CHI-YGSE*SHI
      RETURN
      END
C--------------------------------------------------------------------------
C
       SUBROUTINE SMGSM(XSM,YSM,ZSM,XGSM,YGSM,ZGSM,J)
C
C CONVERTS SOLAR MAGNETIC (SM) TO SOLAR MAGNETOSPHERIC (GSM) COORDINATES
C   OR VICA VERSA.
C                  J>0                 J<0
C-----INPUT: J,XSM,YSM,ZSM        J,XGSM,YGSM,ZGSM
C----OUTPUT:  XGSM,YGSM,ZGSM       XSM,YSM,ZSM
C
C  ATTENTION:  SUBROUTINE RECALC MUST BE CALLED BEFORE SMGSM IN TWO CASES:
C     /A/  BEFORE THE FIRST USE OF SMGSM
C     /B/  IF THE CURRENT VALUES OF IYEAR,IDAY,IHOUR,MIN,ISEC ARE DIFFERENT
C          FROM THOSE IN THE PRECEDING CALL OF SMGSM
C
C
C                AUTHOR: NIKOLAI A. TSYGANENKO
C                        INSTITUTE OF PHYSICS
C                        ST.-PETERSBURG  UNIVERSITY
C                        STARY PETERGOF 198904
C                        ST.-PETERSBURG
C                        U.S.S.R.
C
	IMPLICIT NONE

	REAL XSM,YSM,ZSM,XGSM,YGSM,ZGSM,SPS,CPS,A(10),B(15),AB(8)
	INTEGER J,K,IY

      COMMON/C1/ A,SPS,CPS,B,K,IY,AB
      IF (J.LT.0) GOTO 1
      XGSM=XSM*CPS+ZSM*SPS
      YGSM=YSM
      ZGSM=ZSM*CPS-XSM*SPS
      RETURN
  1   XSM=XGSM*CPS-ZGSM*SPS
      YSM=YGSM
      ZSM=XGSM*SPS+ZGSM*CPS
      RETURN
      END
C---------------------------------------------------------------------------
C
      SUBROUTINE GEOGSM(XGEO,YGEO,ZGEO,XGSM,YGSM,ZGSM,J)
C
C CONVERTS GEOGRAPHIC TO SOLAR MAGNETOSPHERIC COORDINATES OR VICA VERSA.
C
C                   J>0                   J<0
C----- INPUT:  J,XGEO,YGEO,ZGEO    J,XGSM,YGSM,ZGSM
C---- OUTPUT:    XGSM,YGSM,ZGSM      XGEO,YGEO,ZGEO
C  ATTENTION:  SUBROUTINE  RECALC  MUST BE CALLED BEFORE GEOGSM IN TWO CASES:
C     /A/  BEFORE THE FIRST USE OF GEOGSM
C     /B/  IF THE CURRENT VALUES OF IYEAR,IDAY,IHOUR,MIN,ISEC  ARE DIFFERENT
C            FROM THOSE IN THE PREVIOUS CALL OF THIS SUBROUTINE
C
C
C                   AUTHOR: NIKOLAI A. TSYGANENKO
C                           INSTITUTE OF PHYSICS
C                           ST.-PETERSBURG STATE UNIVERSITY
C                           STARY PETERGOF 198904
C                           ST.-PETERSBURG
C                           RUSSIA
C
	IMPLICIT NONE

	REAL XGEO,YGEO,ZGEO,XGSM,YGSM,ZGSM,A11,A21,A31,A12,
     1       A22,A32,A13,A23,A33,D,AA(17),B(8)
	INTEGER J,K,IY

      COMMON/C1/ AA,A11,A21,A31,A12,A22,A32,A13,A23,A33,D,K,IY,B
      IF (J.LT.0) GOTO 1
      XGSM=A11*XGEO+A12*YGEO+A13*ZGEO
      YGSM=A21*XGEO+A22*YGEO+A23*ZGEO
      ZGSM=A31*XGEO+A32*YGEO+A33*ZGEO
      RETURN
  1   XGEO=A11*XGSM+A21*YGSM+A31*ZGSM
      YGEO=A12*XGSM+A22*YGSM+A32*ZGSM
      ZGEO=A13*XGSM+A23*YGSM+A33*ZGSM
      RETURN
      END
C-------------------------------------------------------------------------
C
      SUBROUTINE RHAND(X,Y,Z,R1,R2,R3,IOPT,EXNAME)
C
C COMPUTES RIGHT HAND EXPRESSIONS IN THE GEOMAGNETIC FIELD LINE EQUATION
C      (a subsidiary subroutine for the subr. STEP)
C
C                   AUTHOR: NIKOLAI A. TSYGANENKO
C                           INSTITUTE OF PHYSICS
C                           ST.-PETERSBURG STATE UNIVERSITY
C                           STARY PETERGOF 198904
C                           ST.-PETERSBURG
C                           RUSSIA
C
	IMPLICIT NONE

	REAL X,Y,Z,R1,R2,R3,PSI,DS3,BX,BY,BZ,A(15),AA(10),BB(8),
     1       XG,YG,ZG,R,T,F,BR,BT,BF,FX,FY,FZ,HX,HY,HZ,B
	INTEGER IOPT,K,IY
	EXTERNAL EXNAME

      COMMON/C1/ A,PSI,AA,DS3,K,IY,BB
      CALL EXNAME(IOPT,PSI,X,Y,Z,BX,BY,BZ)
      IF (K.EQ.0) GOTO 1
      CALL GEOGSM(XG,YG,ZG,X,Y,Z,-1)
      CALL SPHCAR(R,T,F,XG,YG,ZG,-1)
      CALL IGRF(IY,K,R,T,F,BR,BT,BF)
      CALL BSPCAR(T,F,BR,BT,BF,FX,FY,FZ)
      CALL GEOGSM(FX,FY,FZ,HX,HY,HZ,1)
      GOTO 2
  1   CALL DIP(PSI,X,Y,Z,HX,HY,HZ)
  2   BX=BX+HX
      BY=BY+HY
      BZ=BZ+HZ
      B=DS3/SQRT(BX**2+BY**2+BZ**2)
      R1=BX*B
      R2=BY*B
      R3=BZ*B
      RETURN
      END
C------------------------------------------------------------------------
C
      SUBROUTINE STEP(N,X,Y,Z,DS,ERRIN,IOPT,EXNAME)
C
C RE-CALCULATES COORDS X,Y,Z FOR ONE STEP ALONG FIELD LINE. N IS MAXIMUM
C ORDER OF HARMONICS IN MAIN FIELD EXPANSION, DS IS STEP SIZE, ERRIN IS
C PERMISSIBLE ERROR VALUE, IOPT AND EXNAME - SEE COMMENTS TO SUBROUTINE TRACE
C  ALL THE PARAMETERS ARE INPUT ONES; OUTPUT IS THE RENEWED TRIPLET X,Y,Z
C
C                   AUTHOR: NIKOLAI A. TSYGANENKO
C                           INSTITUTE OF PHYSICS
C                           ST.-PETERSBURG STATE UNIVERSITY
C                           STARY PETERGOF 198904
C                           ST.-PETERSBURG
C                           RUSSIA
C
	IMPLICIT NONE

	REAL X,Y,Z,DS,ERRIN,DS3,R11,R12,R13,R21,R22,R23,
     1       R31,R32,R33,R41,R42,R43,R51,R52,R53,ERRCUR,A(26),B(8)
	INTEGER N,IOPT,K,IY

      COMMON/C1/ A,DS3,K,IY,B
      EXTERNAL EXNAME
      K=N
  1   DS3=-DS/3.
      CALL RHAND(X,Y,Z,R11,R12,R13,IOPT,EXNAME)
      CALL RHAND(X+R11,Y+R12,Z+R13,R21,R22,R23,IOPT,EXNAME)
      CALL RHAND(X+.5*(R11+R21),Y+.5*(R12+R22),Z+.5*
     *(R13+R23),R31,R32,R33,IOPT,EXNAME)
      CALL RHAND(X+.375*(R11+3.*R31),Y+.375*(R12+3.*R32
     *),Z+.375*(R13+3.*R33),R41,R42,R43,IOPT,EXNAME)
      CALL RHAND(X+1.5*(R11-3.*R31+4.*R41),Y+1.5*(R12-
     *3.*R32+4.*R42),Z+1.5*(R13-3.*R33+4.*R43),
     *R51,R52,R53,IOPT,EXNAME)
      ERRCUR=ABS(R11-4.5*R31+4.*R41-.5*R51)+ABS(R12-4.5*R32+4.*R42-.5*
     *R52)+ABS(R13-4.5*R33+4.*R43-.5*R53)
      IF (ERRCUR.LT.ERRIN) GOTO 2
      DS=DS*.5
      GOTO 1
  2   X=X+.5*(R11+4.*R41+R51)
      Y=Y+.5*(R12+4.*R42+R52)
      Z=Z+.5*(R13+4.*R43+R53)
      IF(ERRCUR.LT.ERRIN*.04.AND.ABS(DS).LT.1.33) DS=DS*1.5
      RETURN
      END
C----------------------------------------------------------------------
C
      SUBROUTINE TRACE(XI,YI,ZI,DIR,RLIM,R0,IHARM,NP,IOPT,EXNAME,
     *XF,YF,ZF,XX,YY,ZZ,L)
C
C   TRACES FIELD LINE FROM ARBITRARY POINT OF SPACE UP TO THE EARTH
C   SURFACE OR UP TO MODEL LIMITING BOUNDARY.
C-------------- INPUT PARAMETERS:
C   XI,YI,ZI - GSM COORDS OF INITIAL POINT (IN EARTH RADII),
C   DIR - SIGN OF TRACING DIRECTION: IF DIR=1. THEN ANTIPARALLEL TO
C     B VECTOR (E.G. FROM NORTHERN TO SOUTHERN CONJUGATE POINT),
C     AND IF DIR=-1. THEN PARALLEL TO B.
C   R0 IS RADIUS OF SPHERE (IN RE) FOR WHICH 'LANDING POINT' COORDINATES
C     XF,YF,ZF  SHOULD BE CALCULATED
C   RLIM - UPPER GEOCENTRIC DISTANCE WHICH LIMITS TRACING REGION.
C   IHARM - MAXIMAL ORDER OF SPH.HARMONICS IN THE MAIN FIELD EXPANSION,
C     IT DEPENDS ON THE CURRENT VERSION OF INTERNAL FIELD MODEL. SUB-
C     ROUTINE IGRF OF THIS ISSUE CORRESPONDS TO IHARM=10. IF THE MAIN
C     FIELD SHOULD BE ASSUMED TO BE PURELY DIPOLAR, THEN PUT IHARM=0
C     AND SPECIFY VALUE OF GEODIPOLE TILT ANGLE PSI (IN RADIANS)
C     IN THE 16-TH ELEMENT OF THE COMMON BLOCK BEFORE CALLING TRACE.
C     OTHERWISE CALL SUBROUTINE RECALC BEFORE CALLING TRACE.
C   NP - UPPER ESTIMATE OF NUMBER OF STEPS ALONG THE FIELD LINE
C     (OF THE ORDER OF SEVERAL HUNDREDS).
C   IOPT - SPECIFIES OPTION OF EXTERNAL FIELD MODEL (E.G. LEVEL OF
C     DISTURBANCE) - SEE COMMENTS TO SUBROUTINES EXTERN AND EXLONG.
C   EXNAME - NAME OF THE EXTRATERRESTRIAL FIELD MODEL SUBROUTINE
C     (EXTERN OR EXLONG OR SOME OTHER ONE).
C-------------- OUTPUT PARAMETERS:
C   XF,YF,ZF - GSM COORDS OF FINAL POINT
C   XX,YY,ZZ - ARRAYS (LENGTH NP) CONTAINING COORDS OF FIELD LINE POINTS
C   L - ACTUAL NUMBER OF FIELD LINE POINTS. IF L EXCEEDS NP, TRACING
C     TERMINATES, AND A WARNING IS DISPLAYED
C
C
C                   AUTHOR: NIKOLAI A. TSYGANENKO
C                           INSTITUTE OF PHYSICS
C                           ST.-PETERSBURG STATE UNIVERSITY
C                           STARY PETERGOF 198904
C                           ST.-PETERSBURG
C                           RUSSIA
C
	IMPLICIT NONE

	REAL XI,YI,ZI,DIR,RLIM,R0,XF,YF,ZF,DD,ERR,DS,AA(26),BB(8),
     1       X,Y,Z,AL,R1,R2,R3,AD,RR,RYZ,R,FC,XR,YR,ZR
	INTEGER IHARM,NP,IOPT,L,K1,K2,J,I

      REAL XX(NP),YY(NP),ZZ(NP)
      COMMON/C1/ AA,DD,K1,K2,BB
      EXTERNAL EXNAME
 10   FORMAT(//,1X,'**** COMPUTATIONS IN THE SUBROUTINE TRACE',
     *' ARE TERMINATED: NP IS TOO SMALL ****'//)
      J=IHARM
      ERR=0.0005
      L=0
      DS=0.5*DIR
      X=XI
      Y=YI
      Z=ZI
      DD=DIR
      K1=IHARM
      AL=0.
      CALL RHAND(X,Y,Z,R1,R2,R3,IOPT,EXNAME)
      AD=SIGN(0.01,X*R1+Y*R2+Z*R3)
      RR=SQRT(X**2+Y**2+Z**2)+AD
  1   L=L+1
      IF(L.GT.NP) GOTO 7
      XX(L)=X
      YY(L)=Y
      ZZ(L)=Z
      RYZ=Y**2+Z**2
      R2=X**2+RYZ
      R=SQRT(R2)
      IF(R.GT.RLIM.OR.RYZ.GT.900..OR.X.GT.15.) GOTO 8
      IF(R.LT.R0.AND.RR.GT.R) GOTO 6
      IF(R.GE.RR) GOTO 5
      IF(R.GT.5.) GOTO 5
      IF(R.GE.3.) GOTO 3
      FC=0.2
      IF(R-R0.LT.0.05) FC=0.05
      AL=FC*(R-R0+0.2)
      DS=DIR*AL
      GOTO 4
  3   DS=DIR
      AL=1.
  4   XR=X
      YR=Y
      ZR=Z
  5   RR=R
      IF(IHARM.NE.0) J=1+10./(R-AL)
      IF(J.GT.IHARM) J=IHARM
      CALL STEP(J,X,Y,Z,DS,ERR,IOPT,EXNAME)
      GOTO 1
  6   R1=(R0-R)/(RR-R)
      X=X-(X-XR)*R1
      Y=Y-(Y-YR)*R1
      Z=Z-(Z-ZR)*R1
      GOTO 8
  7   write(*,10)
      L=NP
      RETURN
  8   XF=X
      YF=Y
      ZF=Z
      DO 9 I=L,NP
      XX(I)=XF
      YY(I)=YF
  9   ZZ(I)=ZF
      RETURN
      END
C------------------------------------------------------------------------


