#include <stdio.h>
#include <dx/dx.h>

#define MAX_NAME_SIZE 256
#define BUFFER 256

float lininterp(float, float, float, float, float);

int debug=0;   /* this is the global flag to turn on/off debugging messages.  */

/* This module implements the IGRF onto a set of points defined by the user
 * This can then be used for comparison with the MHD field coming from the LFM.
 *
 * *****NEW VERSION***** 7/29/2004 by Chia-Lin Huang
 * The routine takes the MHD time (out of yrdayhr) and interpolates through the input file
 * to find the correct inputs to drive the GEOPACK fortran subroutine.
 * 
 *
 * This module has these inputs
 *    In[0] = MHD time (from yrdayhr)
 *    In[1] = Mesh to measure field on
 *    In[2] = last 'linenumber' of input file that matched the MHD Time
 *
 * and these outputs
 *    Out[0] = IGRF Magnetic Field - field float 3 vector
 *    Out[1] = 'linenumber' of the input file that matched MHD Time
 *                 Use SetLocal/GetLocal pair to manage the linenumbers in Dx
 */

Error
m_igrf(Object *in, Object *out)
{
  /* Varible declarations */

  /* Inputs */
  Type TInDataType, TInCoordType;
  Category CInDataCat, CInCoordCat;
  int iInDataItems, iInCoordItems, iInDataRank, iInCoordRank;
  int iInDataShape, iInCoordShape;
  Array aInData, aInCoord, aInConnect;
  float *fpInData, *fpInCoord ;
  int iCount, *ipYear, *ipDoy, inlinenumber, outlinenumber, *ipHr, *ipMin, *ipSec;
  float *fpHr;
  float *fpIGRFLoc, *fpInLoc;
  float *fpIGRFB;
  float fX, fY, fZ;
  int i, j;
  float fIGRFBx, fIGRFBy, fIGRFBz;
  Array returnlinenumber;

  Array simtime_array;
  int  *SimTime = NULL;  /* pointer to the first element of SimTime read in from yrdayhr */
  float SimHr;           /* decimal hour */


  /* Output */
  Array aIGRFData = NULL; 	/* Array for IGRF Magnetic Field*/
  float *fpIGRFData;		/* Ptr to IGRF Mag Field data*/
  Array aIGRFCoord = NULL;		/* Array for IGRF Grid Pts*/
  float *fpIGRFCoord;		/* Ptr to IGRF Mag Field Pts*/
  Field fIGRF = NULL;		/* IGRF Field*/

 
  /* Read in MHD time */
  if (!in[0]) {
    DXSetError(ERROR_BAD_PARAMETER, "Missing UT Time from yrdayhr");
    goto error;
  }
  else {
    simtime_array = (Array) in[0];   /* get the UT time array */
    SimTime = DXGetArrayData(simtime_array);  /* put it into SimTime */
  }

  if (debug) DXMessage("Old Read in SimTime is : %d %d %d %d %d\n", *(SimTime+0),*(SimTime+1),
			 *(SimTime+2),*(SimTime+3),*(SimTime+4));


  /* Process the Input field of locations */
  if (!in[1]) {
    DXSetError(ERROR_BAD_PARAMETER,"Missing Input field");
    goto error;
  } else {
    aInCoord = (Array)DXGetComponentValue((Field)in[1],"positions");
    if (!aInCoord) {
       DXSetError(ERROR_INTERNAL,"No input coord field");
       goto error;
    }
    fpInCoord =  DXGetArrayData(aInCoord);
    DXGetArrayInfo(aInCoord,&iInCoordItems,&TInCoordType,&CInCoordCat,
                   &iInCoordRank,&iInCoordShape);
    DXMessage("Coord Items %d Rank %d Shape %d",iInCoordItems,iInCoordRank,
              iInCoordShape);
    aInConnect = (Array)DXGetComponentValue((Field)in[1],"connections");
    if (!aInConnect) {
       DXSetError(ERROR_INTERNAL,"No input connections field");
       goto error;
    }
 }

  /* now read in the last 'linenumber' which matched the MHD time */
  if (!in[2]) {
    DXMessage("Didn't find an input 'linenumber', starting from the beginning of input file");
    inlinenumber = 0;
  }
  else if (!DXExtractInteger(in[2], &inlinenumber)){
    DXSetError(ERROR_BAD_PARAMETER, "Linenumber must be an INT");
    goto error;
  }

  DXMessage("Got an input linenumber:  %d\n", inlinenumber);


 /* Main Body of Function */

 /* Create the output field and associated arrays */
 fIGRF = DXNewField();
 if (!fIGRF) {
    DXSetError(ERROR_NO_MEMORY,"Creation of IGRF field failed");
    goto error;
  }
  aIGRFData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
  if (!aIGRFData) {
    DXSetError(ERROR_NO_MEMORY,"Creation of IGRFData array failed");
    goto error;
  }
  if(!DXAddArrayData(aIGRFData,0,iInCoordItems,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of IGRF Data Array failed");
    goto error;
  }
  fpIGRFData = DXGetArrayData(aIGRFData);

  aIGRFCoord = DXNewArray(TInCoordType,CInCoordCat,iInCoordRank,iInCoordShape);
  if (!aIGRFCoord) {
    DXSetError(ERROR_NO_MEMORY,"Creation of IGRFCoord array failed");
    goto error;
  }
  if(!DXAddArrayData(aIGRFCoord,0,iInCoordItems,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of IGRF Coord Array failed");
    goto error;
  }
  fpIGRFCoord = DXGetArrayData(aIGRFCoord);

  if(!DXSetStringAttribute((Object)aIGRFData,"dep","positions")) { 
     DXSetError(ERROR_INTERNAL,"Couldn't set IGRFData Array Dep");
     goto error;
  }

  if (!DXSetComponentValue(fIGRF, "positions", (Object)aIGRFCoord)) {
     DXSetError(ERROR_INTERNAL,"Couldn't set IGRF Position Array");
     goto error;
  }

  if (!DXSetComponentValue(fIGRF, "data", (Object)aIGRFData)) {
     DXSetError(ERROR_INTERNAL,"Couldn't set IGRF Data Array");
     goto error;
  }

  if (!DXSetComponentValue(fIGRF, "connections", (Object)aInConnect)) {
     DXSetError(ERROR_INTERNAL,"Couldn't set IGRF Connections Array");
     goto error;
  }

 

 /* Do the IGRF Computation at the Input Grid Locations */
 for(i=0;i<iInCoordItems;i++) {
   fpIGRFB = fpIGRFData + 3*i;  /* Pass to IGRF as 3 elment array*/
   fpIGRFLoc = fpIGRFCoord + 3*i; /* Pass to IGRF as 3 element array*/
   fpInLoc = fpInCoord + 3*i;
   for(j=0;j<3;j++) {
     fpIGRFLoc[j] = fpInLoc[j];
   }
   fX = fpIGRFLoc[0];
   fY = fpIGRFLoc[1];
   fZ = fpIGRFLoc[2];

   recalc_(&SimTime[0], &SimTime[1], &SimTime[2], &SimTime[3], &SimTime[4]);

   igrf_gsm__(&fX,&fY,&fZ,&fIGRFBx,&fIGRFBy,&fIGRFBz);

   if (debug &  (i< 25)) DXMessage("%g %g %g %g %g %g",fX,fY,fZ,fIGRFBx,fIGRFBy,fIGRFBz);
   /* fDipBx = 0.0 ; fDipBy = 0.0 ; fDipBz = 0.0;*/
   fpIGRFB[0] = fIGRFBx;
   fpIGRFB[1] = fIGRFBy;
   fpIGRFB[2] = fIGRFBz;
  
   /* DXMessage("BX BY BZ %g %g %g",fpIGRFB[0],fpIGRFB[1],fpIGRFB[2]);*/
 }

 DXEndField(fIGRF);

/*  DXMessage("Just before passing out %d\n", outlinenumber); */

 /* now need to DX make it into something DX can read */
 returnlinenumber = DXMakeInteger(outlinenumber-1);

 out[0] = (Object) fIGRF;   /* send back the IGRF field  */
 out[1] = (Object) returnlinenumber;  /* send back linenumber where the MHDTime matched  */

/*  DXMessage("passed out 2 things %d\n", outlinenumber); */

 return OK;

/*  DXMessage("After return"); */

error:
  DXDelete((Object) fIGRF); 
  DXDelete((Object) returnlinenumber); 
  DXDelete((Object) aIGRFData); 
  DXDelete((Object) aIGRFCoord); 
 return ERROR;
}



float lininterp(float x1, float y1, float x2, float y2, float x){
  
  float m, b,y;

  m = (y2-y1)/(x2-x1);  /* compute slope  */
  b = y1-(m*x1);        /* compute intercept  */

  y=(m*x)+b;           /* compute interped value */
  return(y);
}
