#include <stdio.h>
#include <dx/dx.h>

#define MAX_NAME_SIZE 256
#define BUFFER 256

float* timeinterpolate(int*, int*, float*, float*, int, int*, int, int*);
float lininterp(float, float, float, float, float);

int debug=0;   /* this is the global flag to turn on/off debugging messages.  */

/* This module implements the T01 model onto a set of points defined by the user
 * This can then be used for comparison with the MHD field coming from the LFM.
 *
 * *****NEW VERSION***** 8/4/03 by TBG. 
 * The new feature of this version has the ability (requirement) to read in a 
 * 'T01.input' file, which has 8 columns of [yyyy doy hh.hhhh swp Dst By Bz DipoleAngle] 
 * for all valid MHD times in a run.  The routine then takes the MHD time (out of yrdayhr) 
 * and interpolates through the input file to find the correct inputs to drive the T01 
 * fortran subroutine.  The input file's SW properties must be propogated to the nose of 
 * the msphere, and must have NO headers.  
 * 
 * Input file units:
 *  yyyy      - integer 4 digit year
 *  doy       - integer day of year
 *  hh.hhhh   - decimal hour
 *  swp       - Solar Wind pressure (rho*v^2) in nP
 *  dst       - nT
 *  By        - nT
 *  Bz        - nT
 *  DipAngle  - radians
 *
 * This module has these inputs
 *    In[0] = t01 input parameter filename (from FileSelector)
 *    In[1] = MHD time (from yrdayhr)
 *    In[2] = Mesh to measure field on
 *    In[3] = last 'linenumber' of input file that matched the MHD Time
 *
 * and these outputs
 *    Out[0] = T01 Field - field float 3 vector
 *    Out[1] = 'linenumber' of the input file that matched MHD Time
 *                 Use SetLocal/GetLocal pair to manage the linenumbers in Dx
 */

Error
m_t01(Object *in, Object *out)
{
  /* Varible declarations */

  /* Inputs */
  Type TInDataType, TInCoordType;
  Category CInDataCat, CInCoordCat;
  int iInDataItems, iInCoordItems, iInDataRank, iInCoordRank;
  int iInDataShape, iInCoordShape;
  Array aInData, aInCoord, aInConnect;
  float *fpInData, *fpInCoord ;

  FILE *FpOpen;    /* to read in input file */
  char *filename;
  char caLine[BUFFER],caWord[BUFFER];  
  int iCount, *ipYear, *ipDoy, inlinenumber, outlinenumber;
  Array returnlinenumber;
  float *fpInputs;    
  float *fpHr;
  int InputColumns = 7;  /* defines the size of the array that holds the inputs */

  Array simtime_array;
  int  *SimTime = NULL;  /* pointer to the first element of SimTime read in from yrdayhr */
  float SimHr;  /* decimal hour */

  float *t01inputs;  

  /* Output */
  Array aT01Data = NULL; 	/* Array for T01 Magnetic Field*/
  float *fpT01Data;		/* Ptr to T01 Mag Field data*/
  Array aT01Coord = NULL;		/* Array for T01 Grid Pts*/
  float *fpT01Coord;		/* Ptr to T01 Mag Field Pts*/
  Field fT01 = NULL;		/* T01 Field*/

  /* T01 Stuff */
  float faParmod[10], fDipAng;
  float *fpT01Loc, *fpT01B, *fpInLoc;
  float fT01Bx,fT01By,fT01Bz;
  float fDipBx,fDipBy,fDipBz;
  float fX,fY,fZ;
  int iOpt = 0;
  int i,j;

  /* read in filename from DX */
  if (!in[0])
    DXMessage("Cant find input file!");
  else {
    DXExtractString(in[0],&filename);
  }
  
  FpOpen = fopen(filename,"r");   /* open file for reading - to get iCount*/
  iCount = 0;
  while(fgets(caLine,BUFFER,FpOpen) != NULL) iCount++; 
  DXMessage("Reading in %s with %d Elements\n",filename, iCount); 
  fclose(FpOpen);

  /* Now have iCount, Allocate space for data arrays */     
  ipYear = (int*)malloc(sizeof(int)*iCount);
  ipDoy = (int*)malloc(sizeof(int)*iCount);
  fpHr = (float*)malloc(sizeof(float)*iCount);

  fpInputs = (float*)malloc(sizeof(float)*iCount*InputColumns);

  FpOpen = fopen(filename,"r");   /* now open file to read in data */

  for(i=0;i<iCount;i++) {

    fscanf(FpOpen,"%d   %d   %f   %f   %f   %f   %f   %f   %f   %f",ipYear+i, 
	   ipDoy+i, fpHr+i, fpInputs+(i*InputColumns), 
	   fpInputs+(i*InputColumns)+1, fpInputs+(i*InputColumns)+2, 
	   fpInputs+(i*InputColumns)+3, fpInputs+(i*InputColumns)+4,
	   fpInputs+(i*InputColumns)+5, fpInputs+(i*InputColumns)+6);
  

    if (debug && i<100) {
      DXMessage("checking...%d   %d   %f   %f   %f   %f   %f   %f   %f   %f\n", 
		*(ipYear+i),*(ipDoy+i), 
		*(fpHr+i), *(fpInputs+(i*InputColumns)), 
		*(fpInputs+(i*InputColumns)+1), *(fpInputs+(i*InputColumns)+2), 
		*(fpInputs+(i*InputColumns)+3), *(fpInputs+(i*InputColumns)+4),
		*(fpInputs+(i*InputColumns)+5), *(fpInputs+(i*InputColumns)+6));
    }
  }
  fclose(FpOpen);

  /* Read in MHD time */
  if (!in[1]) {
    DXSetError(ERROR_BAD_PARAMETER, "Missing UT Time from yrdayhr");
    goto error;
  }
  else {
    simtime_array = (Array) in[1];   /* get the UT time array */
    SimTime = DXGetArrayData(simtime_array);  /* put it into SimTime */
  }

  if (debug) DXMessage("Read in SimTime is : %d %d %d %d %d\n", *(SimTime+0),*(SimTime+1),
			 *(SimTime+2),*(SimTime+3),*(SimTime+4));

  /* Process the Input field of locations */
  if (!in[2]) {
    DXSetError(ERROR_BAD_PARAMETER,"Missing Input field");
    goto error;
  } else {
    aInCoord = (Array)DXGetComponentValue((Field)in[2],"positions");
    if (!aInCoord) {
       DXSetError(ERROR_INTERNAL,"No input coord field");
       goto error;
    }
    fpInCoord =  DXGetArrayData(aInCoord);
    DXGetArrayInfo(aInCoord,&iInCoordItems,&TInCoordType,&CInCoordCat,
                   &iInCoordRank,&iInCoordShape);
    DXMessage("Coord Items %d Rank %d Shape %d",iInCoordItems,iInCoordRank,
              iInCoordShape);
    aInConnect = (Array)DXGetComponentValue((Field)in[2],"connections");
    if (!aInConnect) {
       DXSetError(ERROR_INTERNAL,"No input connections field");
       goto error;
    }
 }

  /* now read in the last 'linenumber' which matched the MHD time */
  if (!in[3]) {
    DXMessage("Didn't find an input 'linenumber', starting from the beginning of input file");
    inlinenumber = 0;
  }
  else if (!DXExtractInteger(in[3], &inlinenumber)){
    DXSetError(ERROR_BAD_PARAMETER, "Linenumber must be an INT");
    goto error;
  }

  DXMessage("Got an input linenumber:  %d\n", inlinenumber);

 DXMessage("T01: Got Inputs");
 
 /* Now need to check the validity of the input file, & call the subroutine */
 /* which will do the interpolation                                         */
 if (debug) {DXMessage("The size of the Inputs arrays is :  %d\n",InputColumns);
 }
  /* convert MHDtime to yyyy doy hh.hhhh */
  SimHr = SimTime[2] + (SimTime[3]/60.0) + (SimTime[4]/3600.0);
  if (debug) DXMessage("SimHr is now %f\n", SimHr);

  if (debug) {
    DXMessage("Input file first year    : %d\n", *ipYear);
    DXMessage("Input file first doy     : %d\n", *ipDoy);
    DXMessage("Input file first hh.hhhh : %f\n", *fpHr);
  }
  /* make sure the MHDTime is within the bounds of the input file */  
  if ((*ipYear!=SimTime[0]) || ((*ipDoy>=SimTime[1]) 
      && (*fpHr>= SimHr))) {
    DXSetError(ERROR_BAD_PARAMETER, 
	       "MHD Time is BEFORE all valid input file times");
    goto error;
  }   
  if (debug) {
    DXMessage("Input file last year    : %d\n", *(ipYear+(iCount-1)));
    DXMessage("Input file last doy     : %d\n", *(ipDoy+(iCount-1)));
    DXMessage("Input file last hh.hhhh : %f\n", *(fpHr+(iCount-1)));
  }
  if ((*(ipYear+(iCount-1))!=SimTime[0]) 
	   || ((*(ipDoy+(iCount-1))<=SimTime[1]) 
	   && (*(fpHr+(iCount-1))<= SimHr))) {
    DXSetError(ERROR_BAD_PARAMETER, 
	       "MHD Time is AFTER all valid input file times");
    goto error;
  }   /* if no error, the MHD time is a valid time in the input file */
 
  /* now call the subroutine which will interpolate the t01 input parameters  */

  t01inputs = timeinterpolate(ipYear, ipDoy, fpHr, fpInputs, iCount, SimTime, inlinenumber, &outlinenumber);
  if (debug) {
    DXMessage("Interpolated T01 input values : %f %f %f %f %f %f %f\n", *t01inputs, 
	    *(t01inputs+1), *(t01inputs+2), *(t01inputs+3), *(t01inputs+4), *(t01inputs+5), 
	      *(t01inputs+6));
    DXMessage("In main(), outlinenumber is : %d\n", outlinenumber);
  }
  /* now set the new values of the t01 inputs into the faParmod array */
  faParmod[0] = *t01inputs;
  faParmod[1] = *(t01inputs+1);
  faParmod[2] = *(t01inputs+2);
  faParmod[3] = *(t01inputs+3);
  faParmod[4] = *(t01inputs+5);
  faParmod[5] = *(t01inputs+6);
  fDipAng = *(t01inputs+4);

  DXMessage("T01: Using SWP= %f nT, Dst=%f nT, By=%f nT, Bz=%f nT, DipAng=%f rad G1=%f, G2=%f, as inputs to T01\n",(float)faParmod[0],(float)faParmod[1],(float)faParmod[2],(float)faParmod[3],(float)fDipAng, (float)faParmod[4], (float)faParmod[5]);

 /* Main Body of Function */

 /* Create the output field and associated arrays */
 fT01 = DXNewField();
 if (!fT01) {
    DXSetError(ERROR_NO_MEMORY,"Creation of T01 field failed");
    goto error;
  }
  aT01Data = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
  if (!aT01Data) {
    DXSetError(ERROR_NO_MEMORY,"Creation of T01Data array failed");
    goto error;
  }
  if(!DXAddArrayData(aT01Data,0,iInCoordItems,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of T01 Data Array failed");
    goto error;
  }
  fpT01Data = DXGetArrayData(aT01Data);

  aT01Coord = DXNewArray(TInCoordType,CInCoordCat,iInCoordRank,iInCoordShape);
  if (!aT01Coord) {
    DXSetError(ERROR_NO_MEMORY,"Creation of T01Coord array failed");
    goto error;
  }
  if(!DXAddArrayData(aT01Coord,0,iInCoordItems,NULL)) {
    DXSetError(ERROR_NO_MEMORY,"Allocation of T01 Coord Array failed");
    goto error;
  }
  fpT01Coord = DXGetArrayData(aT01Coord);

  if(!DXSetStringAttribute((Object)aT01Data,"dep","positions")) { 
     DXSetError(ERROR_INTERNAL,"Couldn't set T01Data Array Dep");
     goto error;
  }

  if (!DXSetComponentValue(fT01, "positions", (Object)aT01Coord)) {
     DXSetError(ERROR_INTERNAL,"Couldn't set T01 Position Array");
     goto error;
  }

  if (!DXSetComponentValue(fT01, "data", (Object)aT01Data)) {
     DXSetError(ERROR_INTERNAL,"Couldn't set T01 Data Array");
     goto error;
  }

  if (!DXSetComponentValue(fT01, "connections", (Object)aInConnect)) {
     DXSetError(ERROR_INTERNAL,"Couldn't set T01 Connections Array");
     goto error;
  }

 /* Do the T01 Computation at the Input Grid Locations */
 for(i=0;i<iInCoordItems;i++) {
   fpT01B = fpT01Data + 3*i;  /* Pass to T01 as 3 elment array*/
   fpT01Loc = fpT01Coord + 3*i; /* Pass to T01 as 3 element array*/
   fpInLoc = fpInCoord + 3*i;
   for(j=0;j<3;j++) {
     fpT01Loc[j] = fpInLoc[j];
   }
   fX = fpT01Loc[0];
   fY = fpT01Loc[1];
   fZ = fpT01Loc[2];
   /* need to figure out how to call t01  */
   t01_01__(&iOpt,faParmod,&fDipAng,&fX,&fY,&fZ,
        &fT01Bx,&fT01By,&fT01Bz);
/*    t96_(&iOpt,faParmod,&fDipAng,&fX,&fY,&fZ, */
/*         &fT01Bx,&fT01By,&fT01Bz); */
   /* fT01Bx = 0.0 ; fT01By = 0.0 ; fT01Bz = 0.0;*/
   if (debug &  (i< 25)) DXMessage("%g %g %g %g %g %g",fX,fY,fZ,fT01Bx,fT01By,fT01Bz);
   dip_(&fDipAng,&fX,&fY,&fZ,
        &fDipBx,&fDipBy,&fDipBz);
   /* fDipBx = 0.0 ; fDipBy = 0.0 ; fDipBz = 0.0;*/
   fpT01B[0] = fDipBx+fT01Bx;
   fpT01B[1] = fDipBy+fT01By;
   fpT01B[2] = fDipBz+fT01Bz;

   /* DXMessage("BX BY BZ %g %g %g",fpT01B[0],fpT01B[1],fpT01B[2]);*/
 }

 DXEndField(fT01);

/*  DXMessage("Just before passing out %d\n", outlinenumber); */

 /* now need to DX make it into something DX can read */
 returnlinenumber = DXMakeInteger(outlinenumber-1);

 out[0] = (Object) fT01;   /* send back the T01 field  */
 out[1] = (Object) returnlinenumber;  /* send back linenumber where the MHDTime matched  */

/*  DXMessage("passed out 2 things %d\n", outlinenumber); */

 return OK;

/*  DXMessage("After return"); */

error:
  DXDelete((Object) fT01); 
  DXDelete((Object) returnlinenumber); 
  DXDelete((Object) aT01Data); 
  DXDelete((Object) aT01Coord); 
 return ERROR;
}


float* timeinterpolate(int *ipYear, int *ipDoy, float *fpHr, float *fpInputs, 
		       int Count, int *ipSimTime, int inlinenumber, int *outlinenumber){
  
  int i, linenumber;
  int TimeColumns = 3, InputColumns = 7;
  float fSimHr, fstarttime, fendtime;
  double *dpStart, *dpEnd;
  float *fpStartInputs, *fpEndInputs, *fpInterpdInputs;

  /* convert SimTime to SimHr here */
  fSimHr = *(ipSimTime+2) + (*(ipSimTime+3)/60.0) + (*(ipSimTime+4)/3600.0); 
 
    /* now search through the time array for the right 2 entries straddling */
  /* the MHD Time   */

  for (i=inlinenumber; i<Count; i++) {   /* this starts from linenumber, and if */
    if(*(ipYear+i) == *ipSimTime) {    /* it's not given, it's set to zero */
      if (*(ipDoy+i) == *(ipSimTime+1)) {
	if (*(fpHr+i) < fSimHr) {
	  *outlinenumber = i;   /* this is the linenumber at the record BEFORE  */
	}	           /* the MHD time */
/* 	else if (*(fpHr+i) > fSimHr){ */
/* 	  *outlinenumber = *outlinenumber+1;  /*only true (maybe) for first record of day */
/* 	} */
      }
    }
  }


if (debug) DXMessage("The outlinenumber found is : %d\n", *outlinenumber);

  fstarttime = *(fpHr+(*outlinenumber));
  fendtime = *(fpHr+(*outlinenumber+1));

  if (debug) DXMessage("The bounding times are : %f, %f\n", fstarttime, fendtime);

  fpStartInputs = (float*)malloc(sizeof(float)*InputColumns);
  fpEndInputs = (float*)malloc(sizeof(float)*InputColumns);
  fpInterpdInputs = (float*)malloc(sizeof(float)*InputColumns);

  if (debug) DXMessage("Start, End, SimHr are :  %f  %f  %f\n",fstarttime,fendtime, fSimHr);

  for (i=0; i<InputColumns; i++) {
    *(fpStartInputs+i) = *(fpInputs+((*outlinenumber)*InputColumns)+i);
    *(fpEndInputs+i) = *(fpInputs+((*outlinenumber+1)*InputColumns)+i);   

    /* this is what's actually doing the interpolating  */
    *(fpInterpdInputs+i) = lininterp(fstarttime, *(fpStartInputs+i),fendtime, 
				     *(fpEndInputs+i), fSimHr);
    
    if (debug) {DXMessage("Start, End, Interpd values are: %d:  %f  %f  %f\n",i,
			  *(fpStartInputs+i), *(fpEndInputs+i),*(fpInterpdInputs+i));
    }
  }


  return(fpInterpdInputs);
}

float lininterp(float x1, float y1, float x2, float y2, float x){
  
  float m, b,y;

  m = (y2-y1)/(x2-x1);  /* compute slope  */
  b = y1-(m*x1);        /* compute intercept  */

  y=(m*x)+b;           /* compute interped value */
  return(y);
}
