#include "netcdf.h"
#include <dx/dx.h>
#include <stdio.h>
#include <math.h>

/* ---------------------------------------------------------------------------
 * readtiegcm
 *
 * INPUTS
 *   filename: Name of file with data
 *   timestep: timestep in data file to read
 *  
 * OUTPUTS
 *   ion: field containing ionospheric data
 *   ut: Vector containing ut array [yr mon day hr min sec]
 */
int GetTIEGCMVar(char *cpName,int iStep,int iFileId, 
                 int iLev,int iLat,int iLon,int iNumVars,
                 int iLoc,float *fpData);
void MonthDay(int, int, int *, int *);
#define ijk(i,j,k) ((k)+iLon*((j)+iLat*(i)))
#define cijk(c,i,j,k) ((c)+3*((k)+(iLon+1)*((j)+iLat*(i))))
#define vijk(c,i,j,k) ((c)+iNumVars*((k)+(iLon+1)*((j)+iLat*(i))))

Error m_ReadTIEGCM(Object *in, Object *out) 
{

  /* Variable Declerations */
  /* Inputs */
  char *cpFileName = NULL; /*Ptr to filename */
  int iStep;               /*Timestep to read */
  char *cpHeight = NULL;   /* Ptr to Height */
  
  /* Outputs */
  Array aGrid = NULL;           /* Array for Grid locations */
  float *fpGrid;                /* Ptr to complete Grid  location data */
  Array aData = NULL;        /* Array for MHD Field data */
  float *fpData;             /* Ptr to MHD data */
  Array aConnect = NULL;     /* Array containing conncetions for MHD field */
  int *ipConnect;            /* Ptr to MHD Connection data */
  Field fIon = NULL;            /* MHD data Field */
  double *dpLev,*dpLon,*dpLat;   /* Temp space for coordinate reads */
  int iLev, iLon, iLat, iSteps;         /* Size of coord dims */
  double *dpTmp;                 /* Variable Data */
  Array aUT = NULL;              /* UT Array */
  int *ipUT;                     /* Ptr to UT array Data */
  
  /* Other varibles */
  int iStatus, iDimId, iVarId, iFileId; /* netCDF vars */
  int iaEdge3d[3], iaStart[2],iaCount[2];  /* netCDF vars */
  int iReadFlag; /* Flag for mallocs completed */
  int iRdPts, iNumPts; /* Size of arrays */
  int h,i,j,k;
  int iNumVars; /* Num of varibles in output field */
  int iaMTime[3]; /* Array containing (d h m) of output */
  int *ipYear; /* Year of output */
  int iMon, iDay;
  float fDeg2Rad, fRe, fReInv;
  
  
   
  /* Get input parameters */
  if(!in[0]) {
    DXSetError(ERROR_BAD_PARAMETER,"Missing Data Filename");
    goto error;
  } else if (!DXExtractString(in[0],&cpFileName)) {
    DXSetError(ERROR_BAD_PARAMETER,"Data Filename must be string");
    goto error;
  }
  if(!in[1]) {
    DXSetError(ERROR_BAD_PARAMETER,"Missing Timestep");
    goto error;
  } else if (!DXExtractInteger(in[1],&iStep)) {
    DXSetError(ERROR_BAD_PARAMETER,"Step must be integer");
    goto error;
  }

  if(!in[2]) {
    DXSetError(ERROR_BAD_PARAMETER,"Missing Height");
    goto error;
  } else if (!DXExtractString(in[2],&cpHeight)) {
    DXSetError(ERROR_BAD_PARAMETER,"Height must be string");
    goto error;
  }
  iReadFlag = 0;
  iStatus = nc_open(cpFileName,NC_NOWRITE,&iFileId);
  if(iStatus != NC_NOERR) {
     DXSetError(ERROR_BAD_PARAMETER,"Unable to open netCDF file %s",cpFileName);
     goto error;
  }
  
  /* Check to see if requested step is valid */
  iStatus = nc_inq_dimid(iFileId,"time",&iDimId);
  iStatus = nc_inq_dimlen(iFileId,iDimId,&iSteps);
  if ((iStep < 0) || (iStep >= iSteps)) {
     DXSetError(ERROR_BAD_PARAMETER,
       "Requested step, %d, not valid.  Max Steps %d",iStep,iSteps-1);
     goto error;
  }
    
  /* Get UT from for step */
  iaStart[0] = iStep ; iaStart[1] = 0;
  iaCount[0] = 1; iaCount[1] = 3;
  iStatus = nc_inq_varid(iFileId,"mtime",&iVarId);
  iStatus = nc_get_vara_int(iFileId, iVarId, iaStart, iaCount, iaMTime);
  iStatus = nc_inq_varid(iFileId,"year",&iVarId);
  ipYear = (int*)malloc(sizeof(int)*iSteps);
  iStatus = nc_get_var_int(iFileId,iVarId,ipYear);
  MonthDay(ipYear[iStep],iaMTime[0],&iMon,&iDay);
  aUT = DXNewArray(TYPE_INT,CATEGORY_REAL,1,6);
  DXAddArrayData(aUT, 0, 1, NULL);
  ipUT = (int *)DXGetArrayData(aUT);
  ipUT[0]= ipYear[iStep];  /* Year */
  ipUT[1] = iMon;  /* Mon */
  ipUT[2] = iDay;  /* Day */
  ipUT[3] = iaMTime[1];  /* Hr */
  ipUT[4] = iaMTime[2];  /* Min */
  ipUT[5] = 0;  /* Sec */

  /* Get coordinate arrays and sizes */
  iStatus = nc_inq_dimid(iFileId,"lev",&iDimId);
  iStatus = nc_inq_dimlen(iFileId,iDimId,&iLev);
  dpLev = (double*)malloc(sizeof(double)*iLev);
  iStatus = nc_inq_varid(iFileId,"lev",&iVarId);
  iStatus = nc_get_var_double(iFileId,iVarId,dpLev);
  
  iStatus = nc_inq_dimid(iFileId,"lon",&iDimId);
  iStatus = nc_inq_dimlen(iFileId,iDimId,&iLon);
  dpLon = (double*)malloc(sizeof(double)*iLon);
  iStatus = nc_inq_varid(iFileId,"lon",&iVarId);
  iStatus = nc_get_var_double(iFileId,iVarId,dpLon);

  iStatus = nc_inq_dimid(iFileId,"lat",&iDimId);
  iStatus = nc_inq_dimlen(iFileId,iDimId,&iLat);
  dpLat = (double*)malloc(sizeof(double)*iLat);
  iStatus = nc_inq_varid(iFileId,"lat",&iVarId);
  iStatus = nc_get_var_double(iFileId,iVarId,dpLat);

  iReadFlag = 1;
 
#ifdef DEBUG
  DXMessage("After coord read %d %d %d %d",iSteps,iLev,iLon,iLat);
#endif

  iaEdge3d[0]=iLev;
  iaEdge3d[1]=iLat; 
  iaEdge3d[2]=iLon+1;  /* For peroidic Phi direction */
  iRdPts = iLev*iLat*iLon;
  iNumPts = iLev*iLat*(iLon+1);  /* For peroidic Phi direction */

 
  iNumVars = 16;
  aData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,iNumVars);
  DXAddArrayData(aData,0,iNumPts,NULL);
  fpData = (float*)DXGetArrayData(aData);

  iStatus = GetTIEGCMVar("TN",iStep,iFileId,iLev,iLat,iLon,iNumVars,0,fpData);
  iStatus = GetTIEGCMVar("UN",iStep,iFileId,iLev,iLat,iLon,iNumVars,1,fpData);
  iStatus = GetTIEGCMVar("VN",iStep,iFileId,iLev,iLat,iLon,iNumVars,2,fpData);
  iStatus = GetTIEGCMVar("O2",iStep,iFileId,iLev,iLat,iLon,iNumVars,3,fpData);
  iStatus = GetTIEGCMVar("O1",iStep,iFileId,iLev,iLat,iLon,iNumVars,4,fpData);
  iStatus = GetTIEGCMVar("N4S",iStep,iFileId,iLev,iLat,iLon,iNumVars,5,fpData);
  iStatus = GetTIEGCMVar("NO",iStep,iFileId,iLev,iLat,iLon,iNumVars,6,fpData);
  iStatus = GetTIEGCMVar("OP",iStep,iFileId,iLev,iLat,iLon,iNumVars,7,fpData);
  iStatus = GetTIEGCMVar("N2D",iStep,iFileId,iLev,iLat,iLon,iNumVars,8,fpData);
  iStatus = GetTIEGCMVar("TI",iStep,iFileId,iLev,iLat,iLon,iNumVars,9,fpData);
  iStatus = GetTIEGCMVar("TE",iStep,iFileId,iLev,iLat,iLon,iNumVars,10,fpData);
  iStatus = GetTIEGCMVar("NE",iStep,iFileId,iLev,iLat,iLon,iNumVars,11,fpData);
  iStatus = GetTIEGCMVar("O2P",iStep,iFileId,iLev,iLat,iLon,iNumVars,12,fpData);
  iStatus = GetTIEGCMVar("W",iStep,iFileId,iLev,iLat,iLon,iNumVars,13,fpData);
  iStatus = GetTIEGCMVar("Z",iStep,iFileId,iLev,iLat,iLon,iNumVars,14,fpData);
  iStatus = GetTIEGCMVar("POTEN",iStep,iFileId,iLev,iLat,iLon,iNumVars,15,
                         fpData);

  /* Do the peroidic copy thing */
  for(j=0;j<iLat;j++) {
    for(i=0;i<iLev;i++) {
      for(h=0;h<iNumVars;h++) {
        fpData[vijk(h,i,j,iLon)] = fpData[vijk(h,i,j,0)];  
      }
    }
  }

  aGrid = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
  DXAddArrayData(aGrid,0,iNumPts,NULL);
  fpGrid = (float*)DXGetArrayData(aGrid);
  fDeg2Rad = M_PI/180.0;
  fRe = 6371.2e5; 
  fReInv = 1.0/fRe;
  /* Use Geopotential Height as radius in calculation */
  if (!strcmp(cpHeight,"Re")) {
    for(i=0;i<iLev;i++) {
      for(j=0;j<iLat;j++) {
        for(k=0;k<iLon;k++) {
          fpGrid[cijk(0,i,j,k)] = (fpData[vijk(14,i,j,k)]+fRe)*
                                  cos(dpLat[j]*fDeg2Rad)*
                                  cos(dpLon[k]*fDeg2Rad)*fReInv;
          fpGrid[cijk(1,i,j,k)] = (fpData[vijk(14,i,j,k)]+fRe)*
                                  cos(dpLat[j]*fDeg2Rad)*
                                  sin(dpLon[k]*fDeg2Rad)*fReInv;
          fpGrid[cijk(2,i,j,k)] = (fpData[vijk(14,i,j,k)]+fRe)*
                                  sin(dpLat[j]*fDeg2Rad)*fReInv;
        }
      }
    }
  } else {
    for(i=0;i<iLev;i++) {
      for(j=0;j<iLat;j++) {
        for(k=0;k<iLon;k++) {
          fpGrid[cijk(0,i,j,k)] = fpData[vijk(14,i,j,k)]*cos(dpLat[j]*fDeg2Rad)*
                                  cos(dpLon[k]*fDeg2Rad)*1.0e-5;
          fpGrid[cijk(1,i,j,k)] = fpData[vijk(14,i,j,k)]*cos(dpLat[j]*fDeg2Rad)*
                                  sin(dpLon[k]*fDeg2Rad)*1.0e-5;
          fpGrid[cijk(2,i,j,k)] = fpData[vijk(14,i,j,k)]*
                                  sin(dpLat[j]*fDeg2Rad)*1.0e-5;
        }
      }
    }
  }
    
  
  /* Since Lon is peroidic we copy 0 into last point to make vis appear
 * continous when displayed.
 */

  for(j=0;j<iLat;j++) {
    for(i=0;i<iLev;i++) {
      fpGrid[cijk(0,i,j,iLon)] = fpGrid[cijk(0,i,j,0)];
      fpGrid[cijk(1,i,j,iLon)] = fpGrid[cijk(1,i,j,0)];
      fpGrid[cijk(2,i,j,iLon)] = fpGrid[cijk(2,i,j,0)];
    }
  }
  fIon = DXNewField();
  if(!fIon) {
    DXSetError(ERROR_NO_MEMORY,"Creation of Ion field failed");
    goto error;
  }
 
  /* Set the dependence of the Data Array to be on positons*/
  if(!DXSetStringAttribute((Object)aData,"dep","positions")) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array Dep");
     goto error;
  }
  /* Set the MHD Data array as the data component of the Ion Field*/
  if(!DXSetComponentValue(fIon,"data",(Object)aData)) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array of Ion Field");
     goto error;
  }
  /* Set the MHD Pts array as the position component of the Ion Field*/
  if(!DXSetComponentValue(fIon,"positions",(Object)aGrid)) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Position Array of Ion Field");
     goto error;
  }
  aConnect = DXMakeGridConnectionsV(3,iaEdge3d);
  if (!aConnect) {
    DXSetError(ERROR_INTERNAL,"Couldn't Make Ion Grid Connections");
    goto error;
  }

  if (!DXSetComponentValue(fIon,"connections",(Object)aConnect)) {
      DXSetError(ERROR_INTERNAL,"Couldn't set Ion Grid Connections");
      goto error;
  }

  if(!DXEndField(fIon)) {
     DXSetError(ERROR_INTERNAL,"Couldn't complete field");
     goto error;
  }

#ifdef DEBUG
  DXMessage("After Field");
#endif
 
  
  out[0] = (Object) fIon;
  out[1] = (Object) aUT;
 
  free(dpLev);
  free(dpLon);
  free(dpLat);
  free(ipYear);
 
  return OK;

  error: 
    DXDelete((Object) fIon);
    DXDelete((Object) aData);
    DXDelete((Object) aGrid);
    DXDelete((Object) aConnect);
    if(1 == iReadFlag) {
      free(dpLev);
      free(dpLon);
      free(dpLat);
      free(dpTmp);
      free(ipYear);
    }
}

int GetTIEGCMVar(char *cpName,int iStep,int iFileId, 
                 int iLev,int iLat,int iLon,int iNumVars,
                 int iLoc,float *fpData)
{
  int i,j,k;
  int iRdPts,iaStart[4],iaCount[4],iVarId,iStatus;
  double *dpTmp;

  iRdPts = iLev*iLat*iLon;
  dpTmp = (double*)malloc(sizeof(double)*iRdPts);
  iaStart[0] = iStep; iaStart[1] = 0 ; iaStart[2] = 0; iaStart[3] = 0;
  iaCount[0] = 1; iaCount[1] = iLev; iaCount[2] = iLat; iaCount[3] = iLon;
  iStatus = nc_inq_varid(iFileId,cpName,&iVarId);
  iStatus = nc_get_vara_double(iFileId, iVarId, iaStart, iaCount, dpTmp);

  for(k=0;k<iLon;k++) {
    for(j=0;j<iLat;j++) {
      for(i=0;i<iLev;i++) {
        fpData[vijk(iLoc,i,j,k)] = dpTmp[ijk(i,j,k)];
      }
    }
  }
  free(dpTmp);
  return iStatus;
}

static int day_tab[2][13] = {
  {0,31,28,31,30,31,30,31,31,30,31,30,31},
  {0,31,29,31,30,31,30,31,31,30,31,30,31}
};

void MonthDay(int iYear, int iYearDay, int *ipMonth, int *ipMonthDay) {
   int i, iLeap;

   iLeap = iYear%4 == 0 && iYear%100 !=0 || iYear%400 ==0 ;
   for(i=1;iYearDay > day_tab[iLeap][i]; i++)
     iYearDay -= day_tab[iLeap][i];
   *ipMonth = i;
   *ipMonthDay = iYearDay;
}

