#include <dx/dx.h>
#include <stdio.h>
#include <math.h>
#include "fitsio.h"

/* --------------------------------------------------------------------------
 * read3dwsa
 *
 * This module reads the data files created by the WSA Solar Wind Model
 *
 * INPUTS
 *   filename: name of input file
 *
 * OUTPUTS
 *   mhd: field containing mhd data
 *
 */

#define cijk(c,i,j,k) ((c)+iNumVars*((i)+iNph*((j)+iNth*(k))))
#define ijk(i,j,k) ((i)+iNph*((j)+iNth*((k))))

void CreateXYZ(float*,float*,float*,float*,int,int,int,int,int,int);
Error m_Read3DWSA(Object *in, Object *out) 
{

  /* Varible Declarations */

  /* Inputs */
  char *cpFileName = NULL; 	/* Ptr to MHD Data file name */
  
  /* Outputs */

  Array aGrid = NULL; 		/* Array for Grid locations */
  float *fpGrid;		/* Ptr to complete Grid  location data */
  Array aMHDData = NULL;	/* Array for MHD Field data */
  float *fpMHDData;		/* Ptr to MHD data */ 
  Array aMHDConnect = NULL;	/* Array containing conncetions for MHD field */
  int *ipMHDConnect;		/* Ptr to MHD Connection data */
  Field fMHD = NULL;		/* MHD data Field */

  /* Other varibles */
  int i,j,k;			    /* loop indices */
  fitsfile *FpIn;		    /* Ptr to input file */
  float fDeg2Rad;		    /* Deg to Rad conversion factor */
  int iStatus,iHDUType,iBitPix,iNum; /* FITS I/O Variables */
  int iNullVal,iAnyNull,iFirst;      /* FITS I/O Variables */
  long laAxis[3];		     /* Image Size */
  float *fpBr,*fpBth,*fpBph;	     /* Temp space for data */
  float *fpR,*fpTh,*fpPh;	     /* Temp space for locs */
  int iSize,iNr,iNth,iNph; 	     /* Array sizes */
  char caKey[] = "RADIUS";
  char caBuf[256], *cpPtr;
  float fRadius;
  int iNumVars;
  int iNumCells,iKoff,iKoff2,iJoff,iJoff2,ip1;
  
  if(!in[0]) {
    DXSetError(ERROR_BAD_PARAMETER,"Missing File Name");
    goto error;
  } else if (!DXExtractString(in[0],&cpFileName)) {
    DXSetError(ERROR_BAD_PARAMETER,"Directory File must be string");
    goto error;
  }
 
  /* Open the FITS file and move to HDU 4 which contains Br */
  
  if(fits_open_file(&FpIn,cpFileName,READONLY,&iStatus)) {
    fits_report_error(stderr,iStatus);
    DXSetError(ERROR_BAD_PARAMETER,"Problem Openning FITS file");
  }
  if(fits_movabs_hdu(FpIn,4,&iHDUType,&iStatus)) {
    fits_report_error(stderr,iStatus);
    DXSetError(ERROR_BAD_PARAMETER,"Problem Accessing Br");
  }
  if(fits_get_img_param(FpIn,3,&iBitPix,&iNum,laAxis,&iStatus)) {
    fits_report_error(stderr,iStatus);
    DXSetError(ERROR_BAD_PARAMETER,"Problem Getting Array size");
  }
  /* Allocate space for arrays */
  iSize = laAxis[0]*laAxis[1]*laAxis[2];
  iNph = laAxis[0];
  iNth = laAxis[1];
  iNr = laAxis[2];
  DXMessage("Sizes %d %d %d",iNph,iNth,iNr);
  fpR = (float*)malloc(sizeof(float)*iNr);
  fpTh = (float*)malloc(sizeof(float)*iNth);
  fpPh = (float*)malloc(sizeof(float)*iNph);
  fpBr = (float*)malloc(sizeof(float)*iSize);
  fpBth = (float*)malloc(sizeof(float)*iSize);
  fpBph = (float*)malloc(sizeof(float)*iSize);
  /* Read Br */
  iFirst = 1;
  if(fits_read_img(FpIn,TFLOAT,iFirst,iSize,&iNullVal,fpBr,&iAnyNull,&iStatus)){
    fits_report_error(stderr,iStatus);
    DXSetError(ERROR_BAD_PARAMETER,"Problem Openning FITS file");
    goto error;
  }
  /* Read Bph */
  if(fits_movabs_hdu(FpIn,5,&iHDUType,&iStatus)) {
    fits_report_error(stderr,iStatus);
    DXSetError(ERROR_BAD_PARAMETER,"Problem Accessing Bph");
  }
  iFirst = 1;
  if(fits_read_img(FpIn,TFLOAT,iFirst,iSize,&iNullVal,fpBph,&iAnyNull,&iStatus)){
    fits_report_error(stderr,iStatus);
    DXSetError(ERROR_BAD_PARAMETER,"Problem Openning FITS file");
  }
  /* Read Bth */
  if(fits_movabs_hdu(FpIn,6,&iHDUType,&iStatus)) {
    fits_report_error(stderr,iStatus);
    DXSetError(ERROR_BAD_PARAMETER,"Problem Accessing Bph");
  }
  iFirst = 1;
  if(fits_read_img(FpIn,TFLOAT,iFirst,iSize,&iNullVal,fpBth,&iAnyNull,&iStatus)){
    fits_report_error(stderr,iStatus);
    DXSetError(ERROR_BAD_PARAMETER,"Problem Openning FITS file");
  }
  
  /* Make the positions array for the DXField */
  DXMessage("Doing Grid");
  aGrid = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,3);
  DXAddArrayData(aGrid,0,iSize,NULL);
  fpGrid = (float*)DXGetArrayData(aGrid);
  iNumVars=3;
  fDeg2Rad = M_PI/180.0;
  if(fits_movabs_hdu(FpIn,1,&iHDUType,&iStatus)) {
    fits_report_error(stderr,iStatus);
    DXSetError(ERROR_BAD_PARAMETER,"Problem Accessing File Information");
  }
  fits_read_key(FpIn,TFLOAT,caKey,&fRadius,caBuf,&iStatus);
  DXMessage("Outer Radius %g",fRadius);
  for(k=0;k<iNr;k++) fpR[k] = 1.0 + ((fRadius-1.0)/(iNr-1))*k; 
  for(j=0;j<iNth;j++) fpTh[j] = (180.0-2.5)*fDeg2Rad-180.0/iNth*fDeg2Rad*j;
  //for(j=0;j<iNth;j++) fpTh[j] = 180.0/iNth*fDeg2Rad*j;
  for(i=0;i<iNph;i++) fpPh[i] = 360.0/iNph*fDeg2Rad*i;
  for(k=0;k<iNr;k++) {
    for(j=0;j<iNth;j++) {
      for(i=0;i<iNph;i++) {
        fpGrid[cijk(0,i,j,k)] = fpR[k]*sin(fpTh[j])*cos(fpPh[i]);
        fpGrid[cijk(1,i,j,k)] = fpR[k]*sin(fpTh[j])*sin(fpPh[i]);
        fpGrid[cijk(2,i,j,k)] = fpR[k]*cos(fpTh[j]);
        //fpGrid[cijk(0,i,j,k)] = fpPh[i];
        //fpGrid[cijk(1,i,j,k)] = fpTh[j];
        //fpGrid[cijk(2,i,j,k)] = fpR[k];
      }
    }
  }


  /* Create the data array for the DX field */

  DXMessage("Doing Data");
  iNumVars = 7;
  aMHDData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,iNumVars);
  DXAddArrayData(aMHDData,0,iSize,NULL);
  fpMHDData = (float*)DXGetArrayData(aMHDData);

  /* Copy the data to the field 
   * 
   * 0 - Br
   * 1 - Btheta
   * 2 - Bphi
   * 3 - Bx
   * 4 - By
   * 5 - Bz
   * 6 - R
   */
  for(k=0;k<iNr;k++) {
    for(j=0;j<iNth;j++) {
      for(i=0;i<iNph;i++) {
         fpMHDData[cijk(0,i,j,k)] = fpBr[ijk(i,j,k)];
         fpMHDData[cijk(1,i,j,k)] = fpBth[ijk(i,j,k)];
         fpMHDData[cijk(2,i,j,k)] = fpBph[ijk(i,j,k)];
         fpMHDData[cijk(6,i,j,k)] = fpR[k];
      }
    }
  }
  CreateXYZ(fpMHDData,fpR,fpTh,fpPh,0,3,iNumVars,iNph,iNth,iNr);

  /* Do the final steps for the DXField */
  DXMessage("Doing Field");
  free(fpR); free(fpTh); free(fpPh);
  free(fpBr); free(fpBth); free(fpBph);
  fMHD = DXNewField();
  if(!fMHD) {
    DXSetError(ERROR_NO_MEMORY,"Creation of MHD field failed");
    goto error;
  }

  /* Set the dependence of the Data Array to be on positons*/
  if(!DXSetStringAttribute((Object)aMHDData,"dep","positions")) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array Dep");
     goto error;
  }

  /* Set the MHD Data array as the data component of the MHD Field*/
  if(!DXSetComponentValue(fMHD,"data",(Object)aMHDData)) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array of MHD Field");
     goto error;
  }

  /* Set the MHD Pts array as the position component of the MHD Field*/
  if(!DXSetComponentValue(fMHD,"positions",(Object)aGrid)) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Position Array of MHD Field");
     goto error;
  }
  
  /* This code makes regular connections */
/*
  aMHDConnect = DXMakeGridConnections(3,iNr,iNth,iNph);

  if (!aMHDConnect) {
    DXSetError(ERROR_INTERNAL,"Could Make MHD Grid Connections");
    goto error;
  }

  if (!DXSetComponentValue(fMHD,"connections",(Object)aMHDConnect)) {
      DXSetError(ERROR_INTERNAL,"Could set MHD Grid Connections");
      goto error;
  }
*/
  
  DXMessage("Doing connections");
  iNumVars = 8;
  iNumCells = (iNr-1)*(iNth-1)*(iNph);
#ifdef DEBUG
  DXMessage("NumVars %d, NumPts %d",iNumVars,iNumCells);
#endif
  aMHDConnect = DXNewArray(TYPE_INT,CATEGORY_REAL,1,iNumVars);
  if (!aMHDConnect) {
    DXSetError(ERROR_INTERNAL,"Could Make MHD Grid Connections");
    goto error;
  }
  DXAddArrayData(aMHDConnect,0,iNumCells,NULL);
  ipMHDConnect = (int*)DXGetArrayData(aMHDConnect);
//  for(k=0;k<iNr-1;k++) {
//     iKoff = iNph*iNth*k; /* Offset for back lower left corner */
//     iKoff2 = iKoff+iNph*iNth; /* Offset for back lower right corner */
//    for(j=0;j<iNth-1;j++) {
//     iJoff = iNph*j; /* Offset for front lower left corner */
//     iJoff2 = iJoff+iNph; /* Offset for front lower right corner */
//      for(i=0;i<iNph;i++) {
//       ip1 = i+1;
//       if (iNr-1 == i) ip1 = 0; /* Use start as right for last set of cubes */
//       *ipMHDConnect = i + iJoff + iKoff;
//       *(ipMHDConnect+1) = ip1 + iJoff + iKoff;
//       *(ipMHDConnect+2) = i + iJoff2 + iKoff;
//       *(ipMHDConnect+3) = ip1 + iJoff2 + iKoff;
//       *(ipMHDConnect+4) = (i + iJoff + iKoff2)%iSize;
//       *(ipMHDConnect+5) = (ip1 + iJoff + iKoff2)%iSize;
//       *(ipMHDConnect+6) = (i + iJoff2 + iKoff2)%iSize;
//       *(ipMHDConnect+7) = (ip1 + iJoff2 + iKoff2)%iSize;
//#ifdef DEBUG
//       if(k < 1)
//         DXMessage("%d %d %d %d %d %d %d %d",
//         *(ipMHDConnect+0),*(ipMHDConnect+1),*(ipMHDConnect+2),
//         *(ipMHDConnect+3),*(ipMHDConnect+4),*(ipMHDConnect+5),
//         *(ipMHDConnect+6),*(ipMHDConnect+7));
//#endif
//       ipMHDConnect +=8;
//      }
//    }
//  }  /* close loop over cell corners */
  if(!DXSetConnections(fMHD,"cubes",(Array)aMHDConnect)) {
    DXSetError(ERROR_INTERNAL,"Couldn't set connections");
    goto error;
  }
//  if (!DXSetComponentValue(fMHD,"connections",(Object)aMHDConnect)) {
//      DXSetError(ERROR_INTERNAL,"Could set MHD Grid Connections");
//      goto error;
//  }
  if(!DXEndField(fMHD)) {
     cpPtr = DXGetErrorMessage();
     strcpy(caBuf,cpPtr);
     DXMessage("Error %s",caBuf);
     DXSetError(ERROR_INTERNAL,"Couldn't complete field - mjw");
     goto error;
  }

  DXMessage("Doing finish");
  out[0] = (Object) fMHD;
  /* Cleanup temporary space */

  return OK;

  error:
    DXMessage("Doing error");
    DXDelete((Object)fMHD);
//    DXDelete((Object)aMHDConnect);
//    DXDelete((Object)aMHDData);
//    DXDelete((Object)aGrid);
  return ERROR;
}

void CreateXYZ(float *fpVar, float *fpR, float *fpTh, float *fpPhi,
               int iFrom, int iTo,
               int iNumVars,int iNph,int iNth,int iNr)
{
   int i,j,k;

   for(k=0;k<iNr;k++) {
     for(j=0;j<iNth;j++) {
       for(i=0;i<iNph;i++) {
         /* For Br use r = i sin(Th)cos(Phi)+j sin(Th)sin(Phi)+k cos(Th) */
         fpVar[cijk(iTo+0,i,j,k)] =
            fpVar[cijk(iFrom+0,i,j,k)]*sin(fpTh[j])*cos(fpPhi[i]);
         fpVar[cijk(iTo+1,i,j,k)] =
            fpVar[cijk(iFrom+0,i,j,k)]*sin(fpTh[j])*sin(fpPhi[i]);
         fpVar[cijk(iTo+2,i,j,k)] =
            fpVar[cijk(iFrom+0,i,j,k)]*cos(fpTh[j]);
         /* For Bth use r = i cos(Th)cos(Phi)+j cos(Th)sin(Phi)-k sin(Th) */
         fpVar[cijk(iTo+0,i,j,k)] +=
            fpVar[cijk(iFrom+1,i,j,k)]*cos(fpTh[j])*cos(fpPhi[i]);
         fpVar[cijk(iTo+1,i,j,k)] +=
            fpVar[cijk(iFrom+1,i,j,k)]*cos(fpTh[j])*sin(fpPhi[i]);
         fpVar[cijk(iTo+2,i,j,k)] +=
            -1.0*fpVar[cijk(iFrom+1,i,j,k)]*sin(fpTh[j]);
         /* For Bphi use r = -i sin(Phi)+j cos(Phi) */
         fpVar[cijk(iTo+0,i,j,k)] +=
            -1.0*fpVar[cijk(iFrom+2,i,j,k)]*sin(fpPhi[i]);
         fpVar[cijk(iTo+1,i,j,k)] +=
            fpVar[cijk(iFrom+2,i,j,k)]*cos(fpPhi[i]);
       }
     }
   }

   return;
}

 
