#include <dx/dx.h>
#include <stdio.h>
#include <math.h>

/* --------------------------------------------------------------------------
 * readwsa
 *
 * This module reads the data files created by the WSA Solar Wind Model
 *
 * INPUTS
 *   filename: name of input file
 *
 * OUTPUTS
 *   mhd: field containing mhd data
 *
 */

#define MAXLINE 256
#define fij(f,i,j) ((f)+iNumVars*((j)+iNLongp1*((i))))

Error m_ReadWSA(Object *in, Object *out) 

{

  /* Varible Declarations */

  /* Inputs */
  char *cpFileName = NULL; 	/* Ptr to MHD Data file name */
  
  /* Outputs */

  Array aGrid = NULL; 		/* Array for Grid locations */
  float *fpGrid;		/* Ptr to complete Grid  location data */
  Array aMHDData = NULL;	/* Array for MHD Field data */
  float *fpMHDData;		/* Ptr to MHD data */ 
  Array aMHDConnect = NULL;	/* Array containing conncetions for MHD field */
  int *ipMHDConnect;		/* Ptr to MHD Connection data */
  Field fMHD = NULL;		/* MHD data Field */

  /* Other varibles */
  int i,j,k;			    /* loop indices */
  int iNLong,iNLongp1,iNRad;	    /* Array Sizes  */
  int iNumPts,iNumVars;	  	    /* Usefule Sizes */
  float fTime, fCarNum, fCarLong, fBAngle; /* Carrington rotation info */
  float fDum;			    /* Temp for reading data */
  char caDum[MAXLINE]; 		    /* Temp array for reading lines */
  FILE *FpIn;			    /* Ptr to input file */
  float fDeg2Rad;		    /* Deg to Rad conversion factor */
  
  if(!in[0]) {
    DXSetError(ERROR_BAD_PARAMETER,"Missing File Name");
    goto error;
  } else if (!DXExtractString(in[0],&cpFileName)) {
    DXSetError(ERROR_BAD_PARAMETER,"Directory File must be string");
    goto error;
  }

 /* WSA has 5 degree angular resolution and 14 Rs radial resolution.  The files
  * have a fixed number of grid points (72 in phi and 15 in radius)  The
  * longitude varies most rapidly starting along the Earth-Sun line.  The
  * Carrington rotation Longitude can be added to the phase to keep the location
  * of the fixed and move into heliographic coordinates.  Each file has the
  * entire velocity field followed by the magnetic polarity field.
  */

  iNLong = 72;
  iNLongp1 = iNLong+1;
  iNRad = 17;
  iNumVars = 2;
#ifdef DEBUG
  DXMessage("Openning file %s",cpFileName);
#endif
  FpIn = fopen(cpFileName,"r");
  if (NULL == FpIn) {
    DXSetError(ERROR_BAD_PARAMETER,"File %s does not exist",cpFileName);
    goto error;
  }
  fgets(caDum,MAXLINE,FpIn);
  DXMessage("Read %s",caDum);
  fscanf(FpIn,"%f %f %f %f",&fTime,&fCarNum,&fCarLong,&fBAngle);
  DXMessage("Read %f %f %f %f",fTime,fCarNum,fCarLong,fBAngle);
   
  /* Make the positions array for the DXField */
  iNumPts=(iNLong+1)*iNRad;  /* Fake peroidic by making one longer */
  aGrid = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,2);
  DXAddArrayData(aGrid,0,iNumPts,NULL);
  fpGrid = (float*)DXGetArrayData(aGrid);
  iNumVars=2;
  fDeg2Rad = M_PI/180.0;
  for(j=0;j<iNLong;j++) {
    for(i=0;i<iNRad;i++) {
        fpGrid[fij(0,i,j)] = (5+i*14)*sin(j*5*fDeg2Rad);
        fpGrid[fij(1,i,j)] = (5+i*14)*cos(j*5*fDeg2Rad);
        /*  Just a quick ordering test
        fpGrid[fij(0,i,j)] = 5+i*14;
        fpGrid[fij(1,i,j)] = j*5;
        */
    }
  }

  /* Since Phi is peroid copy 0 into last point to make via appear continous */

  for(i=0;i<iNRad;i++) {
        fpGrid[fij(0,i,iNLong)] = fpGrid[fij(0,i,0)];
        fpGrid[fij(1,i,iNLong)] = fpGrid[fij(1,i,0)];
  }
    

  /* Create the data array for the DX field */

  iNumVars = 2;
  aMHDData = DXNewArray(TYPE_FLOAT,CATEGORY_REAL,1,iNumVars);
  DXAddArrayData(aMHDData,0,iNumPts,NULL);
  fpMHDData = (float*)DXGetArrayData(aMHDData);

  /* Read the data from the file */
  for(i=0;i<iNRad;i++) {
    for(j=0;j<iNLong;j++) {
        fscanf(FpIn,"%f",&fDum);
        fpMHDData[fij(0,i,j)] = fDum;
    }
  }
  for(i=0;i<iNRad;i++) {
    for(j=0;j<iNLong;j++) {
        fscanf(FpIn,"%f",&fDum);
        fpMHDData[fij(1,i,j)] = fDum;
    }
  }

  /* Since Phi is peroid copy 0 into last point to make via appear continous */

  for(i=0;i<iNRad;i++) {
        fpMHDData[fij(0,i,iNLong)] = fpMHDData[fij(0,i,0)];
        fpMHDData[fij(1,i,iNLong)] = fpMHDData[fij(1,i,0)];
  }
  /* Do the final steps for the DXField */
  fMHD = DXNewField();
  if(!fMHD) {
    DXSetError(ERROR_NO_MEMORY,"Creation of MHD field failed");
    goto error;
  }

  /* Set the dependence of the Data Array to be on positons*/
  if(!DXSetStringAttribute((Object)aMHDData,"dep","positions")) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array Dep");
     goto error;
  }

  /* Set the MHD Data array as the data component of the MHD Field*/
  if(!DXSetComponentValue(fMHD,"data",(Object)aMHDData)) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Data Array of MHD Field");
     goto error;
  }

  /* Set the MHD Pts array as the position component of the MHD Field*/
  if(!DXSetComponentValue(fMHD,"positions",(Object)aGrid)) {
     DXSetError(ERROR_INTERNAL,"Couldn't set Position Array of MHD Field");
     goto error;
  }
  
  /* This code makes regular connections */
  aMHDConnect = DXMakeGridConnections(2,iNRad,iNLongp1);

  if (!aMHDConnect) {
    DXSetError(ERROR_INTERNAL,"Could Make MHD Grid Connections");
    goto error;
  }

  if (!DXSetComponentValue(fMHD,"connections",(Object)aMHDConnect)) {
      DXSetError(ERROR_INTERNAL,"Could set MHD Grid Connections");
      goto error;
  }
  
  if(!DXEndField(fMHD)) {
     DXSetError(ERROR_INTERNAL,"Couldn't complete field");
     goto error;
  }

  out[0] = (Object) fMHD;
  /* Cleanup temporary space */

  return OK;

  error:
    DXDelete((Object) fMHD);
    DXDelete((Object) aMHDData);
    DXDelete((Object) aGrid);
    DXDelete((Object) aMHDConnect);
  return ERROR;
}

 
