function parse_swepam_data()
%PARSE_SWEPAM_DATA 
%
%   PARSE_SWEPAM_DATA Reads in file all_data_1m.dat created by
%   cat_swepam_data.pl and outputs daily averaged solar wind 
%   velocity in a file named YYYY_MM_DD_HH_MM_SS_V.dat with
%   columns of Year, Month, Day, Hour, Minute, Second V.
%
%   This function is called by GET_REAL_TIME_DATA.
%
%   See also GET_REAL_TIME_DATA.
  
% R.S. Weigel 05/14/2004

CISM_DX_GLOBALS;
 
% Load the old data file. 
fid = fopen('./YYYY_MM_DD_HH_MM_SS_V.dat','r');
if (fid == -1)
  old_data_exists = 0;
else
  old_data_exists = 1;
  fclose(fid);
  load('./YYYY_MM_DD_HH_MM_SS_V.dat');
  old_data = YYYY_MM_DD_HH_MM_SS_V;
end

% Load the most recent data file; this step takes the longest
load('./all_data_1m.dat');

% Grab every 120th data point to speed up the formatted write execution time.
I           = [1:120:size(all_data_1m,1)]';
all_data_1m = all_data_1m(I,:);     

% Sort the data.
DN          = day_number(all_data_1m(:,1:3));
[junk,I]    = sort(DN);
all_data_1m = all_data_1m(I,:);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Convert time in this file to standard format
YYYY = all_data_1m(:,1);
MO   = all_data_1m(:,2);
DD   = all_data_1m(:,3);
HH   = floor(all_data_1m(:,4)/100);
MM   = all_data_1m(:,4)-100*HH;
SS   = 59*ones(size(all_data_1m,1),1);
Vsw  = all_data_1m(:,9);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create data file with standard time stamps.
fid     = fopen('./YYYY_MM_DD_HH_MM_SS_V.dat','w');
output  = [YYYY,MO,DD,HH,MM,SS,Vsw];
          fprintf(fid,'%4d %2d %2d %2d %2d %2d %8.2f\n',output');
          fclose(fid);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create a file of daily averages
TODAY      = ymd(now);
DAY_F      = day_number(TODAY);
DAY_I      = day_number(output(1,1),output(1,2),output(1,3));
DAYS_AVAIL = day_number(output(:,1),output(:,2),output(:,3));
DAYS_OUT   = [DAY_I:DAY_F];

% This is the slow loop that needs to be vectorized!
for i = 1:length(DAYS_OUT)
  I       = find(DAYS_AVAIL == DAYS_OUT(i));
  if ~isempty(I)
    output(I,7)= reflag(output(I,7),-9999.9,CISM_DX_FLAG);
    V(i,1)     = mean_nonflag(output(I,7),1,CISM_DX_FLAG);  
    Date(i,:)  = output(I(end),1:6);
  else
    V(i,1)     = CISM_DX_FLAG;
    temp       = ymd(DAYS_OUT(i));
    Date(i,:)  = [temp,12,1,1];
  end
end  
output = [Date V];


fid = fopen('./YYYY_MM_DD_HH_MM_SS_V.daily.dat','w');
for i = 1:size(output,1)
  if (i < size(output,1))
    fprintf(fid,'%4d %2d %2d %2d %2d %2d %8.2f\n',...
	  output(i,1),output(i,2),output(i,3),...
	  23,59,59,output(i,7));
  else
    fprintf(fid,'%4d %2d %2d %2d %2d %2d %8.2f\n',...
	  output(i,1),output(i,2),output(i,3),...
	  output(i,4),output(i,5),output(i,6),output(i,7));
  end
end
fclose(fid);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create archive of raw data in standard form
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (is_octave)
  load -force ./YYYY_MM_DD_HH_MM_SS_V.dat;
else
  load ./YYYY_MM_DD_HH_MM_SS_V.dat;
end

if (old_data_exists == 1)
  wsopred   = [old_data;YYYY_MM_DD_HH_MM_SS_V];
else
  wsopred   = YYYY_MM_DD_HH_MM_SS_V;
end

wsopred   = unique(wsopred,'rows');

fid = fopen('./YYYY_MM_DD_HH_MM_SS_V.dat.archive','w');
      fprintf(fid,'%4d %2d %2d %2d %2d %2d %8.2f\n',wsopred');
      fclose(fid);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
