function x = shift(x,b)
%SHIFT Circular shift of elements of vector or matrix.
% 
%   If x is a vector, perform a circular shift of length b of
%   the elements of x.
%
%   If x is a matrix, do the same for each column of x.
%

% Copyright (C) 1995, 1996  Kurt Hornik
%
% This file is part of Octave.
%
% Octave is free software; you can redistribute it and/or modify it
% under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
%
% Octave is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with Octave; see the file COPYING.  If not, write to the Free
% Software Foundation, 59 Temple Place - Suite 330, Boston, MA
% 02111-1307, USA.

% -*- texinfo -*-
% @deftypefn {Function File} {} shift (@var{x}, @var{b})
% If @var{x} is a vector, perform a circular shift of length @var{b} of
% the elements of @var{x}.
%
% If @var{x} is a matrix, do the same for each column of @var{x}.
% @end deftypefn

% Author: AW <Andreas.Weingessel@ci.tuwien.ac.at>
% Created: 14 September 1994
% Adapted-By: jwe
% Adapted-For Matlab: rsw, 11/22/2005.

function y = shift (x, b)

  if (nargin != 2)
    usage(shift (X, b)");
  endif

  [nr, nc] = size (x);

  if (nr == 0 || nc == 0)
    error ("shift: x must not be empty");
  elseif (nr == 1)
    x = x.';
    nr = nc;
    nc = 0;
  endif

  if (! (isscalar (b) && b == round (b)))
    error ("shift: b must be an integer");
  endif

  elo = empty_list_elements_ok;

  unwind_protect

    empty_list_elements_ok = 1;

    if (b >= 0)
      b = rem (b, nr);
      t1 = x (nr-b+1:nr, :);
      t2 = x (1:nr-b, :);
      y = [t1; t2];
    elseif (b < 0)
      b = rem (abs (b), nr);
      t1 = x (b+1:nr, :);
      t2 = x (1:b, :);
      y = [t1; t2];
    endif

  unwind_protect_cleanup

    empty_list_elements_ok = elo;

  end_unwind_protect

  if (nc == 0)
    y = reshape (y, 1, nr);
  endif

endfunction
