function Pd = diff_nonflag(A,DIM,FLAG,EXACT)
%MEAN_NONFLAG Average of elements 
%
%   Adiff = diff_nonflag(A) for 1-D array is the first difference between
%   elements not equal to FLAG = NaN.  Adiff(end) is set equal
%   to FLAG so that Adiff has the same size as A.
%
%   Adiff = diff_nonflag(A,DIM) is differrence of elements not equal to NaN.
%           along dimension DIM.
%
%   Adiff = diff_nonflag(A,DIM,FLAG) is difference of elements not equal to FLAG.
%
%   Adiff = diff_nonzero(A,DIM,FLAG,EXACT) If EXACT = 0, returns the difference
%           between elements less than FLAG.  If EXACT = 1, returns the
%           difference between elements not equal to flag.  If EXACT = 2, 
%           returns the difference between elements with absolute value 
%           less than FLAG.  Default is EXACT = 1.
%
%   If all elements along dimension DIM are equal to FLAG, a fill of FLAG is
%   used.
%
%   Maximum DIM is 2.  (DIFF_NONFLAG is not N-dimensional like function
%   DIFF.)
%
%   See also DIFF, *_NONFLAG.

% Need to vectorize for loops.
% R.S. Weigel, 04/02/2004.  
  
if (nargin < 2)
  if (min(size(A) > 1))
    error('DIM must be specified');
  else
    [junk,DIM] = max(size(A));
  end
end

N = 1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
if (nargin < 3)
  FLAG     = NaN;
end
if (nargin < 4)
  EXACT = 1;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
if (prod(size(A)) == 1)
  Pd = FLAG;  % If one element, return FILL
  return;
end

if ( isempty(A) )
  Pd = [];  % If empty, return empty
  return;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

if (size(A,1)==1)
  DIM = 2;
end
if (size(A,2)==1)
  DIM = 1;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
if (DIM == 1)
  for i = 1:size(A,2)
    if (EXACT == 0)
      I = find(A(:,i) > FLAG);
    end
    if (EXACT == 1)
      I = find(A(:,i) == FLAG);
    end
    if (EXACT == 2)
      I = find(abs(A(:,i)) > FLAG);
    end
    if (isnan(FLAG))
      I = find(isnan(A(:,i)) == 1);
    end
 
    temp = diff(A(:,i),N);    
    temp = [temp ; FLAG*ones(N,1)];

    if (~isempty(I))
      if (I(1) == 1)
	temp(1)   = FLAG;
	if (length(I) > 1)
	  I       = I(2:end);
	end
      end
      if (I(end) == length(temp))
	temp(end) = FLAG;
	if (length(I) > 1)
	  I       = I(1:end-1);
	end
      end
      temp(I)    = FLAG;
      if (length(I)>1)
	temp(I-1)  = FLAG;
      end
    end
    Pd(:,i) = temp;
  end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
if (DIM == 2)
  for i = 1:size(A,1)
    if (EXACT == 0)
      I = find(A(i,:) > FLAG);
    end
    if (EXACT == 1)
      I = find(A(i,:) == FLAG);
    end
    if (EXACT == 2)
      I = find(abs(A(i,:)) > FLAG);
    end
    if (isnan(FLAG))
      I = find(isnan(A(i,:)) == 1);
    end
 
    temp = diff(A(i,:),N);    
    temp = [temp , FLAG*ones(N,1)];

    if (~isempty(I))
      if (I(1) == 1)
	temp(1)   = FLAG;
	if (length(I) > 1)
	  I       = I(2:end);
	end
      end
      if (I(end) == length(temp))
	temp(end) = FLAG;
	if (length(I) > 1)
	  I       = I(1:end-1);
	end
      end
      temp(I)    = FLAG;
      if (length(I)>1)
	temp(I-1)  = FLAG;
      end
    end
    Pd(i,:) = temp;
  end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

