function Pd = max_nonflag(A,DIM,FLAG,EXACT)
%MAX_NONFLAG Largest non-FLAG component.
%
%   Amax = max_nonflag(A) for 1-D array is the max of elements not equal to
%   NaN.  
%  
%   Amax = max_nonflag(A,DIM) is max of elements not equal to NaN.
%
%   Amax = max_nonflag(A,DIM,FLAG) is max of elements not equal to FLAG.
%
%   Amax = max_nonzero(A,DIM,FLAG,EXACT) If EXACT = 0, returns max of
%   elements less than FLAG.  EXACT = 1, returns max of elements not equal
%   to flag.  EXACT = 2, returns max of elements with absolute value less
%   than FLAG.
%
%   If all elements along dimension DIM are equal to FLAG, a fill of FLAG is
%   used.
%
%   Maximum DIM is 3.  If three dimensional matrix, DIM must be 3.
%   (MAX_NONFLAG is not N-dimensional like function MAX.)
%
%   See also MAX, *_NONFLAG.

% Need to vectorize for loops.
% R.S. Weigel, 04/02/2004.  

if (nargin < 2)
  if (min(size(A) > 1))
    error('DIM must be specified');
  else
    [junk,DIM] = max(size(A));
  end
end
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
if (nargin < 3)
  FLAG  = NaN;
end
if (nargin < 4)
  EXACT = 1;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

if (prod(size(A)) == 1)
  Pd = FLAG;  % If one element and is zero, return FLAG.
  return;
end

if ( isempty(A) )
  Pd = [];  % If empty, return empty.
  return;
end

if (DIM == 1)
  Pd = repmat(FLAG,1,size(A,2));  
  for i = 1:size(A,2)
      I = [];
    if (EXACT == 0)
      I = find(A(:,i) < FLAG);
    end
    if (EXACT == 1)
      I = find(A(:,i) ~= FLAG);
    end
    if (EXACT == 2)
      I = find(abs(A(:,i)) < FLAG);
    end
    if (isnan(FLAG))
      I = find(isnan(A(:,i)) == 0);
    end

    if (~isempty(I))
      Pd(1,i) = max(A(I,i));
    end
  end
end

if (DIM == 2)
  Pd = repmat(FLAG,size(A,1),1);
  for i = 1:size(A,1)
      I = [];
    if (EXACT == 0)
      I = find(A(i,:) < FLAG);
    end
    if (EXACT == 1)
      I = find(A(i,:) ~= FLAG);
    end
    if (EXACT == 2)
      I = find(abs(A(i,:)) < FLAG);
    end
    if (isnan(FLAG))
      I = find(isnan(A(i,:)) == 0);
    end


    if (~isempty(I))
      Pd(i,1) = max(A(i,I));
    end
  end
end

if (DIM == 3)
  Pd = repmat(FLAG,[size(A,1) size(A,2) 1]);
  for i = 1:size(A,1)
    for j = 1:size(A,2)      
        I = [];
      if (EXACT == 0)
	I = find(A(i,j,:) < FLAG);
      end
      if (EXACT == 1)
	I = find(A(i,j,:) ~= FLAG);
      end
      if (EXACT == 2)
	I = find(abs(A(i,j,:)) < FLAG);
      end
      if (isnan(FLAG))
	I = find(isnan(A(i,j,:)) == 0);
      end

      
      if (~isempty(I))
	Pd(i,j,1) = max(A(i,j,I));
      end
    end
  end
end

