function MSE = mse_nonflag(y,f,FLAG,EXACT)
%MSE_NONFLAG  Mean square error between non-FLAG elements of actual and predicted vector.
%
% 
%   MSE(A,P) returns (1/N)*sum( (A-P).^2 )./(std(A,1))^2;
%
%   MSE is the mean square error scaled to the variance of the
%   actual vector, A.  It can be interpreted as the fraction of the
%   variance of A that is not predicted by P.
%
%   For matrices, MSE(A,P) returns a column vector with elements containing
%   the MSE between the respective columns of A and P.
%
%   MSE_NONFLAG(A,P,FLAG,EXACT) If EXACT = 0, returns mse of elements less
%   than FLAG.  EXACT = 1, returns mse of elements not equal to flag.  EXACT
%   = 2, returns mse of elements with absolute value less than FLAG.
%
%   If all elements along dimension DIM are equal to FLAG, a fill of FLAG is
%   used.
%   
%   See also MSE, *_NONFLAG.

% R.S. Weigel, 04/02/2004.
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
if (nargin < 3)
  FLAG     = NaN;
end
if (nargin < 4)
  EXACT = 1;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

if (prod(size(y)) ~= prod(size(f)))
  error('Inputs must be the same size');
end
  
if (size(y,1) == 1)
  y = y';
  f = f';
  if (length(y) < 2)
    error('Inputs must have 2 or more elements');
  end
end

if (nargin < 3)
  MSE = mse(y,f);
end

if (nargin >= 3)
  for i = 1:size(y,2)
      Iy = []; If = [];
    if (EXACT == 0)
      Iy = find(y(:,i) < FLAG);
      If = find(f(:,i) < FLAG);
    end
    if (EXACT == 1)
      Iy = find(y(:,i) ~= FLAG);
      If = find(f(:,i) ~= FLAG);
    end
    if (EXACT == 2)
      Iy = find(abs(y(:,i)) < FLAG);
      If = find(abs(f(:,i)) < FLAG);
    end
    if (isnan(FLAG))
      Iy = find(isnan(y(:,i)) == 0);
      If = find(isnan(f(:,i)) == 0);
    end
    
    I  = intersect(Iy,If);
    if (~isempty(I) | length(I) > 1)
      MSE(1,i) = sum((y(I,i) - f(I,i)).^2)/length(I);
    else
      MSE(1,i) = NaN;      
    end
  end
end


