function [L,dL,dl] = arc_length(x,method,dL1)
%ARC_LENGTH Estimate of the length of a curve
%
%   [L,dL,dl] = ARC_LENGTH(x)
%   [L,dL,dl] = ARC_LENGTH(x,method)
%
%   L is the total cumulative arc length, dL is the vector tanget to the arc
%   and dl is the magnitude of dL.  Default method is 1.
%
%   For method = 1, three assumptions are made: (1) the elements are
%   straight, (2) the length of the first and second elements are the same,
%   and (3) the length of element i is given by the magnitude of the
%   difference between the vectors that connect the *center* of elements i+1
%   and i (and centers are located at values given in x).
%
%   For method = 2, the first two assumptions in method 1 are the same but
%   the third assumption is not needed.  This method should generally be
%   more accurate if dL1 is given.  Requires a for loop, however.
%
%   [L,dL,dl] = ARC_LENGTH(x,method,dL1)
%
%   If dL1 is given for either methods, it is used as the dL(1,:).
%
%   See also ARC_LENGTH_DEMO.  
  
if (nargin < 2)
  method = 1;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (method == 1)

  dL = x(2:end,:)-x(1:end-1,:);
  dl = ( sum(dL.^2,2) ).^0.5;
  dl = [dl(1) ; dl];
  
  if (nargin == 3)
    dL = [dL1 ; dL];  
  else
    dL = [dL(1,:) ; dL];  
  end

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (method == 2)

  if (nargin == 3)
    dL(1,:) = dL1;
  else
    dL(1,:) = x(2,:)-x(1,:);
  end

  for i = 2:size(x,1)
    dL(i,:) = 2*(x(i,:)-x(i-1,:))-dL(i-1,:);
  end

  dl = mag(dL);

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

L = cumsum(dl);