function Bo = biot_savart(x,IdL,xo,coordsys,use_shared,use_cartesian)
%BIOT_SAVART Evaluate Biot-Savart integral
%
%   Bo = biot_savart(x,I,xo) computes
%
%   Bo = sum_i^N ( IdL(i,:) cross (xo-x(i,:)) ) / ( mag(xo-x(i,:)) )^3
%
%   where IdL is the current magnitude multiplied by dL. Each row of x
%   represents the Cartesian position of the current element in the
%   corresponding row of IdL.
%
%   To put B in nT (if inputs in MKS units), multiply by mu_o/(4*pi).  Bo is
%   the integral evaluated at observation point xo.  Multiple observation
%   points may be given.  All values must be given in the same cartesian
%   coordinate system.
%
%   ________________________________________________________________________
%   Special geometry:
%
%   (This is an alternative to using TRANSFORM to put the inputs in cartesian
%   and the output into observer's spherical coordinate system.)
%
%   The currrent element vector (IdL) is given in a coordinate system
%   "Spherical2" that has its origin translated along the vector that
%   connects the element to the origin of a spherical system.
%
%   Bo = biot_savart(x,IdL,xo,coordsys) with coordsys = 1 and
%
%   x  = Pos of element in spherical coordinates.
%   xo = Pos of observer in spherical coordinates.
%   IdL = Current density vector in element's spherical coordinates.
%   Bo = Magnetic field in observer's (spherical) coordinates.
%
%   See Kisabeth and Rostoker, Geophys. J. R. astr. Soc. (1977) 49, 655-683.
%   ________________________________________________________________________
%
%   See also ARC_LENGTH, BIOT_SAVART_DEMO, B_RING, B_WIRE.

if (nargin < 6)
  use_cartesian = 1;
end
if (nargin < 5)
  use_shared = 1;
end
if (nargin < 4)
  coordsys = 0;
end

if (size(x,2)~=3) | (size(x,2)~=3) | (size(IdL,2)~=3)
  error('x, IdL, and xo must have three columns.');
end
if (size(x)~=size(IdL))
  error('The sizes x and IdL must be equal');
end
if (prod(size(xo))<3)
  error('xo must have three or more elements');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Matlab
if (exist('matlabpath')) & (use_shared == 1)
  if (exist('biot_savart_mex') == 3) 

    try
      Bo = biot_savart_mex(x',IdL',xo',coordsys)';
    catch % If .mex file exist but wrong version mex compile
      curr_dir = pwd;
      % Need to be in directory to compile
      tmpstr = which('libsp.c');
      eval(sprintf('cd %s',tmpstr(1:end-8))); 
      Makefile
      Bo = biot_savart_mex(x',IdL',xo',coordsys)';
      cd(curr_dir);
      Bo = biot_savart_mex(x',IdL',xo',coordsys)';
    end      
    return;
  
  else

    try
      curr_dir = pwd;
      % Need to be in directory to compile
      tmpstr = which('libsp.c');
      eval(sprintf('cd %s',tmpstr(1:end-8))); 
      Makefile
      Bo = biot_savart_mex(x',IdL',xo',coordsys)';
      cd(curr_dir);
      return;
    catch
      warning(['biot_savart: Compilation of shared library failed.'...
	       ' using script for computation instead of shared library']);
      Bo = biot_savart(x,IdL,xo,coordsys,0);
    end

  end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (use_shared == 0) 

  if (coordsys == 1) & (use_cartesian == 1)
    IdL = transform('Spherical2','Cartesian',IdL,x); % I now in Cartesian
    x   = transform('Spherical','Cartesian',x); % x now in Cartesian
    xoc = xo; % xoc is in Cartesian
    xo  = transform('Spherical','Cartesian',xoc); % xo is now in Cartesian
  end

  Bo = zeros(size(xo));  
  for i = 1:size(xo,1)
    Xo      = repmat(xo(i,:),size(x,1),1);
    X       = Xo - x;
    Rd      = (sum( X.^2 , 2)).^1.5;
    Bo(i,1) = sum( ( X(:,3).*IdL(:,2)-X(:,2).*IdL(:,3) )./Rd );
    Bo(i,2) = sum( ( X(:,1).*IdL(:,3)-X(:,3).*IdL(:,1) )./Rd );
    Bo(i,3) = sum( ( X(:,2).*IdL(:,1)-X(:,1).*IdL(:,2) )./Rd );
  end

  if (coordsys == 1) & (use_cartesian == 1)
    Bo  = transform('Cartesian','Spherical2',Bo,xoc); % Bo now in Spherical2
  end
  
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This method is orders of magnitude slower than use_cartesion = 1 method.
% because the transforms are done before and after the Biot-Savart
% computation.  The faster method requires of order size(x,1) + size(xo,1)
% transform operations whereas using doing the transforms in the loop
% requires of order size(x,1)*size(xo,1) transform operations.  Actual
% speedup for shared library is near zero when transforms are done before
% calling the mex function, but for script is orders of magnitude faster.
% The mex function could be sped up by doing the before/after transforms in
% the .c code.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (use_shared == 0) & (coordsys == 1) & (use_cartesian == 0)

  Bo = zeros(size(xo));  
  for i = 1:size(xo,1)
    for j = 1:size(x,1)
    
      r_o  = xo(i,1); th_o = xo(i,2); ph_o = xo(i,3);
      r    = x(j,1);  th   = x(j,2);  ph   = x(j,3);
      
      A(1,1) = sin(th_o)*sin(th)*cos(ph_o-ph) + cos(th_o)*cos(th);
      A(1,2) = sin(th_o)*cos(th)*cos(ph_o-ph) - cos(th_o)*sin(th);
      A(1,3) = sin(th_o)*sin(ph_o-ph);
      A(2,1) = cos(th_o)*sin(th)*cos(ph_o-ph) - sin(th_o)*cos(th);
      A(2,2) = cos(th_o)*cos(th)*cos(ph_o-ph) + sin(th_o)*sin(th);
      A(2,3) = cos(th_o)*sin(ph_o-ph);
      A(3,1) = -sin(th)*sin(ph_o-ph);
      A(3,2) = -cos(th)*sin(ph_o-ph);
      A(3,3) = cos(ph_o-ph);
      
      R  = ( r_o^2 + r^2 - 2*r*r_o*A(1,1) )^1.5;
      
      K(1,1) = 0;
      K(1,2) = r*A(1,3);
      K(1,3) = -r*A(1,2);
      K(2,1) = r_o*A(3,1);
      K(2,2) = r*A(2,3) + r_o*A(3,2);
      K(2,3) = r_o*A(3,3)-r*A(2,2);
      K(3,1) = -r_o*A(2,1);
      K(3,2) = r*A(3,3) - r_o*A(2,2);
      K(3,3) = -r_o*A(2,3) - r*A(3,2);

      K      = K/R;

      idL = IdL(j,:);

      bo(j,:) = [K*idL']';
    
    end % for j

    Bo(i,:) = sum(bo,1);

    if (size(xo,1) > 100) 
      if (mod(i,100)==1) | (i == 1)
	if (~exist('matlabpath'))
	  fprintf('%s',repmat('\b',1,80));
	else % Matlab prints literal \b for some reason ...
%	  fprintf(['\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b'...
%		   '\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b'...
%		   '\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b'...
%		   '\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b']);
	fprintf('\n');
	end
	  
	fprintf('biot_savart: Computing field at position %d of %d',...
		i,size(xo,1));fflush(stdout);
      end
    end
    
  end % for i
  if (size(xo,1) > 100) 
    fprintf('\n');fflush(stdout);
  end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
