#include "libsp.h"
#include <math.h>
#include <stdio.h>

/* x is an array with first Cartesian x,y,z position given by x[0],
   x[1], x[2], second cartesian x,y,z position given by x[3], x[4],
   x[5], etc. IdL, xo, and Bo are similarly ordered.
*/

int libsp_biot_savart(double *x,
		      double *IdL, int ncols_IdL,
		      double *xo, int ncols_xo,
		      double *Bo, int coordsys)
{

  int i,j,k=0,K;
  double tmpBo_x=0.0, tmpBo_y=0.0, tmpBo_z=0.0;
  double Xx,Xy,Xz,Rd;
  int *IdL_nonzero;

  IdL_nonzero = (int *) malloc(ncols_IdL*sizeof(int));
  if (IdL_nonzero == 0) {
    printf("libsp.c: virtual memory exhausted. Aborting");
    return 0;
  }

  if ( (pow(2.0,2.0) != 4.0) || (pow(4.0,0.5) != 2.0) ) {
    printf("libsp.c: in function libsp_biot_savart:");
    printf("Problem with power function. Aborting\n");
    return 0;
  }

  /* Find IdL segments that are zero.  They will be skipped in inner loops. */
  for (j=0;j<ncols_IdL;j++) {
    if ( (IdL[0+3*j] != 0.0) || (IdL[1+3*j] != 0.0) || (IdL[2+3*j] != 0.0)) {
      IdL_nonzero[k] = j;
      k = k+1;
    }
  }
  K = k;


  /*
  printf("libsp.c: coordsys = %d\n",coordsys);
  printf("libsp.c: Fraction of zero IdL values = %f\n",
	 1 - ( (double) K )/( (double) ncols_IdL ));
  printf("libsp.c: Nx x Nxo = %.1e\n",(float) ncols_IdL*ncols_xo);
  */

  /**************************************************************************/
  /* Cartesian */
  if (coordsys == 0) {

    for (i=0;i<ncols_xo;i++) {

      tmpBo_x = 0.0;
      tmpBo_y = 0.0;
      tmpBo_z = 0.0;
      
      for (k=0;k<K;k++) {

	j = IdL_nonzero[k];
	
	Xx = xo[0+3*i]-x[0+3*j];
	Xy = xo[1+3*i]-x[1+3*j];
	Xz = xo[2+3*i]-x[2+3*j];
	
	Rd = pow( Xx*Xx + Xy*Xy + Xz*Xz, 1.5);
	
	tmpBo_x = tmpBo_x + (Xz*IdL[1+3*j] - Xy*IdL[2+3*j])/Rd;
	tmpBo_y = tmpBo_y + (Xx*IdL[2+3*j] - Xz*IdL[0+3*j])/Rd;
	tmpBo_z = tmpBo_z + (Xy*IdL[0+3*j] - Xx*IdL[1+3*j])/Rd;
	
      }
      
      Bo[0+3*i] = tmpBo_x;
      Bo[1+3*i] = tmpBo_y;
      Bo[2+3*i] = tmpBo_z;
      
    }

  } /* end coordsys == 0 */
  /**************************************************************************/ 

  /**************************************************************************/ 
  /* Kisabeth and Rostoker, 1977. */
  if (coordsys == 1) {

    double A11,A12,A13,A21,A22,A23,A31,A32,A33;
    double K11,K12,K13,K21,K22,K23,K31,K32,K33;
    double r,th,ph,r_o,th_o,ph_o,R;
    
    for (i=0;i<ncols_xo;i++) {
      
      tmpBo_x = 0.0;
      tmpBo_y = 0.0;
      tmpBo_z = 0.0;

      for (k=0;k<K;k++) {

	j = IdL_nonzero[k];
	
	Xx = xo[0+3*i]-x[0+3*j];
	Xy = xo[1+3*i]-x[1+3*j];
	Xz = xo[2+3*i]-x[2+3*j];
	
	r_o  = xo[0+3*i]; th_o = xo[1+3*i]; ph_o = xo[2+3*i];
	r    = x[0+3*j];  th   = x[1+3*j];  ph = x[2+3*j];
	
	A11 = sin(th_o)*sin(th)*cos(ph_o-ph) + cos(th_o)*cos(th);
	A12 = sin(th_o)*cos(th)*cos(ph_o-ph) - cos(th_o)*sin(th);
	A13 = sin(th_o)*sin(ph_o-ph);
	A21 = cos(th_o)*sin(th)*cos(ph_o-ph) - sin(th_o)*cos(th);
	A22 = cos(th_o)*cos(th)*cos(ph_o-ph) + sin(th_o)*sin(th);
	A23 = cos(th_o)*sin(ph_o-ph);
	A31 = -sin(th)*sin(ph_o-ph);
	A32 = -cos(th)*sin(ph_o-ph);
	A33 = cos(ph_o-ph);
	
	R = pow( r_o*r_o + r*r - 2*r*r_o*A11 , 1.5 );
	
	K11 = 0;
	K12 = r*A13/R;
	K13 = -r*A12/R;
	K21 = r_o*A31/R;
	K22 = (r*A23 + r_o*A32)/R;
	K23 = (r_o*A33-r*A22)/R;
	K31 = -r_o*A21/R;
	K32 = (r*A33 - r_o*A22)/R;
	K33 = (-r_o*A23 - r*A32)/R;

	tmpBo_x = tmpBo_x + K11*IdL[0+3*j] + K12*IdL[1+3*j] + K13*IdL[2+3*j];
	tmpBo_y = tmpBo_y + K21*IdL[0+3*j] + K22*IdL[1+3*j] + K23*IdL[2+3*j];
	tmpBo_z = tmpBo_z + K31*IdL[0+3*j] + K32*IdL[1+3*j] + K33*IdL[2+3*j];
	
      }
      
      Bo[0+3*i] = tmpBo_x;
      Bo[1+3*i] = tmpBo_y;
      Bo[2+3*i] = tmpBo_z;
      
    }

  } /* end coordsys == 1 */
  /**************************************************************************/

  free(IdL_nonzero);

  return 1;


}
