function V_out = transform(inSys,outSys,V_in,X_in)
%TRANSFORM Vector and coordinate transforms
% 
%   V_out = TRANSFORM(inSys,outSys,V_in)
%
%   Where inSys and outSys can be 'Cartesian', 'Polar', 'Spherical'.
%
%   Other use:
%  
%   If Spherical2 is a coordinate system with origin at X_in = [r,theta,phi]
%   and unit vector [rhat,theta_hat,phi_hat], V_in is a vector in this
%   system, then
%  
%   V_out = TRANSFORM('Spherical2','Cartesian',V_in,X_in) is V_in in a
%   cartesian system.
%  
%   The inverse transform is
%   V_out = TRANSFORM('Cartesian','Spherical2',V_in,X_in) where V_in and
%   X_in are in Cartesian.
%   
%   See also TRANSFORM_DEMO.
  
Systems = {'Cartesian','Polar','Spherical','Spherical2'};

if ( isempty(strmatch(inSys,Systems)) )
  error('Coordinate system %s does not exist',inSys);
end
if ( isempty(strmatch(outSys,Systems)) )
  error('Coordinate system %s does not exist',outSys);
end

if ( strmatch(lower(inSys),'cartesian','exact') ) & ...
      ( strmatch(lower(outSys),'spherical','exact') )

  x = V_in(:,1);
  y = V_in(:,2);
  z = V_in(:,3);

  r  = sqrt(x.^2+y.^2+z.^2);

  th = zeros(size(r));
  I  = find(r ~= 0);
  if ~isempty(I)
    th(I) = acos(z(I)./r(I));
  end

  ph = (pi/2)*ones(size(x));
  I = find((x == 0) & (y == 0));
  if ~isempty(I)
    ph(I) = 0;
  end
  I = find(x ~= 0);
    if ~isempty(I)
    ph(I) = atan(y(I)./x(I));
  end

  V_out = [r,th,ph];

end

if ( strmatch(lower(inSys),'spherical','exact') ) & ...
     ( strmatch(lower(outSys),'cartesian','exact') )

  r  = V_in(:,1);
  th = V_in(:,2);
  ph = V_in(:,3);
  
  V_out = [r.*sin(th).*cos(ph),r.*sin(th).*sin(ph),r.*cos(th)];

end  

if ( strmatch(lower(inSys),'spherical2','exact') ) & ...
     ( strmatch(lower(outSys),'cartesian','exact') )

  th = X_in(:,2);
  ph = X_in(:,3);
  
  % Cartesian unit vectors in spherical at point X_in
  xhat  = [sin(th).*cos(ph),cos(th).*cos(ph),-sin(ph)];
  yhat  = [sin(th).*sin(ph),cos(th).*sin(ph),cos(ph)];
  zhat  = [cos(th),-sin(th),zeros(length(th),1)];

  V_x   = dot(xhat,V_in);
  V_y   = dot(yhat,V_in);
  V_z   = dot(zhat,V_in);

  V_out = [V_x,V_y,V_z];
  
end

if ( strmatch(lower(inSys),'cartesian','exact') ) & ...
     ( strmatch(lower(outSys),'spherical2','exact') )
  
  X_out = transform('Cartesian','Spherical',X_in);
  
  th = X_out(:,2);
  ph = X_out(:,3);
  
  % Spherical unit vectors in cartesian at point X_in
  rhat  = [sin(th).*cos(ph),sin(th).*sin(ph),cos(th)];
  thhat = [cos(th).*cos(ph),cos(th).*sin(ph),-sin(th)];
  phhat = [-sin(ph),cos(ph),zeros(length(ph),1)];

  V_r  = dot(V_in,rhat);
  V_th = dot(V_in,thhat);
  V_ph = dot(V_in,phhat);  
  
  V_out = [V_r,V_th,V_ph];

end
