function Y = qqchunk(xin,thresh,qtime,plot_it)
%QQCHUNK Extract chunks from columns of matrix corresponding to times when
%    first column of matrix is preceeded and followed by a quiet time.
%
%    The data in chunk may cross below threshold for times less than qtime.
%    QQCHUNK2 does not allow below threshold crossings.
%
%    Y = QQCHUNK(X,THRESH,QTIME) where X is a matrix, THRESH is the value below 
%    which is considered a quiet time, and QTIME is the length of time
%    below thresh to be considered a quiet time.
%
%    Y.chunks a cell matrix.  Each cell contains a chunk of X.
%    Y.mean   a matrix that contains the mean of time series in Y.chunks.
%    Y.max    a matrix that contains the maximum value of t.s. in Y.chunks.
%    Y.area   a matrix that contains the area under t.s. in Y.chunks.
%    Y.j1     a matrix.  Each element contains left boundaries J1.
%    Y.j2     a matrix.  Each element contains right boundaries J2.
%
%    Y = QQCHUNK(X,THRESH,PLOT_IT) executes PLOT_QQCHUNK(xin,Y,thesh,qtime)
%    if PLOT_IT = 1.
%   
%    See also QQCHUNK2, BDL, CHUNK, QQCHUNK_DEMO, QQCHUNK_PLOT.

if (nargin < 4)
  plot_it = 0;
end

Y      = [];
y      = [];

x      = [thresh;xin(:,1);thresh];
J1     = find(x >= thresh);
D1     = diff(J1);
Id1    = find(D1 > qtime);
bound  = J1(Id1) + 1;

if (length(bound) < 2)
  return;
else 
  if (mod(length(bound),2) == 1)
    B = bound(1:end-1);
  else 
    B = bound;
  end
end
%B'
B1 = B;
B2 = B;

for i = 0:length(B)/2 - 1
  k       = 1+2*i;
  I       = find(x(B(k):end) >= thresh);
  B1(k)   = B(k) + I(1) - 2;
  I       = find(x(B(k+1):end) >= thresh);
  if (isempty(I)),k,B(k+1),end;
  B2(k+1) = B(k+1) + I(1) - 2; 
end

if (mod(length(bound),2) == 1)
  B2 = [B2(2:end); bound(end)];
else
  B2 = B2(2:end-1);
end

%B1'
%B2'
B3 = sort([B1;B2]);

keven = [1:2:length(B3)-1];
a     = B3(keven);
b     = B3(keven + 1) - 2;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for k = 1:length(a)
  for i = 1:size(xin,2)
    y{k}(:,i)    = xin(a(k):b(k),i);
    ymean(k,i)   = mean(y{k}(:,i));
    ymax(k,i)    = max(y{k}(:,i));
    yarea(k,i)   = sum(y{k}(:,i));
  end
end

if (~isempty(y))
  Y.chunk = y;
  Y.mean  = ymean;
  Y.max   = ymax;
  Y.area  = yarea;
  Y.j1    = a;
  Y.j2    = b;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


if (plot_it == 1)
  qqchunk_plot(xin,Y,thresh,qtime)
end
  
