function [DT,J1,J2,S] = thresh_cross0(R,Thresh,Time)
%THRESH_CROSS0 Threshold crossings intime series
%
%    [DT,J1,J2,S] = THRESH_CROSS0(X,THRESH) returns J1, the times at which
%    the vector X crosses THRESH upwards, J2, the time of downward
%    crossings, and DT, the amount of time spent above THRESH.  X must be a
%    vector.  S is a structure with statistics on each event.  THRESH0 is
%    much faster if S is not requested.
%
%    J1 is floor(t_up) and J2 is ceil(t_down), where t_up is actual time of
%    upward crossing and t_down is actual time of downward crossing, i.e.,
%    if X = [0;0;1;0;0] and THRESH = 0.1, the upward crossing time t_up must
%    be between 2 and 3 and the downward crossing time t_down must be
%    between 3 and 4.  THRESH_CROSS0(X,THRESH) returns DT = 1, J1 = 2 and J3
%    = 4;
%
%    [DT,J1,J2,S] = THRESH_CROSS0(X,THRESH,Tquiet,mintime,maxtime) requires
%    at least Tquiet intervals exist before and including J1 and only
%    intervals with DT >= mintime and DT <= maxtime.  Default value of
%    Tquiet, mintime, and maxtime is 1.  If maxtime == mintime, S.interval
%    is a matrix (and execution is fastest), otherwise it is a cell array.
%    
%    See also THRESH_CROSS0_DEMO.

if (prod(size(R))>length(R))
  error('X must be a vector');
end

mintime = [];
maxtime = [];
qtimeBefore = 1;
qtimeAfter  = 1;

if (nargin > 2)
  qtimeBefore = Time(1);
  if (length(Time) > 1)
    mintime = Time(2);
  end
  if (length(Time) > 2)
    maxtime = Time(3);  
    if maxtime < mintime
      error('thresh_cross0: maxtime must be equal to or greater than mintime');
    end
  end
  if (length(Time) > 3)
    qtimeAfter = Time(4);
  end
end

J1 = find((R(1:end-1) <= Thresh) & (R(2:end) >  Thresh));
J2 = find((R(1:end-1) >  Thresh) & (R(2:end) <= Thresh));
J2 = J2+1;

%J1(1:8)'
%J2(1:8)'

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ((isempty(J1)) | (isempty(J2)))

  J1 = [];
  J2 = [];
  DT = [];

else	

  if (R(1) <= Thresh)
    if (size(J2,1) < size(J1,1))
      DT  = J2(1:end) - J1(1:end-1);
      J1  = J1(1:end-1);
      J2  = J2(1:end);
    elseif (size(J2,1) == size(J1,1))
      DT  = J2 - J1;
    end
  end

  if (R(1) > Thresh)

    if (size(J1,1) < size(J2,1))
      DT  = J2(2:end) - J1(1:end);
      J1  = J1(1:end);
      J2  = J2(2:end);
    elseif (size(J2,1) == size(J1,1))

      if (length(J2) > 1)
	DT  = J2(2:end) - J1(1:end-1);
	J1  = J1(1:end-1);
	J2  = J2(2:end);
      else 
	DT  = [];
	J1  = [];
	J2  = [];
	return;
      end
    end
  
  end

DT = DT-1;

%J1(1:8)'
%J2(1:8)'

if (~isempty(mintime))
  I  = find(DT >= mintime);
  DT = DT(I);
  J1 = J1(I);
  J2 = J2(I);
else
  mintime = -1;
end

if (~isempty(maxtime))
  I  = find(DT <= maxtime);
  DT = DT(I);
  J1 = J1(I);
  J2 = J2(I);
else
  maxtime = -1;
end

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Remove crossings not preceeded by quiet times
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (~isempty(J1)) & (qtimeBefore > 1)

  a = find( J1 - qtimeBefore >= 0 );

  if isempty(a)
    DT  = [];
    J1  = [];
    J2  = [];
    return;
  end
  
  J1  = J1(a(1):end);
  J2  = J2(a(1):end);
  DT  = DT(a(1):end);

  JJ1  = cumsum([J1' ; -1*ones(qtimeBefore-1,length(J1))]);

  Z    = reshape(R(JJ1(:)),qtimeBefore,length(JJ1(:))/qtimeBefore);
  I    = find(max(Z) <= Thresh);

  J1   = J1(I);
  J2   = J2(I);
  DT   = DT(I);
  
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Remove crossings not followed by quiet times
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (~isempty(J1)) & (qtimeAfter > 1)

  a   = find( J2 + qtimeAfter - 1 <= length(R) );

  if isempty(a)
    DT  = [];
    J1  = [];
    J2  = [];
    return;
  end

  J1  = J1(1:a(end));
  J2  = J2(1:a(end));
  DT  = DT(1:a(end));

  JJ2  = cumsum([J2' ; +1*ones(qtimeAfter-1,length(J2))]);

  Z    = reshape(R(JJ2(:)),qtimeAfter,length(JJ2(:))/qtimeAfter);
  I    = find(max(Z) <= Thresh);

  J1   = J1(I);
  J2   = J2(I);
  DT   = DT(I);
  
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%plot(R(1:337))

%J1(1:8)'
%J2(1:8)'

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Return statistics
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (nargout>3)
  
if (~isempty(J1))

  % If all intervals same size, return matrix
  if (maxtime > 0) & (mintime > 0) & (maxtime == mintime) 


    JJ1      = cumsum([J1' ; ones(DT(1)+1,length(J1))]);
    interval = reshape(R(JJ1(:)),DT(1)+2,length(JJ1(:))/(DT(1)+2));
    time     = reshape(JJ1(:),DT(1)+2,length(JJ1(:))/(DT(1)+2));  
    maxR     = max(interval);
    meanR    = mean(interval);
    areaR    = sum(interval);
  
  else % If all intervals not same size, return matrix

    nstr = '';
  
    for k = 1:length(J1)
      
      interval{k} = R(J1(k):J2(k));
      
      if size(R,2) > size(R,1)
	time{k}     = [J1(k):J2(k)];
      else
	time{k}     = [J1(k):J2(k)]';  
      end
      
      meanR(k)    = mean(R(J1(k):J2(k)));
      maxR(k)     = max(R(J1(k):J2(k)));
      areaR(k)    = sum(R(J1(k):J2(k)));
      
      if mod(k,100) == 0
	nstr = ...
	    sprintf(['thresh_cross0: Generating statistics for event '...
		     '%d of %d'],k,length(J1));
	fprintf('%s%s',repmat('\b',length(nstr),1),nstr);
	fflush(stdout);
      end
      
    end
    
    if (k >= 100)
      fprintf('%s%s%s',...
	      repmat('\b',length(nstr),1),...
	      repmat(' ',length(nstr),1),...
      	      repmat('\b',length(nstr),1));
    end
  end
  
  S.interval = interval;
  S.time     = time;
  S.t1       = J1;
  S.t2       = J2;
  S.mean     = meanR;
  S.max      = maxR;
  S.area     = areaR;

end

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
