function [X,dX] = mag_data_load(MAGLIST,years,COMPONENT,datastore,FLAGVAL)
%MAG_DATA_LOAD Returns magnetometer data over a given time span.
%
%   X = MAG_DATA_LOAD(MAGLIST,years,COMPONENT) 
% 
%   If no data is found for years a warning is generated and data for that year
%   is set to FLAGVAL.
%
%   X = MAG_DATA_LOAD(MAGLIST,years,COMPONENT,datastore,FLAGVAL) 
%
%   [X,dX] = MAG_DATA_LOAD(MAGLIST,years,COMPONENT) gives properly
%   flagged dX.
%
%   If magnetometer data exists in more than one data directory, data is
%   taken from the DMI directory.
%  
%   Other usage: MAG_DATA_LOAD(MAGLIST,years,COMPONENT,datastore,FLAGVAL)
%   Replaces any abs(X) above FLAGVAL with FLAGVAL.  Default setting of FLAGVAL
%   is 99999.
%
%   See also MAG_DATA_INFO, DMI_DEMO, DMI_DOWNLOAD, DMI_PLOT.

mag_data_info;

MAGLIST = lower(MAGLIST);

if (~exist('FLAGVAL')),FLAGVAL=99999;,end  

for j  = 1:size(MAGLIST,1)  
  MAG = MAGLIST(j,:);
  if ~isempty(strmatch(MAG,MAGLIST_FMI))
    if (~exist('datastore'))
      datastore=directory_fmi;
    end  
  end
  if ~isempty(strmatch(MAG,MAGLIST_DMI))
    if (~exist('datastore'))
      datastore=directory_dmi;
    end  
  end
  if ~isempty(strmatch(MAG,MAGLIST_SPI))
    if (~exist('datastore'))
      datastore=directory_spi;
    end  
  end
  
  x   = []; 
  dx  = [];
  
for yr = 1:length(years)

    I        = [];
    %D        = dir(datastore);
    D        = readdir(datastore);
    filename = sprintf('%s_%s_%d.mat',lower(MAG),COMPONENT,years(yr));
    Ifname   = strmatch(filename,D);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if (~isempty(Ifname)) % If .mat file exists

      fname   = sprintf('%s/%s_%s_%d.mat',datastore,lower(MAG),COMPONENT,years(yr));
      Yc      = load(fname);
      fprintf('loading %s\n',fname);
      if (is_octave)
	NAME  = struct_elements(Yc);
	        eval(sprintf('temp = Yc.%s',NAME));
      else
	NAME  = fieldnames(Yc);
	temp  = getfield(Yc,NAME{1});
      end

    else % If .mat file does not exist, fill with FLAGVAL.
      fname   = sprintf('%s/%s_%s_%d.mat',datastore,lower(MAG),COMPONENT,years(yr));      
      fprintf('Data file %s not found.  Inserting fill values.\n',fname);
      temp      = FLAGVAL*ones(1440*number_days(years(yr),years(yr)),1);
    
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    x  = [x ; temp];    

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if (nargout == 2) % If dX requested
      dx = diff_nonflag(x,1,FLAG);
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    
end

X(:,j)  = x;

if (nargout == 2) % If dX requested
  dX(:,j) = dx;
end

end
