function [Ddetrended,DQr] = quiet_day_detrend(Q,D,years,FLAG1)
%QUIET_DAY_DETREND Detrend data using quiet days within a month
%
%    D = QUIET_DAY_DETREND(Q,D,years), where
%    
%    Q     = 1 if day was quiet and 0 otherwise
%    D     = data to detrend
%    years = years the data covers.  Q must start on first
%            day of years(1) and end on last day of years(2).
%
%    If length(D)/length(Q) = 1, the average of the quiet days in every
%    month is subtracted from every value of D within that month.
%
%    If length(D)/length(Q) = 24 or 1440, the quiet days in every month are
%    averaged to give a UT-dependent quiet day profile of length 24 or 1440.
%    This profile is subtracted from UT-dependent profiles of length 24 or
%    of each day within that month. 
%
%    Dd = QUIET_DAY_DETREND(Q,D,years,FLAG) specifies that the flag value
%         for D is FLAG.  Default is NaN.
%
%    [Dd,DQ] = QUIET_DAY_DETREND(Q,D,years) returns the quiet day vector,
%              where Dd = D-DQ.
%
%    See also QUIET_DAY_DETREND_DEMO.

% R.S. Weigel 10/14/2004
  
if (nargin < 4)
  FLAG1 = NaN;
else
  D     = reflag(D,FLAG1,NaN); % Change the flag value to NaN.
end

% Compute number of intervals per day.
if (length(D)/length(Q) == 1440)
  Npd = 1440;
elseif (length(D)/length(Q) == 24)
  Npd = 24;
elseif (length(D)/length(Q) == 1)
  Npd = 1;
else
  error('length(D)/length(Q) must be 1440, 24, or 1');
end

if (length(years) > 2)
  error('years vector must have only two elements');
end

if (length(years) == 2)
  years = [years(1):years(2)];
end

if (diff(years) ~= 1)
  error('years vector must be monotonically increasing.');
end

if (length(D) ~= Npd*number_days(years(1),years(end)))
  error('years range is not consistent with length of Q or D');
end

% Replace quiet day values labeled with zeros with NaN.
Q    = reflag(Q,0,NaN);

% Resample Q onto same time grid as D.  Then expand and reshape Q so rows
% are minutes of day (for Npd = 1440). Columns now correspond to days. 
Q    = repmat(Q',Npd,1);

% Flatten Q so it is same length as D and multiply to get values
% on quiet days.
DQ   = Q(:).*D;

% Reshape DQ (Days Quiet) so it has rows of minutes of day (for Npd = 1440)
DQ   = reshape(DQ,Npd,length(DQ)/Npd);
%plot(mean_nonflag(DQ,2))

% Compute DQr.  
DQr = [];
b   = 0;
for i = 1:length(years)
  a    = 1 + b;
  b    = a - 1 + number_days(years(i));
  % a is first data point in year
  % b is last data point in year
  bb   = 0;
  TEMP = DQ(:,a:b);
  temp = [];
  for j = 1:12
    Dpm   = days_per_month(j,years(i));
    aa    = 1 + bb;
    bb    = aa - 1 + Dpm;
    % aa is index of first day in month
    % bb is index of last day in month
    qq    = mean_nonflag(TEMP(:,aa:bb),2); 
    % qq is now a Npd x 1 matrix
    % Expand qq to be a Npd x Dpm matrix
    qq    = repmat(qq,1,Dpm); 
    temp  = [temp ; qq(:)];  % Append flattened qq to previous months data
  end
  DQr = [DQr ; temp]; % Append temp to previous years data
end

Ddetrended = D-DQr; % Detrend the input D

if (nargin > 3)
  Ddetrended = reflag(Ddetrended,NaN,FLAG1);
  if (nargout > 1)
    DQr = reflag(DQr,NaN,FLAG1);
  end
end