## Copyright (C) 2005 Bill Denney
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##
## Based on code Copyright (C) 2000 Etienne Grossmann 

## -*- texinfo -*-
## @deftypefn {Function File} {} xaddpath (@var{dir1}, @dots{})
## Prepend @var{dir1}, @dots{} to the current @code{LOADPATH}.
## If a directory is already in the path, it is moved to the specified
## location, prepending by default.
##
## Use xaddpath(@var{dir1}, @dots{}, "-end") to append directories.
## This function will not add directories named
## @samp{-end} or @samp{-begin} (case insensitively) to the path.
##
## Compound paths are allowed, for example, 
## @example
## addpath ("/path1:/path2","/path3")
## @result{} prepends path1, path2, and path3 to LOADPATH.
## @end example
##
## Duplicates are ignored such that 
## @example
## addpath ("/path1:/path2:/path1") or addpath ("/path1:/path2:/path1","-end")
## @result{} puts path1 before path2 in LOADPATH.
## @end example
##
## A warning will be returned if the string is not a directory, 
## does not exist, is named @samp{-begin} or @samp{-end} (case insensitively),
## or you do not have read access to it.
##
## Prior to execution, "cleanpath" is used to remove unusual strings such
## as multiple consecutive colons or forward slashes.  
##
## The current directory "." is always put first in the pathlist.
##
## Used as above, this function equivalent to Matlab's addpath except 
## that "//" syntax (see LOADPATH help) is allowed.
## Use of "//" syntax will result in a warning if warn_matlab_incompatable 
## = 1.
##
## P = xaddpath(DIR1,...) returns LOADPATH prior to modification.
##
## @seealso{LOADPATH, rmpath, savepath, setpath, cleanpath}
## @end deftypefn

## Author: Etienne Grossmann <etienne@cs.uky.edu>
## Modified-By: Bill Denney <bill@givebillmoney.com>
## Modified-By: R.S. Weigel <rweigel@gmu.edu>

function ret = xaddpath (varargin)

  global traditional_path_mode;

  if (nargout)
    ret = LOADPATH;
  end

  tmp = cleanpath([LOADPATH,DEFAULT_LOADPATH]);
  LOADPATH = ['.',tmp(1:end-1)];
  DEFAULT_LOADPATH = ''; ## (Setting to '' generates warning)

  if (nargout)
    ret = path;
  end

  arg = 1;
  appendp = 0;

  switch varargin{end}
   case { 0, "0", "-begin", "-BEGIN" }
    varargin = varargin(1:end-1);
    appendp = 0;
   case { 1, "1", "-end", "-END" }
    varargin = varargin(1:end-1);
    appendp = 1;
   endswitch

   varargin = cleanpath(varargin);

   if (appendp == 0)
     varargin = fliplr (varargin);
   end

   np = '';
   for arg = 1:length (varargin)
     
     p = varargin{arg};

     dupe = 0;
     if (arg > 1) && (appendp == 1)
       if (strmatch(p,varargin(1:arg-1),"exact"))
	 warning("xaddpath: Multiple occurences of %s in input",p(2:end-1));
	 dupe = 1;
       end
     end
     if (arg < length (varargin)) && (appendp == 0)
       if (strmatch(p,varargin(arg+1:end),"exact"))
	 warning("xaddpath: Multiple occurences of %s in input",p(2:end-1));
	 dupe = 1;
       end
     end

     if (~isempty(p)) && (dupe==0)
       p = p(2:end-1);
       p = checkpath(p,'addpath'); # check path
       xrmpath(p,0); ## 0 tells rmpath to not warn if p is not in path

	 if (appendp)
	   LOADPATH = sprintf ("%s:%s", LOADPATH , p);
	 else
	   LOADPATH = sprintf (".:%s:%s", p , LOADPATH(3:end));
	 end
     end
     
   end

endfunction

%!assert(addpath
%!assert(addpath('','hello','world'),'hello:world')
%!assert(addpath(':','hello'),'hello:');
%!assert(addpath(':','hello','-end'),':hello');
%!assert(addpath('hello','hello'),'hello');
%!assert(addpath('hello','world'),'world:hello')
%!assert(addpath('hello','world','-end'),'hello:world')
%!assert(addpath('hello:','world','-end'),'hello::world')
%!assert(addpath('hello:','hello','world','-end'),':hello:world')

%!assert(addpath('',''),':')
%!assert(addpath(':',''),':')
%!assert(addpath('hello',''),':hello')
%!assert(addpath('hello:world',''),':hello:world')
%!assert(addpath('hello:world:',''),':hello:world')
%!assert(addpath('hello::world',''),':hello:world')
