function [B,Ng] = moving_mean_nonflag(A,N,DIM,FLAG,COND,force_internal_fn)
%MOVING_MEAN_NONFLAG Moving average of non-FLAG elements.
%  
%    [B,Ng] = MOVING_MEAN_NONFLAG(A,N) If A is a vector, each element of B
%    is average of N points: B(i) = (1/N)*(A(i) + ... + A(i+N)), where Ng(i)
%    is the number of non-NaN elements in the interval and the sum omit any
%    x(i) that is NaN.  If A is a matrix, computation on each column.  B
%    is the same size as A.
%
%    MOVING_MEAN_NONFLAG(X,N,DIM) does computation across dimension DIM.
%
%    MOVING_MEAN_NONFLAG(X,N,DIM,FLAG) uses a flag of FLAG instead of NaN.
%
%    MOVING_MEAN_NONFLAG(X,N,DIM,FLAG,COND) uses a flag condition COND
%    (see IS_FLAG for COND values).
%
%   Computation is done with a mex or oct program if either
%   moving_mean_nonflag_oct.oct or moving_mean_nonflag_mex.mex are found in
%   path.  Use exist('moving_mean_nonflag_mex') or
%   exist('moving_mean_nonflag_oct') to see if these binaries are in your
%   path.  Use moving_mean_nonflag(A,DIM,FLAG,COND,1) to prevent usage of
%   mex or oct programs.
%
%    See also MEAN_NONFLAG, FILTER.

if (nargin < 4)
  FLAG = NaN;
end
if (nargin < 5)
  COND = 1;
end

if (nargin < 3)
  if (min(size(A) > 1))
    DIM = 1;
  else
    [junk,DIM] = max(size(A));
    if (DIM > 2)
      error('DIM must be 1 or 2.');
    end
  end
end

if (nargin < 6)
  force_internal_fn = 0;
end

if (isempty(DIM))
  DIM = 1;
end

if (isempty(FLAG))
  FLAG = NaN;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
if (size(A,1)==1)
  DIM = 2;
end
if (size(A,2)==1)
  DIM = 1;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
if (prod(size(A)) == 1)
  B   = FLAG;  % If one element and is zero, return FLAG
  Ng   = 0;
  return;
end
if ( isempty(A) )
  B    = [];  % If empty, return empty
  Ng   = [];
  return;
end
if (N > size(A,DIM))
  error('Averaging number is greater than size(A,DIM)');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

external_fn = 1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
if (is_octave == 0)
if (exist('moving_mean_nonflag_mex') ~= 3)
  fprintf(['moving_mean_nonflag: Did not find mex function '...
	   'mean_nonflag_mex (will be slower).\n']);
  %system('mex moving_mean_nonflag_mex.c');
  external_fn = 0;
end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
if (is_octave == 1)
if (exist('moving_mean_nonflag_oct') ~= 3)
  fprintf(['moving_mean_nonflag: Did not find oct function '...
	   'mean_nonflag_oct (will be slower).\n']);
  %system('mkoctfile mean_nonflag_oct.cc')
  external_fn = 0;
end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

if (external_fn == 1) & (force_internal_fn == 0)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
if (DIM == 1)
  if (is_octave == 0)
    [B,Ng] = moving_mean_nonflag_mex(A,N,FLAG,COND);
  else
    [B,Ng] = moving_mean_nonflag_oct(A,N,FLAG,COND);
  end
end

if (DIM == 2)
  if (is_octave == 0)
    [B,Ng] = moving_mean_nonflag_mex(A',N,FLAG,COND);
    B  = B';
    Ng = Ng';
  else
    [B,Ng] = moving_mean_nonflag_oct(A',N,FLAG,COND);
    B  = B';
    Ng = Ng';
  end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
end

if (external_fn == 0) | (force_internal_fn == 1)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
if (DIM == 2)
  A = A';
end  

B  = zeros(size(A));
Ng = zeros(size(A));

mrows = size(A,1);
ncols = size(A,2);

for j=1:ncols

  backup = repmat('\b',1,80);
  fprintf(['%smoving_mean_nonflag:'...
	   ' Evaluating column %5d of %d'],backup,j,ncols);fflush(stdout);

  for i = 1:N
    if (is_flag(A(i+(j-1)*mrows),FLAG,COND) == 0)
      Ng(1 + (j-1)*mrows) = Ng(1 + (j-1)*mrows) + 1;
      B(1 + (j-1)*mrows)  = B(1 + (j-1)*mrows) + A(i + (j-1)*mrows);
    end
  end

  for k = 2:(mrows-N+1)
    Im1   = k - 1 + (j-1)*mrows;
    newx  = is_flag(A(Im1+N),FLAG,COND);
    oldx  = is_flag(A(Im1),FLAG,COND);
    
    if (newx==0)
      if (oldx==0)
	B(k + (j-1)*mrows)  = B(Im1) - A(Im1) + A(Im1+N);
	Ng(k + (j-1)*mrows) = Ng(Im1);
      else
	B(k + (j-1)*mrows)  = B(Im1) + A(Im1+N);
	Ng(k + (j-1)*mrows) = Ng(Im1)+1;
      end
    end
    
    if (newx==1)
      if (oldx==0)
	B(k + (j-1)*mrows)  = B(Im1) - A(Im1);
	Ng(k + (j-1)*mrows) = Ng(Im1) - 1;
      else
	B(k + (j-1)*mrows)  = B(Im1);% - A(Im1);
	Ng(k + (j-1)*mrows) = Ng(Im1);
      end
    end
    
  end
  
  if (N == mrows)
    k = 2;
  else
    k = k + 1;
  end

  for i = k:mrows
    B(i+(j-1)*mrows)  = FLAG;
    Ng(i+(j-1)*mrows) = 0;
  end

  if (Ng(1 + (j-1)*mrows) == 0)
    B(1 + (j-1)*mrows) = FLAG;
  end

  for k = 1:(mrows-N+1)
    if (Ng(k + (j-1)*mrows) > 0)
      B(k + (j-1)*mrows)  = B(k + (j-1)*mrows)/Ng(k + (j-1)*mrows);
    else
      B(k + (j-1)*mrows)  = FLAG;
    end
  end
  
end

fprintf('\n');

if (DIM == 2)
  B  = B';
  Ng = Ng';
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
end
