function [I,A,B] = perlop(B,regexp,WIDTH,HEIGHT,isafile,filenameout,padval)
%PERLOP Do Perl regexp operation on each line of matrix, cell array or file.
%
%    A fast replacement for the Matlab native functions STRMATCH, FINDSTR,
%    STRVCAT, STRCMP, FGETL, etc.  
%
%    PERLOP looks calls the perl processor returned in the location returned
%    by PERLPATH.  perlpath.m file must be edited so it contains the
%    location of perl on your system.
%
%    --- Matrix and Cell operations ------------------------------------------
%    I     = PERLOP(B,REGEXP) returns the row numbers of B that matched REGEXP.
%  
%    [I,A] = PERLOP(B,REGEXP) Does REGEXP on each row of matrix or cell
%    array B.  A is the matrix of these lines after being operated on by
%    REGEXP.  If A is all numeric data, A is a double array, otherwise A is
%    a padded char array.  For example, if REGEXP = '=~ m/test/', vector I
%    will contain lines of B that had word "test"; A = B(I,:).  
%
%    [I,A,Bm] = PERLOP(B,REGEXP) Returns the full modified matrix Bm provided
%    that A is a character array.
%
%    ...      = PERLOP(B,REGEXP,WIDTH,HEIGHT) Sets left boundary at
%    WIDTH(1), right boundary at WIDTH(2), first line at HEIGHT(1), and
%    last line at HEIGHT(2).
%
%    ...      = PERLOP(B,REGEXP,WIDTH,HEIGHT,[],FILENAMEOUT) Writes the
%    data in A to FILENAMEOUT.
%
%    --- File operations ----------------------------------------------------
%    [I,A] = PERLOP(FILENAME,REGEXP,[],[],'file') Does same as above on 
%    the file FILENAME using the default values of WIDTH and HEIGHT 
%    (WIDTH = [1 1e9], HEIGHT = [1 1e9]).  
%  
%    [I,A] = PERLOP(FILENAME,REGEXP,[],[],'file',FILENAMEOUT) Writes
%    the changed file to FILENAMEOUT.  FILENAMEOUT may be the same as FILENAME.
%
%    [I,A] = PERLOP(FILENAME,REGEXP,[],[],'file',FILENAMEOUT,padval) Writes
%    the changed file to FILENAMEOUT with leading and trailing whitespace
%    stripped *before* padding is done if padval = 1.  If padval = 2,
%    leading and trailing white space on each line is kept.  
%
% 
%    Newlines that are not at the end of a line are removed.
%
%    If REGEXP contains ' in unix or " in w32, they need to be prefaced with
%    a \.  See perlop_demo.m. 
% 
%   
%    See also PERLOP_DEMO, PERLPATH.

%    R.S. Weigel, October, 2001.  

%    Note that behavior of load command has changed between Matlab V5 and V6:
%    The load command will now accept data files containing dates and time,   
%    i.e., 1999-01-01 00:01 384.0 will be read as 1999.00 0 384.0
  
%    Put a switch to test for this condition:
%    If REGEXP contains ' in unix or " in w32, they need to be prefaced with
%    a \.  See perlop_demo.m. 

%    To do: 
%           Change so that perlpath returns location of perl.exe also
%           by searching matlab path directories.

%           Need to add support for returning Bm in file operations.  
%           Need to add support for writing out data in Bm to FILENAMEOUT.
%           
  
%    Need to document surprising behavior caused by newlines not at end
%    of line.  This may have been fixed by newline_regexp switch.
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% Below problem is now fixed
% Fix this:
% if enddate  = '05o03o2002' 
% [I,endd]    = perlop(enddate,'=~ s/o/d/g') gives 5000 since load command
% does not fail if file is one line with non-numeric data; It loads some
% of the numbers and ignores all of the string parts.  (try /o/xx/, for
% example)
% load command also loads file 
% 0 9X 1B
% 1 2  4 
% and ignores the letters if no two consecutive non letters and provided
% letters are on the end.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (is_octave)
  error('Not (yet) compatable with Octave');
end

if (~exist('padval')),padval = 1;,end;  

if (padval ~= 1 | padval ~= 2)
  padval = 1;
end
if (nargout == 1 & ~exist('isafile'))
  padval = 0;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set up default values.
if ((nargin < 3) | isempty(WIDTH))
  left_c   = 0;
  right_c  = 1e9-1;  % Max width of file is 1 billion columns.
else
  left_c   = WIDTH(1)-1; % -1 since perl index starts at 0.
  right_c  = WIDTH(2)-1;
end

if ((nargin < 4) | isempty(HEIGHT))
  a = 0;
  b = 1e9-1;         % Max file length is 1 billion lines. 
else
  a = HEIGHT(1);
  b = HEIGHT(2);
end

if (isinf(b)),b = 1e9;,end
if (isinf(right_c)),right_c = 1e9;,end

if (nargin > 4)
  if (strmatch(isafile,'file') & ~isempty(isafile))
    file       = 1;
  else
    file = 0;
  end
else
  file = 0;
end

if (file == 1)
  filename    = B;
end

if (isempty(B))
  I = [];
  B = [];
  disp('perlop Warning: Input matrix is empty');
  return;
end

newline_regexp = 0;
if (~isempty(findstr('\n',regexp)))
  newline_regexp = 1;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check if B is a cell matrix.  If it is, turn cell into padded matrix.
if (iscell(B))
  if (size(B,2) > 1)
    disp('??? B must be a matrix or a one column cell arrary');
    I = [];
    A = [];
    return;
  end
  B = strvcat(B{:});
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% If cell or matrix, create temp file using time stamp as unique identifier.
if (file == 0)
  filename    = sprintf('%d',fix(clock));
  filename    = strcat('temp_',filename);
  fid         = fopen(filename,'w');

  % If the matrix does not have newlines at end, add them so
  % in temp file, each row of matrix is a line.
  
  if (isempty(strmatch(B(1,end),sprintf('\n'))))   % First line does not have
                                                   % newline character at end.
    a  = 1;
    NL = repmat(sprintf('\n'),size(B,1),1);
    if (~isempty(strmatch(sprintf('\n'),B(:,end))))
      errstr = sprintf('??? All rows in matrix or cell must be ended');
      errstr = sprintf('%s with newline if any one\nrow is ended with',errstr);
      errstr = sprintf('%s newline\n',errstr);
      disp(errstr);
      I = [];
      A = [];
      return;
    end

    B = [B NL];     % Add newline to end of each row.
     
  end

  Nc = findstr(B(1,:),sprintf('\n'));  % Change this so it checks all lines 

  %Also, if Nc(1) < size(B(1,:),2) and B(1,Nc+1:end-1) are blank, leave
  %it alone.
  
  % Do this only if attempting to do operation other than s/\n//g 
  if (Nc(1) < size(B(1,:),2))
    %disp('perlop Warning: Newlines exist in matrix B that are not at the end of each row.');
    %disp('These newlines are being removed.');
    %disp('This problem may have been caused by using strvcat instead of strvcat2.');
    %I = [];
    %A = [];
    %return;
  end    
  fprintf(fid,'%s',B'); % Write matrix B to a file.
  fclose(fid);
  %!more temp*
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Call the Perl program perlop.pl
path_perlpart = which('perlop.m');  % Assume perlop.pl in same
				    % directory as perlop.m
				    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Guess the directory
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
path_perlpart = which('perlop.m');
path_perlpart = [path_perlpart(1:end-1),'pl'];

if (isunix)
%  path_perlpart = [home_dir,'CISM_DX/','perlfiles/perlop.pl'];
  com = sprintf('!%s %s %s %d %d ''%s'' %d %d %d',...
	perlpath,path_perlpart,filename,a,b,regexp,left_c,right_c,padval);
  eval(com);
else
%  path_perlpart = [home_dir,'CISM_DX\','perlfiles\perlop.pl'];
  com = sprintf('%s "%s" "%s" %d %d "%s" %d %d %d',...
	perlpath,path_perlpart,filename,a,b,regexp,left_c,right_c,padval);
  
  lasterr(''); % Sets lasterr to ''
  s1 = sprintf('!%s',com);
  eval(s1,'');

  if (~isempty(lasterr))
    error('Requested file does not exist.');
  end

  if (0)
    fid = fopen('perlop_command.bat','w');
    fprintf(fid,'%s\n',com);
    fprintf(fid,'echo off\n%s\n',com); % echo off is still written to screen
    fclose(fid);
    !perlop_command.bat;
    !del perlop_command.bat;
  end

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load processed matrix or file
% First check to see if matlab LOAD function can load file.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
filenameA = strcat(filename,'_processed');
A         = [];
%!head -1 *_processed

if (nargout > 1) % If A is not called for, don't load.
  s1 = sprintf('A = load(''%s_processed'');',filename);
  eval(s1,'');
  
  % Need this because Matlab load command will sometimes 
  % read files with text (dates with colons, etc.).
  if (~isempty(A) & (~isnumeric(A)))
    % If load command did not fail when we wanted it to
    % (because line contained non-numeric characters).
    A = [];
  end
  if (isempty(A))    % If load command failed, load as text file
    if (0)
      fprintf('\nLoading as text file (Slow Way) since this error\n');
      fprintf('encountered using load on file %s_processed:\n\n',filename);
      S = lasterr;
      I = findstr('file',S);
      fprintf('%s\n',S(1:I+4));
      fprintf('%s\n',S(I+5:end));
      fprintf('\n');
      end
    if (newline_regexp)        % Then REGEXP must have done a splitting
                               % and we need to reprocess file so it is
                               % padded correctly.
      Ijunk = perlop(filenameA,'=~ m/.*/g',[],[],'file',filenameA);	   
    end
    fid       = fopen(filenameA);
    cols      = size(fgets(fid),2);          % Get number of columns of 
					     % first line in file.
    tmp       = fclose(fid);
    fid       = fopen(filenameA);
    A         = fscanf(fid,'%c',[cols,inf]); % Read in whole file in one gulp.
    A         = A';                          % Matlab reads column to
                                             % column not row to row.
					     
    tmp       = fclose(fid);
    %    A = A(:,1:end-1); % Remove the newline character for better printing.
    %    No.  Removing the newline causes problems.  When operating on a matrix
    %    with no newlines, perlop will write a file with only one line.  This
    %    newline behavior needs to be worked on.
  end

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load index file

filenameI = strcat(filename,'_index');
I         = load(filenameI);
if (isempty(I))
  warning('REGEXP resulted in empty modified file.');
  Bm = A;
  perlop_cleanup(filename,filenameA,filenameI,file);
  return;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create modified matrix
if (ischar(A) & (nargout == 3))
  if (newline_regexp)
    error('Cannot return modified matrix because REGEXP has newline');
  end
  MA      = size(A,2);
  MB      = size(B,2);
  if (MB < MA)
    Blanks  = blanks(MA-MB);
    Blanks  = repmat(Blanks,size(B,1),1);
    B       = [B(:,1:end-1) Blanks B(:,end)];       
    B(I,:)  = A;
  end
  if (MB > MA)
        %B(I,1:MA-1)    = A(:,1:MA-1);      % Remove newline from A.   
        %Blanks         = blanks(MB-MA);    % (-1) seems correct.
	%B(I,MA:MB-1)   = Blanks;

	B(I,1:MA-2)    = A(:,1:MA-2);       % Remove newline from A.  (-2) works.
        Blanks         = blanks(MB-MA+1);   % Is this due to DOS having 2 
	                                    % characters represent linefeed?
        Blanks         = repmat(Blanks,size(I,1),1);

	B(I,MA-1:MB-1) = Blanks;
  end
  if (MB == MA)
    B(I,:) = A;
  end
else
  B      = [];
end
perlop_cleanup(filename,filenameA,filenameI,file);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


