
%PERLOP_DEMO Examples of PERLOP usage.
%
%
%    To run: type "which perlop_demo" to see where the perlop_demo file is
%    located and then view the file with an editor. Run the examples by
%    pasting different parts to command line to see what happens.
%
%    See also REMOVE_LINES_DEMO, REPLACE_LINES_DEMO, GRAB_LINES_DEMO,
%    DEMO_LINEWRAP, and DEMO_FIX_MAIL_HEADER for examples that use PERLOP or
%    related functions.
%
%    A fast replacement for the Matlab native functions STRMATCH, FINDSTR,
%    STRVCAT, STRCMP, FGETL, etc.  It is much more powerful than these
%    Matlab functions because it has all of the Perl regular expression
%    capability.  This program can also be used for probing or extracting
%    numeric data from a file with interspersed text.  For large files it is
%    much faster than using fgetl, fgets and the native Matlab string
%    matches.  Allows user to have the power of Perl regular expressions in
%    the easy-to-use matlab programing interface.
%
%    I wrote this so that I would never have to write another Perl program.
%
%    PERLOP communicates with perl via files NOT pipes.  It would be nice
%    to change this to speed things up, but this program is already MUCH
%    faster than the alternatives of using native Matlab functions (fgetl
%    and fgets, etc).
%  
%    Speed considerations: [I,A] = PERLOP is fastest if data file that
%    results after REGEXP can be loaded using the Matlab LOAD function.  If
%    LOAD command can be used, A is a double array, otherwise it is a char
%    arrray.  If matrix A is not needed, use I = PERLOP(...) so that the
%    file is not loaded into memory.
%
%    Examples in perlop_demo.m
%
%    1. Analize apache httpd log file.  
%    2. Matrix and cell operations: simple regular expressions
%    3. Matrix, cell, and file operations: complex [sic] regular expressions
%        (a) Convert numeric data not loadable with Matlab's LOAD command.
%        (b) Grab lines containin date and time and remove punctuation characters.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Example 1.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Suppose we want to do some analysis of a web log that contains these lines (standard apache httpd log):
% 216.239.46.12 - - [05/Nov/2001:19:26:04 -0600] "GET /~star/eps.pdf HTTP/1.0" 304 -
% 216.239.46.102 - - [05/Nov/2001:20:20:44 -0600] "GET /~star/Syn.doc HTTP/1.0" 200 222208
% 128.163.111.239 - - [04/Nov/2001:07:13:39 -0600] "GET /cmd.exe?/c+dir HTTP/1.0" 404 306
%
% The usual method of dealing with this file is to write a Perl script to
% extract the numeric data and then load the data into matlab for analysis.
% Here I want to avoid writing a perl script (which was the primary
% motivation for writing perlop.m and perlop.pl) and do all of the analysis
% in matlab.  The most difficult part then is getting the regular
% expressions correct, not re-learning Perl syntax.

L       = which('perlop');
Ldat    = which('perlop_demo.dat');
I       = strmatch(L,Ldat,'exact');

% Copy data file to current directory unless it is already there.
if (~isempty(I))
  if (isunix)
    eval(sprintf('!cp %sperlop_demo.dat .',L(1:end-8)));
  else  
    eval(sprintf('!copy "%sperlop_demo.dat" .',L(1:end-8)));
  end
end

% Lets count the number of "Code Red Virus" hits, which are the lines that
% contain the string "cmd.exe".

regexp = '=~ m/cmd\.exe/';  

% The . is a special character in Perl, so we must escape it with a \ 
% to make it literal .  The "=~ m" indicates match.

I       = perlop('perlop_demo.dat',regexp,[],[],'file');

fprintf('Number of lines with string cmd.exe = %d\n',length(I));

input('Continue?')

% The numbers in I are the line numbers that contain 'cmd.exe'.  Notice that
% we did not specify a second output argument, so the program will run
% faster. (Since the matching lines are not loaded, only the line numbers
% are.)
%
% Now lets find the time-of-day of the Code Red hits.  We want to grab everything
% between the square braces.  If it is a large file, we also want to modify
% the file so that what remains is all numeric data, so that we can use the
% Matlab LOAD command.  This is done in several steps.  If you are very good
% with regular expressions, this may only take one step.  I use several
% steps here not for pedagogical reasons, but because I am not very good
% with regular expressions.

% Copy perlop_demo.dat to perlop_demo_modified.dat:

if (isunix)
  !cp perlop_demo.dat perlop_demo_modified.dat
else
  !copy perlop_demo.dat perlop_demo_modified.dat
end

% Now we will operate on perlop_demo_modified.dat until it contains only numeric
% data.  First grab all lines that contain 'cmd.exe' and overwrite
% perlop_demo_modified.dat.

regexp = '=~ m/cmd\.exe/';  
fname  = 'perlop_demo_modified.dat';
I      = perlop(fname,regexp,[],[],'file','perlop_demo_modified.dat');

% Remove stuff between double quotes and overwrite the old file.  Double
% quotes are a problem for MSDOS (and single quotes for Unix).  The regualar
% expression is passed perlop.pl by the command line.  Therefore we need to
% tell the OS to ignore the double quotes.

if (isunix)
  regexp = '=~ s/"(.*)"/ /';
else
  regexp = '=~ s/\"(.*)\"/ /'; % Need to put \ before " in DOS command
                               % line since " is reserved character.
end

I      = perlop(fname,regexp,[],[],'file',fname);

% Remove any letters, backslashes, -, and colons and overwrite the old file.

regexp = '=~ s/:|\/|[A-Z]|[a-z]|\-|\[|\]|\./ /g';  
I      = perlop(fname,regexp,[],[],'file',fname);

% Load the (all numeric) data 
load perlop_demo_modified.dat

% View matrix of data loaded with the above load command.
perlop_demo_modified

input('Continue?')

% Remove the data file.
if (isunix)
  !rm perlop_demo_modified.dat
else
  !del perlop_demo_modified.dat 
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% For each modification, perlop.pl read the data file
% perlop_demo_modified.dat and ran through every line.  This was somewhat
% wasteful of processing, but the alternative of using a Matlab's fgetl is
% much slower, especially for a large data file.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Example 2. Matrix and Cell Examples -- Simple REGEXPs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

B1           = sprintf('ABCDEF\n');
B2           = sprintf('GHIJKL\n');

% Note, if the newline is not inserted:
% B1           = sprintf('ABCDEF');
% B2           = sprintf('GHIJKL');
% then perlop will assume that the end of each row in a matrix should have a newline
% and will auto-insert one.

B            = [B1;B2]

B_cell{1,1}  = B;
B_cell{2,1}  = B;
a            = 1;
b            = 10;
maxwidth     = 10;

regexp1      = '!~ m/A/';      % Lines that do not match A
regexp2      = '=~ m/A/';      % Lines lines that match A

regexp3      = '=~ s/A/F/';    % Find lines that match A and Substitute
                               % A with F.
regexp4      = '=~ s/A|G/F/g'; % Find lines that match A or G and replace
                               % A's and G's with F (the g means global).
			       
[I,A]        = perlop(B,regexp1)
[I,A]        = perlop(B,regexp2)
[I,A]        = perlop(B,regexp3)
[I,A]        = perlop(B,regexp4)
input('Continue?')
[I,A]        = perlop(B_cell,regexp1)
[I,A]        = perlop(B_cell,regexp2)
[I,A]        = perlop(B_cell,regexp3)
[I,A]        = perlop(B_cell,regexp4)
input('Continue?')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Example 3. Matrix and Cell Examples -- Complex REGEXPs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Take a look at the data file we are about to manipulate.
fname = 'perlop_demo.dat';
type perlop_demo.dat
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This will take lines that have the following (sometimes used) format 
% +0002+0001+0000-0001-0002 and convert to 
% +0002 +0001 +0000 -0001 -0002 
% which is loadable using the LOAD command.
regexp0       = '=~ s/([0-9])(\+|\-)([0-9])/$1 $2$3/g';
[I,A]         = perlop(fname,regexp0,[],[],'file');
% Pattern kept ones with colon
% This keep lines without colon
regexp0   = '!~ m/:/g';
[I,A,Bm]  = perlop(A,regexp0);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

input('Continue?')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This will  match lines such as '01-02-1999 14:22:01.01'
% and remove non-numeric chars   '01 02 1999 14 22 01.01';
regexp1       = '=~ s/(..)-(..)-(....) (..):(..):(..)/$1 $2 $3 $4 $5 $6/';
[I,A]         = perlop(fname,regexp1,[],[],'file');
A
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

input('Continue?')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This will match lines that begin with two numbers and a space and 
% prefix these numbers with a 19.  For example, '98 ' gets turned
% into '1998 ' if 98 was the first element in the row (the ^ operator).  
regexp2       = '=~ s/^([0-9])([0-9]) /19$1$2 /';
[I,A]         = perlop(fname,regexp2,[],[],'file');
A
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Demo of returning full modified matrix.
REGEXP  = '=~ s/123/111/';
A       = ['X123';'B456';'C789'];
[I,A,B] = perlop(A,REGEXP);
B
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Find lines that don't have any letters in them except 'E'.  Use this if
% for numeric files that use scientific notation is used in the numbers
% (i.e., 1E-01). (the i means any case allowed in match, the g means global,
% the | means or).  This loads rows with all numbers, unless there is a line
% with one or more E's and no other letters.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% regexp3       = '!~ m/[A-D]|[F-Z]/i';
% [I,A]         = perlop(fname,regexp3,[],[],'file');
% A
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


