function [T,Xin] = time_delay(T,Xin,Nc,Ts,Na,padstr)
%TIME_DELAY Create time delay vectors
%
%   [T,X] = TIME_DELAY(T,X,Nc)
%   Example: 
%             [T,X] = time_delay([1:10]',[1:10]',4) ; [T,X]
%   returns
%   X = [X(4:end-1,:) X(3:end-2,:) X(2:end-3,:) X(1:end-4,:)]
%   T = T(5:end,:) 
% 
%   [T,X] = TIME_DELAY(T,X,Nc,Ts) (Ts>=0) Shifts X back Ts
%   Example: [T,X] = time_delay([1:10]',[1:10]',5,1) ; [T,X]
%
%   [T,X] = TIME_DELAY(T,X,Nc,Ts,Na) Puts in Na acausal terms
%   Example: [T,X] = time_delay([1:10]',[1:10]',5,0,2) ; [T,X]
%   
%   Set X to [] if it is not needed.
%   Example: T = time_delay([1:10]',[],5,0,2) ; T
%
%   TIME_DELAY(T,X,Nc,Ts,Na,'pad') Adds rows of NaNs to Xin and T so outputs
%   has same size as inputs and Tout(1,1)=Tin(1,1).  Example: [T,X] =
%   time_delay([1:10]',[1:10]',5,0,2,'pad') ; [T,X]
%   
%
  
if (nargin < 4)  
  Ts = 0;
end
if (nargin < 5)  
  Na = 0;
end

if (size(T,1) != size(Xin,1)) & (~isempty(Xin))
  error('T and Xin must have the same number of rows');
end

sT = size(T,1);

if (nargin == 6)
  T   = [repmat(NaN,Nc+Ts,size(T,2)) ; T ; repmat(NaN,Na,size(T,2))];
  Xin = [repmat(NaN,Nc+Ts,size(Xin,2)) ; Xin ; repmat(NaN,Na,size(Xin,2))];
end

temp = [];
if (nargout > 1)
  if (~isempty(Xin))
    for i = 1:(Nc+Na)
      temp = [Xin(i:end-(Ts+Na+Nc-i+1),:) temp];
    end  
    Xin = temp; 
  end
end

T = T(Nc+1+Ts:end-Na,:);
